@@ -340,7 +340,8 @@ do
 done
 
 # Currently fmt::format is faster both at compile and runtime
-find $ROOT_PATH/{src,base,programs,utils} -name '*.h' -or -name '*.cpp' | grep -vP $EXCLUDE | xargs grep -l "std::format" | while read -r file;
+EXCLUDE_STD_FORMAT='HTTPHandler'
+find $ROOT_PATH/{src,base,programs,utils} -name '*.h' -or -name '*.cpp' | grep -vP $EXCLUDE | grep -vP $EXCLUDE_STD_FORMAT | xargs grep -l "std::format" | while read -r file;
 do
     echo "Found the usage of std::format in '${file}'. Please use fmt::format instead"
 done
