@@ -69,10 +69,21 @@ class Runner:
             filtered_jobs={},
             custom_data={},
         )
+        # Extract repository name from git remote (format: owner/repo)
+        repo_url = Shell.get_output("git config --get remote.origin.url")
+        repo_name = ""
+        if repo_url:
+            # Handle both HTTPS and SSH formats
+            # HTTPS: https://github.com/owner/repo.git
+            # SSH: git@github.com:owner/repo.git
+            match = re.search(r"[:/]([^/]+/[^/]+?)(\.git)?$", repo_url)
+            if match:
+                repo_name = match.group(1)
+
         _Environment(
             WORKFLOW_NAME=workflow.name,
             JOB_NAME=job.name,
-            REPOSITORY="",
+            REPOSITORY=repo_name,
             BRANCH=branch,
             SHA=sha or Shell.get_output("git rev-parse HEAD"),
             PR_NUMBER=pr if not branch else 0,
@@ -94,7 +105,7 @@ class Runner:
             FORK_NAME="",
             PR_LABELS=[],
             EVENT_TIME="",
-            WORKFLOW_DATA={
+            WORKFLOW_STATUS_DATA={
                 Utils.normalize_string(Settings.CI_CONFIG_JOB_NAME): {
                     "outputs": {
                         "data": json.dumps(
@@ -239,6 +250,7 @@ class Runner:
         debug=False,
         path="",
         path_1="",
+        workers=None,
     ):
         # re-set envs for local run
         env = _Environment.get()
@@ -265,6 +277,9 @@ class Runner:
                     f"Custom param for local tests must be of type str, got [{type(param)}]"
                 )
 
+        if job.enable_gh_auth:
+            _GH_Auth(workflow=workflow)
+
         if job.run_in_docker and not no_docker:
             job.run_in_docker, docker_settings = (
                 job.run_in_docker.split("+")[0],
@@ -303,6 +318,11 @@ class Runner:
                 "docker ps -a --format '{{.Names}}' | grep -q praktika && docker rm -f praktika",
                 verbose=True,
             )
+            if job.enable_gh_auth:
+                # pass gh auth seamlessly into the docker container
+                gh_mount = "--volume ~/.config/gh:/ghconfig -e GH_CONFIG_DIR=/ghconfig"
+            else:
+                gh_mount = ""
             # enable tty mode & interactive for docker if we have real tty
             tty = ""
             if preserve_stdio:
@@ -313,7 +333,7 @@ class Runner:
             for p_ in [path, path_1]:
                 if p_ and Path(p_).exists() and p_.startswith("/"):
                     extra_mounts += f" --volume {p_}:{p_}"
-            cmd = f"docker run {tty} --rm --name praktika {'--user $(id -u):$(id -g)' if not from_root else ''} -e PYTHONUNBUFFERED=1 -e PYTHONPATH='.:./ci' --volume ./:{current_dir} {extra_mounts} --workdir={current_dir} {' '.join(settings)} {docker} {job.command}"
+            cmd = f"docker run {tty} --rm --name praktika {'--user $(id -u):$(id -g)' if not from_root else ''} -e PYTHONUNBUFFERED=1 -e PYTHONPATH='.:./ci' --volume ./:{current_dir} {extra_mounts} {gh_mount} --workdir={current_dir} {' '.join(settings)} {docker} {job.command}"
         else:
             cmd = job.command
             python_path = os.getenv("PYTHONPATH", ":")
@@ -337,6 +357,9 @@ class Runner:
         if path_1:
             print(f"Custom --path_1 [{path_1}] will be passed to job's script")
             cmd += f" --path_1 {path_1}"
+        if workers is not None:
+            print(f"Custom --workers [{workers}] will be passed to job's script")
+            cmd += f" --workers {workers}"
         print(f"--- Run command [{cmd}]")
 
         with TeePopen(
@@ -572,6 +595,7 @@ class Runner:
                                     test_case_result.name,
                                     url=Settings.CI_DB_READ_URL,
                                     user=Settings.CI_DB_READ_USER,
+                                    job_name=job.name,
                                 ),
                             )
                     result.dump()
@@ -704,6 +728,7 @@ class Runner:
         debug=False,
         path="",
         path_1="",
+        workers=None,
     ):
         res = True
         setup_env_code = -10
@@ -732,7 +757,7 @@ class Runner:
                 workflow, job, pr=pr, sha=sha, branch=branch
             )
 
-        if res and (not local_run or pr or sha or branch):
+        if res and (not local_run or ((pr or branch) and sha)):
             res = False
             print(f"=== Pre run script [{job.name}], workflow [{workflow.name}] ===")
             try:
@@ -761,6 +786,7 @@ class Runner:
                     debug=debug,
                     path=path,
                     path_1=path_1,
+                    workers=workers,
                 )
                 res = run_code == 0
                 if not res:
