@@ -2,6 +2,9 @@
 #define CURL_DISABLE_FTP
 #define CURL_DISABLE_TFTP
 #define CURL_DISABLE_LDAP
+#define CURL_DISABLE_SMB
+#define CURL_DISABLE_POP3
+#define CURL_DISABLE_SMTP
 #define CURL_EXTERN_SYMBOL __attribute__ ((__visibility__ ("default")))
 
 #define SIZEOF_SHORT 2
@@ -15,11 +18,13 @@
 #define HAVE_GETADDRINFO
 #define HAVE_LONGLONG
 #define HAVE_POLL_FINE
+#define HAVE_SELECT
 #define HAVE_SIGACTION
 #define HAVE_SIGNAL
 #define HAVE_SIGSETJMP
 #define HAVE_SOCKET
 #define HAVE_STRUCT_TIMEVAL
+#define HAVE_POLL
 
 #define HAVE_RECV
 #define RECV_TYPE_ARG1 int
@@ -38,13 +43,23 @@
 
 #define HAVE_ARPA_INET_H
 #define HAVE_ERRNO_H
+#define HAVE_GETSOCKNAME
 #define HAVE_FCNTL_H
 #define HAVE_NETDB_H
 #define HAVE_NETINET_IN_H
+#define HAVE_SELECT_H
+#define HAVE_SETJMP_H
 #define HAVE_SETJMP_H
-#define HAVE_SYS_STAT_H
 #define HAVE_UNISTD_H
+#define HAVE_POLL_H
+#define HAVE_PTHREAD_H
 
 #define ENABLE_IPV6
 #define USE_OPENSSL
 #define USE_THREADS_POSIX
+#define USE_ARES
+
+#ifdef __illumos__
+#define HAVE_POSIX_STRERROR_R 1
+#define HAVE_STRERROR_R 1
+#endif
