@@ -1,6 +1,22 @@
 #!/bin/sh
 set -e
 
+# Parse optional --max-procs argument (defaults to 64)
+MAX_PROCS=64
+while [ $# -gt 0 ]; do
+    case "$1" in
+        --max-procs)
+            MAX_PROCS="$2"
+            shift 2
+            ;;
+        *)
+            echo "Unknown argument: $1"
+            echo "Usage: $0 [--max-procs NUM]"
+            exit 1
+            ;;
+    esac
+done
+
 SCRIPT_PATH=$(realpath "$0")
 SCRIPT_DIR=$(dirname "${SCRIPT_PATH}")
 GIT_DIR=$(git -C "$SCRIPT_DIR" rev-parse --show-toplevel)
@@ -19,7 +35,7 @@ git submodule sync
 #       It may cause unexpected behavior. Instead you need to commit a new SHA1 for a submodule.
 #
 #       [1] - https://git-scm.com/book/en/v2/Git-Tools-Submodules
-git config --file .gitmodules --get-regexp '.*path' | sed 's/[^ ]* //' | xargs -I _ --max-procs 64 git submodule update --depth=1 --single-branch _
+git config --file .gitmodules --get-regexp '.*path' | sed 's/[^ ]* //' | xargs -I _ --max-procs $MAX_PROCS git submodule update --depth=1 --single-branch _
 
 # We don't want to depend on any third-party CMake files.
 # To check it, find and delete them.
