@@ -1,14 +1,20 @@
 #include <Access/ExternalAuthenticators.h>
 #include <Access/LDAPClient.h>
+#include <Access/SettingsAuthResponseParser.h>
+#include <Access/resolveSetting.h>
 #include <Common/Exception.h>
+#include <Common/SettingsChanges.h>
+#include <Common/SipHash.h>
 #include <Common/quoteString.h>
-#include <Poco/Util/AbstractConfiguration.h>
+#include <Common/typeid_cast.h>
+#include <Interpreters/ClientInfo.h>
+
 #include <boost/algorithm/string/case_conv.hpp>
+#include <Poco/Util/AbstractConfiguration.h>
 
 #include <optional>
 #include <utility>
 
-
 namespace DB
 {
 
@@ -46,15 +52,16 @@ void parseLDAPSearchParams(LDAPClient::SearchParams & params, const Poco::Util::
         else if (scope == "subtree")   params.scope = LDAPClient::SearchParams::Scope::SUBTREE;
         else if (scope == "children")  params.scope = LDAPClient::SearchParams::Scope::CHILDREN;
         else
-            throw Exception("Invalid value for 'scope' field of LDAP search parameters in '" + prefix +
-                "' section, must be one of 'base', 'one_level', 'subtree', or 'children'", ErrorCodes::BAD_ARGUMENTS);
+            throw Exception(ErrorCodes::BAD_ARGUMENTS,
+                            "Invalid value for 'scope' field of LDAP search parameters "
+                            "in '{}' section, must be one of 'base', 'one_level', 'subtree', or 'children'", prefix);
     }
 }
 
 void parseLDAPServer(LDAPClient::Params & params, const Poco::Util::AbstractConfiguration & config, const String & name)
 {
     if (name.empty())
-        throw Exception("LDAP server name cannot be empty", ErrorCodes::BAD_ARGUMENTS);
+        throw Exception(ErrorCodes::BAD_ARGUMENTS, "LDAP server name cannot be empty");
 
     const String ldap_server_config = "ldap_servers." + name;
 
@@ -73,26 +80,27 @@ void parseLDAPServer(LDAPClient::Params & params, const Poco::Util::AbstractConf
     const bool has_tls_ca_cert_file = config.has(ldap_server_config + ".tls_ca_cert_file");
     const bool has_tls_ca_cert_dir = config.has(ldap_server_config + ".tls_ca_cert_dir");
     const bool has_tls_cipher_suite = config.has(ldap_server_config + ".tls_cipher_suite");
+    const bool has_search_limit = config.has(ldap_server_config + ".search_limit");
 
     if (!has_host)
-        throw Exception("Missing 'host' entry", ErrorCodes::BAD_ARGUMENTS);
+        throw Exception(ErrorCodes::BAD_ARGUMENTS, "Missing 'host' entry");
 
     params.host = config.getString(ldap_server_config + ".host");
 
     if (params.host.empty())
-        throw Exception("Empty 'host' entry", ErrorCodes::BAD_ARGUMENTS);
+        throw Exception(ErrorCodes::BAD_ARGUMENTS, "Empty 'host' entry");
 
     if (has_bind_dn)
     {
         if (has_auth_dn_prefix || has_auth_dn_suffix)
-            throw Exception("Deprecated 'auth_dn_prefix' and 'auth_dn_suffix' entries cannot be used with 'bind_dn' entry", ErrorCodes::BAD_ARGUMENTS);
+            throw Exception(ErrorCodes::BAD_ARGUMENTS, "Deprecated 'auth_dn_prefix' and 'auth_dn_suffix' entries cannot be used with 'bind_dn' entry");
 
         params.bind_dn = config.getString(ldap_server_config + ".bind_dn");
     }
     else if (has_auth_dn_prefix || has_auth_dn_suffix)
     {
-        const auto auth_dn_prefix = config.getString(ldap_server_config + ".auth_dn_prefix");
-        const auto auth_dn_suffix = config.getString(ldap_server_config + ".auth_dn_suffix");
+        std::string auth_dn_prefix = config.getString(ldap_server_config + ".auth_dn_prefix");
+        std::string auth_dn_suffix = config.getString(ldap_server_config + ".auth_dn_suffix");
         params.bind_dn = auth_dn_prefix + "{user_name}" + auth_dn_suffix;
     }
 
@@ -118,7 +126,7 @@ void parseLDAPServer(LDAPClient::Params & params, const Poco::Util::AbstractConf
         if (enable_tls_lc_str == "starttls")
             params.enable_tls = LDAPClient::Params::TLSEnable::YES_STARTTLS;
         else if (config.getBool(ldap_server_config + ".enable_tls"))
-            params.enable_tls = LDAPClient::Params::TLSEnable::YES; //-V1048
+            params.enable_tls = LDAPClient::Params::TLSEnable::YES;
         else
             params.enable_tls = LDAPClient::Params::TLSEnable::NO;
     }
@@ -137,9 +145,11 @@ void parseLDAPServer(LDAPClient::Params & params, const Poco::Util::AbstractConf
         else if (tls_minimum_protocol_version_lc_str == "tls1.1")
             params.tls_minimum_protocol_version = LDAPClient::Params::TLSProtocolVersion::TLS1_1;
         else if (tls_minimum_protocol_version_lc_str == "tls1.2")
-            params.tls_minimum_protocol_version = LDAPClient::Params::TLSProtocolVersion::TLS1_2; //-V1048
+            params.tls_minimum_protocol_version = LDAPClient::Params::TLSProtocolVersion::TLS1_2;
         else
-            throw Exception("Bad value for 'tls_minimum_protocol_version' entry, allowed values are: 'ssl2', 'ssl3', 'tls1.0', 'tls1.1', 'tls1.2'", ErrorCodes::BAD_ARGUMENTS);
+            throw Exception(ErrorCodes::BAD_ARGUMENTS,
+                            "Bad value for 'tls_minimum_protocol_version' entry, allowed values are: "
+                            "'ssl2', 'ssl3', 'tls1.0', 'tls1.1', 'tls1.2'");
     }
 
     if (has_tls_require_cert)
@@ -154,9 +164,11 @@ void parseLDAPServer(LDAPClient::Params & params, const Poco::Util::AbstractConf
         else if (tls_require_cert_lc_str == "try")
             params.tls_require_cert = LDAPClient::Params::TLSRequireCert::TRY;
         else if (tls_require_cert_lc_str == "demand")
-            params.tls_require_cert = LDAPClient::Params::TLSRequireCert::DEMAND; //-V1048
+            params.tls_require_cert = LDAPClient::Params::TLSRequireCert::DEMAND;
         else
-            throw Exception("Bad value for 'tls_require_cert' entry, allowed values are: 'never', 'allow', 'try', 'demand'", ErrorCodes::BAD_ARGUMENTS);
+            throw Exception(ErrorCodes::BAD_ARGUMENTS,
+                            "Bad value for 'tls_require_cert' entry, allowed values are: "
+                            "'never', 'allow', 'try', 'demand'");
     }
 
     if (has_tls_cert_file)
@@ -176,14 +188,17 @@ void parseLDAPServer(LDAPClient::Params & params, const Poco::Util::AbstractConf
 
     if (has_port)
     {
-        const auto port = config.getInt64(ldap_server_config + ".port");
-        if (port < 0 || port > 65535)
-            throw Exception("Bad value for 'port' entry", ErrorCodes::BAD_ARGUMENTS);
+        UInt32 port = config.getUInt(ldap_server_config + ".port");
+        if (port > 65535)
+            throw Exception(ErrorCodes::BAD_ARGUMENTS, "Bad value for 'port' entry");
 
         params.port = port;
     }
     else
         params.port = (params.enable_tls == LDAPClient::Params::TLSEnable::YES ? 636 : 389);
+
+    if (has_search_limit)
+        params.search_limit = static_cast<UInt32>(config.getUInt64(ldap_server_config + ".search_limit"));
 }
 
 void parseKerberosParams(GSSAcceptorContext::Params & params, const Poco::Util::AbstractConfiguration & config)
@@ -207,16 +222,46 @@ void parseKerberosParams(GSSAcceptorContext::Params & params, const Poco::Util::
     }
 
     if (reealm_key_count > 0 && principal_keys_count > 0)
-        throw Exception("Realm and principal name cannot be specified simultaneously", ErrorCodes::BAD_ARGUMENTS);
+        throw Exception(ErrorCodes::BAD_ARGUMENTS, "Realm and principal name cannot be specified simultaneously");
 
     if (reealm_key_count > 1)
-        throw Exception("Multiple realm sections are not allowed", ErrorCodes::BAD_ARGUMENTS);
+        throw Exception(ErrorCodes::BAD_ARGUMENTS, "Multiple realm sections are not allowed");
 
     if (principal_keys_count > 1)
-        throw Exception("Multiple principal sections are not allowed", ErrorCodes::BAD_ARGUMENTS);
+        throw Exception(ErrorCodes::BAD_ARGUMENTS, "Multiple principal sections are not allowed");
 
     params.realm = config.getString("kerberos.realm", "");
     params.principal = config.getString("kerberos.principal", "");
+    params.keytab = config.getString("kerberos.keytab", "");
+}
+
+HTTPAuthClientParams parseHTTPAuthParams(const Poco::Util::AbstractConfiguration & config, const String & prefix)
+{
+    HTTPAuthClientParams http_auth_params;
+
+    http_auth_params.uri = config.getString(prefix + ".uri");
+
+    size_t connection_timeout_ms = config.getInt(prefix + ".connection_timeout_ms", 1000);
+    size_t receive_timeout_ms = config.getInt(prefix + ".receive_timeout_ms", 1000);
+    size_t send_timeout_ms = config.getInt(prefix + ".send_timeout_ms", 1000);
+    http_auth_params.timeouts = ConnectionTimeouts()
+        .withConnectionTimeout(Poco::Timespan(connection_timeout_ms * 1000))
+        .withReceiveTimeout(Poco::Timespan(receive_timeout_ms * 1000))
+        .withSendTimeout(Poco::Timespan(send_timeout_ms * 1000));
+
+    http_auth_params.max_tries = config.getInt(prefix + ".max_tries", 3);
+    http_auth_params.retry_initial_backoff_ms = config.getInt(prefix + ".retry_initial_backoff_ms", 50);
+    http_auth_params.retry_max_backoff_ms = config.getInt(prefix + ".retry_max_backoff_ms", 1000);
+
+    Strings forward_headers;
+    config.keys(prefix + ".forward_headers", forward_headers);
+    for (const auto & header : forward_headers)
+    {
+        String name = config.getString(prefix + ".forward_headers." + header);
+        http_auth_params.forward_headers.push_back(name);
+    }
+
+    return http_auth_params;
 }
 
 }
@@ -231,24 +276,32 @@ void parseLDAPRoleSearchParams(LDAPClient::RoleSearchParams & params, const Poco
         params.prefix = config.getString(prefix + ".prefix");
 }
 
-void ExternalAuthenticators::reset()
+void ExternalAuthenticators::resetImpl()
 {
-    std::scoped_lock lock(mutex);
     ldap_client_params_blueprint.clear();
     ldap_caches.clear();
     kerberos_params.reset();
 }
 
-void ExternalAuthenticators::setConfiguration(const Poco::Util::AbstractConfiguration & config, Poco::Logger * log)
+void ExternalAuthenticators::reset()
 {
-    std::scoped_lock lock(mutex);
-    reset();
+    std::lock_guard lock(mutex);
+    resetImpl();
+}
+
+void ExternalAuthenticators::setConfiguration(const Poco::Util::AbstractConfiguration & config, LoggerPtr log)
+{
+    std::lock_guard lock(mutex);
+    resetImpl();
 
     Poco::Util::AbstractConfiguration::Keys all_keys;
     config.keys("", all_keys);
 
     std::size_t ldap_servers_key_count = 0;
     std::size_t kerberos_keys_count = 0;
+    std::size_t http_auth_server_keys_count = 0;
+
+    const String http_auth_servers_config = "http_authentication_servers";
 
     for (auto key : all_keys)
     {
@@ -260,13 +313,33 @@ void ExternalAuthenticators::setConfiguration(const Poco::Util::AbstractConfigur
 
         ldap_servers_key_count += (key == "ldap_servers");
         kerberos_keys_count += (key == "kerberos");
+        http_auth_server_keys_count += (key == http_auth_servers_config);
     }
 
     if (ldap_servers_key_count > 1)
-        throw Exception("Multiple ldap_servers sections are not allowed", ErrorCodes::BAD_ARGUMENTS);
+        throw Exception(ErrorCodes::BAD_ARGUMENTS, "Multiple ldap_servers sections are not allowed");
 
     if (kerberos_keys_count > 1)
-        throw Exception("Multiple kerberos sections are not allowed", ErrorCodes::BAD_ARGUMENTS);
+        throw Exception(ErrorCodes::BAD_ARGUMENTS, "Multiple kerberos sections are not allowed");
+
+    if (http_auth_server_keys_count > 1)
+        throw Exception(ErrorCodes::BAD_ARGUMENTS, "Multiple http_authentication_servers sections are not allowed");
+
+    Poco::Util::AbstractConfiguration::Keys http_auth_server_names;
+    config.keys(http_auth_servers_config, http_auth_server_names);
+    http_auth_servers.clear();
+    for (const auto & http_auth_server_name : http_auth_server_names)
+    {
+        String prefix = fmt::format("{}.{}", http_auth_servers_config, http_auth_server_name);
+        try
+        {
+            http_auth_servers[http_auth_server_name] = parseHTTPAuthParams(config, prefix);
+        }
+        catch (...)
+        {
+            tryLogCurrentException(log, "Could not parse HTTP auth server" + backQuote(http_auth_server_name));
+        }
+    }
 
     Poco::Util::AbstractConfiguration::Keys ldap_server_names;
     config.keys("ldap_servers", ldap_server_names);
@@ -279,8 +352,8 @@ void ExternalAuthenticators::setConfiguration(const Poco::Util::AbstractConfigur
             if (bracket_pos != std::string::npos)
                 ldap_server_name.resize(bracket_pos);
 
-            if (ldap_client_params_blueprint.count(ldap_server_name) > 0)
-                throw Exception("Multiple LDAP servers with the same name are not allowed", ErrorCodes::BAD_ARGUMENTS);
+            if (ldap_client_params_blueprint.contains(ldap_server_name))
+                throw Exception(ErrorCodes::BAD_ARGUMENTS, "Multiple LDAP servers with the same name are not allowed");
 
             LDAPClient::Params ldap_client_params_tmp;
             parseLDAPServer(ldap_client_params_tmp, config, ldap_server_name);
@@ -308,32 +381,40 @@ void ExternalAuthenticators::setConfiguration(const Poco::Util::AbstractConfigur
     }
 }
 
+static UInt128 computeParamsHash(const LDAPClient::Params & params, const LDAPClient::RoleSearchParamsList * role_search_params)
+{
+    SipHash hash;
+    params.updateHash(hash);
+    if (role_search_params)
+    {
+        for (const auto & params_instance : *role_search_params)
+        {
+            params_instance.updateHash(hash);
+        }
+    }
+
+    return hash.get128();
+}
+
 bool ExternalAuthenticators::checkLDAPCredentials(const String & server, const BasicCredentials & credentials,
     const LDAPClient::RoleSearchParamsList * role_search_params, LDAPClient::SearchResultsList * role_search_results) const
 {
     std::optional<LDAPClient::Params> params;
-    std::size_t params_hash = 0;
+    UInt128 params_hash = 0;
 
     {
-        std::scoped_lock lock(mutex);
+        std::lock_guard lock(mutex);
 
         // Retrieve the server parameters.
         const auto pit = ldap_client_params_blueprint.find(server);
         if (pit == ldap_client_params_blueprint.end())
-            throw Exception("LDAP server '" + server + "' is not configured", ErrorCodes::BAD_ARGUMENTS);
+            throw Exception(ErrorCodes::BAD_ARGUMENTS, "LDAP server '{}' is not configured", server);
 
         params = pit->second;
         params->user = credentials.getUserName();
         params->password = credentials.getPassword();
 
-        params->combineCoreHash(params_hash);
-        if (role_search_params)
-        {
-            for (const auto & params_instance : *role_search_params)
-            {
-                params_instance.combineHash(params_hash);
-            }
-        }
+        params_hash = computeParamsHash(*params, role_search_params);
 
         // Check the cache, but only if the caching is enabled at all.
         if (params->verification_cooldown > std::chrono::seconds{0})
@@ -392,7 +473,7 @@ bool ExternalAuthenticators::checkLDAPCredentials(const String & server, const B
     // Update the cache, but only if this is the latest check and the server is still configured in a compatible way.
     if (result)
     {
-        std::scoped_lock lock(mutex);
+        std::lock_guard lock(mutex);
 
         // If the server was removed from the config while we were checking the password, we discard the current result.
         const auto pit = ldap_client_params_blueprint.find(server);
@@ -403,15 +484,7 @@ bool ExternalAuthenticators::checkLDAPCredentials(const String & server, const B
         new_params.user = credentials.getUserName();
         new_params.password = credentials.getPassword();
 
-        std::size_t new_params_hash = 0;
-        new_params.combineCoreHash(new_params_hash);
-        if (role_search_params)
-        {
-            for (const auto & params_instance : *role_search_params)
-            {
-                params_instance.combineHash(new_params_hash);
-            }
-        }
+        const UInt128 new_params_hash = computeParamsHash(new_params, role_search_params);
 
         // If the critical server params have changed while we were checking the password, we discard the current result.
         if (params_hash != new_params_hash)
@@ -447,10 +520,10 @@ bool ExternalAuthenticators::checkLDAPCredentials(const String & server, const B
 
 bool ExternalAuthenticators::checkKerberosCredentials(const String & realm, const GSSAcceptorContext & credentials) const
 {
-    std::scoped_lock lock(mutex);
+    std::lock_guard lock(mutex);
 
     if (!kerberos_params.has_value())
-        throw Exception("Kerberos is not enabled", ErrorCodes::BAD_ARGUMENTS);
+        throw Exception(ErrorCodes::BAD_ARGUMENTS, "Kerberos is not enabled");
 
     if (!credentials.isReady())
         return false;
@@ -466,12 +539,35 @@ bool ExternalAuthenticators::checkKerberosCredentials(const String & realm, cons
 
 GSSAcceptorContext::Params ExternalAuthenticators::getKerberosParams() const
 {
-    std::scoped_lock lock(mutex);
+    std::lock_guard lock(mutex);
 
     if (!kerberos_params.has_value())
-        throw Exception("Kerberos is not enabled", ErrorCodes::BAD_ARGUMENTS);
+        throw Exception(ErrorCodes::BAD_ARGUMENTS, "Kerberos is not enabled");
 
     return kerberos_params.value();
 }
 
+HTTPAuthClientParams ExternalAuthenticators::getHTTPAuthenticationParams(const String& server) const
+{
+    std::lock_guard lock{mutex};
+
+    const auto it = http_auth_servers.find(server);
+    if (it == http_auth_servers.end())
+        throw Exception(ErrorCodes::BAD_ARGUMENTS, "HTTP server '{}' is not configured", server);
+    return it->second;
+}
+
+bool ExternalAuthenticators::checkHTTPBasicCredentials(
+    const String & server, const BasicCredentials & credentials, const ClientInfo & client_info, SettingsChanges & settings) const
+{
+    auto params = getHTTPAuthenticationParams(server);
+    HTTPBasicAuthClient<SettingsAuthResponseParser> client(params);
+
+    auto [is_ok, settings_from_auth_server] = client.authenticate(credentials.getUserName(), credentials.getPassword(), client_info.http_headers);
+
+    if (is_ok)
+        std::ranges::move(settings_from_auth_server, std::back_inserter(settings));
+
+    return is_ok;
+}
 }
