@@ -20,6 +20,15 @@ struct SettingsProfile : public IAccessEntity
     std::shared_ptr<IAccessEntity> clone() const override { return cloneImpl<SettingsProfile>(); }
     static constexpr const auto TYPE = AccessEntityType::SETTINGS_PROFILE;
     AccessEntityType getType() const override { return TYPE; }
+
+    std::vector<UUID> findDependencies() const override;
+    bool hasDependencies(const std::unordered_set<UUID> & ids) const override;
+    void replaceDependencies(const std::unordered_map<UUID, UUID> & old_to_new_ids) override;
+    void copyDependenciesFrom(const IAccessEntity & src, const std::unordered_set<UUID> & ids) override;
+    void removeDependencies(const std::unordered_set<UUID> & ids) override;
+    void clearAllExceptDependencies() override;
+
+    bool isBackupAllowed() const override { return elements.isBackupAllowed(); }
 };
 
 using SettingsProfilePtr = std::shared_ptr<const SettingsProfile>;
