@@ -27,7 +27,6 @@
 #include <Poco/JSON/Object.h>
 #include <Poco/JSON/Stringifier.h>
 #include <cstring>
-#include <filesystem>
 #include <base/FnTraits.h>
 #include <base/range.h>
 #include <unordered_set>
@@ -48,7 +47,7 @@ namespace ErrorCodes
 namespace
 {
     template <typename T>
-    void parseGrant(T & entity, const String & string_query, const std::unordered_set<UUID> & allowed_role_ids)
+    void parseGrant(T & entity, const String & string_query, const std::unordered_set<UUID> & role_ids_from_users_config, const AccessControl & access_control, LoggerPtr log)
     {
         ParserGrantQuery parser;
         parser.setParseWithoutGrantees();
@@ -87,8 +86,13 @@ namespace
             for (const auto & role_name : query.roles->names)
             {
                 auto role_id = UsersConfigParser::generateID(AccessEntityType::ROLE, role_name);
-                if (!allowed_role_ids.contains(role_id))
-                    throw Exception(ErrorCodes::THERE_IS_NO_PROFILE, "Role {} was not found", role_name);
+                if (!role_ids_from_users_config.contains(role_id))
+                {
+                    if (const auto role = access_control.find<Role>(role_name))
+                        role_id = *role;
+                    else
+                        LOG_WARNING(log, "Role {} is not defined and will be ignored for grant query '{}'.", role_name, string_query);
+                }
 
                 roles_to_grant.push_back(role_id);
             }
@@ -104,9 +108,11 @@ namespace
         const Poco::Util::AbstractConfiguration & config,
         String user_name,
         const std::unordered_set<UUID> & allowed_profile_ids,
-        const std::unordered_set<UUID> & allowed_role_ids,
+        const std::unordered_set<UUID> & role_ids_from_users_config,
+        const AccessControl & access_control,
         bool allow_no_password,
-        bool allow_plaintext_password)
+        bool allow_plaintext_password,
+        LoggerPtr log)
     {
         const bool validate = true;
         auto user = std::make_shared<User>();
@@ -378,7 +384,7 @@ namespace
         if (grant_queries)
         {
             for (const auto & string_query : *grant_queries)
-                parseGrant(*user, string_query, allowed_role_ids);
+                parseGrant(*user, string_query, role_ids_from_users_config, access_control, log);
         }
         else
         {
@@ -429,7 +435,9 @@ namespace
     RolePtr parseRole(
         const Poco::Util::AbstractConfiguration & config,
         const String & role_name,
-        const std::unordered_set<UUID> & allowed_role_ids)
+        const std::unordered_set<UUID> & role_ids_from_users_config,
+        const AccessControl & access_control,
+        LoggerPtr log)
     {
         auto role = std::make_shared<Role>();
         role->setName(role_name);
@@ -444,7 +452,7 @@ namespace
             for (const auto & key : keys)
             {
                 const auto query = config.getString(grants_config + "." + key);
-                parseGrant(*role, query, allowed_role_ids);
+                parseGrant(*role, query, role_ids_from_users_config, access_control, log);
             }
         }
 
@@ -611,7 +619,11 @@ namespace
 }
 
 
-UsersConfigParser::UsersConfigParser(AccessControl & access_control_) : access_control(access_control_) {}
+UsersConfigParser::UsersConfigParser(AccessControl & access_control_)
+    : access_control(access_control_)
+    , log(getLogger("UsersConfigParser"))
+{
+}
 
 UUID UsersConfigParser::generateID(AccessEntityType type, const String & name)
 {
@@ -632,7 +644,7 @@ UUID UsersConfigParser::generateID(const IAccessEntity & entity) { return genera
 std::vector<AccessEntityPtr> UsersConfigParser::parseUsers(
     const Poco::Util::AbstractConfiguration & config,
     const std::unordered_set<UUID> & allowed_profile_ids,
-    const std::unordered_set<UUID> & allowed_role_ids) const
+    const std::unordered_set<UUID> & role_ids_from_users_config) const
 {
     Poco::Util::AbstractConfiguration::Keys user_names;
     config.keys("users", user_names);
@@ -646,7 +658,7 @@ std::vector<AccessEntityPtr> UsersConfigParser::parseUsers(
     {
         try
         {
-            users.push_back(parseUser(config, user_name, allowed_profile_ids, allowed_role_ids, no_password_allowed, plaintext_password_allowed));
+            users.push_back(parseUser(config, user_name, allowed_profile_ids, role_ids_from_users_config, access_control, no_password_allowed, plaintext_password_allowed, log));
         }
         catch (Exception & e)
         {
@@ -660,7 +672,7 @@ std::vector<AccessEntityPtr> UsersConfigParser::parseUsers(
 
 std::vector<AccessEntityPtr> UsersConfigParser::parseRoles(
     const Poco::Util::AbstractConfiguration & config,
-    const std::unordered_set<UUID> & allowed_role_ids) const
+    const std::unordered_set<UUID> & role_ids_from_users_config) const
 {
     Poco::Util::AbstractConfiguration::Keys role_names;
     config.keys("roles", role_names);
@@ -671,7 +683,7 @@ std::vector<AccessEntityPtr> UsersConfigParser::parseRoles(
     {
         try
         {
-            roles.push_back(parseRole(config, role_name, allowed_role_ids));
+            roles.push_back(parseRole(config, role_name, role_ids_from_users_config, access_control, log));
         }
         catch (Exception & e)
         {
