@@ -7,6 +7,8 @@
 #include <Client/BuzzHouse/Utils/HugeInt.h>
 #include <Client/BuzzHouse/Utils/UHugeInt.h>
 
+#include <IO/ReadBufferFromFile.h>
+#include <IO/WriteBufferFromString.h>
 #include <IO/copyData.h>
 #include <base/scope_guard.h>
 #include <Poco/Net/HTTPClientSession.h>
@@ -69,7 +71,7 @@ void ClickHouseIntegratedDatabase::swapTableDefinitions(RandomGenerator & rg, Cr
         {
             SetValue & sv = const_cast<SetValue &>(i == 0 ? svs.set_value() : svs.other_values(i - 1));
 
-            if (allSettings.find(sv.property()) != allSettings.end())
+            if (allSettings.contains(sv.property()))
             {
                 const CHSetting & chs = allSettings.at(sv.property());
 
@@ -140,13 +142,16 @@ void ClickHouseIntegratedDatabase::swapTableDefinitions(RandomGenerator & rg, Cr
     }
     if (newt.has_table_def())
     {
-        const TableDef & def = newt.table_def();
+        std::vector<TableDefItem> items_to_keep;
+        TableDef & def = const_cast<TableDef &>(newt.table_def());
 
         for (int i = 0; i < def.table_defs_size(); i++)
         {
-            if (def.table_defs(i).has_col_def())
+            const auto & next = def.table_defs(i);
+
+            if (next.has_col_def())
             {
-                ColumnDef & cdef = const_cast<ColumnDef &>(def.table_defs(i).col_def());
+                ColumnDef & cdef = const_cast<ColumnDef &>(next.col_def());
                 TopTypeName & ttn = const_cast<TopTypeName &>(cdef.type().type());
 
                 if (cdef.has_codecs() && rg.nextBool())
@@ -184,7 +189,7 @@ void ClickHouseIntegratedDatabase::swapTableDefinitions(RandomGenerator & rg, Cr
                         {
                             SetValue & sv = const_cast<SetValue &>(j == 0 ? svs.set_value() : svs.other_values(j - 1));
 
-                            if (allSettings.find(sv.property()) != allSettings.end())
+                            if (allSettings.contains(sv.property()))
                             {
                                 const CHSetting & chs = allSettings.at(sv.property());
 
@@ -197,7 +202,18 @@ void ClickHouseIntegratedDatabase::swapTableDefinitions(RandomGenerator & rg, Cr
                         }
                     }
                 }
+                /// Keep all columns
+                items_to_keep.emplace_back(next);
             }
+            else if (rg.nextSmallNumber() < 8)
+            {
+                items_to_keep.emplace_back(next);
+            }
+        }
+        def.clear_table_defs();
+        for (const auto & item : items_to_keep)
+        {
+            *def.add_table_defs() = item;
         }
     }
     if (newt.has_cluster() && rg.nextSmallNumber() < 4)
@@ -250,7 +266,7 @@ bool ClickHouseIntegratedDatabase::performCreatePeerTable(
             {
                 t.db->setName(est.mutable_database());
             }
-            if (rg.nextMediumNumber() < 91)
+            if (!fc.measure_performance && !fc.compare_explains && rg.nextMediumNumber() < 91)
             {
                 this->swapTableDefinitions(rg, newt);
             }
@@ -367,7 +383,8 @@ bool MySQLIntegration::optimizeTableForOracle(const PeerTableDatabase pt, const
     if (is_clickhouse && t.isMergeTreeFamily())
     {
         /// Sometimes the optimize step doesn't have to do anything, then throws error. Ignore it
-        const auto u = performQueryOnServerOrRemote(pt, fmt::format("ALTER TABLE {} APPLY DELETED MASK;", getTableName(t.db, t.tname)));
+        const auto u = performQueryOnServerOrRemote(
+            pt, fmt::format("ALTER TABLE {} APPLY DELETED MASK SETTINGS mutations_sync = 2;", getTableName(t.db, t.tname)));
         const auto v = performQueryOnServerOrRemote(
             pt, fmt::format("OPTIMIZE TABLE {}{};", getTableName(t.db, t.tname), t.supportsFinal() ? " FINAL" : ""));
         UNUSED(u);
@@ -1189,7 +1206,7 @@ void MongoDBIntegration::documentAppendBottomType(RandomGenerator & rg, const St
     }
     else
     {
-        chassert(0);
+        UNREACHABLE();
     }
 }
 
@@ -1335,7 +1352,7 @@ void MongoDBIntegration::documentAppendAnyValue(
     }
     else
     {
-        chassert(0);
+        UNREACHABLE();
     }
 }
 
@@ -1495,7 +1512,7 @@ bool DolorIntegration::performDatabaseIntegration(RandomGenerator & rg, SQLDatab
             catalog = "unity";
             break;
         default:
-            chassert(0);
+            UNREACHABLE();
     }
     buf += fmt::format(
         R"({{"seed":{},"database_name":"{}","storage":"{}","lake":"{}","catalog":"{}"}})",
@@ -1513,19 +1530,20 @@ bool DolorIntegration::reRunCreateDatabase(const String & body)
     return httpPut("/sparkdatabase", body);
 }
 
+static const DB::Strings & catalogs = {"glue", "hive", "rest", "unity"};
+
 void DolorIntegration::setDatabaseDetails(RandomGenerator & rg, const SQLDatabase & d, DatabaseEngine * de, SettingValues * svs)
 {
+    String catalog_str;
     const Catalog * cat = nullptr;
-    SetValue * sv1 = svs->has_set_value() ? svs->add_other_values() : svs->mutable_set_value();
-    SetValue * sv2 = svs->add_other_values();
 
-    sv1->set_property("catalog_type");
+    chassert(d.storage == LakeStorage::S3);
     switch (d.catalog)
     {
         case LakeCatalog::Glue:
             cat = &sc.glue_catalog.value();
             de->add_params()->set_svalue(fmt::format("http://{}:{}", cat->server_hostname, cat->port));
-            sv1->set_value("'glue'");
+            catalog_str = "glue";
             break;
         case LakeCatalog::Hive:
             cat = &sc.hive_catalog.value();
@@ -1533,50 +1551,91 @@ void DolorIntegration::setDatabaseDetails(RandomGenerator & rg, const SQLDatabas
             {
                 de->add_params()->set_svalue(fmt::format("thrift://{}:{}", cat->server_hostname, cat->port));
             }
-            sv1->set_value("'hive'");
+            catalog_str = "hive";
             break;
         case LakeCatalog::REST:
             cat = &sc.rest_catalog.value();
             de->add_params()->set_svalue(fmt::format("http://{}:{}{}", cat->server_hostname, cat->port, cat->path));
-            sv1->set_value("'rest'");
+            catalog_str = "rest";
             break;
         case LakeCatalog::Unity:
             cat = &sc.unity_catalog.value();
             de->add_params()->set_svalue(
                 fmt::format(
                     "http://{}:{}{}{}", cat->server_hostname, cat->port, cat->path, d.format == LakeFormat::Iceberg ? "/iceberg" : ""));
-            sv1->set_value(fmt::format("'{}'", d.format == LakeFormat::Iceberg ? "rest" : "unity"));
+            catalog_str = d.format == LakeFormat::Iceberg ? "rest" : "unity";
             break;
         default:
-            chassert(0);
+            UNREACHABLE();
     }
-    sv2->set_property("warehouse");
-    sv2->set_value("'" + d.getName() + "'");
-    chassert(d.storage == LakeStorage::S3);
-    if (d.format == LakeFormat::Iceberg)
+
+    if (rg.nextMediumNumber() < 6)
     {
-        const ServerCredentials & minio = fc.minio_server.value();
-
-        de->add_params()->set_svalue(minio.user);
-        de->add_params()->set_svalue(minio.secret);
-
-        SetValue * sv3 = svs->add_other_values();
-        sv3->set_property("storage_endpoint");
-        sv3->set_value(fmt::format("'http://{}:{}/{}'", minio.server_hostname, minio.port, cat->warehouse));
+        /// Mess up with the catalog
+        catalog_str = rg.pickRandomly(catalogs);
     }
-    if (!cat->region.empty())
-    {
-        SetValue * sv4 = svs->add_other_values();
+    uint32_t added_type = 0;
+    uint32_t added_warehouse = 0;
+    uint32_t added_endpoint = 0;
+    uint32_t added_region = 0;
+    uint32_t added_credentials = 0;
+    const uint32_t toadd_type = rg.nextMediumNumber() < 96;
+    const uint32_t toadd_warehouse = rg.nextMediumNumber() < 96;
+    const uint32_t toadd_endpoint = d.format == LakeFormat::Iceberg && rg.nextMediumNumber() < 96;
+    const uint32_t toadd_region = !cat->region.empty() && rg.nextMediumNumber() < 96;
+    const uint32_t toadd_credentials = rg.nextSmallNumber() < 4;
+    const uint32_t total_to_add = toadd_type + toadd_warehouse + toadd_endpoint + toadd_region + toadd_credentials;
 
-        sv4->set_property("region");
-        sv4->set_value("'" + cat->region + "'");
-    }
-    if (d.catalog == LakeCatalog::Unity || rg.nextSmallNumber() < 4)
+    for (uint32_t i = 0; i < total_to_add; i++)
     {
-        SetValue * sv5 = svs->add_other_values();
+        SetValue * sv = svs->has_set_value() ? svs->add_other_values() : svs->mutable_set_value();
+        const uint32_t add_type = 3 * static_cast<uint32_t>(added_type < toadd_type);
+        const uint32_t add_warehouse = 3 * static_cast<uint32_t>(added_warehouse < toadd_warehouse);
+        const uint32_t add_endpoint = 3 * static_cast<uint32_t>(added_endpoint < toadd_endpoint);
+        const uint32_t add_region = 3 * static_cast<uint32_t>(added_region < toadd_region);
+        const uint32_t add_credentials = 3 * static_cast<uint32_t>(added_credentials < toadd_credentials);
+        const uint32_t prob_space = add_type + add_warehouse + add_endpoint + add_region + add_credentials;
+        std::uniform_int_distribution<uint32_t> next_dist(1, prob_space);
+        const uint32_t nopt = next_dist(rg.generator);
 
-        sv5->set_property("vended_credentials");
-        sv5->set_value(d.catalog == LakeCatalog::Unity || rg.nextBool() ? "1" : "0");
+        if (add_type && nopt < (add_type + 1))
+        {
+            sv->set_property("catalog_type");
+            sv->set_value("'" + catalog_str + "'");
+            added_type++;
+        }
+        else if (add_warehouse && nopt < (add_type + add_warehouse + 1))
+        {
+            sv->set_property("warehouse");
+            sv->set_value("'" + d.getName() + "'");
+            added_warehouse++;
+        }
+        else if (add_endpoint && nopt < (add_type + add_warehouse + add_endpoint + 1))
+        {
+            const ServerCredentials & minio = fc.minio_server.value();
+
+            de->add_params()->set_svalue(minio.user);
+            de->add_params()->set_svalue(minio.secret);
+            sv->set_property("storage_endpoint");
+            sv->set_value(fmt::format("'http://{}:{}/{}'", minio.server_hostname, minio.port, cat->warehouse));
+            added_endpoint++;
+        }
+        else if (add_region && nopt < (add_type + add_warehouse + add_endpoint + add_region + 1))
+        {
+            sv->set_property("region");
+            sv->set_value("'" + cat->region + "'");
+            added_region++;
+        }
+        else if (add_credentials && nopt < (add_type + add_warehouse + add_endpoint + add_region + add_credentials + 1))
+        {
+            sv->set_property("vended_credentials");
+            sv->set_value(rg.nextBool() ? "1" : "0");
+            added_credentials++;
+        }
+        else
+        {
+            UNREACHABLE();
+        }
     }
 }
 
@@ -1622,7 +1681,7 @@ bool DolorIntegration::reRunCreateTable(const String & body)
     return httpPut("/sparktable", body);
 }
 
-void DolorIntegration::setTableEngineDetails(RandomGenerator &, const SQLTable & t, TableEngine * te)
+void DolorIntegration::setTableEngineDetails(RandomGenerator & rg, const SQLTable & t, TableEngine * te)
 {
     const LakeCatalog catalog = t.getLakeCatalog();
 
@@ -1634,71 +1693,120 @@ void DolorIntegration::setTableEngineDetails(RandomGenerator &, const SQLTable &
     else
     {
         const Catalog * cat = nullptr;
+        String catalog_url;
+        String catalog_str;
         SettingValues * svs = te->mutable_setting_values();
-        SetValue * sv1 = svs->has_set_value() ? svs->add_other_values() : svs->mutable_set_value();
-        SetValue * sv2 = svs->add_other_values();
-        SetValue * sv3 = svs->add_other_values();
-        SetValue * sv4 = svs->add_other_values();
 
-        sv1->set_property("storage_catalog_type");
-        sv2->set_property("storage_warehouse");
-        sv3->set_property("object_storage_endpoint");
-        sv4->set_property("storage_catalog_url");
-        sv2->set_value("'" + t.getDatabaseName() + "'");
         switch (catalog)
         {
             case LakeCatalog::Glue:
                 cat = &sc.glue_catalog.value();
-                sv1->set_value("'glue'");
-                sv4->set_value(fmt::format("'http://{}:{}'", cat->server_hostname, cat->port));
+                catalog_str = "glue";
+                catalog_url = fmt::format("http://{}:{}", cat->server_hostname, cat->port);
                 break;
             case LakeCatalog::REST:
                 cat = &sc.rest_catalog.value();
-                sv1->set_value("'rest'");
-                sv4->set_value(fmt::format("'http://{}:{}{}'", cat->server_hostname, cat->port, cat->path));
+                catalog_str = "rest";
+                catalog_url = fmt::format("http://{}:{}{}", cat->server_hostname, cat->port, cat->path);
                 break;
             case LakeCatalog::Hive:
                 cat = &sc.hive_catalog.value();
-                sv1->set_value("'hive'");
-                sv4->set_value(fmt::format("thrift://{}:{}", cat->server_hostname, cat->port));
+                catalog_str = "hive";
+                catalog_url = fmt::format("thrift://{}:{}", cat->server_hostname, cat->port);
                 break;
             case LakeCatalog::Unity:
                 cat = &sc.unity_catalog.value();
-                sv1->set_value(fmt::format("'{}'", t.getPossibleLakeFormat() == LakeFormat::Iceberg ? "rest" : "unity"));
-                sv4->set_value(
-                    fmt::format(
-                        "'http://{}:{}{}{}'",
-                        cat->server_hostname,
-                        cat->port,
-                        cat->path,
-                        t.getPossibleLakeFormat() == LakeFormat::Iceberg ? "/iceberg" : ""));
+                catalog_str = t.getPossibleLakeFormat() == LakeFormat::Iceberg ? "rest" : "unity";
+                catalog_url = fmt::format(
+                    "http://{}:{}{}{}",
+                    cat->server_hostname,
+                    cat->port,
+                    cat->path,
+                    t.getPossibleLakeFormat() == LakeFormat::Iceberg ? "/iceberg" : "");
                 break;
             default:
-                chassert(0);
+                UNREACHABLE();
         }
-        /// The key-value format is not well supported for catalogs at the moment
-        const ServerCredentials & minio = fc.minio_server.value();
 
-        /// I don't know how to set for other storages
+        /// The other storages are not tested yet
         chassert(t.isOnS3());
-        te->add_params()->set_svalue(t.getTablePath(fc));
-        te->add_params()->set_svalue(minio.password);
-        te->add_params()->set_svalue(minio.secret);
-        if (t.isAnyIcebergEngine() && t.file_format.has_value())
+        if (rg.nextMediumNumber() < 6)
         {
-            te->add_params()->set_svalue(InOutFormat_Name(t.file_format.value()).substr(6));
-            if (t.file_comp.has_value())
-            {
-                te->add_params()->set_svalue(t.file_comp.value());
-            }
+            /// Mess up with the catalog
+            catalog_str = rg.pickRandomly(catalogs);
         }
-        sv3->set_value(fmt::format("'http://{}:{}/{}'", minio.server_hostname, minio.port, cat->warehouse));
-        if (!cat->region.empty())
-        {
-            SetValue * sv5 = svs->add_other_values();
+        uint32_t added_sct = 0;
+        uint32_t added_warehouse = 0;
+        uint32_t added_endpoint = 0;
+        uint32_t added_region = 0;
+        uint32_t added_url = 0;
+        const uint32_t toadd_sct = rg.nextMediumNumber() < 96;
+        const uint32_t toadd_warehouse = rg.nextMediumNumber() < 96;
+        const uint32_t toadd_endpoint = rg.nextMediumNumber() < 96;
+        const uint32_t toadd_region = !cat->region.empty() && rg.nextMediumNumber() < 96;
+        const uint32_t toadd_url = rg.nextMediumNumber() < 96;
+        const uint32_t total_to_add = toadd_sct + toadd_warehouse + toadd_endpoint + toadd_region + toadd_url;
 
-            sv5->set_property("storage_region");
-            sv5->set_value("'" + cat->region + "'");
+        for (uint32_t i = 0; i < total_to_add; i++)
+        {
+            SetValue * sv = svs->has_set_value() ? svs->add_other_values() : svs->mutable_set_value();
+            const uint32_t add_sct = 3 * static_cast<uint32_t>(added_sct < toadd_sct);
+            const uint32_t add_warehouse = 3 * static_cast<uint32_t>(added_warehouse < toadd_warehouse);
+            const uint32_t add_endpoint = 3 * static_cast<uint32_t>(added_endpoint < toadd_endpoint);
+            const uint32_t add_region = 3 * static_cast<uint32_t>(added_region < toadd_region);
+            const uint32_t add_url = 3 * static_cast<uint32_t>(added_url < toadd_url);
+            const uint32_t prob_space = add_sct + add_warehouse + add_endpoint + add_region + add_url;
+            std::uniform_int_distribution<uint32_t> next_dist(1, prob_space);
+            const uint32_t nopt = next_dist(rg.generator);
+
+            if (add_sct && nopt < (add_sct + 1))
+            {
+                sv->set_property("storage_catalog_type");
+                sv->set_value("'" + catalog_str + "'");
+                added_sct++;
+            }
+            else if (add_warehouse && nopt < (add_sct + add_warehouse + 1))
+            {
+                sv->set_property("storage_warehouse");
+                sv->set_value("'" + t.getDatabaseName() + "'");
+                added_warehouse++;
+            }
+            else if (add_endpoint && nopt < (add_sct + add_warehouse + add_endpoint + 1))
+            {
+                /// The key-value format is not well supported for catalogs at the moment
+                const ServerCredentials & minio = fc.minio_server.value();
+
+                te->add_params()->set_svalue(t.getTablePath(fc));
+                te->add_params()->set_svalue(minio.password);
+                te->add_params()->set_svalue(minio.secret);
+                if (t.isAnyIcebergEngine() && t.file_format.has_value() && rg.nextMediumNumber() < 96)
+                {
+                    te->add_params()->set_svalue(InOutFormat_Name(t.file_format.value()).substr(6));
+                    if (t.file_comp.has_value() && rg.nextMediumNumber() < 96)
+                    {
+                        te->add_params()->set_svalue(t.file_comp.value());
+                    }
+                }
+                sv->set_property("object_storage_endpoint");
+                sv->set_value(fmt::format("'http://{}:{}/{}'", minio.server_hostname, minio.port, cat->warehouse));
+                added_endpoint++;
+            }
+            else if (add_region && nopt < (add_sct + add_warehouse + add_endpoint + add_region + 1))
+            {
+                sv->set_property("storage_region");
+                sv->set_value("'" + cat->region + "'");
+                added_region++;
+            }
+            else if (add_url && nopt < (add_sct + add_warehouse + add_endpoint + add_region + add_url + 1))
+            {
+                sv->set_property("storage_catalog_url");
+                sv->set_value("'" + catalog_url + "'");
+                added_url++;
+            }
+            else
+            {
+                UNREACHABLE();
+            }
         }
     }
 }
@@ -1765,8 +1873,7 @@ void ExternalIntegrations::createExternalDatabase(RandomGenerator & rg, SQLDatab
             next = dolor.get();
             break;
         default:
-            chassert(0);
-            break;
+            UNREACHABLE();
     }
     requires_external_call_check++;
     next_calls_succeeded.emplace_back(next->performDatabaseIntegration(rg, d));
@@ -1808,8 +1915,7 @@ void ExternalIntegrations::createExternalDatabaseTable(
             next = dolor.get();
             break;
         default:
-            chassert(0);
-            break;
+            UNREACHABLE();
     }
     requires_external_call_check++;
     next_calls_succeeded.emplace_back(next->performTableIntegration(rg, t, true, entries));
@@ -1826,8 +1932,7 @@ bool ExternalIntegrations::reRunCreateDatabase(const IntegrationCall ic, const S
             next = dolor.get();
             break;
         default:
-            chassert(0);
-            break;
+            UNREACHABLE();
     }
     return next ? next->reRunCreateDatabase(body) : false;
 }
@@ -1842,8 +1947,7 @@ bool ExternalIntegrations::reRunCreateTable(const IntegrationCall ic, const Stri
             next = dolor.get();
             break;
         default:
-            chassert(0);
-            break;
+            UNREACHABLE();
     }
     return next ? next->reRunCreateTable(body) : false;
 }
@@ -1859,8 +1963,7 @@ bool ExternalIntegrations::performExternalCommand(
             next = dolor.get();
             break;
         default:
-            chassert(0);
-            break;
+            UNREACHABLE();
     }
     if (next)
     {
@@ -1935,8 +2038,7 @@ void ExternalIntegrations::setBackupDetails(const IntegrationCall dc, const Stri
             azurite->setBackupDetails(filename, br);
             break;
         default:
-            chassert(0);
-            break;
+            UNREACHABLE();
     }
 }
 
