@@ -117,7 +117,7 @@ const std::vector<CHAggregate> CHAggrs = {
     CHAggregate(SQLFunc::FUNCsumMapWithOverflow, 0, 0, 1, 2, false),
     CHAggregate(SQLFunc::FUNCsumMapFiltered, 1, 1, 2, 2, false),
     CHAggregate(SQLFunc::FUNCsumMapFilteredWithOverflow, 1, 1, 2, 2, false),
-    CHAggregate(SQLFunc::FUNCminMap, 0, 0, 2, 2, false),
+    CHAggregate(SQLFunc::FUNCminMap, 0, 0, 1, 2, false),
     CHAggregate(SQLFunc::FUNCmaxMap, 0, 0, 1, 2, false),
     CHAggregate(SQLFunc::FUNCskewSamp, 0, 0, 1, 1, false),
     CHAggregate(SQLFunc::FUNCskewPop, 0, 0, 1, 1, false),
@@ -178,7 +178,7 @@ const std::vector<CHAggregate> CHAggrs = {
     CHAggregate(SQLFunc::FUNCsparkBar, 1, 3, 2, 2, false),
     CHAggregate(SQLFunc::FUNCsumCount, 0, 0, 1, 1, false),
     CHAggregate(SQLFunc::FUNClargestTriangleThreeBuckets, 1, 1, 2, 2, false),
-    CHAggregate(SQLFunc::FUNCestimateCompressionRatio, 2, 2, 1, 1, false),
+    CHAggregate(SQLFunc::FUNCestimateCompressionRatio, 0, 2, 1, 1, false),
     CHAggregate(SQLFunc::FUNCgroupConcat, 0, 2, 1, 1, false),
     /// JSON and Dynamic tyoes
     CHAggregate(SQLFunc::FUNCdistinctDynamicTypes, 0, 0, 1, 1, false),
@@ -234,6 +234,7 @@ const std::vector<CHFunction> CHFuncs = {
     CHFunction(SQLFunc::FUNCmoduloOrNull, 0, 0, 2, 2),
     CHFunction(SQLFunc::FUNCpositiveModuloOrNull, 0, 0, 2, 2),
     CHFunction(SQLFunc::FUNCicebergBucket, 0, 0, 2, 2),
+    CHFunction(SQLFunc::FUNCicebergHash, 0, 0, 1, 1),
     CHFunction(SQLFunc::FUNCicebergTruncate, 0, 0, 2, 2),
     /// String and array functions
     CHFunction(SQLFunc::FUNCempty, 0, 0, 1, 1),
@@ -334,6 +335,10 @@ const std::vector<CHFunction> CHFuncs = {
     CHFunction(SQLFunc::FUNCarrayLevenshteinDistanceWeighted, 0, 0, 4, 4),
     CHFunction(SQLFunc::FUNCarraySimilarity, 0, 0, 4, 4),
     CHFunction(SQLFunc::FUNCarrayExcept, 0, 0, 2, 2),
+    CHFunction(SQLFunc::FUNCgroupUniqArrayArray, 0, 1, 1, 1),
+    CHFunction(SQLFunc::FUNCsumMappedArrays, 0, 0, 1, ulimited_params),
+    CHFunction(SQLFunc::FUNCminMappedArrays, 0, 0, 1, ulimited_params),
+    CHFunction(SQLFunc::FUNCmaxMappedArrays, 0, 0, 1, ulimited_params),
     /// Bit functions
     CHFunction(SQLFunc::FUNCbitAnd, 0, 0, 2, 2),
     CHFunction(SQLFunc::FUNCbitOr, 0, 0, 2, 2),
@@ -609,11 +614,22 @@ const std::vector<CHFunction> CHFuncs = {
     CHFunction(SQLFunc::FUNCJSONExtractBool, 0, 0, 1, ulimited_params),
     CHFunction(SQLFunc::FUNCJSONExtractString, 0, 0, 1, ulimited_params),
     CHFunction(SQLFunc::FUNCJSONExtract, 0, 0, 1, ulimited_params),
-    CHFunction(SQLFunc::FUNCJSONExtractKeysAndValues, 0, 0, 2, ulimited_params),
+    CHFunction(SQLFunc::FUNCJSONExtractKeysAndValues, 0, 0, 1, ulimited_params),
     CHFunction(SQLFunc::FUNCJSONExtractKeys, 0, 0, 1, ulimited_params),
     CHFunction(SQLFunc::FUNCJSONExtractRaw, 0, 0, 1, ulimited_params),
     CHFunction(SQLFunc::FUNCJSONExtractArrayRaw, 0, 0, 1, ulimited_params),
     CHFunction(SQLFunc::FUNCJSONExtractKeysAndValuesRaw, 0, 0, 1, 5),
+    CHFunction(SQLFunc::FUNCJSONExtractArrayRawCaseInsensitive, 0, 0, 1, ulimited_params),
+    CHFunction(SQLFunc::FUNCJSONExtractBoolCaseInsensitive, 0, 0, 1, ulimited_params),
+    CHFunction(SQLFunc::FUNCJSONExtractCaseInsensitive, 0, 0, 1, ulimited_params),
+    CHFunction(SQLFunc::FUNCJSONExtractFloatCaseInsensitive, 0, 0, 1, ulimited_params),
+    CHFunction(SQLFunc::FUNCJSONExtractIntCaseInsensitive, 0, 0, 1, ulimited_params),
+    CHFunction(SQLFunc::FUNCJSONExtractKeysAndValuesCaseInsensitive, 0, 0, 1, ulimited_params),
+    CHFunction(SQLFunc::FUNCJSONExtractKeysAndValuesRawCaseInsensitive, 0, 0, 1, ulimited_params),
+    CHFunction(SQLFunc::FUNCJSONExtractKeysCaseInsensitive, 0, 0, 1, ulimited_params),
+    CHFunction(SQLFunc::FUNCJSONExtractRawCaseInsensitive, 0, 0, 1, ulimited_params),
+    CHFunction(SQLFunc::FUNCJSONExtractStringCaseInsensitive, 0, 0, 1, ulimited_params),
+    CHFunction(SQLFunc::FUNCJSONExtractUIntCaseInsensitive, 0, 0, 1, ulimited_params),
     CHFunction(SQLFunc::FUNCJSON_EXISTS, 0, 0, 2, 2),
     CHFunction(SQLFunc::FUNCJSON_QUERY, 0, 0, 2, 2),
     CHFunction(SQLFunc::FUNCJSON_VALUE, 0, 0, 2, 2),
@@ -907,6 +923,10 @@ const std::vector<CHFunction> CHFuncs = {
     CHFunction(SQLFunc::FUNCtryBase32Decode, 0, 0, 1, 1),
     CHFunction(SQLFunc::FUNCbech32Encode, 0, 0, 1, 3),
     CHFunction(SQLFunc::FUNCbech32Decode, 0, 0, 1, 1),
+    CHFunction(SQLFunc::FUNCendsWithCaseInsensitive, 0, 0, 2, 2),
+    CHFunction(SQLFunc::FUNCendsWithCaseInsensitiveUTF8, 0, 0, 2, 2),
+    CHFunction(SQLFunc::FUNCstartsWithCaseInsensitive, 0, 0, 2, 2),
+    CHFunction(SQLFunc::FUNCstartsWithCaseInsensitiveUTF8, 0, 0, 2, 2),
     /// Timeseries
     CHFunction(SQLFunc::FUNCseriesOutliersDetectTukey, 0, 0, 1, 1),
     CHFunction(SQLFunc::FUNCseriesPeriodDetectFFT, 0, 0, 1, 1),
@@ -1116,6 +1136,7 @@ const std::vector<CHFunction> CHFuncs = {
     CHFunction(SQLFunc::FUNCsnowflakeIDToDateTime64, 0, 0, 1, 3),
     CHFunction(SQLFunc::FUNCdateTimeToSnowflakeID, 0, 0, 1, 2),
     CHFunction(SQLFunc::FUNCdateTime64ToSnowflakeID, 0, 0, 1, 2),
+    CHFunction(SQLFunc::FUNCdateTimeToUUIDv7, 0, 0, 1, 1),
     /// UniqTheta
     CHFunction(SQLFunc::FUNCuniqThetaUnion, 0, 0, 2, 2),
     CHFunction(SQLFunc::FUNCuniqThetaIntersect, 0, 0, 2, 2),
@@ -1216,7 +1237,6 @@ const std::vector<CHFunction> CHFuncs = {
     CHFunction(SQLFunc::FUNCfinancialNetPresentValue, 0, 0, 2, 3),
     CHFunction(SQLFunc::FUNCfinancialNetPresentValueExtended, 0, 0, 3, 4),
     /// Other
-    CHFunction(SQLFunc::FUNChostname, 0, 0, 0, 0),
     CHFunction(SQLFunc::FUNCfqdn, 0, 0, 0, 0),
     CHFunction(SQLFunc::FUNCbasename, 0, 0, 1, 1),
     CHFunction(SQLFunc::FUNCvisibleWidth, 0, 0, 1, 1),
@@ -1230,7 +1250,8 @@ const std::vector<CHFunction> CHFuncs = {
     CHFunction(SQLFunc::FUNCformatReadableSize, 0, 0, 1, 1),
     CHFunction(SQLFunc::FUNCformatReadableQuantity, 0, 0, 1, 1),
     CHFunction(SQLFunc::FUNCformatReadableTimeDelta, 0, 0, 1, 3),
-    CHFunction(SQLFunc::FUNCneighbor, 0, 0, 2, 3),
+    CHFunction(SQLFunc::FUNCcolorOKLCHToSRGB, 0, 0, 1, 2),
+    CHFunction(SQLFunc::FUNCcolorSRGBToOKLCH, 0, 0, 1, 2),
     CHFunction(SQLFunc::FUNCMACNumToString, 0, 0, 1, 1),
     CHFunction(SQLFunc::FUNCMACStringToNum, 0, 0, 1, 1),
     CHFunction(SQLFunc::FUNCMACStringToOUI, 0, 0, 1, 1),
@@ -1301,9 +1322,11 @@ const std::vector<CHFunction> CHFuncs = {
     CHFunction(SQLFunc::FUNCrandFisherF, 0, 0, 2, 2),
     CHFunction(SQLFunc::FUNCfuzzBits, 0, 0, 2, 2),
     /// Other
+    CHFunction(SQLFunc::FUNChostname, 0, 0, 0, 0),
     CHFunction(SQLFunc::FUNCshowCertificate, 0, 0, 0, 0),
     CHFunction(SQLFunc::FUNCdisplayName, 0, 0, 0, 0),
     CHFunction(SQLFunc::FUNCcurrentDatabase, 0, 0, 0, 0),
+    CHFunction(SQLFunc::FUNCauthenticatedUser, 0, 0, 0, 0),
     CHFunction(SQLFunc::FUNCcurrentUser, 0, 0, 0, 0),
     CHFunction(SQLFunc::FUNCcurrentSchemas, 0, 0, 1, 1),
     CHFunction(SQLFunc::FUNCindexHint, 0, 0, 1, 1),
@@ -1324,7 +1347,6 @@ const std::vector<CHFunction> CHFuncs = {
     CHFunction(SQLFunc::FUNCblockNumber, 0, 0, 0, 0),
     CHFunction(SQLFunc::FUNCrowNumberInBlock, 0, 0, 0, 0),
     CHFunction(SQLFunc::FUNCrowNumberInAllBlocks, 0, 0, 0, 0),
-    CHFunction(SQLFunc::FUNCrunningDifference, 0, 0, 1, 1),
     CHFunction(SQLFunc::FUNCrunningConcurrency, 0, 0, 2, 2),
     CHFunction(SQLFunc::FUNCfilesystemAvailable, 0, 0, 0, 0),
     CHFunction(SQLFunc::FUNCfilesystemUnreserved, 0, 0, 0, 0),
@@ -1336,6 +1358,7 @@ const std::vector<CHFunction> CHFuncs = {
     CHFunction(SQLFunc::FUNCshardCount, 0, 0, 0, 0),
     CHFunction(SQLFunc::FUNCzookeeperSessionUptime, 0, 0, 0, 0),
     CHFunction(SQLFunc::FUNCgenerateRandomStructure, 0, 0, 0, 2),
+    CHFunction(SQLFunc::FUNCgenerateSerialID, 0, 0, 1, 2),
     CHFunction(SQLFunc::FUNCconnectionId, 0, 0, 0, 0),
     CHFunction(SQLFunc::FUNCtransactionID, 0, 0, 0, 0),
     CHFunction(SQLFunc::FUNCtransactionLatestSnapshot, 0, 0, 0, 0),
