@@ -402,7 +402,7 @@ private:
     void prepareNextExplain(RandomGenerator & rg, ExplainQuery * eq);
     void generateOrderBy(RandomGenerator & rg, uint32_t ncols, bool allow_settings, bool is_window, OrderByStatement * ob);
     void generateLimitExpr(RandomGenerator & rg, Expr * expr);
-    void generateLimit(RandomGenerator & rg, bool has_order_by, uint32_t ncols, LimitStatement * ls);
+    void generateLimit(RandomGenerator & rg, bool has_order_by, LimitStatement * ls);
     void generateOffset(RandomGenerator & rg, bool has_order_by, OffsetStatement * off);
     void generateGroupByExpr(
         RandomGenerator & rg,
@@ -493,16 +493,17 @@ private:
         uint32_t added_structure = 0;
         uint32_t added_storage_class_name = 0;
         const uint32_t toadd_path = 1;
-        const uint32_t toadd_format = (b.file_format.has_value() || this->allow_not_deterministic) && rg.nextMediumNumber() < 91;
-        const uint32_t toadd_compression = (b.file_comp.has_value() || this->allow_not_deterministic) && rg.nextMediumNumber() < 51;
-        const uint32_t toadd_partition_strategy
-            = (b.partition_strategy.has_value() || ((b.isS3Engine() || b.isAzureEngine()) && this->allow_not_deterministic))
-            && rg.nextMediumNumber() < 21;
+        const uint32_t toadd_format = (b.file_format.has_value() && !this->allow_not_deterministic)
+            || (this->allow_not_deterministic && rg.nextMediumNumber() < 91);
+        const uint32_t toadd_compression
+            = (b.file_comp.has_value() && !this->allow_not_deterministic) || (this->allow_not_deterministic && rg.nextMediumNumber() < 51);
+        const uint32_t toadd_partition_strategy = (b.partition_strategy.has_value() && !this->allow_not_deterministic)
+            || ((b.isS3Engine() || b.isAzureEngine()) && this->allow_not_deterministic && rg.nextMediumNumber() < 21);
         const uint32_t toadd_partition_columns_in_data_file
-            = (b.partition_columns_in_data_file.has_value() || ((b.isS3Engine() || b.isAzureEngine()) && this->allow_not_deterministic))
-            && rg.nextMediumNumber() < 21;
-        const uint32_t toadd_storage_class_name
-            = (b.storage_class_name.has_value() || (b.isS3Engine() && this->allow_not_deterministic)) && rg.nextMediumNumber() < 21;
+            = (b.partition_columns_in_data_file.has_value() && !this->allow_not_deterministic)
+            || ((b.isS3Engine() || b.isAzureEngine()) && this->allow_not_deterministic && rg.nextMediumNumber() < 21);
+        const uint32_t toadd_storage_class_name = (b.storage_class_name.has_value() && !this->allow_not_deterministic)
+            || (b.isS3Engine() && this->allow_not_deterministic && rg.nextMediumNumber() < 21);
         const uint32_t toadd_structure = !std::is_same_v<U, TableEngine> && (!this->allow_not_deterministic || rg.nextMediumNumber() < 91);
         const uint32_t total_to_add = toadd_path + toadd_format + toadd_compression + toadd_partition_strategy
             + toadd_partition_columns_in_data_file + toadd_storage_class_name + toadd_structure;
@@ -612,7 +613,7 @@ private:
                 next->set_key("structure");
                 if constexpr (std::is_same_v<U, TableEngine>)
                 {
-                    chassert(0);
+                    UNREACHABLE();
                 }
                 else
                 {
@@ -622,7 +623,7 @@ private:
             }
             else
             {
-                chassert(0);
+                UNREACHABLE();
             }
         }
     }
