@@ -475,17 +475,16 @@ void ColumnAggregateFunction::get(size_t n, Field & res) const
     res = operator[](n);
 }
 
-std::pair<String, DataTypePtr> ColumnAggregateFunction::getValueNameAndType(size_t n) const
+DataTypePtr ColumnAggregateFunction::getValueNameAndTypeImpl(WriteBufferFromOwnString & name_buf, size_t n, const Options & options) const
 {
-    String state;
+    if (options.notFull(name_buf))
     {
         WriteBufferFromOwnString buffer;
         func->serialize(data[n], buffer, version);
-        WriteBufferFromString wb(state);
-        writeQuoted(buffer.str(), wb);
+        writeQuoted(buffer.str(), name_buf);
     }
 
-    return {state, DataTypeFactory::instance().get(type_string)};
+    return DataTypeFactory::instance().get(type_string);
 }
 
 StringRef ColumnAggregateFunction::getDataAt(size_t n) const
@@ -609,7 +608,7 @@ StringRef ColumnAggregateFunction::serializeValueIntoArena(size_t n, Arena & are
     return out.complete();
 }
 
-const char * ColumnAggregateFunction::deserializeAndInsertFromArena(const char * src_arena)
+void ColumnAggregateFunction::deserializeAndInsertFromArena(ReadBuffer & in)
 {
     ensureOwnership();
 
@@ -618,21 +617,10 @@ const char * ColumnAggregateFunction::deserializeAndInsertFromArena(const char *
       */
     Arena & dst_arena = createOrGetArena();
     pushBackAndCreateState(data, dst_arena, func.get());
-
-    /** We will read from src_arena.
-      * There is no limit for reading - it is assumed, that we can read all that we need after src_arena pointer.
-      * Buf ReadBufferFromMemory requires some bound. We will use arbitrary big enough number, that will not overflow pointer.
-      * NOTE Technically, this is not compatible with C++ standard,
-      *  as we cannot legally compare pointers after last element + 1 of some valid memory region.
-      *  Probably this will not work under UBSan.
-      */
-    ReadBufferFromMemory read_buffer(src_arena, std::numeric_limits<char *>::max() - src_arena - 1);
-    func->deserialize(data.back(), read_buffer, version, &dst_arena);
-
-    return read_buffer.position();
+    func->deserialize(data.back(), in, version, &dst_arena);
 }
 
-const char * ColumnAggregateFunction::skipSerializedInArena(const char *) const
+void ColumnAggregateFunction::skipSerializedInArena(ReadBuffer &) const
 {
     throw Exception(ErrorCodes::NOT_IMPLEMENTED, "Method skipSerializedInArena is not supported for {}", getName());
 }
