@@ -240,6 +240,8 @@ static void checkIndexesAreLimited(const IColumn & indexes, UInt64 limit)
 
         const auto & data = column_ptr->getData();
         size_t num_rows = data.size();
+        if (num_rows == 0)
+            return true;
         UInt64 max_position = 0;
         for (size_t i = 0; i < num_rows; ++i)
             max_position = std::max<UInt64>(max_position, data[i]);
@@ -307,29 +309,21 @@ void ColumnLowCardinality::collectSerializedValueSizes(PaddedPODArray<UInt64> &
     idx.collectSerializedValueSizes(sizes, dict_sizes);
 }
 
-const char * ColumnLowCardinality::deserializeAndInsertFromArena(const char * pos)
+void ColumnLowCardinality::deserializeAndInsertFromArena(ReadBuffer & in)
 {
     compactIfSharedDictionary();
-
-    const char * new_pos;
-    idx.insertIndex(getDictionary().uniqueDeserializeAndInsertFromArena(pos, new_pos));
-
-    return new_pos;
+    idx.insertIndex(getDictionary().uniqueDeserializeAndInsertFromArena(in));
 }
 
-const char * ColumnLowCardinality::deserializeAndInsertAggregationStateValueFromArena(const char * pos)
+void ColumnLowCardinality::deserializeAndInsertAggregationStateValueFromArena(ReadBuffer & in)
 {
     compactIfSharedDictionary();
-
-    const char * new_pos;
-    idx.insertIndex(getDictionary().uniqueDeserializeAndInsertAggregationStateValueFromArena(pos, new_pos));
-
-    return new_pos;
+    idx.insertIndex(getDictionary().uniqueDeserializeAndInsertAggregationStateValueFromArena(in));
 }
 
-const char * ColumnLowCardinality::skipSerializedInArena(const char * pos) const
+void ColumnLowCardinality::skipSerializedInArena(ReadBuffer & in) const
 {
-    return getDictionary().skipSerializedInArena(pos);
+    getDictionary().skipSerializedInArena(in);
 }
 
 WeakHash32 ColumnLowCardinality::getWeakHash32() const
