@@ -14,6 +14,7 @@ namespace DB
 
 namespace ErrorCodes
 {
+    extern const int ATTEMPT_TO_READ_AFTER_EOF;
     extern const int NOT_IMPLEMENTED;
     extern const int LOGICAL_ERROR;
 }
@@ -268,79 +269,90 @@ void ColumnObject::get(size_t n, Field & res) const
     res = (*this)[n];
 }
 
-std::pair<String, DataTypePtr> ColumnObject::getValueNameAndType(size_t n) const
+DataTypePtr ColumnObject::getValueNameAndTypeImpl(WriteBufferFromOwnString & name_buf, size_t n, const Options & options) const
 {
-    WriteBufferFromOwnString wb;
-    wb << '{';
+    if (options.notFull(name_buf))
+        name_buf << '{';
 
     bool first = true;
 
-    for (const auto & [path, column] : typed_paths)
+    if (options.notFull(name_buf))
     {
-        const auto & [value, type] = column->getValueNameAndType(n);
-
-        if (first)
-            first = false;
-        else
-            wb << ", ";
-
-        writeDoubleQuoted(path, wb);
-        wb << ": " << value;
-    }
-
-    for (const auto & [path, column] : dynamic_paths_ptrs)
-    {
-        /// Output only non-null values from dynamic paths. We cannot distinguish cases when
-        /// dynamic path has Null value and when it's absent in the row and consider them equivalent.
-        if (column->isNullAt(n))
-            continue;
-
-        const auto & [value, type] = column->getValueNameAndType(n);
-
-        if (first)
-            first = false;
-        else
-            wb << ", ";
-
-        writeDoubleQuoted(path, wb);
-        wb << ": " << value;
-    }
-
-    const auto & shared_data_offsets = getSharedDataOffsets();
-    const auto [shared_paths, shared_values] = getSharedDataPathsAndValues();
-    size_t start = shared_data_offsets[static_cast<ssize_t>(n) - 1];
-    size_t end = shared_data_offsets[n];
-    for (size_t i = start; i != end; ++i)
-    {
-        if (first)
-            first = false;
-        else
-            wb << ", ";
-
-        String path = shared_paths->getDataAt(i).toString();
-        writeDoubleQuoted(path, wb);
-
-        auto value_data = shared_values->getDataAt(i);
-        ReadBufferFromMemory buf(value_data.data, value_data.size);
-        auto decoded_type = decodeDataType(buf);
-
-        if (isNothing(decoded_type))
+        for (const auto & [path, column] : typed_paths)
         {
-            wb << ": NULL";
-            continue;
+            if (!options.notFull(name_buf))
+                break;
+
+            if (first)
+                first = false;
+            else
+                name_buf << ", ";
+            writeDoubleQuoted(path, name_buf);
+            column->getValueNameAndTypeImpl(name_buf, n, options);
         }
-
-        const auto column = decoded_type->createColumn();
-        decoded_type->getDefaultSerialization()->deserializeBinary(*column, buf, getFormatSettings());
-
-        const auto & [value, type] = column->getValueNameAndType(0);
-
-        wb << ": " << value;
     }
 
-    wb << "}";
+    if (options.notFull(name_buf))
+    {
+        for (const auto & [path, column] : dynamic_paths_ptrs)
+        {
+            if (!options.notFull(name_buf))
+                break;
 
-    return {wb.str(), std::make_shared<DataTypeObject>(DataTypeObject::SchemaFormat::JSON)};
+            /// Output only non-null values from dynamic paths. We cannot distinguish cases when
+            /// dynamic path has Null value and when it's absent in the row and consider them equivalent.
+            if (column->isNullAt(n))
+                continue;
+
+            if (first)
+                first = false;
+            else
+                name_buf << ", ";
+            writeDoubleQuoted(path, name_buf);
+            column->getValueNameAndTypeImpl(name_buf, n, options);
+        }
+    }
+
+    if (options.notFull(name_buf))
+    {
+        const auto & shared_data_offsets = getSharedDataOffsets();
+        const auto [shared_paths, shared_values] = getSharedDataPathsAndValues();
+        size_t start = shared_data_offsets[static_cast<ssize_t>(n) - 1];
+        size_t end = shared_data_offsets[n];
+        for (size_t i = start; i != end; ++i)
+        {
+            if (!options.notFull(name_buf))
+                break;
+
+            if (first)
+                first = false;
+            else
+                name_buf << ", ";
+
+            String path = shared_paths->getDataAt(i).toString();
+            writeDoubleQuoted(path, name_buf);
+
+            auto value_data = shared_values->getDataAt(i);
+            ReadBufferFromMemory buf(value_data.data, value_data.size);
+            auto decoded_type = decodeDataType(buf);
+
+            if (isNothing(decoded_type))
+            {
+                name_buf << ": NULL";
+                continue;
+            }
+
+            const auto column = decoded_type->createColumn();
+            decoded_type->getDefaultSerialization()->deserializeBinary(*column, buf, getFormatSettings());
+
+            column->getValueNameAndTypeImpl(name_buf, 0, options);
+        }
+    }
+
+    if (options.notFull(name_buf))
+        name_buf << "}";
+
+    return std::make_shared<DataTypeObject>(DataTypeObject::SchemaFormat::JSON);
 }
 
 bool ColumnObject::isDefaultAt(size_t n) const
@@ -631,7 +643,6 @@ void ColumnObject::doInsertFrom(const IColumn & src, size_t n)
 
     /// Finally, insert paths from shared data.
     insertFromSharedDataAndFillRemainingDynamicPaths(src_object_column, std::move(src_dynamic_paths_for_shared_data), n, 1);
-    validateDynamicPathsSizes();
 }
 
 #if !defined(DEBUG_OR_SANITIZER_BUILD)
@@ -666,7 +677,6 @@ void ColumnObject::doInsertRangeFrom(const IColumn & src, size_t start, size_t l
 
     /// Finally, insert paths from shared data.
     insertFromSharedDataAndFillRemainingDynamicPaths(src_object_column, std::move(src_dynamic_paths_for_shared_data), start, length);
-    validateDynamicPathsSizes();
 }
 
 void ColumnObject::insertFromSharedDataAndFillRemainingDynamicPaths(const DB::ColumnObject & src_object_column, std::vector<std::string_view> && src_dynamic_paths_for_shared_data, size_t start, size_t length)
@@ -729,9 +739,16 @@ void ColumnObject::insertFromSharedDataAndFillRemainingDynamicPaths(const DB::Co
             {
                 /// Deserialize binary value into dynamic column from shared data.
                 if (it->second->size() != current_size)
-                    throw Exception(ErrorCodes::LOGICAL_ERROR, "Unexpected size of dynamic path {}: {} != {}. It may indicate duplicated data for this path", path, it->second->size(), current_size);
+                {
+                    src_object_column.validateDynamicPathsAndSharedData();
+                    throw Exception(ErrorCodes::LOGICAL_ERROR, "Unexpected size of dynamic path {}: {} != {}", path, it->second->size(), current_size);
+                }
                 deserializeValueFromSharedData(src_shared_data_values, i, *it->second);
             }
+            else if (auto * dynamic_path_column = tryToAddNewDynamicPath(path))
+            {
+                deserializeValueFromSharedData(src_shared_data_values, i, *dynamic_path_column);
+            }
             else
             {
                 /// Before inserting this path into shared data check if we need to
@@ -984,59 +1001,73 @@ void ColumnObject::serializePathAndValueIntoArena(DB::Arena & arena, const char
     res.size += sizeof(size_t) + path_size + sizeof(size_t) + value_size;
 }
 
-const char * ColumnObject::deserializeAndInsertFromArena(const char * pos)
+void ColumnObject::deserializeAndInsertFromArena(ReadBuffer & in)
 {
     /// First deserialize typed paths. They come first.
     for (auto path : sorted_typed_paths)
-        pos = typed_paths.find(path)->second->deserializeAndInsertFromArena(pos);
+        typed_paths.find(path)->second->deserializeAndInsertFromArena(in);
 
     /// Second deserialize all other paths and values and insert them into dynamic paths or shared data.
-    return deserializeDynamicPathsAndSharedDataFromArena(pos);
+    deserializeDynamicPathsAndSharedDataFromArena(in);
 }
 
-const char * ColumnObject::deserializeAndInsertAggregationStateValueFromArena(const char * pos)
+void ColumnObject::deserializeAndInsertAggregationStateValueFromArena(ReadBuffer & in)
 {
     /// First deserialize typed paths. They come first.
     for (auto path : sorted_typed_paths)
-        pos = typed_paths.find(path)->second->deserializeAndInsertAggregationStateValueFromArena(pos);
+        typed_paths.find(path)->second->deserializeAndInsertAggregationStateValueFromArena(in);
 
     /// Second deserialize all other paths and values and insert them into dynamic paths or shared data.
-    return deserializeDynamicPathsAndSharedDataFromArena(pos);
+    deserializeDynamicPathsAndSharedDataFromArena(in);
 }
 
-const char * ColumnObject::deserializeDynamicPathsAndSharedDataFromArena(const char * pos)
+void ColumnObject::deserializeDynamicPathsAndSharedDataFromArena(ReadBuffer & in)
 {
     size_t current_size = size();
-    auto num_paths = unalignedLoad<size_t>(pos);
-    pos += sizeof(size_t);
+    size_t num_paths;
+    readBinaryLittleEndian<size_t>(num_paths, in);
+
     const auto [shared_data_paths, shared_data_values] = getSharedDataPathsAndValues();
     for (size_t i = 0; i != num_paths; ++i)
     {
-        auto path_size = unalignedLoad<size_t>(pos);
-        pos += sizeof(size_t);
-        std::string_view path(pos, path_size);
-        pos += path_size;
+        size_t path_size;
+        readBinaryLittleEndian<size_t>(path_size, in);
+
+        if (in.available() < path_size)
+            throw Exception(
+                ErrorCodes::ATTEMPT_TO_READ_AFTER_EOF,
+                "Attempt to read {} bytes for the path size, but only {} bytes are available",
+                path_size,
+                in.available());
+        std::string_view path(in.position(), path_size);
+        in.ignore(path_size);
+
         /// Deserialize binary value and try to insert it to dynamic paths or shared data.
-        auto value_size = unalignedLoad<size_t>(pos);
-        pos += sizeof(size_t);
-        std::string_view value(pos, value_size);
-        pos += value_size;
+        size_t value_size;
+        readBinaryLittleEndian<size_t>(value_size, in);
+
         /// Check if we have this path in dynamic paths.
         if (auto dynamic_it = dynamic_paths.find(path); dynamic_it != dynamic_paths.end())
         {
-            ReadBufferFromMemory buf(value.data(), value.size());
-            getDynamicSerialization()->deserializeBinary(*dynamic_it->second, buf, getFormatSettings());
+            getDynamicSerialization()->deserializeBinary(*dynamic_it->second, in, getFormatSettings());
         }
         /// Try to add a new dynamic path.
         else if (auto * dynamic_path_column = tryToAddNewDynamicPath(path))
         {
-            ReadBufferFromMemory buf(value.data(), value.size());
-            getDynamicSerialization()->deserializeBinary(*dynamic_path_column, buf, getFormatSettings());
+            getDynamicSerialization()->deserializeBinary(*dynamic_path_column, in, getFormatSettings());
         }
         /// Limit on dynamic paths is reached, add this path to shared data.
         /// Serialized paths are sorted, so we can insert right away.
         else
         {
+            if (in.available() < value_size)
+                throw Exception(
+                    ErrorCodes::ATTEMPT_TO_READ_AFTER_EOF,
+                    "Attempt to read {} bytes for the value size, but only {} bytes are available",
+                    value_size,
+                    in.available());
+            std::string_view value(in.position(), value_size);
+            in.ignore(value_size);
             shared_data_paths->insertData(path.data(), path.size());
             shared_data_values->insertData(value.data(), value.size());
         }
@@ -1050,30 +1081,28 @@ const char * ColumnObject::deserializeDynamicPathsAndSharedDataFromArena(const c
         if (column->size() == current_size)
             column->insertDefault();
     }
-
-    return pos;
 }
 
-const char * ColumnObject::skipSerializedInArena(const char * pos) const
+void ColumnObject::skipSerializedInArena(ReadBuffer & in) const
 {
     /// First, skip all values of typed paths;
     for (auto path : sorted_typed_paths)
-        pos = typed_paths.find(path)->second->skipSerializedInArena(pos);
+        typed_paths.find(path)->second->skipSerializedInArena(in);
 
     /// Second, skip all other paths and values.
-    auto num_paths = unalignedLoad<size_t>(pos);
-    pos += sizeof(size_t);
+    size_t num_paths;
+    readBinaryLittleEndian<size_t>(num_paths, in);
+
     for (size_t i = 0; i != num_paths; ++i)
     {
-        auto path_size = unalignedLoad<size_t>(pos);
-        pos += sizeof(size_t);
-        std::string_view path(pos, path_size);
-        pos += path_size;
-        auto value_size = unalignedLoad<size_t>(pos);
-        pos += sizeof(size_t) + value_size;
-    }
+        size_t path_size;
+        readBinaryLittleEndian<size_t>(path_size, in);
+        in.ignore(path_size);
 
-    return pos;
+        size_t value_size;
+        readBinaryLittleEndian<size_t>(value_size, in);
+        in.ignore(value_size);
+    }
 }
 
 void ColumnObject::updateHashWithValue(size_t n, SipHash & hash) const
@@ -1660,7 +1689,7 @@ bool ColumnObject::dynamicStructureEquals(const IColumn & rhs) const
     return true;
 }
 
-void ColumnObject::takeDynamicStructureFromSourceColumns(const DB::Columns & source_columns)
+void ColumnObject::takeDynamicStructureFromSourceColumns(const Columns & source_columns, std::optional<size_t> max_dynamic_subcolumns)
 {
     if (!empty())
         throw Exception(ErrorCodes::LOGICAL_ERROR, "takeDynamicStructureFromSourceColumns should be called only on empty Object column");
@@ -1718,7 +1747,8 @@ void ColumnObject::takeDynamicStructureFromSourceColumns(const DB::Columns & sou
     dynamic_paths.clear();
     dynamic_paths_ptrs.clear();
     sorted_dynamic_paths.clear();
-    max_dynamic_paths = global_max_dynamic_paths;
+    /// If max_dynamic_subcolumns is set, use it as max number of paths.
+    max_dynamic_paths = max_dynamic_subcolumns ? std::min(*max_dynamic_subcolumns, global_max_dynamic_paths) : global_max_dynamic_paths;
     Statistics new_statistics(Statistics::Source::MERGE);
 
     /// Check if the number of all dynamic paths exceeds the limit.
@@ -1782,7 +1812,7 @@ void ColumnObject::takeDynamicStructureFromSourceColumns(const DB::Columns & sou
             if (it != source_object.dynamic_paths.end())
                 dynamic_path_source_columns.push_back(it->second);
         }
-        column->takeDynamicStructureFromSourceColumns(dynamic_path_source_columns);
+        column->takeDynamicStructureFromSourceColumns(dynamic_path_source_columns, max_dynamic_subcolumns);
     }
 
     /// Typed paths also can contain types with dynamic structure.
@@ -1792,7 +1822,7 @@ void ColumnObject::takeDynamicStructureFromSourceColumns(const DB::Columns & sou
         typed_path_source_columns.reserve(source_columns.size());
         for (const auto & source_column : source_columns)
             typed_path_source_columns.push_back(assert_cast<const ColumnObject &>(*source_column).typed_paths.at(path));
-        column->takeDynamicStructureFromSourceColumns(typed_path_source_columns);
+        column->takeDynamicStructureFromSourceColumns(typed_path_source_columns, max_dynamic_subcolumns);
     }
 }
 
@@ -1891,148 +1921,131 @@ void ColumnObject::fillPathColumnFromSharedData(IColumn & path_column, StringRef
     }
 }
 
-/// Class that allows to iterate over paths inside single row in ColumnObject in sorted order.
-class ColumnObject::SortedPathsIterator
+ColumnObject::SortedPathsIterator::SortedPathsIterator(const ColumnObject & column_object_, size_t row_)
+    : column_object(column_object_)
+    , typed_paths_it(column_object.sorted_typed_paths.begin())
+    , typed_paths_end(column_object.sorted_typed_paths.end())
+    , dynamic_paths_it(column_object.sorted_dynamic_paths.begin())
+    , dynamic_paths_end(column_object.sorted_dynamic_paths.end())
+    , row(row_)
 {
-public:
-    SortedPathsIterator(const ColumnObject & column_object_, size_t row_)
-        : column_object(column_object_)
-        , typed_paths_it(column_object.sorted_typed_paths.begin())
-        , typed_paths_end(column_object.sorted_typed_paths.end())
-        , dynamic_paths_it(column_object.sorted_dynamic_paths.begin())
-        , dynamic_paths_end(column_object.sorted_dynamic_paths.end())
-        , row(row_)
+    std::tie(shared_data_paths, shared_data_values) = column_object.getSharedDataPathsAndValues();
+    const auto & shared_data_offsets = column_object.getSharedDataOffsets();
+    shared_data_it = shared_data_offsets[row - 1];
+    shared_data_end = shared_data_offsets[row];
+    setCurrentPath();
+}
+
+void ColumnObject::SortedPathsIterator::next()
+{
+    switch (current_path_type)
     {
-        std::tie(shared_data_paths, shared_data_values) = column_object.getSharedDataPathsAndValues();
-        const auto & shared_data_offsets = column_object.getSharedDataOffsets();
-        shared_data_it = shared_data_offsets[row - 1];
-        shared_data_end = shared_data_offsets[row];
-        setCurrentPath();
-    }
-
-    void next()
-    {
-        switch (current_path_type)
-        {
-            case PathType::DYNAMIC:
-                ++dynamic_paths_it;
-                break;
-            case PathType::TYPED:
-                ++typed_paths_it;
-                break;
-            case PathType::SHARED_DATA:
-                ++shared_data_it;
-                break;
-        }
-
-        setCurrentPath();
-    }
-
-    /// Compare paths and values of 2 iterators.
-    /// Returns -1, 0, 1 if this iterator is less, equal or greater than rhs.
-    int compare(const SortedPathsIterator & rhs, int nan_direction_hint) const
-    {
-        /// First, compare paths as strings.
-        auto path = getCurrentPath();
-        auto rhs_path = rhs.getCurrentPath();
-        if (path != rhs_path)
-            return path < rhs_path ? -1 : 1;
-
-        /// If paths are equal, compare their values.
-        auto [column, n] = getCurrentPathColumnAndRow();
-        auto [rhs_column, rhs_n] = rhs.getCurrentPathColumnAndRow();
-        return column->compareAt(n, rhs_n, *rhs_column, nan_direction_hint);
-    }
-
-    bool end()
-    {
-        return typed_paths_it == typed_paths_end && dynamic_paths_it == dynamic_paths_end && shared_data_it == shared_data_end;
-    }
-
-private:
-    void setCurrentPath()
-    {
-        /// We have 3 sorted lists of paths - typed paths, dynamic paths and paths in shared data.
-        /// We store iterators for each of these lists. Here we try to find the iterator with the lexicographically smallest path.
-
-        /// Null in dynamic path is considered as absence of this path.
-        while (dynamic_paths_it != dynamic_paths_end && column_object.dynamic_paths.find(*dynamic_paths_it)->second->isNullAt(row))
+        case PathType::DYNAMIC:
             ++dynamic_paths_it;
-
-        std::array<std::pair<PathType, std::optional<std::string_view>>, 3> paths{
-            std::pair{PathType::TYPED, typed_paths_it == typed_paths_end ? std::nullopt : std::optional<std::string_view>(*typed_paths_it)},
-            std::pair{PathType::DYNAMIC, dynamic_paths_it == dynamic_paths_end ? std::nullopt : std::optional<std::string_view>(*dynamic_paths_it)},
-            std::pair{
-                PathType::SHARED_DATA, shared_data_it == shared_data_end ? std::nullopt : std::optional<std::string_view>(shared_data_paths->getDataAt(shared_data_it).toView())},
-        };
-
-        auto min_path = std::ranges::min(
-            paths,
-            [](auto first_path, auto second_path)
-            {
-                if (!second_path.second)
-                    return true;
-
-                if (!first_path.second)
-                    return false;
-
-                return first_path.second < second_path.second;
-            });
-
-        if (min_path.second)
-            current_path_type = min_path.first;
+            break;
+        case PathType::TYPED:
+            ++typed_paths_it;
+            break;
+        case PathType::SHARED_DATA:
+            ++shared_data_it;
+            break;
     }
 
-    std::string_view getCurrentPath() const
-    {
-        switch (current_path_type)
-        {
-            case PathType::DYNAMIC:
-                return *dynamic_paths_it;
-            case PathType::TYPED:
-                return *typed_paths_it;
-            case PathType::SHARED_DATA:
-                return shared_data_paths->getDataAt(shared_data_it).toView();
-        }
-    }
+    setCurrentPath();
+}
 
-    std::pair<ColumnPtr, size_t> getCurrentPathColumnAndRow() const
-    {
-        switch (current_path_type)
-        {
-            case PathType::DYNAMIC:
-                return {column_object.dynamic_paths.find(*dynamic_paths_it)->second, row};
-            case PathType::TYPED:
-                return {column_object.typed_paths.find(*typed_paths_it)->second, row};
-            case PathType::SHARED_DATA:
-            {
-                auto tmp_column = ColumnDynamic::create();
-                ColumnObject::deserializeValueFromSharedData(shared_data_values, shared_data_it, *tmp_column);
-                return {std::move(tmp_column), 0};
-            }
-        }
-    }
+int ColumnObject::SortedPathsIterator::compare(const SortedPathsIterator & rhs, int nan_direction_hint) const
+{
+    /// First, compare paths as strings.
+    auto path = getCurrentPath();
+    auto rhs_path = rhs.getCurrentPath();
+    if (path != rhs_path)
+        return path < rhs_path ? -1 : 1;
 
-    enum class PathType
-    {
-        TYPED,
-        DYNAMIC,
-        SHARED_DATA,
+    /// If paths are equal, compare their values.
+    auto [column, n] = getCurrentPathColumnAndRow();
+    auto [rhs_column, rhs_n] = rhs.getCurrentPathColumnAndRow();
+    return column->compareAt(n, rhs_n, *rhs_column, nan_direction_hint);
+}
+
+bool ColumnObject::SortedPathsIterator::end()
+{
+    return typed_paths_it == typed_paths_end && dynamic_paths_it == dynamic_paths_end && shared_data_it == shared_data_end;
+}
+
+void ColumnObject::SortedPathsIterator::setCurrentPath()
+{
+    /// We have 3 sorted lists of paths - typed paths, dynamic paths and paths in shared data.
+    /// We store iterators for each of these lists. Here we try to find the iterator with the lexicographically smallest path.
+
+    /// Null in dynamic path is considered as absence of this path.
+    while (dynamic_paths_it != dynamic_paths_end && column_object.dynamic_paths.find(*dynamic_paths_it)->second->isNullAt(row))
+        ++dynamic_paths_it;
+
+    std::array<std::pair<PathType, std::optional<std::string_view>>, 3> paths{
+        std::pair{PathType::TYPED, typed_paths_it == typed_paths_end ? std::nullopt : std::optional<std::string_view>(*typed_paths_it)},
+        std::pair{PathType::DYNAMIC, dynamic_paths_it == dynamic_paths_end ? std::nullopt : std::optional<std::string_view>(*dynamic_paths_it)},
+        std::pair{
+            PathType::SHARED_DATA, shared_data_it == shared_data_end ? std::nullopt : std::optional<std::string_view>(shared_data_paths->getDataAt(shared_data_it).toView())},
     };
 
-    const ColumnObject & column_object;
-    decltype(column_object.sorted_typed_paths)::const_iterator typed_paths_it;
-    decltype(column_object.sorted_typed_paths)::const_iterator typed_paths_end;
-    decltype(column_object.sorted_dynamic_paths)::const_iterator dynamic_paths_it;
-    decltype(column_object.sorted_dynamic_paths)::const_iterator dynamic_paths_end;
-    size_t shared_data_it;
-    size_t shared_data_end;
-    const ColumnString * shared_data_paths;
-    const ColumnString * shared_data_values;
-    PathType current_path_type;
-    size_t row;
+    auto min_path = std::ranges::min(
+        paths,
+        [](auto first_path, auto second_path)
+        {
+            if (!second_path.second)
+                return true;
+
+            if (!first_path.second)
+                return false;
+
+            return first_path.second < second_path.second;
+        });
+
+    if (min_path.second)
+        current_path_type = min_path.first;
+}
+
+std::string_view ColumnObject::SortedPathsIterator::getCurrentPath() const
+{
+    switch (current_path_type)
+    {
+        case PathType::DYNAMIC:
+            return *dynamic_paths_it;
+        case PathType::TYPED:
+            return *typed_paths_it;
+        case PathType::SHARED_DATA:
+            return shared_data_paths->getDataAt(shared_data_it).toView();
+    }
+}
+
+std::pair<ColumnPtr, size_t> ColumnObject::SortedPathsIterator::getCurrentPathColumnAndRow() const
+{
+    switch (current_path_type)
+    {
+        case PathType::DYNAMIC:
+            return {column_object.dynamic_paths.find(*dynamic_paths_it)->second, row};
+        case PathType::TYPED:
+            return {column_object.typed_paths.find(*typed_paths_it)->second, row};
+        case PathType::SHARED_DATA:
+        {
+            auto tmp_column = ColumnDynamic::create();
+            tmp_column->reserve(1);
+            ColumnObject::deserializeValueFromSharedData(shared_data_values, shared_data_it, *tmp_column);
+            return {std::move(tmp_column), 0};
+        }
+    }
 };
 
+ColumnObject::SortedPathsIterator::PathInfo ColumnObject::SortedPathsIterator::getCurrentPathInfo() const
+{
+    PathInfo path_info;
+    path_info.type = current_path_type;
+    path_info.path = getCurrentPath();
+    std::tie(path_info.column, path_info.row) = getCurrentPathColumnAndRow();
+    return path_info;
+}
+
 #if !defined(DEBUG_OR_SANITIZER_BUILD)
 int ColumnObject::compareAt(size_t n, size_t m, const IColumn & rhs, int nan_direction_hint) const
 #else
@@ -2062,14 +2075,28 @@ int ColumnObject::doCompareAt(size_t n, size_t m, const IColumn & rhs, int nan_d
     return 1;
 }
 
-void ColumnObject::validateDynamicPathsSizes() const
+void ColumnObject::validateDynamicPathsAndSharedData(size_t shared_data_offset) const
 {
+    if (dynamic_paths.empty())
+        return;
+
     size_t expected_size = shared_data->size();
     for (const auto & [path, column] : dynamic_paths)
     {
         if (column->size() != expected_size)
             throw Exception(ErrorCodes::LOGICAL_ERROR, "Unexpected size of dynamic path {}: {} != {}", path, column->size(), expected_size);
     }
+
+    const auto & shared_data_offsets = getSharedDataOffsets();
+    const auto [shared_data_paths, _] = getSharedDataPathsAndValues();
+    size_t shared_data_paths_start = shared_data_offsets[ssize_t(shared_data_offset) - 1];
+    size_t shared_data_paths_end = shared_data_offsets.back();
+    for (size_t i = shared_data_paths_start; i != shared_data_paths_end; ++i)
+    {
+        auto path = shared_data_paths->getDataAt(i);
+        if (dynamic_paths.contains(path))
+            throw Exception(ErrorCodes::LOGICAL_ERROR, "Path {} is present both in dynamic paths and in shared data", path.toString());
+    }
 }
 
 }
