@@ -122,7 +122,7 @@ public:
 
     Field operator[](size_t n) const override;
     void get(size_t n, Field & res) const override;
-    std::pair<String, DataTypePtr> getValueNameAndType(size_t n) const override;
+    DataTypePtr getValueNameAndTypeImpl(WriteBufferFromOwnString &, size_t n, const Options &) const override;
 
     bool isDefaultAt(size_t n) const override;
     StringRef getDataAt(size_t n) const override;
@@ -145,9 +145,9 @@ public:
 
     StringRef serializeValueIntoArena(size_t n, Arena & arena, char const *& begin) const override;
     StringRef serializeAggregationStateValueIntoArena(size_t n, Arena & arena, char const *& begin) const override;
-    const char * deserializeAndInsertFromArena(const char * pos) override;
-    const char * deserializeAndInsertAggregationStateValueFromArena(const char * pos) override;
-    const char * skipSerializedInArena(const char * pos) const override;
+    void deserializeAndInsertFromArena(ReadBuffer & in) override;
+    void deserializeAndInsertAggregationStateValueFromArena(ReadBuffer & in) override;
+    void skipSerializedInArena(ReadBuffer & in) const override;
     std::optional<size_t> getSerializedValueSize(size_t) const override { return std::nullopt; }
 
     void updateHashWithValue(size_t n, SipHash & hash) const override;
@@ -203,7 +203,7 @@ public:
 
     bool hasDynamicStructure() const override { return true; }
     bool dynamicStructureEquals(const IColumn & rhs) const override;
-    void takeDynamicStructureFromSourceColumns(const Columns & source_columns) override;
+    void takeDynamicStructureFromSourceColumns(const Columns & source_columns, std::optional<size_t> max_dynamic_subcolumns) override;
     void takeDynamicStructureFromColumn(const ColumnPtr & source_column) override;
 
     const PathToColumnMap & getTypedPaths() const { return typed_paths; }
@@ -282,16 +282,63 @@ public:
     /// Insert all the data from shared data with specified path to dynamic column.
     static void fillPathColumnFromSharedData(IColumn & path_column, StringRef path, const ColumnPtr & shared_data_column, size_t start, size_t end);
 
-    /// Validate that all dynamic paths have correct sizes.
-    void validateDynamicPathsSizes() const;
+    /// Validate that all dynamic paths have correct sizes and that shared data doesn't contain any dynamic paths.
+    void validateDynamicPathsAndSharedData(size_t shared_data_offset = 0) const;
+
+    /// Class that allows to iterate over paths inside single row in ColumnObject in sorted order.
+    class SortedPathsIterator
+    {
+    public:
+        enum class PathType
+        {
+            TYPED,
+            DYNAMIC,
+            SHARED_DATA,
+        };
+
+        struct PathInfo
+        {
+            PathType type;
+            std::string_view path;
+            ColumnPtr column;
+            size_t row;
+        };
+
+        SortedPathsIterator(const ColumnObject & column_object_, size_t row_);
+
+        void next();
+        bool end();
+
+        /// Compare paths and values of 2 iterators.
+        /// Returns -1, 0, 1 if this iterator is less, equal or greater than rhs.
+        int compare(const SortedPathsIterator & rhs, int nan_direction_hint) const;
+
+        PathInfo getCurrentPathInfo() const;
+
+    private:
+        void setCurrentPath();
+        std::string_view getCurrentPath() const;
+        std::pair<ColumnPtr, size_t> getCurrentPathColumnAndRow() const;
+
+        const ColumnObject & column_object;
+        std::vector<std::string_view>::const_iterator typed_paths_it;
+        std::vector<std::string_view>::const_iterator typed_paths_end;
+        std::set<std::string_view>::const_iterator dynamic_paths_it;
+        std::set<std::string_view>::const_iterator dynamic_paths_end;
+        size_t shared_data_it;
+        size_t shared_data_end;
+        const ColumnString * shared_data_paths;
+        const ColumnString * shared_data_values;
+        PathType current_path_type;
+        size_t row;
+    };
 
 private:
-    class SortedPathsIterator;
 
     void insertFromSharedDataAndFillRemainingDynamicPaths(const ColumnObject & src_object_column, std::vector<std::string_view> && src_dynamic_paths_for_shared_data, size_t start, size_t length);
     void serializePathAndValueIntoArena(Arena & arena, const char *& begin, StringRef path, StringRef value, StringRef & res) const;
     void serializeDynamicPathsAndSharedDataIntoArena(size_t n, Arena & arena, const char *& begin, StringRef & res) const;
-    const char * deserializeDynamicPathsAndSharedDataFromArena(const char * pos);
+    void deserializeDynamicPathsAndSharedDataFromArena(ReadBuffer & in);
 
     /// Map path -> column for paths with explicitly specified types.
     /// This set of paths is constant and cannot be changed.
