@@ -82,9 +82,9 @@ void ColumnReplicated::get(size_t n, Field & res) const
     nested_column->get(indexes.getIndexAt(n), res);
 }
 
-std::pair<String, DataTypePtr> ColumnReplicated::getValueNameAndType(size_t n) const
+DataTypePtr ColumnReplicated::getValueNameAndTypeImpl(WriteBufferFromOwnString & name_buf, size_t n, const IColumn::Options & options) const
 {
-    return nested_column->getValueNameAndType(indexes.getIndexAt(n));
+    return nested_column->getValueNameAndTypeImpl(name_buf, indexes.getIndexAt(n), options);
 }
 
 bool ColumnReplicated::getBool(size_t n) const
@@ -153,23 +153,21 @@ std::optional<size_t> ColumnReplicated::getSerializedValueSize(size_t n) const
     return nested_column->getSerializedValueSize(indexes.getIndexAt(n));
 }
 
-const char * ColumnReplicated::deserializeAndInsertFromArena(const char * pos)
+void ColumnReplicated::deserializeAndInsertFromArena(ReadBuffer & in)
 {
-    const auto * res = nested_column->deserializeAndInsertFromArena(pos);
+    nested_column->deserializeAndInsertFromArena(in);
     indexes.insertIndex(nested_column->size() - 1);
-    return res;
 }
 
-const char * ColumnReplicated::deserializeAndInsertAggregationStateValueFromArena(const char * pos)
+void ColumnReplicated::deserializeAndInsertAggregationStateValueFromArena(ReadBuffer & in)
 {
-    const auto * res = nested_column->deserializeAndInsertAggregationStateValueFromArena(pos);
+    nested_column->deserializeAndInsertAggregationStateValueFromArena(in);
     indexes.insertIndex(nested_column->size() - 1);
-    return res;
 }
 
-const char * ColumnReplicated::skipSerializedInArena(const char * pos) const
+void ColumnReplicated::skipSerializedInArena(ReadBuffer & in) const
 {
-    return nested_column->skipSerializedInArena(pos);
+    nested_column->skipSerializedInArena(in);
 }
 
 #if !defined(DEBUG_OR_SANITIZER_BUILD)
@@ -638,7 +636,7 @@ bool ColumnReplicated::structureEquals(const IColumn & rhs) const
     return false;
 }
 
-void ColumnReplicated::takeDynamicStructureFromSourceColumns(const Columns & source_columns)
+void ColumnReplicated::takeDynamicStructureFromSourceColumns(const Columns & source_columns, std::optional<size_t> max_dynamic_subcolumns)
 {
     Columns source_nested_columns;
     source_nested_columns.reserve(source_columns.size());
@@ -650,7 +648,7 @@ void ColumnReplicated::takeDynamicStructureFromSourceColumns(const Columns & sou
             source_nested_columns.emplace_back(source_column);
     }
 
-    nested_column->takeDynamicStructureFromSourceColumns(source_nested_columns);
+    nested_column->takeDynamicStructureFromSourceColumns(source_nested_columns, max_dynamic_subcolumns);
 }
 
 void ColumnReplicated::takeDynamicStructureFromColumn(const ColumnPtr & source_column)
