@@ -96,9 +96,9 @@ void ColumnSparse::get(size_t n, Field & res) const
     values->get(getValueIndex(n), res);
 }
 
-std::pair<String, DataTypePtr>  ColumnSparse::getValueNameAndType(size_t n) const
+DataTypePtr  ColumnSparse::getValueNameAndTypeImpl(WriteBufferFromOwnString & name_buf, size_t n, const Options & options) const
 {
-    return values->getValueNameAndType(getValueIndex(n));
+    return values->getValueNameAndTypeImpl(name_buf, getValueIndex(n), options);
 }
 
 bool ColumnSparse::getBool(size_t n) const
@@ -179,23 +179,19 @@ std::optional<size_t> ColumnSparse::getSerializedValueSize(size_t n) const
     return values->getSerializedValueSize(getValueIndex(n));
 }
 
-const char * ColumnSparse::deserializeAndInsertFromArena(const char * pos)
+void ColumnSparse::deserializeAndInsertFromArena(ReadBuffer & in)
 {
-    const char * res = nullptr;
-    insertSingleValue([&](IColumn & column) { res = column.deserializeAndInsertFromArena(pos); });
-    return res;
+    insertSingleValue([&](IColumn & column) { column.deserializeAndInsertFromArena(in); });
 }
 
-const char * ColumnSparse::deserializeAndInsertAggregationStateValueFromArena(const char * pos)
+void ColumnSparse::deserializeAndInsertAggregationStateValueFromArena(ReadBuffer & in)
 {
-    const char * res = nullptr;
-    insertSingleValue([&](IColumn & column) { res = column.deserializeAndInsertAggregationStateValueFromArena(pos); });
-    return res;
+    insertSingleValue([&](IColumn & column) { column.deserializeAndInsertAggregationStateValueFromArena(in); });
 }
 
-const char * ColumnSparse::skipSerializedInArena(const char * pos) const
+void ColumnSparse::skipSerializedInArena(ReadBuffer & in) const
 {
-    return values->skipSerializedInArena(pos);
+    values->skipSerializedInArena(in);
 }
 
 #if !defined(DEBUG_OR_SANITIZER_BUILD)
@@ -891,13 +887,13 @@ ColumnSparse::Iterator ColumnSparse::getIterator(size_t n) const
     return Iterator(offsets_data, _size, current_offset, n);
 }
 
-void ColumnSparse::takeDynamicStructureFromSourceColumns(const Columns & source_columns)
+void ColumnSparse::takeDynamicStructureFromSourceColumns(const Columns & source_columns, std::optional<size_t> max_dynamic_subcolumns)
 {
     Columns values_source_columns;
     values_source_columns.reserve(source_columns.size());
     for (const auto & source_column : source_columns)
         values_source_columns.push_back(assert_cast<const ColumnSparse &>(*source_column).getValuesPtr());
-    values->takeDynamicStructureFromSourceColumns(values_source_columns);
+    values->takeDynamicStructureFromSourceColumns(values_source_columns, max_dynamic_subcolumns);
 }
 
 void ColumnSparse::takeDynamicStructureFromColumn(const ColumnPtr & source_column)
