@@ -1,12 +1,10 @@
 #include <gtest/gtest.h>
 
-#include <Columns/ColumnsNumber.h>
 #include <Columns/ColumnString.h>
 #include <Columns/ColumnFixedString.h>
 #include <Columns/ColumnArray.h>
 #include <Columns/ColumnConst.h>
 #include <Columns/ColumnDecimal.h>
-#include <Columns/ColumnLowCardinality.h>
 #include <Columns/ColumnNullable.h>
 #include <Columns/ColumnTuple.h>
 
@@ -14,7 +12,6 @@
 #include <DataTypes/DataTypesNumber.h>
 
 #include <Common/WeakHash.h>
-#include <Common/hex.h>
 
 #include <unordered_map>
 #include <iostream>
@@ -24,11 +21,10 @@
 using namespace DB;
 
 template <typename T>
-void checkColumn(
+void checkColumnImpl(
     const WeakHash32::Container & hash,
     const PaddedPODArray<T> & eq_class,
-    size_t allowed_collisions = 0,
-    size_t max_collisions_to_print = 10)
+    UInt32 scale = 0)
 {
     ASSERT_EQ(hash.size(), eq_class.size());
 
@@ -46,47 +42,34 @@ void checkColumn(
             else
             {
                 if (it->second != hash[i])
-                    std::cout << "Different hashes for the same equivalent class (" << toString(val) << ")\n";
+                {
+                    if constexpr (is_decimal<T>)
+                        std::cout << "Different hashes for the same equivalent class (" << toString(val, scale) << ")\n";
+                    else
+                        std::cout << "Different hashes for the same equivalent class (" << toString(val) << ")\n";
+                }
 
                 ASSERT_EQ(it->second, hash[i]);
             }
         }
     }
+}
 
-    /// Check have not many collisions.
-    {
-        std::unordered_map<UInt32, T> map;
-        size_t num_collisions = 0;
+template <typename T>
+void checkColumn(
+    const WeakHash32::Container & hash,
+    const PaddedPODArray<T> & eq_class)
+{
+    checkColumnImpl(hash, eq_class);
+}
 
-        std::stringstream collisions_str;       // STYLE_CHECK_ALLOW_STD_STRING_STREAM
-        collisions_str.exceptions(std::ios::failbit);
-
-        for (size_t i = 0; i < eq_class.size(); ++i)
-        {
-            auto & val = eq_class[i];
-            auto it = map.find(hash[i]);
-
-            if (it == map.end())
-                map[hash[i]] = val;
-            else if (it->second != val)
-            {
-                ++num_collisions;
-
-                if (num_collisions <= max_collisions_to_print)
-                {
-                    collisions_str << "Collision:\n";
-                }
-
-                if (num_collisions > allowed_collisions)
-                {
-                    std::cerr << collisions_str.rdbuf();
-                    break;
-                }
-            }
-        }
-
-        ASSERT_LE(num_collisions, allowed_collisions);
-    }
+template <is_decimal T>
+void checkColumn(
+    const WeakHash32::Container & hash,
+    const PaddedPODArray<T> & eq_class,
+    UInt32 scale)
+{
+    checkColumnImpl(hash, eq_class, scale);
 }
 
 TEST(WeakHash32, ColumnVectorU8)
@@ -100,8 +83,7 @@ TEST(WeakHash32, ColumnVectorU8)
             data.push_back(i);
     }
 
-    WeakHash32 hash(col->size());
-    col->updateWeakHash32(hash);
+    WeakHash32 hash = col->getWeakHash32();
 
     checkColumn(hash.getData(), col->getData());
 }
@@ -117,8 +99,7 @@ TEST(WeakHash32, ColumnVectorI8)
             data.push_back(i);
     }
 
-    WeakHash32 hash(col->size());
-    col->updateWeakHash32(hash);
+    WeakHash32 hash = col->getWeakHash32();
 
     checkColumn(hash.getData(), col->getData());
 }
@@ -134,8 +115,7 @@ TEST(WeakHash32, ColumnVectorU16)
             data.push_back(i);
     }
 
-    WeakHash32 hash(col->size());
-    col->updateWeakHash32(hash);
+    WeakHash32 hash = col->getWeakHash32();
 
     checkColumn(hash.getData(), col->getData());
 }
@@ -151,8 +131,7 @@ TEST(WeakHash32, ColumnVectorI16)
             data.push_back(i);
     }
 
-    WeakHash32 hash(col->size());
-    col->updateWeakHash32(hash);
+    WeakHash32 hash = col->getWeakHash32();
 
     checkColumn(hash.getData(), col->getData());
 }
@@ -164,12 +143,11 @@ TEST(WeakHash32, ColumnVectorU32)
 
     for (int idx [[maybe_unused]] : {1, 2})
     {
-        for (uint64_t i = 0; i < 65536; ++i)
+        for (uint32_t i = 0; i < 65536; ++i)
             data.push_back(i << 16u);
     }
 
-    WeakHash32 hash(col->size());
-    col->updateWeakHash32(hash);
+    WeakHash32 hash = col->getWeakHash32();
 
     checkColumn(hash.getData(), col->getData());
 }
@@ -181,12 +159,11 @@ TEST(WeakHash32, ColumnVectorI32)
 
     for (int idx [[maybe_unused]] : {1, 2})
     {
-        for (int64_t i = -32768; i < 32768; ++i)
-            data.push_back(i << 16); //-V610
+        for (int32_t i = -32768; i < 32768; ++i)
+            data.push_back(i << 16);
     }
 
-    WeakHash32 hash(col->size());
-    col->updateWeakHash32(hash);
+    WeakHash32 hash = col->getWeakHash32();
 
     checkColumn(hash.getData(), col->getData());
 }
@@ -202,8 +179,7 @@ TEST(WeakHash32, ColumnVectorU64)
             data.push_back(i << 32u);
     }
 
-    WeakHash32 hash(col->size());
-    col->updateWeakHash32(hash);
+    WeakHash32 hash = col->getWeakHash32();
 
     checkColumn(hash.getData(), col->getData());
 }
@@ -216,11 +192,10 @@ TEST(WeakHash32, ColumnVectorI64)
     for (int idx [[maybe_unused]] : {1, 2})
     {
         for (int64_t i = -32768; i < 32768; ++i)
-            data.push_back(i << 32); //-V610
+            data.push_back(i << 32);
     }
 
-    WeakHash32 hash(col->size());
-    col->updateWeakHash32(hash);
+    WeakHash32 hash = col->getWeakHash32();
 
     checkColumn(hash.getData(), col->getData());
 }
@@ -240,12 +215,11 @@ TEST(WeakHash32, ColumnVectorU128)
             val.items[0] = i << 32u;
             val.items[1] = i << 32u;
             data.push_back(val);
-            eq_data.push_back(i);
+            eq_data.push_back(static_cast<UInt32>(i));
         }
     }
 
-    WeakHash32 hash(col->size());
-    col->updateWeakHash32(hash);
+    WeakHash32 hash = col->getWeakHash32();
 
     checkColumn(hash.getData(), eq_data);
 }
@@ -258,11 +232,10 @@ TEST(WeakHash32, ColumnVectorI128)
     for (int idx [[maybe_unused]] : {1, 2})
     {
         for (int64_t i = -32768; i < 32768; ++i)
-            data.push_back(i << 32); //-V610
+            data.push_back(i << 32);
     }
 
-    WeakHash32 hash(col->size());
-    col->updateWeakHash32(hash);
+    WeakHash32 hash = col->getWeakHash32();
 
     checkColumn(hash.getData(), col->getData());
 }
@@ -274,14 +247,13 @@ TEST(WeakHash32, ColumnDecimal32)
 
     for (int idx [[maybe_unused]] : {1, 2})
     {
-        for (int64_t i = -32768; i < 32768; ++i)
-            data.push_back(i << 16); //-V610
+        for (int32_t i = -32768; i < 32768; ++i)
+            data.push_back(i << 16);
     }
 
-    WeakHash32 hash(col->size());
-    col->updateWeakHash32(hash);
+    WeakHash32 hash = col->getWeakHash32();
 
-    checkColumn(hash.getData(), col->getData());
+    checkColumn(hash.getData(), col->getData(), col->getScale());
 }
 
 TEST(WeakHash32, ColumnDecimal64)
@@ -292,13 +264,12 @@ TEST(WeakHash32, ColumnDecimal64)
     for (int idx [[maybe_unused]] : {1, 2})
     {
         for (int64_t i = -32768; i < 32768; ++i)
-            data.push_back(i << 32); //-V610
+            data.push_back(i << 32);
     }
 
-    WeakHash32 hash(col->size());
-    col->updateWeakHash32(hash);
+    WeakHash32 hash = col->getWeakHash32();
 
-    checkColumn(hash.getData(), col->getData());
+    checkColumn(hash.getData(), col->getData(), col->getScale());
 }
 
 TEST(WeakHash32, ColumnDecimal128)
@@ -309,13 +280,12 @@ TEST(WeakHash32, ColumnDecimal128)
     for (int idx [[maybe_unused]] : {1, 2})
     {
         for (int64_t i = -32768; i < 32768; ++i)
-            data.push_back(i << 32); //-V610
+            data.push_back(i << 32);
     }
 
-    WeakHash32 hash(col->size());
-    col->updateWeakHash32(hash);
+    WeakHash32 hash = col->getWeakHash32();
 
-    checkColumn(hash.getData(), col->getData());
+    checkColumn(hash.getData(), col->getData(), col->getScale());
 }
 
 TEST(WeakHash32, ColumnString1)
@@ -326,7 +296,7 @@ TEST(WeakHash32, ColumnString1)
 
     for (int idx [[maybe_unused]] : {1, 2})
     {
-        for (int64_t i = 0; i < 65536; ++i)
+        for (int32_t i = 0; i < 65536; ++i)
         {
             data.push_back(i);
             auto str = std::to_string(i);
@@ -334,8 +304,7 @@ TEST(WeakHash32, ColumnString1)
         }
     }
 
-    WeakHash32 hash(col->size());
-    col->updateWeakHash32(hash);
+    WeakHash32 hash = col->getWeakHash32();
 
     checkColumn(hash.getData(), data);
 }
@@ -359,7 +328,7 @@ TEST(WeakHash32, ColumnString2)
     {
         size_t max_size = 3000;
         char letter = 'a';
-        for (int64_t i = 0; i < 65536; ++i)
+        for (int32_t i = 0; i < 65536; ++i)
         {
             data.push_back(i);
             size_t s = (i % max_size) + 1;
@@ -371,13 +340,9 @@ TEST(WeakHash32, ColumnString2)
         }
     }
 
-    WeakHash32 hash(col->size());
-    col->updateWeakHash32(hash);
+    WeakHash32 hash = col->getWeakHash32();
 
-    /// Now there is single collision between 'k' * 544 and 'q' * 2512 (which is calculated twice)
-    size_t allowed_collisions = 4;
-
-    checkColumn(hash.getData(), data, allowed_collisions);
+    checkColumn(hash.getData(), data);
 }
 
 TEST(WeakHash32, ColumnString3)
@@ -401,7 +366,7 @@ TEST(WeakHash32, ColumnString3)
         char letter = 'a';
         for (int64_t i = 0; i < 65536; ++i)
         {
-            data.push_back(i);
+            data.push_back(static_cast<UInt32>(i));
             size_t s = (i % max_size) + 1;
             std::string str(s,'\0');
             str[0] = letter;
@@ -412,8 +377,7 @@ TEST(WeakHash32, ColumnString3)
         }
     }
 
-    WeakHash32 hash(col->size());
-    col->updateWeakHash32(hash);
+    WeakHash32 hash = col->getWeakHash32();
 
     checkColumn(hash.getData(), data);
 }
@@ -430,7 +394,7 @@ TEST(WeakHash32, ColumnFixedString)
         char letter = 'a';
         for (int64_t i = 0; i < 65536; ++i)
         {
-            data.push_back(i);
+            data.push_back(static_cast<UInt32>(i));
             size_t s = (i % max_size) + 1;
             std::string str(s, letter);
             col->insertData(str.data(), str.size());
@@ -440,8 +404,7 @@ TEST(WeakHash32, ColumnFixedString)
         }
     }
 
-    WeakHash32 hash(col->size());
-    col->updateWeakHash32(hash);
+    WeakHash32 hash = col->getWeakHash32();
 
     checkColumn(hash.getData(), data);
 }
@@ -471,7 +434,7 @@ TEST(WeakHash32, ColumnArray)
         UInt32 cur = 0;
         for (int64_t i = 0; i < 65536; ++i)
         {
-            eq_data.push_back(i);
+            eq_data.push_back(static_cast<UInt32>(i));
             size_t s = (i % max_size) + 1;
 
             cur_off += s;
@@ -487,8 +450,7 @@ TEST(WeakHash32, ColumnArray)
 
     auto col_arr = ColumnArray::create(std::move(val), std::move(off));
 
-    WeakHash32 hash(col_arr->size());
-    col_arr->updateWeakHash32(hash);
+    WeakHash32 hash = col_arr->getWeakHash32();
 
     checkColumn(hash.getData(), eq_data);
 }
@@ -505,9 +467,9 @@ TEST(WeakHash32, ColumnArray2)
     UInt64 cur_off = 0;
     for (int idx [[maybe_unused]] : {1, 2})
     {
-        for (int64_t i = 0; i < 1000; ++i)
+        for (int32_t i = 0; i < 1000; ++i)
         {
-            for (size_t j = 0; j < 1000; ++j)
+            for (uint32_t j = 0; j < 1000; ++j)
             {
                 eq_data.push_back(i * 1000 + j);
 
@@ -522,8 +484,7 @@ TEST(WeakHash32, ColumnArray2)
 
     auto col_arr = ColumnArray::create(std::move(val), std::move(off));
 
-    WeakHash32 hash(col_arr->size());
-    col_arr->updateWeakHash32(hash);
+    WeakHash32 hash = col_arr->getWeakHash32();
 
     checkColumn(hash.getData(), eq_data);
 }
@@ -556,7 +517,7 @@ TEST(WeakHash32, ColumnArrayArray)
         UInt32 cur = 1;
         for (int64_t i = 0; i < 3000; ++i)
         {
-            eq_data.push_back(i);
+            eq_data.push_back(static_cast<UInt32>(i));
             size_t s = (i % max_size) + 1;
 
             cur_off2 += s;
@@ -579,8 +540,7 @@ TEST(WeakHash32, ColumnArrayArray)
     auto col_arr = ColumnArray::create(std::move(val), std::move(off));
     auto col_arr_arr = ColumnArray::create(std::move(col_arr), std::move(off2));
 
-    WeakHash32 hash(col_arr_arr->size());
-    col_arr_arr->updateWeakHash32(hash);
+    WeakHash32 hash = col_arr_arr->getWeakHash32();
 
     checkColumn(hash.getData(), eq_data);
 }
@@ -598,8 +558,7 @@ TEST(WeakHash32, ColumnConst)
 
     auto col_const = ColumnConst::create(std::move(inner_col), 256);
 
-    WeakHash32 hash(col_const->size());
-    col_const->updateWeakHash32(hash);
+    WeakHash32 hash = col_const->getWeakHash32();
 
     checkColumn(hash.getData(), data);
 }
@@ -619,8 +578,7 @@ TEST(WeakHash32, ColumnLowcardinality)
         }
     }
 
-    WeakHash32 hash(col->size());
-    col->updateWeakHash32(hash);
+    WeakHash32 hash = col->getWeakHash32();
 
     checkColumn(hash.getData(), data);
 }
@@ -645,8 +603,7 @@ TEST(WeakHash32, ColumnNullable)
 
     auto col_null = ColumnNullable::create(std::move(col), std::move(mask));
 
-    WeakHash32 hash(col_null->size());
-    col_null->updateWeakHash32(hash);
+    WeakHash32 hash = col_null->getWeakHash32();
 
     checkColumn(hash.getData(), eq);
 }
@@ -667,7 +624,7 @@ TEST(WeakHash32, ColumnTupleUInt64UInt64)
         {
             data1.push_back(l);
             data2.push_back(i << 32u);
-            eq.push_back(l * 65536 + i);
+            eq.push_back(static_cast<UInt32>(l * 65536 + i));
         }
     }
 
@@ -676,8 +633,7 @@ TEST(WeakHash32, ColumnTupleUInt64UInt64)
     columns.emplace_back(std::move(col2));
     auto col_tuple = ColumnTuple::create(std::move(columns));
 
-    WeakHash32 hash(col_tuple->size());
-    col_tuple->updateWeakHash32(hash);
+    WeakHash32 hash = col_tuple->getWeakHash32();
 
     checkColumn(hash.getData(), eq);
 }
@@ -695,7 +651,7 @@ TEST(WeakHash32, ColumnTupleUInt64String)
 
         size_t max_size = 3000;
         char letter = 'a';
-        for (int64_t i = 0; i < 65536; ++i)
+        for (int32_t i = 0; i < 65536; ++i)
         {
             data1.push_back(l);
             eq.push_back(l * 65536 + i);
@@ -714,11 +670,9 @@ TEST(WeakHash32, ColumnTupleUInt64String)
     columns.emplace_back(std::move(col2));
     auto col_tuple = ColumnTuple::create(std::move(columns));
 
-    WeakHash32 hash(col_tuple->size());
-    col_tuple->updateWeakHash32(hash);
+    WeakHash32 hash = col_tuple->getWeakHash32();
 
-    size_t allowed_collisions = 8;
-    checkColumn(hash.getData(), eq, allowed_collisions);
+    checkColumn(hash.getData(), eq);
 }
 
 TEST(WeakHash32, ColumnTupleUInt64FixedString)
@@ -737,7 +691,7 @@ TEST(WeakHash32, ColumnTupleUInt64FixedString)
         for (int64_t i = 0; i < 65536; ++i)
         {
             data1.push_back(l);
-            eq.push_back(l * 65536 + i);
+            eq.push_back(static_cast<Int32>(l * 65536 + i));
 
             size_t s = (i % max_size) + 1;
             std::string str(s, letter);
@@ -753,8 +707,7 @@ TEST(WeakHash32, ColumnTupleUInt64FixedString)
     columns.emplace_back(std::move(col2));
     auto col_tuple = ColumnTuple::create(std::move(columns));
 
-    WeakHash32 hash(col_tuple->size());
-    col_tuple->updateWeakHash32(hash);
+    WeakHash32 hash = col_tuple->getWeakHash32();
 
     checkColumn(hash.getData(), eq);
 }
@@ -778,7 +731,7 @@ TEST(WeakHash32, ColumnTupleUInt64Array)
         auto l = idx % 2;
 
         UInt32 cur = 0;
-        for (int64_t i = 0; i < 65536; ++i)
+        for (int32_t i = 0; i < 65536; ++i)
         {
             data1.push_back(l);
             eq_data.push_back(l * 65536 + i);
@@ -800,13 +753,7 @@ TEST(WeakHash32, ColumnTupleUInt64Array)
     columns.emplace_back(ColumnArray::create(std::move(val), std::move(off)));
     auto col_tuple = ColumnTuple::create(std::move(columns));
 
-    WeakHash32 hash(col_tuple->size());
-    col_tuple->updateWeakHash32(hash);
+    WeakHash32 hash = col_tuple->getWeakHash32();
 
-    /// There are 2 collisions right now (repeated 2 times each):
-    /// (0, [array of size 1212 with values 7]) vs (0, [array of size 2265 with values 17])
-    /// (0, [array of size 558 with values 5]) vs (1, [array of size 879 with values 21])
-
-    size_t allowed_collisions = 8;
-    checkColumn(hash.getData(), eq_data, allowed_collisions);
+    checkColumn(hash.getData(), eq_data);
 }
