@@ -18,9 +18,11 @@ void AlignedBuffer::alloc(size_t size, size_t alignment)
     void * new_buf;
     int res = ::posix_memalign(&new_buf, std::max(alignment, sizeof(void*)), size);
     if (0 != res)
-        throwFromErrno(fmt::format("Cannot allocate memory (posix_memalign), size: {}, alignment: {}.",
-            ReadableSize(size), ReadableSize(alignment)),
-            ErrorCodes::CANNOT_ALLOCATE_MEMORY, res);
+        throw ErrnoException(
+            ErrorCodes::CANNOT_ALLOCATE_MEMORY,
+            "Cannot allocate memory (posix_memalign), size: {}, alignment: {}.",
+            ReadableSize(size),
+            ReadableSize(alignment));
     buf = new_buf;
 }
 
