@@ -639,6 +639,8 @@
     M(757, CANNOT_GET_THREAD_PRIORITY) \
     M(758, TOO_MANY_NAMED_COLLECTIONS) \
     M(759, LICENSING_EXCEPTION) \
+    M(760, UNICODE_ERROR) \
+    M(761, ACME_ERROR) \
 \
     M(900, DISTRIBUTED_CACHE_ERROR) \
     M(901, CANNOT_USE_DISTRIBUTED_CACHE) \
