@@ -4,7 +4,7 @@
 #include <boost/noncopyable.hpp>
 
 
-#if defined(__linux__)
+#if defined(OS_LINUX)
 struct taskstats;
 
 namespace DB
@@ -13,13 +13,13 @@ namespace DB
 class ProcfsMetricsProvider : private boost::noncopyable
 {
 public:
-    ProcfsMetricsProvider(const pid_t /*tid*/);
+    explicit ProcfsMetricsProvider(pid_t /*tid*/);
     ~ProcfsMetricsProvider();
 
     /// Updates only a part of taskstats struct's fields:
     ///  - cpu_run_virtual_total, cpu_delay_total (when /proc/thread-self/schedstat is available)
     ///  - blkio_delay_total                      (when /proc/thread-self/stat is available)
-    ///  - rchar, wchar, read_bytes, write_bytes  (when /prod/thread-self/io is available)
+    ///  - rchar, wchar, read_bytes, write_bytes  (when /proc/thread-self/io is available)
     /// See: man procfs
     void getTaskStats(::taskstats & out_stats) const;
 
@@ -31,13 +31,12 @@ private:
     void readParseAndSetThreadBlkIOStat(::taskstats & out_stats, char *, size_t) const;
     void readParseAndSetThreadIOStat(::taskstats & out_stats, char *, size_t) const;
 
-private:
     int thread_schedstat_fd = -1;
     int thread_stat_fd = -1;
     int thread_io_fd = -1;
 
     /// This field is used for compatibility with TasksStatsCounters::incrementProfileEvents()
-    unsigned short stats_version = 1;
+    unsigned short stats_version = 1; /// NOLINT
 };
 
 }
