@@ -1,8 +1,6 @@
 #pragma once
 
-
-#include <string.h>
-#if !defined(__APPLE__) && !defined(__FreeBSD__)
+#if !defined(OS_DARWIN) && !defined(OS_FREEBSD)
 #include <malloc.h>
 #endif
 #include <algorithm>
@@ -15,8 +13,10 @@
 
 #include <base/bit_cast.h>
 #include <base/extended_types.h>
-#include <Core/Defines.h>
+#include <base/sort.h>
 
+#include <Common/AllocatorWithMemoryTracking.h>
+#include <Common/TargetSpecific.h>
 
 /** Radix sort, has the following functionality:
   *
@@ -33,22 +33,6 @@
   */
 
 
-/** Used as a template parameter. See below.
-  */
-struct RadixSortAllocator
-{
-    void * allocate(size_t size)
-    {
-        return ::operator new(size);
-    }
-
-    void deallocate(void * ptr, size_t size)
-    {
-        ::operator delete(ptr, size);
-    }
-};
-
-
 /** A transformation that transforms the bit representation of a key into an unsigned integer number,
   *  that the order relation over the keys will match the order relation over the obtained unsigned numbers.
   * For floats this conversion does the following:
@@ -90,18 +74,13 @@ struct RadixSortFloatTraits
     using CountType = uint32_t;
 
     /// The type to which the key is transformed to do bit operations. This UInt is the same size as the key.
-    using KeyBits = std::conditional_t<sizeof(Key) == 8, uint64_t, uint32_t>;
+    using KeyBits = std::conditional_t<sizeof(Key) == 8, uint64_t, std::conditional_t<sizeof(Key) == 4, uint32_t, uint16_t>>;
 
     static constexpr size_t PART_SIZE_BITS = 8;    /// With what pieces of the key, in bits, to do one pass - reshuffle of the array.
 
     /// Converting a key into KeyBits is such that the order relation over the key corresponds to the order relation over KeyBits.
     using Transform = RadixSortFloatTransform<KeyBits>;
 
-    /// An object with the functions allocate and deallocate.
-    /// Can be used, for example, to allocate memory for a temporary array on the stack.
-    /// To do this, the allocator itself is created on the stack.
-    using Allocator = RadixSortAllocator;
-
     /// The function to get the key from an array element.
     static Key & extractKey(Element & elem) { return elem; }
 
@@ -114,6 +93,11 @@ struct RadixSortFloatTraits
     {
         return x < y;
     }
+
+    static bool greater(Key x, Key y)
+    {
+        return x > y;
+    }
 };
 
 
@@ -139,7 +123,6 @@ struct RadixSortUIntTraits
     static constexpr size_t PART_SIZE_BITS = 8;
 
     using Transform = RadixSortIdentityTransform<KeyBits>;
-    using Allocator = RadixSortAllocator;
 
     static Key & extractKey(Element & elem) { return elem; }
     static Result & extractResult(Element & elem) { return elem; }
@@ -148,6 +131,11 @@ struct RadixSortUIntTraits
     {
         return x < y;
     }
+
+    static bool greater(Key x, Key y)
+    {
+        return x > y;
+    }
 };
 
 
@@ -173,7 +161,6 @@ struct RadixSortIntTraits
     static constexpr size_t PART_SIZE_BITS = 8;
 
     using Transform = RadixSortSignedTransform<KeyBits>;
-    using Allocator = RadixSortAllocator;
 
     static Key & extractKey(Element & elem) { return elem; }
     static Result & extractResult(Element & elem) { return elem; }
@@ -182,6 +169,11 @@ struct RadixSortIntTraits
     {
         return x < y;
     }
+
+    static bool greater(Key x, Key y)
+    {
+        return x > y;
+    }
 };
 
 
@@ -209,11 +201,28 @@ private:
     static constexpr size_t PART_BITMASK = HISTOGRAM_SIZE - 1;
     static constexpr size_t KEY_BITS = sizeof(Key) * 8;
     static constexpr size_t NUM_PASSES = (KEY_BITS + (Traits::PART_SIZE_BITS - 1)) / Traits::PART_SIZE_BITS;
-
+    static constexpr size_t PREFETCH_DISTANCE = std::max(1UL, 64 / sizeof(Element) / 2);
+    static constexpr size_t UNROLL_DISTANCE = 512 / (8 * sizeof(Element));
 
     static KeyBits keyToBits(Key x) { return bit_cast<KeyBits>(x); }
     static Key bitsToKey(KeyBits x) { return bit_cast<Key>(x); }
 
+    struct LessComparator
+    {
+        ALWAYS_INLINE bool operator()(Element & lhs, Element & rhs)
+        {
+            return Traits::less(Traits::extractKey(lhs), Traits::extractKey(rhs));
+        }
+    };
+
+    struct GreaterComparator
+    {
+        ALWAYS_INLINE bool operator()(Element & lhs, Element & rhs)
+        {
+            return Traits::greater(Traits::extractKey(lhs), Traits::extractKey(rhs));
+        }
+    };
+
     static ALWAYS_INLINE KeyBits getPart(size_t N, KeyBits x)
     {
         if (Traits::Transform::transform_is_simple)
@@ -244,8 +253,10 @@ private:
         }
     }
 
-
-    template <bool DIRECT_WRITE_TO_DESTINATION>
+    /// SOFTWARE_PREFETCH is used only on Intel CPUs, where software prefetching is beneficial
+    /// On AMD x86 CPUs and on ARM, it does not help or even slows down
+    /// See more measurements in https://github.com/ClickHouse/ClickHouse/pull/86378
+    template <bool DIRECT_WRITE_TO_DESTINATION, bool SOFTWARE_PREFETCH>
     static NO_INLINE void radixSortLSDInternal(Element * arr, size_t size, bool reverse, Result * destination)
     {
         /// If the array is smaller than 256, then it is better to use another algorithm.
@@ -255,10 +266,10 @@ private:
         /// For each of the NUM_PASSES bit ranges of the key, consider how many times each value of this bit range met.
         std::unique_ptr<CountType[]> histograms{new CountType[HISTOGRAM_SIZE * NUM_PASSES]{}};
 
-        typename Traits::Allocator allocator;
+        AllocatorWithMemoryTracking<typename Traits::Element> allocator;
 
         /// We will do several passes through the array. On each pass, the data is transferred to another array. Let's allocate this temporary array.
-        Element * swap_buffer = reinterpret_cast<Element *>(allocator.allocate(size * sizeof(Element)));
+        Element * swap_buffer = allocator.allocate(size);
 
         /// Transform the array and calculate the histogram.
         /// NOTE This is slightly suboptimal. Look at https://github.com/powturbo/TurboHist
@@ -272,15 +283,15 @@ private:
         }
 
         {
-            /// Replace the histograms with the accumulated sums: the value in position i is the sum of the previous positions minus one.
-            size_t sums[NUM_PASSES] = {0};
+            /// Replace the histograms with the accumulated sums: the value in position i is the sum of the previous positions.
+            CountType sums[NUM_PASSES] = {0};
 
             for (size_t i = 0; i < HISTOGRAM_SIZE; ++i)
             {
                 for (size_t pass = 0; pass < NUM_PASSES; ++pass)
                 {
-                    size_t tmp = histograms[pass * HISTOGRAM_SIZE + i] + sums[pass];
-                    histograms[pass * HISTOGRAM_SIZE + i] = sums[pass] - 1;
+                    CountType tmp = histograms[pass * HISTOGRAM_SIZE + i] + sums[pass];
+                    histograms[pass * HISTOGRAM_SIZE + i] = sums[pass];
                     sums[pass] = tmp;
                 }
             }
@@ -292,17 +303,59 @@ private:
             Element * writer = pass % 2 ? arr : swap_buffer;
             Element * reader = pass % 2 ? swap_buffer : arr;
 
-            for (size_t i = 0; i < size; ++i)
+            size_t unrolled_end = (size / UNROLL_DISTANCE * UNROLL_DISTANCE);
+
+            size_t i = 0;
+            /// Unrolling allows for vectorized code, but prefetching disables it
+            if constexpr (!SOFTWARE_PREFETCH && UNROLL_DISTANCE >= 2)
             {
-                size_t pos = extractPart(pass, reader[i]);
+                for (; i < unrolled_end; i += UNROLL_DISTANCE)
+                {
+                    size_t positions[UNROLL_DISTANCE];
+
+                    for (size_t p = 0; p < UNROLL_DISTANCE; p++)
+                        positions[p] = extractPart(pass, reader[i + p]);
+
+                    for (size_t p = 0; p < UNROLL_DISTANCE; p++)
+                    {
+                        auto element = reader[i + p];
+
+                        /// Place the element on the next free position.
+                        auto & dest = writer[histograms[pass * HISTOGRAM_SIZE + positions[p]]];
+                        ++histograms[pass * HISTOGRAM_SIZE + positions[p]];
+                        dest = element;
+
+                        /// On the last pass, we do the reverse transformation.
+                        if (!Traits::Transform::transform_is_simple && pass == NUM_PASSES - 1)
+                            Traits::extractKey(dest) = bitsToKey(Traits::Transform::backward(keyToBits(Traits::extractKey(element))));
+                    }
+                }
+            }
+
+            for (; i < size; i++)
+            {
+                auto element = reader[i];
+                size_t pos = extractPart(pass, element);
+
+                if constexpr (SOFTWARE_PREFETCH)
+                {
+                    /// Note that for prefetch to be effective, the distance must be large enough so that the memory access is ready
+                    /// when we actually need it. This depends on CPU and memory subsystem.
+                    if (i + PREFETCH_DISTANCE < size) [[likely]]
+                    {
+                        size_t next_pos = extractPart(pass, reader[i + PREFETCH_DISTANCE]);
+                        __builtin_prefetch(&writer[histograms[pass * HISTOGRAM_SIZE + next_pos]], 1);
+                    }
+                }
 
                 /// Place the element on the next free position.
-                auto & dest = writer[++histograms[pass * HISTOGRAM_SIZE + pos]];
-                dest = reader[i];
+                auto & dest = writer[histograms[pass * HISTOGRAM_SIZE + pos]];
+                ++histograms[pass * HISTOGRAM_SIZE + pos];
+                dest = element;
 
                 /// On the last pass, we do the reverse transformation.
                 if (!Traits::Transform::transform_is_simple && pass == NUM_PASSES - 1)
-                    Traits::extractKey(dest) = bitsToKey(Traits::Transform::backward(keyToBits(Traits::extractKey(reader[i]))));
+                    Traits::extractKey(dest) = bitsToKey(Traits::Transform::backward(keyToBits(Traits::extractKey(element))));
             }
         }
 
@@ -316,16 +369,36 @@ private:
             {
                 for (size_t i = 0; i < size; ++i)
                 {
-                    size_t pos = extractPart(pass, reader[i]);
-                    writer[size - 1 - (++histograms[pass * HISTOGRAM_SIZE + pos])] = Traits::extractResult(reader[i]);
+                    auto element = reader[i];
+                    size_t pos = extractPart(pass, element);
+                    if constexpr (SOFTWARE_PREFETCH)
+                    {
+                        if (i + PREFETCH_DISTANCE < size) [[likely]]
+                        {
+                            size_t next_pos = extractPart(pass, reader[i + PREFETCH_DISTANCE]);
+                            __builtin_prefetch(&writer[size - 1 - histograms[pass * HISTOGRAM_SIZE + next_pos]], 1);
+                        }
+                    }
+                    writer[size - 1 - (histograms[pass * HISTOGRAM_SIZE + pos])] = Traits::extractResult(element);
+                    ++histograms[pass * HISTOGRAM_SIZE + pos];
                 }
             }
             else
             {
                 for (size_t i = 0; i < size; ++i)
                 {
-                    size_t pos = extractPart(pass, reader[i]);
-                    writer[++histograms[pass * HISTOGRAM_SIZE + pos]] = Traits::extractResult(reader[i]);
+                    auto element = reader[i];
+                    size_t pos = extractPart(pass, element);
+                    if constexpr (SOFTWARE_PREFETCH)
+                    {
+                        if (i + PREFETCH_DISTANCE < size)
+                        {
+                            size_t next_pos = extractPart(pass, reader[i + PREFETCH_DISTANCE]);
+                            __builtin_prefetch(&writer[histograms[pass * HISTOGRAM_SIZE + next_pos]], 1);
+                        }
+                    }
+                    writer[histograms[pass * HISTOGRAM_SIZE + pos]] = Traits::extractResult(element);
+                    ++histograms[pass * HISTOGRAM_SIZE + pos];
                 }
             }
         }
@@ -340,7 +413,7 @@ private:
                 std::reverse(arr, arr + size);
         }
 
-        allocator.deallocate(swap_buffer, size * sizeof(Element));
+        allocator.deallocate(swap_buffer, size);
     }
 
 
@@ -353,10 +426,8 @@ private:
      * PASS is counted from least significant (0), so the first pass is NUM_PASSES - 1.
      */
     template <size_t PASS>
-    static inline void radixSortMSDInternal(Element * arr, size_t size, size_t limit)
+    static void radixSortMSDInternal(Element * arr, size_t size, size_t limit)
     {
-//        std::cerr << PASS << ", " << size << ", " << limit << "\n";
-
         /// The beginning of every i-1-th bucket. 0th element will be equal to 1st.
         /// Last element will point to array end.
         std::unique_ptr<Element *[]> prev_buckets{new Element*[HISTOGRAM_SIZE + 1]};
@@ -498,7 +569,7 @@ private:
 
     // A helper to choose sorting algorithm based on array length
     template <size_t PASS>
-    static inline void radixSortMSDInternalHelper(Element * arr, size_t size, size_t limit)
+    static void radixSortMSDInternalHelper(Element * arr, size_t size, size_t limit)
     {
         if (size <= INSERTION_SORT_THRESHOLD)
             insertionSortInternal(arr, size);
@@ -506,13 +577,56 @@ private:
             radixSortMSDInternal<PASS>(arr, size, limit);
     }
 
+    template <bool DIRECT_WRITE_TO_DESTINATION, typename Comparator>
+    static void executeLSDWithTrySortInternal(Element * arr, size_t size, bool reverse, Comparator comparator, Result * destination)
+    {
+        bool try_sort = ::trySort(arr, arr + size, comparator);
+        if (try_sort)
+        {
+            if constexpr (DIRECT_WRITE_TO_DESTINATION)
+            {
+                for (size_t i = 0; i < size; ++i)
+                    destination[i] = Traits::extractResult(arr[i]);
+            }
+
+            return;
+        }
+#if USE_MULTITARGET_CODE
+        if (DB::isArchSupported(DB::TargetArch::GenuineIntel))
+        {
+            radixSortLSDInternal<DIRECT_WRITE_TO_DESTINATION, true>(arr, size, reverse, destination);
+            return;
+        }
+#endif
+        radixSortLSDInternal<DIRECT_WRITE_TO_DESTINATION, false>(arr, size, reverse, destination);
+    }
+
 public:
     /** Least significant digit radix sort (stable).
       * This function will sort inplace (modify 'arr')
       */
     static void executeLSD(Element * arr, size_t size)
     {
-        radixSortLSDInternal<false>(arr, size, false, nullptr);
+#if USE_MULTITARGET_CODE
+        if (DB::isArchSupported(DB::TargetArch::GenuineIntel))
+        {
+            radixSortLSDInternal<false, true>(arr, size, false, nullptr);
+            return;
+        }
+#endif
+        radixSortLSDInternal<false, false>(arr, size, false, nullptr);
+    }
+
+    static void executeLSD(Element * arr, size_t size, bool reverse)
+    {
+#if USE_MULTITARGET_CODE
+        if (DB::isArchSupported(DB::TargetArch::GenuineIntel))
+        {
+            radixSortLSDInternal<false, true>(arr, size, reverse, nullptr);
+            return;
+        }
+#endif
+        radixSortLSDInternal<false, false>(arr, size, reverse, nullptr);
     }
 
     /** This function will start to sort inplace (modify 'arr')
@@ -523,7 +637,42 @@ public:
       */
     static void executeLSD(Element * arr, size_t size, bool reverse, Result * destination)
     {
-        radixSortLSDInternal<true>(arr, size, reverse, destination);
+#if USE_MULTITARGET_CODE
+        if (DB::isArchSupported(DB::TargetArch::GenuineIntel))
+        {
+            radixSortLSDInternal<true, true>(arr, size, reverse, destination);
+            return;
+        }
+#endif
+        radixSortLSDInternal<true, false>(arr, size, reverse, destination);
+    }
+
+    /** Tries to fast sort elements for common sorting patterns (unstable).
+      * If fast sort cannot be performed, execute least significant digit radix sort.
+      */
+    static void executeLSDWithTrySort(Element * arr, size_t size)
+    {
+        return executeLSDWithTrySort(arr, size, false);
+    }
+
+    static void executeLSDWithTrySort(Element * arr, size_t size, bool reverse)
+    {
+        return executeLSDWithTrySort(arr, size, reverse, nullptr);
+    }
+
+    static void executeLSDWithTrySort(Element * arr, size_t size, bool reverse, Result * destination)
+    {
+        if (reverse)
+        {
+
+            if (destination)
+                return executeLSDWithTrySortInternal<true>(arr, size, reverse, GreaterComparator(), destination);
+            return executeLSDWithTrySortInternal<false>(arr, size, reverse, GreaterComparator(), destination);
+        }
+
+        if (destination)
+            return executeLSDWithTrySortInternal<true>(arr, size, reverse, LessComparator(), destination);
+        return executeLSDWithTrySortInternal<false>(arr, size, reverse, LessComparator(), destination);
     }
 
     /* Most significant digit radix sort
