@@ -74,14 +74,17 @@ public:
                 regexp_string_, regexp.error());
     }
 
-    uint64_t apply(std::string & data) const
+    uint64_t applyThrow(std::string & data) const
     {
         auto m = RE2::GlobalReplace(&data, regexp, replacement);
 
         if (throw_on_match && m > 0)
+        {
             throw Exception(ErrorCodes::LOGICAL_ERROR,
-                            "The rule {} was triggered on the log line {}",
-                            name, data);
+                "The rule {} was triggered on the log line {}",
+                name, data);
+        }
+
 
 #ifndef NDEBUG
         matches_count += m;
@@ -89,6 +92,23 @@ public:
         return m;
     }
 
+    uint64_t applyNoThrow(std::string & data) const
+    {
+        /// FIXME(nikitamikhylov): There is a misuse of the SensitiveDataMasker class.
+        /// It is used in some places where we serialize a query to the storage (Keeper for example).
+        /// Effectively breaking it by wiping the crucial information (credentials, etc).
+        /// So, rules that may touch a query (those that mask passwords) should be checked in `applyThrow` method only.
+        if (throw_on_match)
+            return 0;
+
+        auto m = RE2::GlobalReplace(&data, regexp, replacement);
+#ifndef NDEBUG
+        matches_count += m;
+#endif
+        return m;
+
+    }
+
     const std::string & getName() const { return name; }
     const std::string & getReplacementString() const { return replacement_string; }
 #ifndef NDEBUG
@@ -129,15 +149,22 @@ SensitiveDataMasker::SensitiveDataMasker(const Poco::Util::AbstractConfiguration
     LoggerPtr logger = getLogger("SensitiveDataMaskerConfigRead");
 
     std::set<std::string> used_names;
+    std::set<std::string> used_rules;
 
     for (const auto & rule : keys)
     {
+        /// Rules names are expected to be unique and be in a form of "rule1", "rule2", etc.
         if (startsWith(rule, "rule"))
         {
             auto rule_config_prefix = config_prefix + "." + rule;
 
-            auto rule_name = config.getString(rule_config_prefix + ".name", rule_config_prefix);
+            if (!used_rules.insert(rule).second)
+            {
+                throw Exception(ErrorCodes::INVALID_CONFIG_PARAMETER,
+                    "There are at least two rules with the same prefix '{}' in the query_masking_rules configuration", rule);
+            }
 
+            auto rule_name = config.getString(rule_config_prefix + ".name", rule_config_prefix);
             if (!used_names.insert(rule_name).second)
             {
                 throw Exception(ErrorCodes::INVALID_CONFIG_PARAMETER,
@@ -187,11 +214,19 @@ void SensitiveDataMasker::addMaskingRule(
 }
 
 
+size_t SensitiveDataMasker::wipeSensitiveDataThrow(std::string & data) const
+{
+    size_t matches = 0;
+    for (const auto & rule : all_masking_rules)
+        matches += rule->applyThrow(data);
+    return matches;
+}
+
 size_t SensitiveDataMasker::wipeSensitiveData(std::string & data) const
 {
     size_t matches = 0;
     for (const auto & rule : all_masking_rules)
-        matches += rule->apply(data);
+        matches += rule->applyNoThrow(data);
 
     if (matches)
         ProfileEvents::increment(ProfileEvents::QueryMaskingRulesMatch, matches);
@@ -216,12 +251,13 @@ size_t SensitiveDataMasker::rulesCount() const
 }
 
 
-std::string wipeSensitiveDataAndCutToLength(std::string str, size_t max_length)
+std::string wipeSensitiveDataAndCutToLength(std::string str, size_t max_length, bool wipe_sensitive)
 {
     std::string res = std::move(str);
 
-    if (auto masker = SensitiveDataMasker::getInstance())
-        masker->wipeSensitiveData(res);
+    if (wipe_sensitive)
+        if (auto masker = SensitiveDataMasker::getInstance())
+            masker->wipeSensitiveData(res);
 
     size_t length = res.length();
     if (max_length && (length > max_length))
