@@ -1,10 +1,13 @@
-#include <Common/MemoryTrackerBlockerInThread.h>
-#include <Common/TraceSender.h>
-
 #include <IO/WriteBufferFromFileDescriptorDiscardOnFailure.h>
 #include <IO/WriteHelpers.h>
-#include <Common/StackTrace.h>
 #include <Common/CurrentThread.h>
+#include <Common/MemoryTrackerBlockerInThread.h>
+#include <Common/StackTrace.h>
+#include <Common/TraceSender.h>
+#include <Common/setThreadName.h>
+#include <base/defines.h>
+
+#include <string_view>
 
 namespace
 {
@@ -45,6 +48,7 @@ void TraceSender::send(TraceType trace_type, const StackTrace & stack_trace, Ext
         + sizeof(StackTrace::FramePointers)  /// Collected stack trace, maximum capacity
         + sizeof(TraceType)                  /// trace type
         + sizeof(UInt64)                     /// thread_id
+        + sizeof(ThreadName)                /// thread name enum
         + sizeof(Int64)                      /// size
         + sizeof(void *)                     /// ptr
         + sizeof(UInt8)                      /// memory_context
@@ -89,6 +93,8 @@ void TraceSender::send(TraceType trace_type, const StackTrace & stack_trace, Ext
 
     writePODBinary(trace_type, out);
     writePODBinary(thread_id, out);
+    writePODBinary(UInt8(getThreadName()), out);
+
     writePODBinary(extras.size, out);
     writePODBinary(UInt64(extras.ptr), out);
     if (extras.memory_context.has_value())
