@@ -93,6 +93,11 @@ namespace DB
     class AggregatedZooKeeperLog;
 }
 
+namespace HistogramMetrics
+{
+    struct Metric;
+}
+
 namespace Coordination
 {
 
@@ -272,7 +277,6 @@ private:
         ZooKeeperRequestPtr request;
         ResponseCallback callback;
         WatchCallbackPtrOrEventPtr watch;
-        clock::time_point time;
     };
 
     using RequestsQueue = ConcurrentBoundedQueue<RequestInfo>;
