@@ -1,10 +1,10 @@
-#include "getMappedArea.h"
+#include <Common/getMappedArea.h>
 #include <Common/Exception.h>
 
-#if defined(__linux__)
+#if defined(OS_LINUX)
 
-#include <Common/StringUtils/StringUtils.h>
-#include <Common/hex.h>
+#include <Common/StringUtils.h>
+#include <base/hex.h>
 #include <IO/ReadBufferFromFile.h>
 #include <IO/ReadHelpers.h>
 
@@ -58,7 +58,7 @@ std::pair<void *, size_t> getMappedArea(void * ptr)
             return {reinterpret_cast<void *>(begin), end - begin};
     }
 
-    throw Exception("Cannot find mapped area for pointer", ErrorCodes::LOGICAL_ERROR);
+    throw Exception(ErrorCodes::LOGICAL_ERROR, "Cannot find mapped area for pointer");
 }
 
 }
@@ -75,7 +75,7 @@ namespace ErrorCodes
 
 std::pair<void *, size_t> getMappedArea(void *)
 {
-    throw Exception("The function getMappedArea is implemented only for Linux", ErrorCodes::NOT_IMPLEMENTED);
+    throw Exception(ErrorCodes::NOT_IMPLEMENTED, "The function getMappedArea is implemented only for Linux");
 }
 
 }
