@@ -7,25 +7,33 @@ namespace DB
 {
 String quoteString(std::string_view x)
 {
-    String res(x.size(), '\0');
+    String res(2 + x.size(), '\0');
     WriteBufferFromString wb(res);
     writeQuotedString(x, wb);
     return res;
 }
 
-
-String doubleQuoteString(const StringRef & x)
+String quoteStringSingleQuoteWithSingleQuote(std::string_view x)
 {
-    String res(x.size, '\0');
+    String res(2 + x.size(), '\0');
+    WriteBufferFromString wb(res);
+    writeQuotedStringPostgreSQL(x, wb);
+    return res;
+}
+
+
+String doubleQuoteString(StringRef x)
+{
+    String res(2 + x.size, '\0');
     WriteBufferFromString wb(res);
     writeDoubleQuotedString(x, wb);
     return res;
 }
 
 
-String backQuote(const StringRef & x)
+String backQuote(StringRef x)
 {
-    String res(x.size, '\0');
+    String res(2 + x.size, '\0');
     {
         WriteBufferFromString wb(res);
         writeBackQuotedString(x, wb);
@@ -34,13 +42,25 @@ String backQuote(const StringRef & x)
 }
 
 
-String backQuoteIfNeed(const StringRef & x)
+String backQuoteIfNeed(StringRef x)
 {
-    String res(x.size, '\0');
+    String res(2 + x.size, '\0');
     {
         WriteBufferFromString wb(res);
         writeProbablyBackQuotedString(x, wb);
     }
     return res;
 }
+
+
+String backQuoteMySQL(StringRef x)
+{
+    String res(2 + x.size, '\0');
+    {
+        WriteBufferFromString wb(res);
+        writeBackQuotedStringMySQL(x, wb);
+    }
+    return res;
+}
+
 }
