@@ -1,3 +1,9 @@
+#if !defined(SANITIZER)
+
+/// This test is slow due to exhaustive checking of time zones.
+/// Better to replace with randomization.
+/// Also, recommended to replace with a functional test for better maintainability.
+
 #include <Common/DateLUT.h>
 #include <Common/DateLUTImpl.h>
 
@@ -10,9 +16,7 @@
 
 
 /// For the expansion of gtest macros.
-#if defined(__clang__)
-    #pragma clang diagnostic ignored "-Wused-but-marked-unused"
-#endif
+#pragma clang diagnostic ignored "-Wused-but-marked-unused"
 
 // All timezones present at build time and embedded into ClickHouse binary.
 extern const char * auto_time_zones[];
@@ -58,7 +62,7 @@ FailuresCount countFailures(const ::testing::TestResult & test_result)
     const size_t count = test_result.total_part_count();
     for (size_t i = 0; i < count; ++i)
     {
-        const auto & part = test_result.GetTestPartResult(i);
+        const auto & part = test_result.GetTestPartResult(static_cast<int>(i));
         if (part.nonfatally_failed())
         {
             ++failures.non_fatal;
@@ -79,8 +83,13 @@ FailuresCount countFailures(const ::testing::TestResult & test_result)
 TEST(DateLUTTest, makeDayNumTest)
 {
     const DateLUTImpl & lut = DateLUT::instance("UTC");
-    EXPECT_EQ(0, lut.makeDayNum(2500, 12, 25));
-    EXPECT_EQ(0, lut.makeDayNum(1924, 12, 31));
+    EXPECT_EQ(0, lut.makeDayNum(1899, 12, 31));
+    EXPECT_EQ(-1, lut.makeDayNum(1899, 12, 31, -1));
+    EXPECT_EQ(-25567, lut.makeDayNum(1900, 1, 1));
+    EXPECT_EQ(-16436, lut.makeDayNum(1925, 1, 1));
+    EXPECT_EQ(0, lut.makeDayNum(1970, 1, 1));
+    EXPECT_EQ(120529, lut.makeDayNum(2300, 12, 31));
+    EXPECT_EQ(120529, lut.makeDayNum(2500, 12, 25));
 }
 
 
@@ -115,7 +124,7 @@ TEST(DateLUTTest, TimeValuesInMiddleOfRange)
     EXPECT_EQ(lut.toSecond(time), 11 /*unsigned*/);
     EXPECT_EQ(lut.toMinute(time), 20 /*unsigned*/);
     EXPECT_EQ(lut.toStartOfMinute(time), 1568650800 /*time_t*/);
-    EXPECT_EQ(lut.toStartOfFiveMinute(time), 1568650800 /*time_t*/);
+    EXPECT_EQ(lut.toStartOfFiveMinutes(time), 1568650800 /*time_t*/);
     EXPECT_EQ(lut.toStartOfFifteenMinutes(time), 1568650500 /*time_t*/);
     EXPECT_EQ(lut.toStartOfTenMinutes(time), 1568650800 /*time_t*/);
     EXPECT_EQ(lut.toStartOfHour(time), 1568649600 /*time_t*/);
@@ -129,6 +138,7 @@ TEST(DateLUTTest, TimeValuesInMiddleOfRange)
     EXPECT_EQ(lut.toRelativeMonthNum(time), 24237 /*unsigned*/);
     EXPECT_EQ(lut.toRelativeQuarterNum(time), 8078 /*unsigned*/);
     EXPECT_EQ(lut.toRelativeHourNum(time), 435736 /*time_t*/);
+    EXPECT_EQ(lut.toStableRelativeHourNum(time), 435757 /*time_t*/);
     EXPECT_EQ(lut.toRelativeMinuteNum(time), 26144180 /*time_t*/);
     EXPECT_EQ(lut.toStartOfMinuteInterval(time, 6), 1568650680 /*time_t*/);
     EXPECT_EQ(lut.toStartOfSecondInterval(time, 7), 1568650811 /*time_t*/);
@@ -142,6 +152,10 @@ TEST(DateLUTTest, TimeValuesInMiddleOfRange)
     EXPECT_EQ(lut.addYears(time, 10), 1884270011 /*time_t*/);
     EXPECT_EQ(lut.timeToString(time), "2019-09-16 19:20:11" /*std::string*/);
     EXPECT_EQ(lut.dateToString(time), "2019-09-16" /*std::string*/);
+    EXPECT_EQ(lut.toLastDayOfWeek(time), 1569099600 /*time_t*/);
+    EXPECT_EQ(lut.toLastDayNumOfWeek(time), DayNum(18161) /*DayNum*/);
+    EXPECT_EQ(lut.toLastDayOfMonth(time), 1569790800 /*time_t*/);
+    EXPECT_EQ(lut.toLastDayNumOfMonth(time), DayNum(18169) /*DayNum*/);
 }
 
 
@@ -175,7 +189,7 @@ TEST(DateLUTTest, TimeValuesAtLeftBoderOfRange)
     EXPECT_EQ(lut.toSecond(time), 0 /*unsigned*/);
     EXPECT_EQ(lut.toMinute(time), 0 /*unsigned*/);
     EXPECT_EQ(lut.toStartOfMinute(time), 0 /*time_t*/);
-    EXPECT_EQ(lut.toStartOfFiveMinute(time), 0 /*time_t*/);
+    EXPECT_EQ(lut.toStartOfFiveMinutes(time), 0 /*time_t*/);
     EXPECT_EQ(lut.toStartOfFifteenMinutes(time), 0 /*time_t*/);
     EXPECT_EQ(lut.toStartOfTenMinutes(time), 0 /*time_t*/);
     EXPECT_EQ(lut.toStartOfHour(time), 0 /*time_t*/);
@@ -189,6 +203,7 @@ TEST(DateLUTTest, TimeValuesAtLeftBoderOfRange)
     EXPECT_EQ(lut.toRelativeMonthNum(time), 23641 /*unsigned*/); // ?
     EXPECT_EQ(lut.toRelativeQuarterNum(time), 7880 /*unsigned*/); // ?
     EXPECT_EQ(lut.toRelativeHourNum(time), 0 /*time_t*/);
+    EXPECT_EQ(lut.toStableRelativeHourNum(time), 24 /*time_t*/);
     EXPECT_EQ(lut.toRelativeMinuteNum(time), 0 /*time_t*/);
     EXPECT_EQ(lut.toStartOfMinuteInterval(time, 6), 0 /*time_t*/);
     EXPECT_EQ(lut.toStartOfSecondInterval(time, 7), 0 /*time_t*/);
@@ -202,6 +217,10 @@ TEST(DateLUTTest, TimeValuesAtLeftBoderOfRange)
     EXPECT_EQ(lut.addYears(time, 10), 315532800 /*time_t*/);
     EXPECT_EQ(lut.timeToString(time), "1970-01-01 00:00:00" /*std::string*/);
     EXPECT_EQ(lut.dateToString(time), "1970-01-01" /*std::string*/);
+    EXPECT_EQ(lut.toLastDayOfWeek(time), 259200 /*time_t*/);
+    EXPECT_EQ(lut.toLastDayNumOfWeek(time), DayNum(3) /*DayNum*/);
+    EXPECT_EQ(lut.toLastDayOfMonth(time), 2592000 /*time_t*/);
+    EXPECT_EQ(lut.toLastDayNumOfMonth(time), DayNum(30) /*DayNum*/);
 }
 
 TEST(DateLUTTest, TimeValuesAtRightBoderOfRangeOfOldLUT)
@@ -221,7 +240,7 @@ TEST(DateLUTTest, TimeValuesAtRightBoderOfRangeOfOldLUT)
 
     EXPECT_EQ(lut.toFirstDayOfWeek(time), 4293820800 /*time_t*/);
     EXPECT_EQ(lut.toFirstDayNumOfWeek(time), DayNum(49697));
-    EXPECT_EQ(lut.toFirstDayOfMonth(time), 4291747200 /*time_t*/); // 2016-01-01
+    EXPECT_EQ(lut.toFirstDayOfMonth(time), 4291747200 /*time_t*/); // 2106-01-01
     EXPECT_EQ(lut.toFirstDayNumOfMonth(time), DayNum(49673));
     EXPECT_EQ(lut.toFirstDayNumOfQuarter(time), DayNum(49673) /*DayNum*/);
     EXPECT_EQ(lut.toFirstDayOfQuarter(time), 4291747200 /*time_t*/);
@@ -236,7 +255,7 @@ TEST(DateLUTTest, TimeValuesAtRightBoderOfRangeOfOldLUT)
     EXPECT_EQ(lut.toMinute(time), 17 /*unsigned*/);
     EXPECT_EQ(lut.toSecond(time), 53 /*unsigned*/);
     EXPECT_EQ(lut.toStartOfMinute(time), 4294343820 /*time_t*/);
-    EXPECT_EQ(lut.toStartOfFiveMinute(time), 4294343700 /*time_t*/);
+    EXPECT_EQ(lut.toStartOfFiveMinutes(time), 4294343700 /*time_t*/);
     EXPECT_EQ(lut.toStartOfFifteenMinutes(time), 4294343700 /*time_t*/);
     EXPECT_EQ(lut.toStartOfTenMinutes(time), 4294343400 /*time_t*/);
     EXPECT_EQ(lut.toStartOfHour(time), 4294342800 /*time_t*/);
@@ -250,6 +269,7 @@ TEST(DateLUTTest, TimeValuesAtRightBoderOfRangeOfOldLUT)
     EXPECT_EQ(lut.toRelativeMonthNum(time), 25273 /*unsigned*/);
     EXPECT_EQ(lut.toRelativeQuarterNum(time), 8424 /*unsigned*/);
     EXPECT_EQ(lut.toRelativeHourNum(time), 1192873 /*time_t*/);
+    EXPECT_EQ(lut.toStableRelativeHourNum(time), 1192897 /*time_t*/);
     EXPECT_EQ(lut.toRelativeMinuteNum(time), 71572397 /*time_t*/);
     EXPECT_EQ(lut.toStartOfMinuteInterval(time, 6), 4294343520 /*time_t*/);
     EXPECT_EQ(lut.toStartOfSecondInterval(time, 7), 4294343872 /*time_t*/);
@@ -264,6 +284,10 @@ TEST(DateLUTTest, TimeValuesAtRightBoderOfRangeOfOldLUT)
 
     EXPECT_EQ(lut.timeToString(time), "2106-01-31 01:17:53" /*std::string*/);
     EXPECT_EQ(lut.dateToString(time), "2106-01-31" /*std::string*/);
+    EXPECT_EQ(lut.toLastDayOfWeek(time), 4294339200 /*time_t*/);
+    EXPECT_EQ(lut.toLastDayNumOfWeek(time), DayNum(49703) /*DayNum*/);
+    EXPECT_EQ(lut.toLastDayOfMonth(time), 4294339200 /*time_t*/); // 2106-01-01
+    EXPECT_EQ(lut.toLastDayNumOfMonth(time), DayNum(49703));
 }
 
 
@@ -529,3 +553,4 @@ INSTANTIATE_TEST_SUITE_P(AllTimezones_Year1970,
         }))
 );
 
+#endif
