@@ -1,4 +1,5 @@
 #include <Common/ThreadPool.h>
+#include <Common/CurrentMetrics.h>
 
 #include <gtest/gtest.h>
 
@@ -7,6 +8,13 @@
   */
 
 
+namespace CurrentMetrics
+{
+    extern const Metric LocalThread;
+    extern const Metric LocalThreadActive;
+    extern const Metric LocalThreadScheduled;
+}
+
 TEST(ThreadPool, ConcurrentWait)
 {
     auto worker = []
@@ -18,14 +26,14 @@ TEST(ThreadPool, ConcurrentWait)
     constexpr size_t num_threads = 4;
     constexpr size_t num_jobs = 4;
 
-    ThreadPool pool(num_threads);
+    ThreadPool pool(CurrentMetrics::LocalThread, CurrentMetrics::LocalThreadActive, CurrentMetrics::LocalThreadScheduled, num_threads);
 
     for (size_t i = 0; i < num_jobs; ++i)
         pool.scheduleOrThrowOnError(worker);
 
     constexpr size_t num_waiting_threads = 4;
 
-    ThreadPool waiting_pool(num_waiting_threads);
+    ThreadPool waiting_pool(CurrentMetrics::LocalThread, CurrentMetrics::LocalThreadActive, CurrentMetrics::LocalThreadScheduled, num_waiting_threads);
 
     for (size_t i = 0; i < num_waiting_threads; ++i)
         waiting_pool.scheduleOrThrowOnError([&pool] { pool.wait(); });
