@@ -11,7 +11,6 @@
 #include <Core/Types.h>
 #include <bit>
 
-
 namespace DB
 {
 
@@ -500,14 +499,18 @@ UInt32 compressData(const char * src, UInt32 bytes_size, char * dst)
     static constexpr const UInt32 matrix_size = 64;
     static constexpr const UInt32 header_size = 2 * sizeof(UInt64);
 
-    if (bytes_size % sizeof(T))
-        throw Exception(ErrorCodes::CANNOT_COMPRESS, "Cannot compress with T64 codec, data size {} is not multiplier of {}",
-                        bytes_size, sizeof(T));
+    UInt8 bytes_to_skip = bytes_size % sizeof(T);
+    bytes_size -= bytes_to_skip;
+    memcpy(dst, src, bytes_to_skip);
+    src += bytes_to_skip;
+    dst += bytes_to_skip;
+
+    if (bytes_size == 0)
+        return bytes_to_skip;
 
     UInt32 src_size = bytes_size / sizeof(T);
     UInt32 num_full = src_size / matrix_size;
     UInt32 tail = src_size % matrix_size;
-
     T min;
     T max;
     findMinMax<T>(src, bytes_size, min, max);
@@ -523,7 +526,7 @@ UInt32 compressData(const char * src, UInt32 bytes_size, char * dst)
 
     UInt32 num_bits = getValuableBitsNumber(min64, max64);
     if (!num_bits)
-        return header_size;
+        return header_size + bytes_to_skip;
 
     T buf[matrix_size];
     UInt32 src_shift = sizeof(T) * matrix_size;
@@ -545,7 +548,7 @@ UInt32 compressData(const char * src, UInt32 bytes_size, char * dst)
         dst_bytes += dst_shift;
     }
 
-    return header_size + dst_bytes;
+    return header_size + dst_bytes + bytes_to_skip;
 }
 
 template <typename T, bool full>
@@ -556,15 +559,23 @@ void decompressData(const char * src, UInt32 bytes_size, char * dst, UInt32 unco
     static constexpr const UInt32 matrix_size = 64;
     static constexpr const UInt32 header_size = 2 * sizeof(UInt64);
 
-    if (bytes_size < header_size)
-        throw Exception(ErrorCodes::CANNOT_DECOMPRESS, "Cannot decompress T64-encoded data, data size ({}) is less than the size of T64 header",
-                        bytes_size);
 
-    if (uncompressed_size % sizeof(T))
+    UInt8 bytes_to_skip = uncompressed_size % sizeof(T);
+    memcpy(dst, src, bytes_to_skip);
+
+    uncompressed_size -= bytes_to_skip;
+    bytes_size -= bytes_to_skip;
+    src += bytes_to_skip;
+    dst += bytes_to_skip;
+
+    if (uncompressed_size % sizeof(T) != 0)
         throw Exception(ErrorCodes::CANNOT_DECOMPRESS, "Cannot decompress T64-encoded data, unexpected uncompressed size ({})"
                         " isn't a multiple of the data type size ({})",
                         uncompressed_size, sizeof(T));
 
+    if (uncompressed_size == 0)
+        return;
+
     UInt64 num_elements = uncompressed_size / sizeof(T);
     MinMaxType min;
     MinMaxType max;
@@ -662,7 +673,6 @@ UInt32 CompressionCodecT64::doCompressData(const char * src, UInt32 src_size, ch
     UInt8 cookie = static_cast<UInt8>(serializeTypeId(type_idx)) | (static_cast<UInt8>(variant) << 7);
     memcpy(dst, &cookie, 1);
     dst += 1;
-
     switch (baseType(*type_idx))
     {
         case TypeIndex::Int8:
