@@ -1,21 +1,18 @@
-#include "LZ4_decompress_faster.h"
+#include <Compression/LZ4_decompress_faster.h>
 
-#include <string.h>
-#include <iostream>
-#include <Core/Defines.h>
-#include <Common/Stopwatch.h>
+#include <base/defines.h>
+#include <base/MemorySanitizer.h>
 #include <base/types.h>
 #include <base/unaligned.h>
+#include <Common/Stopwatch.h>
 
-#ifdef __SSE2__
-#include <emmintrin.h>
-#endif
+#include <cstring>
 
 #ifdef __SSSE3__
 #include <tmmintrin.h>
 #endif
 
-#ifdef __aarch64__
+#if defined(__aarch64__) && defined(__ARM_NEON)
 #include <arm_neon.h>
 #endif
 
@@ -25,52 +22,56 @@ namespace LZ4
 namespace
 {
 
-template <size_t N> [[maybe_unused]] void copy(UInt8 * dst, const UInt8 * src);
-template <size_t N> [[maybe_unused]] void wildCopy(UInt8 * dst, const UInt8 * src, UInt8 * dst_end);
-template <size_t N, bool USE_SHUFFLE> [[maybe_unused]] void copyOverlap(UInt8 * op, const UInt8 *& match, size_t offset);
-
-
-inline void copy8(UInt8 * dst, const UInt8 * src)
+ALWAYS_INLINE UInt16 LZ4_readLE16(const void * mem_ptr)
 {
-    memcpy(dst, src, 8);
+    const UInt8* p = reinterpret_cast<const UInt8*>(mem_ptr);
+    return static_cast<UInt16>(p[0]) + (p[1] << 8);
 }
 
-inline void wildCopy8(UInt8 * dst, const UInt8 * src, const UInt8 * dst_end)
+template <size_t block_size>
+ALWAYS_INLINE void copyFromOutput(UInt8 * dst, UInt8 * src)
+{
+    __builtin_memcpy(dst, src, block_size);
+}
+
+template <size_t block_size>
+ALWAYS_INLINE void wildCopyFromInput(UInt8 * __restrict dst, const UInt8 * __restrict src, size_t size)
 {
     /// Unrolling with clang is doing >10% performance degrade.
-#if defined(__clang__)
+    size_t i = 0;
     #pragma nounroll
-#endif
     do
     {
-        copy8(dst, src);
-        dst += 8;
-        src += 8;
-    } while (dst < dst_end);
+        __builtin_memcpy(dst, src, block_size);
+        dst += block_size;
+        src += block_size;
+        i += block_size;
+    } while (i < size);
 }
 
-inline void copyOverlap8(UInt8 * op, const UInt8 *& match, size_t offset)
+
+template <size_t block_size>
+ALWAYS_INLINE void wildCopyFromOutput(UInt8 * dst, const UInt8 * src, size_t size)
 {
-    /// 4 % n.
-    /// Or if 4 % n is zero, we use n.
-    /// It gives equivalent result, but is better CPU friendly for unknown reason.
-    static constexpr int shift1[] = { 0, 1, 2, 1, 4, 4, 4, 4 };
-
-    /// 8 % n - 4 % n
-    static constexpr int shift2[] = { 0, 0, 0, 1, 0, -1, -2, -3 };
-
-    op[0] = match[0];
-    op[1] = match[1];
-    op[2] = match[2];
-    op[3] = match[3];
-
-    match += shift1[offset];
-    memcpy(op + 4, match, 4);
-    match += shift2[offset];
+    /// Unrolling with clang is doing >10% performance degrade.
+    size_t i = 0;
+    #pragma nounroll
+    do
+    {
+        __builtin_memcpy(dst, src, block_size);
+        dst += block_size;
+        src += block_size;
+        i += block_size;
+    } while (i < size);
 }
 
+template <size_t block_size>
+void ALWAYS_INLINE copyOverlap(UInt8 * op, UInt8 *& match, size_t offset);
 
-#if defined(__x86_64__) || defined(__PPC__) || defined(__riscv)
+template <>
+[[maybe_unused]] void ALWAYS_INLINE copyOverlap<8>(UInt8 * op, UInt8 *& match, size_t offset)
+{
+#if defined(__SSSE3__)
 
 /** We use 'xmm' (128bit SSE) registers here to shuffle 16 bytes.
   *
@@ -149,10 +150,6 @@ inline void copyOverlap8(UInt8 * op, const UInt8 *& match, size_t offset)
   * Actually it's possible to use 'emms' instruction after decompression routine.
   * But it's more easy to just use 'xmm' registers and avoid using 'mm' registers.
   */
-inline void copyOverlap8Shuffle(UInt8 * op, const UInt8 *& match, const size_t offset)
-{
-#if defined(__SSSE3__) && !defined(MEMORY_SANITIZER)
-
     static constexpr UInt8 __attribute__((__aligned__(8))) masks[] =
     {
         0, 1, 2, 2, 4, 3, 2, 1, /* offset = 0, not used as mask, but for shift amount instead */
@@ -171,20 +168,13 @@ inline void copyOverlap8Shuffle(UInt8 * op, const UInt8 *& match, const size_t o
             _mm_loadu_si128(reinterpret_cast<const __m128i *>(match)),
             _mm_loadu_si128(reinterpret_cast<const __m128i *>(masks + 8 * offset))));
 
+    /// MSAN does not recognize the store as initializing the memory
+    __msan_unpoison(op, 16);
+
     match += masks[offset];
 
-#else
-    copyOverlap8(op, match, offset);
-#endif
-}
+#elif defined(__aarch64__) && defined(__ARM_NEON)
 
-#endif
-
-
-#ifdef __aarch64__
-
-inline void copyOverlap8Shuffle(UInt8 * op, const UInt8 *& match, const size_t offset)
-{
     static constexpr UInt8 __attribute__((__aligned__(8))) masks[] =
     {
         0, 1, 2, 2, 4, 3, 2, 1, /* offset = 0, not used as mask, but for shift amount instead */
@@ -199,73 +189,30 @@ inline void copyOverlap8Shuffle(UInt8 * op, const UInt8 *& match, const size_t o
 
     unalignedStore<uint8x8_t>(op, vtbl1_u8(unalignedLoad<uint8x8_t>(match), unalignedLoad<uint8x8_t>(masks + 8 * offset)));
     match += masks[offset];
-}
 
-#endif
-
-
-template <> void inline copy<8>(UInt8 * dst, const UInt8 * src) { copy8(dst, src); }
-template <> void inline wildCopy<8>(UInt8 * dst, const UInt8 * src, UInt8 * dst_end) { wildCopy8(dst, src, dst_end); }
-template <> void inline copyOverlap<8, false>(UInt8 * op, const UInt8 *& match, const size_t offset) { copyOverlap8(op, match, offset); }
-template <> void inline copyOverlap<8, true>(UInt8 * op, const UInt8 *& match, const size_t offset) { copyOverlap8Shuffle(op, match, offset); }
-
-
-inline void copy16(UInt8 * dst, const UInt8 * src)
-{
-#ifdef __SSE2__
-    _mm_storeu_si128(reinterpret_cast<__m128i *>(dst),
-        _mm_loadu_si128(reinterpret_cast<const __m128i *>(src)));
 #else
-    memcpy(dst, src, 16);
-#endif
-}
-
-inline void wildCopy16(UInt8 * dst, const UInt8 * src, const UInt8 * dst_end)
-{
-    /// Unrolling with clang is doing >10% performance degrade.
-#if defined(__clang__)
-    #pragma nounroll
-#endif
-    do
-    {
-        copy16(dst, src);
-        dst += 16;
-        src += 16;
-    } while (dst < dst_end);
-}
-
-inline void copyOverlap16(UInt8 * op, const UInt8 *& match, const size_t offset)
-{
     /// 4 % n.
-    static constexpr int shift1[]
-        = { 0,  1,  2,  1,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4 };
+    /// Or if 4 % n is zero, we use n.
+    /// It gives equivalent result, but is better CPU friendly for unknown reason.
+    static constexpr UInt8 shift1[] = {0, 1, 2, 1, 4, 4, 4, 4};
 
-    /// 8 % n - 4 % n
-    static constexpr int shift2[]
-        = { 0,  0,  0,  1,  0, -1, -2, -3, -4,  4,  4,  4,  4,  4,  4,  4 };
-
-    /// 16 % n - 8 % n
-    static constexpr int shift3[]
-        = { 0,  0,  0, -1,  0, -2,  2,  1,  8, -1, -2, -3, -4, -5, -6, -7 };
+    /// Shift amount for match pointer.
+    static constexpr UInt8 shift2[] = {0, 1, 2, 2, 4, 3, 2, 1};
 
     op[0] = match[0];
     op[1] = match[1];
     op[2] = match[2];
     op[3] = match[3];
 
-    match += shift1[offset];
-    memcpy(op + 4, match, 4);
+    memcpy(op + 4, match + shift1[offset], 4);
     match += shift2[offset];
-    memcpy(op + 8, match, 8);
-    match += shift3[offset];
+#endif
 }
 
-
-#if defined(__x86_64__) || defined(__PPC__) || defined (__riscv)
-
-inline void copyOverlap16Shuffle(UInt8 * op, const UInt8 *& match, const size_t offset)
+template <>
+[[maybe_unused]] void ALWAYS_INLINE copyOverlap<16>(UInt8 * op, UInt8 *& match, size_t offset)
 {
-#if defined(__SSSE3__) && !defined(MEMORY_SANITIZER)
+#if defined(__SSSE3__)
 
     static constexpr UInt8 __attribute__((__aligned__(16))) masks[] =
     {
@@ -294,17 +241,11 @@ inline void copyOverlap16Shuffle(UInt8 * op, const UInt8 *& match, const size_t
 
     match += masks[offset];
 
-#else
-    copyOverlap16(op, match, offset);
-#endif
-}
+    /// MSAN does not recognize the store as initializing the memory
+    __msan_unpoison(op, 16);
 
-#endif
+#elif defined(__aarch64__) && defined(__ARM_NEON)
 
-#ifdef __aarch64__
-
-inline void copyOverlap16Shuffle(UInt8 * op, const UInt8 *& match, const size_t offset)
-{
     static constexpr UInt8 __attribute__((__aligned__(16))) masks[] =
     {
         0,  1,  2,  1,  4,  1,  4,  2,  8,  7,  6,  5,  4,  3,  2,  1, /* offset = 0, not used as mask, but for shift amount instead */
@@ -332,91 +273,60 @@ inline void copyOverlap16Shuffle(UInt8 * op, const UInt8 *& match, const size_t
         vtbl2_u8(unalignedLoad<uint8x8x2_t>(match), unalignedLoad<uint8x8_t>(masks + 16 * offset + 8)));
 
     match += masks[offset];
-}
 
-#endif
-
-
-template <> void inline copy<16>(UInt8 * dst, const UInt8 * src) { copy16(dst, src); }
-template <> void inline wildCopy<16>(UInt8 * dst, const UInt8 * src, UInt8 * dst_end) { wildCopy16(dst, src, dst_end); }
-template <> void inline copyOverlap<16, false>(UInt8 * op, const UInt8 *& match, const size_t offset) { copyOverlap16(op, match, offset); }
-template <> void inline copyOverlap<16, true>(UInt8 * op, const UInt8 *& match, const size_t offset) { copyOverlap16Shuffle(op, match, offset); }
-
-
-inline void copy32(UInt8 * dst, const UInt8 * src)
-{
-    /// There was an AVX here but with mash with SSE instructions, we got a big slowdown.
-#if defined(__SSE2__)
-    _mm_storeu_si128(reinterpret_cast<__m128i *>(dst),
-        _mm_loadu_si128(reinterpret_cast<const __m128i *>(src)));
-    _mm_storeu_si128(reinterpret_cast<__m128i *>(dst + 16),
-        _mm_loadu_si128(reinterpret_cast<const __m128i *>(src + 16)));
 #else
-    memcpy(dst, src, 16);
-    memcpy(dst + 16, src + 16, 16);
-#endif
-}
-
-inline void wildCopy32(UInt8 * dst, const UInt8 * src, const UInt8 * dst_end)
-{
-    /// Unrolling with clang is doing >10% performance degrade.
-#if defined(__clang__)
-    #pragma nounroll
-#endif
-    do
-    {
-        copy32(dst, src);
-        dst += 32;
-        src += 32;
-    } while (dst < dst_end);
-}
-
-inline void copyOverlap32(UInt8 * op, const UInt8 *& match, const size_t offset)
-{
     /// 4 % n.
-    static constexpr int shift1[]
-        = { 0,  1,  2,  1,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4 };
+    static constexpr UInt8 shift1[] = {0, 1, 2, 1, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4};
 
-    /// 8 % n - 4 % n
-    static constexpr int shift2[]
-        = { 0,  0,  0,  1,  0, -1, -2, -3, -4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4,  4 };
+    /// 8 % n
+    static constexpr UInt8 shift2[] = {0, 1, 2, 2, 4, 3, 2, 1, 0, 8, 8, 8, 8, 8, 8, 8};
 
-    /// 16 % n - 8 % n
-    static constexpr int shift3[]
-        = { 0,  0,  0, -1,  0, -2,  2,  1,  8, -1, -2, -3, -4, -5, -6, -7,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8,  8 };
-
-    /// 32 % n - 16 % n
-    static constexpr int shift4[]
-        = { 0,  0,  0,  1,  0,  1, -2,  2,  0, -2, -4,  5,  4,  3,  2,  1,  0, -1, -2, -3, -4, -5, -6, -7, -8, -9,-10,-11,-12,-13,-14,-15 };
+    /// 16 % n
+    static constexpr UInt8 shift3[] = {0, 1, 2, 1, 4, 1, 4, 2, 8, 7, 6, 5, 4, 3, 2, 1};
 
     op[0] = match[0];
     op[1] = match[1];
     op[2] = match[2];
     op[3] = match[3];
 
-    match += shift1[offset];
-    memcpy(op + 4, match, 4);
-    match += shift2[offset];
-    memcpy(op + 8, match, 8);
+    memcpy(op + 4, match + shift1[offset], 4);
+    memcpy(op + 8, match + shift2[offset], 8);
     match += shift3[offset];
-    memcpy(op + 16, match, 16);
+#endif
+}
+
+
+template <>
+[[maybe_unused]] void ALWAYS_INLINE copyOverlap<32>(UInt8 * op, UInt8 *& match, size_t offset)
+{
+    /// 4 % n
+    static constexpr UInt8 shift1[] = {0, 1, 2, 1, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4, 4};
+
+    /// 8 % n
+    static constexpr UInt8 shift2[] = {0, 1, 2, 2, 4, 3, 2, 1, 0, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8};
+
+    /// 16 % n
+    static constexpr UInt8 shift3[]
+        = {0, 1, 2, 1, 4, 1, 4, 2, 8, 7, 6, 5, 4, 3, 2, 1, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16, 16};
+
+    /// 32 % n
+    static constexpr UInt8 shift4[]
+        = {0, 1, 2, 2, 4, 2, 2, 4, 8, 5, 2, 10, 8, 6, 4, 2, 16, 15, 14, 13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1};
+
+    op[0] = match[0];
+    op[1] = match[1];
+    op[2] = match[2];
+    op[3] = match[3];
+
+    memcpy(op + 4, match + shift1[offset], 4);
+    memcpy(op + 8, match + shift2[offset], 8);
+    memcpy(op + 16, match + shift3[offset], 16);
     match += shift4[offset];
 }
 
 
-template <> void inline copy<32>(UInt8 * dst, const UInt8 * src) { copy32(dst, src); }
-template <> void inline wildCopy<32>(UInt8 * dst, const UInt8 * src, UInt8 * dst_end) { wildCopy32(dst, src, dst_end); }
-template <> void inline copyOverlap<32, false>(UInt8 * op, const UInt8 *& match, const size_t offset) { copyOverlap32(op, match, offset); }
-
-
-/// See also https://stackoverflow.com/a/30669632
-
-template <size_t copy_amount, bool use_shuffle>
-bool NO_INLINE decompressImpl(
-     const char * const source,
-     char * const dest,
-     size_t source_size,
-     size_t dest_size)
+template <size_t copy_amount>
+bool NO_INLINE decompressImpl(const char * const source, char * const dest, size_t source_size, size_t dest_size)
 {
     const UInt8 * ip = reinterpret_cast<const UInt8 *>(source);
     UInt8 * op = reinterpret_cast<UInt8 *>(dest);
@@ -424,10 +334,6 @@ bool NO_INLINE decompressImpl(
     UInt8 * const output_begin = op;
     UInt8 * const output_end = op + dest_size;
 
-    /// Unrolling with clang is doing >10% performance degrade.
-#if defined(__clang__)
-    #pragma nounroll
-#endif
     while (true)
     {
         size_t length;
@@ -449,6 +355,18 @@ bool NO_INLINE decompressImpl(
 
         const unsigned token = *ip++;
         length = token >> 4;
+
+        UInt8 * copy_end;
+        size_t real_length;
+
+        /// It might be true fairly often for well-compressed columns.
+        /// ATST it may hurt performance in other cases because this condition is hard to predict (especially if the number of zeros is ~50%).
+        /// In such cases this `if` will significantly increase number of mispredicted instructions. But seems like it results in a
+        /// noticeable slowdown only for implementations with `copy_amount` > 8. Probably because they use havier instructions.
+        if constexpr (copy_amount == 8)
+            if (length == 0)
+                goto decompress_match;
+
         if (length == 0x0F)
         {
             if (unlikely(ip + 1 >= input_end))
@@ -458,7 +376,7 @@ bool NO_INLINE decompressImpl(
 
         /// Copy literals.
 
-        UInt8 * copy_end = op + length;
+        copy_end = op + length;
 
         /// input: Hello, world
         ///        ^-ip
@@ -475,7 +393,7 @@ bool NO_INLINE decompressImpl(
             return false;
 
         // Due to implementation specifics the copy length is always a multiple of copy_amount
-        size_t real_length = 0;
+        real_length = 0;
 
         static_assert(copy_amount == 8 || copy_amount == 16 || copy_amount == 32);
         if constexpr (copy_amount == 8)
@@ -486,9 +404,9 @@ bool NO_INLINE decompressImpl(
             real_length = (((length >> 5) + 1) * 32);
 
         if (unlikely(ip + real_length >= input_end + ADDITIONAL_BYTES_AT_END_OF_BUFFER))
-             return false;
+            return false;
 
-        wildCopy<copy_amount>(op, ip, copy_end);    /// Here we can write up to copy_amount - 1 bytes after buffer.
+        wildCopyFromInput<copy_amount>(op, ip, copy_end - op); /// Here we can write up to copy_amount - 1 bytes after buffer.
 
         if (copy_end == output_end)
             return true;
@@ -496,14 +414,16 @@ bool NO_INLINE decompressImpl(
         ip += length;
         op = copy_end;
 
+    decompress_match:
+
         if (unlikely(ip + 1 >= input_end))
             return false;
 
         /// Get match offset.
 
-        size_t offset = unalignedLoad<UInt16>(ip);
+        size_t offset = LZ4_readLE16(ip);
         ip += 2;
-        const UInt8 * match = op - offset;
+        UInt8 * match = op - offset;
 
         if (unlikely(match < output_begin))
             return false;
@@ -523,11 +443,14 @@ bool NO_INLINE decompressImpl(
 
         copy_end = op + length;
 
+        if (unlikely(copy_end > output_end))
+            return false;
+
         /** Here we can write up to copy_amount - 1 - 4 * 2 bytes after buffer.
           * The worst case when offset = 1 and length = 4
           */
 
-        if (unlikely(offset < copy_amount))
+        if (offset < copy_amount)
         {
             /// output: Hello
             ///              ^-op
@@ -540,22 +463,22 @@ bool NO_INLINE decompressImpl(
             /// output: HelloHelloHel
             ///            ^-match   ^-op
 
-            copyOverlap<copy_amount, use_shuffle>(op, match, offset);
+            copyOverlap<copy_amount>(op, match, offset);
         }
         else
         {
-            copy<copy_amount>(op, match);
+            copyFromOutput<copy_amount>(op, match);
             match += copy_amount;
         }
 
         op += copy_amount;
 
-        copy<copy_amount>(op, match);   /// copy_amount + copy_amount - 1 - 4 * 2 bytes after buffer.
+        copyFromOutput<copy_amount>(op, match);   /// copy_amount + copy_amount - 1 - 4 * 2 bytes after buffer.
         if (length > copy_amount * 2)
         {
             if (unlikely(copy_end > output_end))
                 return false;
-            wildCopy<copy_amount>(op + copy_amount, match + copy_amount, copy_end);
+            wildCopyFromOutput<copy_amount>(op + copy_amount, match + copy_amount, copy_end - (op + copy_amount));
         }
 
         op = copy_end;
@@ -564,13 +487,12 @@ bool NO_INLINE decompressImpl(
 
 }
 
-
 bool decompress(
     const char * const source,
     char * const dest,
     size_t source_size,
     size_t dest_size,
-    PerformanceStatistics & statistics [[maybe_unused]])
+    [[maybe_unused]] PerformanceStatistics & statistics)
 {
     if (source_size == 0 || dest_size == 0)
         return true;
@@ -578,113 +500,25 @@ bool decompress(
     /// Don't run timer if the block is too small.
     if (dest_size >= 32768)
     {
-        size_t best_variant = statistics.select();
-
-        /// Run the selected method and measure time.
+        size_t variant_size = 3;
+        size_t best_variant = statistics.select(variant_size);
 
         Stopwatch watch;
-        bool success = true;
+        bool success;
         if (best_variant == 0)
-            success = decompressImpl<16, true>(source, dest, source_size, dest_size);
-        if (best_variant == 1)
-            success = decompressImpl<16, false>(source, dest, source_size, dest_size);
-        if (best_variant == 2)
-            success = decompressImpl<8, true>(source, dest, source_size, dest_size);
-        if (best_variant == 3)
-            success = decompressImpl<32, false>(source, dest, source_size, dest_size);
+            success = decompressImpl<8>(source, dest, source_size, dest_size);
+        else if (best_variant == 1)
+            success = decompressImpl<16>(source, dest, source_size, dest_size);
+        else
+            success = decompressImpl<32>(source, dest, source_size, dest_size);
 
         watch.stop();
-
-        /// Update performance statistics.
-
-        statistics.data[best_variant].update(watch.elapsedSeconds(), dest_size);
+        statistics.data[best_variant].update(watch.elapsedSeconds(), dest_size);  // NOLINT(clang-analyzer-security.ArrayBound)
 
         return success;
     }
-    else
-    {
-        return decompressImpl<8, false>(source, dest, source_size, dest_size);
-    }
-}
 
-
-void StreamStatistics::literal(size_t length)
-{
-    ++num_tokens;
-    sum_literal_lengths += length;
-}
-
-void StreamStatistics::match(size_t length, size_t offset)
-{
-    ++num_tokens;
-    sum_match_lengths += length;
-    sum_match_offsets += offset;
-    count_match_offset_less_8 += offset < 8;
-    count_match_offset_less_16 += offset < 16;
-    count_match_replicate_itself += offset < length;
-}
-
-void StreamStatistics::print() const
-{
-    std::cerr
-        << "Num tokens: " << num_tokens
-        << ", Avg literal length: " << double(sum_literal_lengths) / num_tokens
-        << ", Avg match length: " << double(sum_match_lengths) / num_tokens
-        << ", Avg match offset: " << double(sum_match_offsets) / num_tokens
-        << ", Offset < 8 ratio: " << double(count_match_offset_less_8) / num_tokens
-        << ", Offset < 16 ratio: " << double(count_match_offset_less_16) / num_tokens
-        << ", Match replicate itself: " << double(count_match_replicate_itself) / num_tokens
-        << "\n";
-}
-
-void statistics(
-    const char * const source,
-    char * const dest,
-    size_t dest_size,
-    StreamStatistics & stat)
-{
-    const UInt8 * ip = reinterpret_cast<const UInt8 *>(source);
-    UInt8 * op = reinterpret_cast<UInt8 *>(dest);
-    UInt8 * const output_end = op + dest_size;
-    while (true)
-    {
-        size_t length;
-
-        auto continue_read_length = [&]
-        {
-            unsigned s;
-            do
-            {
-                s = *ip++;
-                length += s;
-            } while (unlikely(s == 255));
-        };
-
-        auto token = *ip++;
-        length = token >> 4;
-        if (length == 0x0F)
-            continue_read_length();
-
-        stat.literal(length);
-
-        ip += length;
-        op += length;
-
-        if (op > output_end)
-            return;
-
-        size_t offset = unalignedLoad<UInt16>(ip);
-        ip += 2;
-
-        length = token & 0x0F;
-        if (length == 0x0F)
-            continue_read_length();
-        length += 4;
-
-        stat.match(length, offset);
-
-        op += length;
-    }
+    return decompressImpl<8>(source, dest, source_size, dest_size);
 }
 
 }
