@@ -129,7 +129,7 @@ KeeperDispatcher::KeeperDispatcher()
 
 void KeeperDispatcher::requestThread()
 {
-    setThreadName("KeeperReqT");
+    DB::setThreadName(ThreadName::KEEPER_REQUEST);
 
     /// Result of requests batch from previous iteration
     RaftAppendResult prev_result = nullptr;
@@ -333,7 +333,8 @@ void KeeperDispatcher::requestThread()
 
 void KeeperDispatcher::responseThread()
 {
-    setThreadName("KeeperRspT");
+    DB::setThreadName(ThreadName::KEEPER_RESPONSE);
+
     const auto & shutdown_called = keeper_context->isShutdownCalled();
     while (!shutdown_called)
     {
@@ -360,7 +361,8 @@ void KeeperDispatcher::responseThread()
 
 void KeeperDispatcher::snapshotThread()
 {
-    setThreadName("KeeperSnpT");
+    DB::setThreadName(ThreadName::KEEPER_SNAPSHOT);
+
     const auto & shutdown_called = keeper_context->isShutdownCalled();
     CreateSnapshotTask task;
     while (snapshots_queue.pop(task))
@@ -747,6 +749,7 @@ void KeeperDispatcher::addErrorResponses(const KeeperRequestsForSessions & reque
         response->xid = request_for_session.request->xid;
         response->zxid = 0;
         response->error = error;
+        response->enqueue_ts = std::chrono::steady_clock::now();
         if (!responses_queue.push(DB::KeeperResponseForSession{request_for_session.session_id, response}))
             throw Exception(ErrorCodes::SYSTEM_ERROR,
                 "Could not push error response xid {} zxid {} error message {} to responses queue",
