@@ -15,6 +15,7 @@
 #include <Common/ThreadPool_fwd.h>
 #include <Common/ZooKeeper/Types.h>
 #include <Common/callOnce.h>
+#include <Common/setThreadName.h>
 #include <Core/BackgroundSchedulePoolTaskHolder.h>
 
 namespace DB
@@ -54,7 +55,7 @@ public:
     /// be error prone. We support only increasing number of threads at runtime.
     void increaseThreadsCount(size_t new_threads_count);
 
-    static BackgroundSchedulePoolPtr create(size_t size, size_t max_parallel_tasks_per_type, CurrentMetrics::Metric tasks_metric, CurrentMetrics::Metric size_metric, const char * thread_name);
+    static BackgroundSchedulePoolPtr create(size_t size, size_t max_parallel_tasks_per_type, CurrentMetrics::Metric tasks_metric, CurrentMetrics::Metric size_metric, ThreadName thread_name);
     ~BackgroundSchedulePool();
 
     /// Shutdown the pool (set flag, destroy threads)
@@ -69,7 +70,7 @@ private:
     using Threads = std::vector<ThreadFromGlobalPoolNoTracingContextPropagation>;
 
     /// @param thread_name_ cannot be longer then 13 bytes (2 bytes is reserved for "/D" suffix for delayExecutionThreadFunction())
-    BackgroundSchedulePool(size_t size_, size_t max_parallel_tasks_per_type_, CurrentMetrics::Metric tasks_metric_, CurrentMetrics::Metric size_metric_, const char * thread_name_);
+    BackgroundSchedulePool(size_t size_, size_t max_parallel_tasks_per_type_, CurrentMetrics::Metric tasks_metric_, CurrentMetrics::Metric size_metric_, ThreadName thread_name_);
 
     void threadFunction();
     void delayExecutionThreadFunction();
@@ -87,6 +88,7 @@ private:
     /// Tasks.
     std::condition_variable tasks_cond_var;
     std::mutex tasks_mutex;
+    LoggerPtr logger;
 
     struct TasksGroup
     {
@@ -110,7 +112,7 @@ private:
 
     CurrentMetrics::Metric tasks_metric;
     CurrentMetrics::Increment size_metric;
-    std::string thread_name;
+    ThreadName thread_name;
 
     size_t max_parallel_tasks_per_type;
 };
