@@ -1,18 +1,26 @@
-#include <Common/Exception.h>
-#include <Common/FieldVisitorToString.h>
-
-#include <Core/Block.h>
-
-#include <IO/WriteBufferFromString.h>
-#include <IO/Operators.h>
-
-#include <Common/assert_cast.h>
-
+#include <AggregateFunctions/IAggregateFunction.h>
+#include <Columns/ColumnAggregateFunction.h>
 #include <Columns/ColumnConst.h>
 #include <Columns/ColumnSparse.h>
+#include <Columns/ColumnReplicated.h>
+#include <Core/Block.h>
+#include <Core/UUID.h>
+#include <DataTypes/DataTypesNumber.h>
+#include <DataTypes/IDataType.h>
+#include <DataTypes/NestedUtils.h>
+#include <DataTypes/Serializations/SerializationInfo.h>
+#include <IO/Operators.h>
+#include <IO/WriteBufferFromString.h>
+#include <base/sort.h>
+#include <Common/Exception.h>
+#include <Common/FieldVisitorToString.h>
+#include <Common/assert_cast.h>
 
 #include <iterator>
-#include <base/sort.h>
+#include <ranges>
+
+#include <boost/algorithm/string.hpp>
+#include <fmt/ranges.h>
 
 
 namespace DB
@@ -27,58 +35,94 @@ namespace ErrorCodes
     extern const int AMBIGUOUS_COLUMN_NAME;
 }
 
-template <typename ReturnType>
-static ReturnType onError(const std::string & message [[maybe_unused]], int code [[maybe_unused]])
+template <typename ReturnType, typename... FmtArgs>
+static ReturnType onError(int code [[maybe_unused]],
+                          FormatStringHelper<FmtArgs...> fmt_string [[maybe_unused]],
+                          FmtArgs && ...fmt_args [[maybe_unused]])
 {
     if constexpr (std::is_same_v<ReturnType, void>)
-        throw Exception(message, code);
+        throw Exception(code, std::move(fmt_string), std::forward<FmtArgs>(fmt_args)...);
     else
         return false;
-};
+}
 
+/// Omit Const, Sparse and Replicated and get actual column.
+static const IColumn * getActualColumn(const IColumn * column)
+{
+    const IColumn * actual_column = column;
+    if (const auto * column_const = typeid_cast<const ColumnConst *>(column))
+        return getActualColumn(&column_const->getDataColumn());
+
+    if (const auto * column_replicated = typeid_cast<const ColumnReplicated *>(column))
+        return getActualColumn(column_replicated->getNestedColumn().get());
+
+    if (const auto * column_sparse = typeid_cast<const ColumnSparse *>(column))
+        return getActualColumn(&column_sparse->getValuesColumn());
+
+    return actual_column;
+}
 
 template <typename ReturnType>
 static ReturnType checkColumnStructure(const ColumnWithTypeAndName & actual, const ColumnWithTypeAndName & expected,
     std::string_view context_description, bool allow_materialize, int code)
 {
     if (actual.name != expected.name)
-        return onError<ReturnType>("Block structure mismatch in " + std::string(context_description) + " stream: different names of columns:\n"
-            + actual.dumpStructure() + "\n" + expected.dumpStructure(), code);
+        return onError<ReturnType>(code, "Block structure mismatch in {} stream: different names of columns:\n{}\n{}",
+                                   context_description, actual.dumpStructure(), expected.dumpStructure());
 
-    if (!actual.type->equals(*expected.type))
-        return onError<ReturnType>("Block structure mismatch in " + std::string(context_description) + " stream: different types:\n"
-            + actual.dumpStructure() + "\n" + expected.dumpStructure(), code);
+    if ((actual.type && !expected.type) || (!actual.type && expected.type)
+        || (actual.type && expected.type && !actual.type->equals(*expected.type)))
+        return onError<ReturnType>(code, "Block structure mismatch in {} stream: different types:\n{}\n{}",
+                                   context_description, actual.dumpStructure(), expected.dumpStructure());
 
     if (!actual.column || !expected.column)
         return ReturnType(true);
 
     const IColumn * actual_column = actual.column.get();
+    const IColumn * expected_column = expected.column.get();
 
-    /// If we allow to materialize, and expected column is not const or sparse, then unwrap actual column.
+    /// If we allow to materialize columns, omit Const, Replicated and Sparse columns.
     if (allow_materialize)
     {
-        if (!isColumnConst(*expected.column))
-            if (const auto * column_const = typeid_cast<const ColumnConst *>(actual_column))
-                actual_column = &column_const->getDataColumn();
-
-        if (!expected.column->isSparse())
-            if (const auto * column_sparse = typeid_cast<const ColumnSparse *>(actual_column))
-                actual_column = &column_sparse->getValuesColumn();
+        actual_column = getActualColumn(actual_column);
+        expected_column = getActualColumn(expected_column);
     }
 
-    if (actual_column->getName() != expected.column->getName())
-        return onError<ReturnType>("Block structure mismatch in " + std::string(context_description) + " stream: different columns:\n"
-            + actual.dumpStructure() + "\n" + expected.dumpStructure(), code);
+    const auto * actual_column_maybe_agg = typeid_cast<const ColumnAggregateFunction *>(actual_column);
+    const auto * expected_column_maybe_agg = typeid_cast<const ColumnAggregateFunction *>(expected_column);
 
-    if (isColumnConst(*actual.column) && isColumnConst(*expected.column))
+    if (actual_column_maybe_agg && expected_column_maybe_agg)
+    {
+        if (!actual_column_maybe_agg->getAggregateFunction()->haveSameStateRepresentation(*expected_column_maybe_agg->getAggregateFunction()))
+            return onError<ReturnType>(code,
+                    "Block structure mismatch in {} stream: different columns:\n{}\n{}",
+                    context_description,
+                    actual.dumpStructure(),
+                    expected.dumpStructure());
+    }
+    else if (actual_column->getName() != expected_column->getName())
+    {
+        return onError<ReturnType>(code,
+                "Block structure mismatch in {} stream: different columns:\n{}\n{}",
+                context_description,
+                actual.dumpStructure(),
+                expected.dumpStructure());
+
+    }
+
+    if (isColumnConst(*actual.column) && isColumnConst(*expected.column)
+        && !actual.column->empty() && !expected.column->empty()) /// don't check values in empty columns
     {
         Field actual_value = assert_cast<const ColumnConst &>(*actual.column).getField();
         Field expected_value = assert_cast<const ColumnConst &>(*expected.column).getField();
 
         if (actual_value != expected_value)
-            return onError<ReturnType>("Block structure mismatch in " + std::string(context_description) + " stream: different values of constants, actual: "
-                + applyVisitor(FieldVisitorToString(), actual_value) + ", expected: " + applyVisitor(FieldVisitorToString(), expected_value),
-                code);
+            return onError<ReturnType>(code,
+                    "Block structure mismatch in {} stream: different values of constants in column '{}': actual: {}, expected: {}",
+                    context_description,
+                    actual.name,
+                    applyVisitor(FieldVisitorToString(), actual_value),
+                    applyVisitor(FieldVisitorToString(), expected_value));
     }
 
     return ReturnType(true);
@@ -88,10 +132,14 @@ static ReturnType checkColumnStructure(const ColumnWithTypeAndName & actual, con
 template <typename ReturnType>
 static ReturnType checkBlockStructure(const Block & lhs, const Block & rhs, std::string_view context_description, bool allow_materialize)
 {
+    /// It's common to have common SharedHeaders in the pipeline
+    if (&lhs == &rhs)
+        return ReturnType(true);
+
     size_t columns = rhs.columns();
     if (lhs.columns() != columns)
-        return onError<ReturnType>("Block structure mismatch in " + std::string(context_description) + " stream: different number of columns:\n"
-            + lhs.dumpStructure() + "\n" + rhs.dumpStructure(), ErrorCodes::LOGICAL_ERROR);
+        return onError<ReturnType>(ErrorCodes::LOGICAL_ERROR, "Block structure mismatch in {} stream: different number of columns:\n{}\n{}",
+                                   context_description, lhs.dumpStructure(), rhs.dumpStructure());
 
     for (size_t i = 0; i < columns; ++i)
     {
@@ -122,6 +170,11 @@ Block::Block(const ColumnsWithTypeAndName & data_) : data{data_}
     initializeIndexByName();
 }
 
+Block::Block(ColumnsWithTypeAndName && data_) : data{std::move(data_)}
+{
+    initializeIndexByName();
+}
+
 
 void Block::initializeIndexByName()
 {
@@ -140,11 +193,11 @@ void Block::reserve(size_t count)
 void Block::insert(size_t position, ColumnWithTypeAndName elem)
 {
     if (position > data.size())
-        throw Exception("Position out of bound in Block::insert(), max position = "
-        + toString(data.size()), ErrorCodes::POSITION_OUT_OF_BOUND);
+        throw Exception(ErrorCodes::POSITION_OUT_OF_BOUND, "Position out of bound in Block::insert(), max position = {}",
+        data.size());
 
     if (elem.name.empty())
-        throw Exception("Column name in Block cannot be empty", ErrorCodes::AMBIGUOUS_COLUMN_NAME);
+        throw Exception(ErrorCodes::AMBIGUOUS_COLUMN_NAME, "Column name in Block cannot be empty");
 
     auto [new_it, inserted] = index_by_name.emplace(elem.name, position);
     if (!inserted)
@@ -164,7 +217,7 @@ void Block::insert(size_t position, ColumnWithTypeAndName elem)
 void Block::insert(ColumnWithTypeAndName elem)
 {
     if (elem.name.empty())
-        throw Exception("Column name in Block cannot be empty", ErrorCodes::AMBIGUOUS_COLUMN_NAME);
+        throw Exception(ErrorCodes::AMBIGUOUS_COLUMN_NAME, "Column name in Block cannot be empty");
 
     auto [it, inserted] = index_by_name.emplace(elem.name, data.size());
     if (!inserted)
@@ -178,9 +231,9 @@ void Block::insert(ColumnWithTypeAndName elem)
 void Block::insertUnique(ColumnWithTypeAndName elem)
 {
     if (elem.name.empty())
-        throw Exception("Column name in Block cannot be empty", ErrorCodes::AMBIGUOUS_COLUMN_NAME);
+        throw Exception(ErrorCodes::AMBIGUOUS_COLUMN_NAME, "Column name in Block cannot be empty");
 
-    if (index_by_name.end() == index_by_name.find(elem.name))
+    if (!index_by_name.contains(elem.name))
         insert(std::move(elem));
 }
 
@@ -195,11 +248,11 @@ void Block::erase(const std::set<size_t> & positions)
 void Block::erase(size_t position)
 {
     if (data.empty())
-        throw Exception("Block is empty", ErrorCodes::POSITION_OUT_OF_BOUND);
+        throw Exception(ErrorCodes::POSITION_OUT_OF_BOUND, "Block is empty");
 
     if (position >= data.size())
-        throw Exception("Position out of bound in Block::erase(), max position = "
-            + toString(data.size() - 1), ErrorCodes::POSITION_OUT_OF_BOUND);
+        throw Exception(ErrorCodes::POSITION_OUT_OF_BOUND, "Position out of bound in Block::erase(), max position = {}",
+            data.size() - 1);
 
     eraseImpl(position);
 }
@@ -227,8 +280,7 @@ void Block::erase(const String & name)
 {
     auto index_it = index_by_name.find(name);
     if (index_it == index_by_name.end())
-        throw Exception("No such name in Block::erase(): '"
-            + name + "'", ErrorCodes::NOT_FOUND_COLUMN_IN_BLOCK);
+        throw Exception(ErrorCodes::NOT_FOUND_COLUMN_IN_BLOCK, "No such name in Block::erase(): '{}'", name);
 
     eraseImpl(index_it->second);
 }
@@ -237,13 +289,11 @@ void Block::erase(const String & name)
 ColumnWithTypeAndName & Block::safeGetByPosition(size_t position)
 {
     if (data.empty())
-        throw Exception("Block is empty", ErrorCodes::POSITION_OUT_OF_BOUND);
+        throw Exception(ErrorCodes::POSITION_OUT_OF_BOUND, "Block is empty");
 
     if (position >= data.size())
-        throw Exception("Position " + toString(position)
-            + " is out of bound in Block::safeGetByPosition(), max position = "
-            + toString(data.size() - 1)
-            + ", there are columns: " + dumpNames(), ErrorCodes::POSITION_OUT_OF_BOUND);
+        throw Exception(ErrorCodes::POSITION_OUT_OF_BOUND, "Position {} is out of bound in Block::safeGetByPosition(), "
+                        "max position = {}, there are columns: {}", toString(position), toString(data.size() - 1), dumpNames());
 
     return data[position];
 }
@@ -252,56 +302,121 @@ ColumnWithTypeAndName & Block::safeGetByPosition(size_t position)
 const ColumnWithTypeAndName & Block::safeGetByPosition(size_t position) const
 {
     if (data.empty())
-        throw Exception("Block is empty", ErrorCodes::POSITION_OUT_OF_BOUND);
+        throw Exception(ErrorCodes::POSITION_OUT_OF_BOUND, "Block is empty");
 
     if (position >= data.size())
-        throw Exception("Position " + toString(position)
-            + " is out of bound in Block::safeGetByPosition(), max position = "
-            + toString(data.size() - 1)
-            + ", there are columns: " + dumpNames(), ErrorCodes::POSITION_OUT_OF_BOUND);
+        throw Exception(ErrorCodes::POSITION_OUT_OF_BOUND, "Position {} is out of bound in Block::safeGetByPosition(), "
+                        "max position = {}, there are columns: {}", toString(position), toString(data.size() - 1), dumpNames());
 
     return data[position];
 }
 
 
-const ColumnWithTypeAndName * Block::findByName(const std::string & name) const
+const ColumnWithTypeAndName * Block::findByName(std::string_view name, bool case_insensitive) const
 {
-    auto it = index_by_name.find(name);
-    if (index_by_name.end() == it)
-    {
-        return nullptr;
-    }
-    return &data[it->second];
+    const auto pos = findPositionByName(name, case_insensitive);
+    return pos.has_value() ? &data[pos.value()] : nullptr;
+}
+
+const ColumnWithTypeAndName * Block::findByName(const std::string & name, bool case_insensitive) const
+{
+    return findByName(std::string_view{name}, case_insensitive);
+}
+
+std::optional<ColumnWithTypeAndName> Block::findSubcolumnByName(const std::string & name) const
+{
+    auto [name_in_storage, subcolumn_name] = Nested::splitName(name);
+    if (subcolumn_name.empty())
+        return std::nullopt;
+
+    const auto * column = findByName(name_in_storage, false);
+    if (!column)
+        return std::nullopt;
+
+    auto subcolumn_type = column->type->tryGetSubcolumnType(subcolumn_name);
+    auto subcolumn = column->type->tryGetSubcolumn(subcolumn_name, column->column);
+    if (!subcolumn_type || !subcolumn)
+        return std::nullopt;
+
+    return ColumnWithTypeAndName(subcolumn, subcolumn_type, name);
+}
+
+std::optional<ColumnWithTypeAndName> Block::findColumnOrSubcolumnByName(const std::string & name) const
+{
+    if (const auto * column = findByName(name, false))
+        return *column;
+
+    return findSubcolumnByName(name);
 }
 
 
-const ColumnWithTypeAndName & Block::getByName(const std::string & name) const
+const ColumnWithTypeAndName & Block::getByName(const std::string & name, bool case_insensitive) const
 {
-    const auto * result = findByName(name);
+    size_t pos = getPositionByName(name, case_insensitive);
+    return data[pos];
+}
+
+ColumnWithTypeAndName Block::getSubcolumnByName(const std::string & name) const
+{
+    auto result = findSubcolumnByName(name);
     if (!result)
-        throw Exception("Not found column " + name + " in block. There are only columns: " + dumpNames()
-            , ErrorCodes::NOT_FOUND_COLUMN_IN_BLOCK);
+        throw Exception(
+            ErrorCodes::NOT_FOUND_COLUMN_IN_BLOCK,
+            "Not found subcolumn {} in block. There are only columns: {}",
+            name,
+            dumpNames());
+
+    return *result;
+}
+
+ColumnWithTypeAndName Block::getColumnOrSubcolumnByName(const std::string & name) const
+{
+    auto result = findColumnOrSubcolumnByName(name);
+    if (!result)
+        throw Exception(
+            ErrorCodes::NOT_FOUND_COLUMN_IN_BLOCK,
+            "Not found column or subcolumn {} in block. There are only columns: {}",
+            name,
+            dumpNames());
 
     return *result;
 }
 
 
-bool Block::has(const std::string & name) const
+bool Block::has(const std::string & name, bool case_insensitive) const
 {
-    return index_by_name.end() != index_by_name.find(name);
+    return findPositionByName(name, case_insensitive).has_value();
 }
 
 
-size_t Block::getPositionByName(const std::string & name) const
+std::optional<size_t> Block::findPositionByName(std::string_view name, bool case_insensitive) const
 {
+    if (case_insensitive)
+    {
+        auto found = std::find_if(data.begin(), data.end(), [&](const auto & column) { return boost::iequals(column.name, name); });
+        if (found == data.end())
+        {
+            return std::nullopt;
+        }
+        return found - data.begin();
+    }
+
     auto it = index_by_name.find(name);
     if (index_by_name.end() == it)
-        throw Exception("Not found column " + name + " in block. There are only columns: " + dumpNames()
-            , ErrorCodes::NOT_FOUND_COLUMN_IN_BLOCK);
-
+    {
+        return std::nullopt;
+    }
     return it->second;
 }
 
+size_t Block::getPositionByName(const std::string & name, bool case_insensitive) const
+{
+    const auto pos = findPositionByName(name, case_insensitive);
+    if (!pos.has_value())
+        throw Exception(
+            ErrorCodes::NOT_FOUND_COLUMN_IN_BLOCK, "Not found column {} in block. There are only columns: {}", name, dumpNames());
+    return pos.value();
+}
 
 void Block::checkNumberOfRows(bool allow_null_columns) const
 {
@@ -312,18 +427,15 @@ void Block::checkNumberOfRows(bool allow_null_columns) const
             continue;
 
         if (!elem.column)
-            throw Exception("Column " + elem.name + " in block is nullptr, in method checkNumberOfRows."
-                , ErrorCodes::SIZES_OF_COLUMNS_DOESNT_MATCH);
+            throw Exception(ErrorCodes::SIZES_OF_COLUMNS_DOESNT_MATCH, "Column {} in block is nullptr, in method checkNumberOfRows." , elem.name);
 
         ssize_t size = elem.column->size();
 
         if (rows == -1)
             rows = size;
         else if (rows != size)
-            throw Exception("Sizes of columns doesn't match: "
-                + data.front().name + ": " + toString(rows)
-                + ", " + elem.name + ": " + toString(size)
-                , ErrorCodes::SIZES_OF_COLUMNS_DOESNT_MATCH);
+            throw Exception(ErrorCodes::SIZES_OF_COLUMNS_DOESNT_MATCH, "Sizes of columns doesn't match: {}: {}, {}: {}",
+                data.front().name, rows, elem.name, toString(size));
     }
 }
 
@@ -342,7 +454,8 @@ size_t Block::bytes() const
 {
     size_t res = 0;
     for (const auto & elem : data)
-        res += elem.column->byteSize();
+        if (elem.column)
+            res += elem.column->byteSize();
 
     return res;
 }
@@ -351,7 +464,8 @@ size_t Block::allocatedBytes() const
 {
     size_t res = 0;
     for (const auto & elem : data)
-        res += elem.column->allocatedBytes();
+        if (elem.column)
+            res += elem.column->allocatedBytes();
 
     return res;
 }
@@ -417,6 +531,14 @@ MutableColumns Block::cloneEmptyColumns() const
     return columns;
 }
 
+MutableColumns Block::cloneEmptyColumns(const Serializations & serializations) const
+{
+    size_t num_columns = data.size();
+    MutableColumns columns(num_columns);
+    for (size_t i = 0; i < num_columns; ++i)
+        columns[i] = data[i].type->createColumn(*serializations[i]);
+    return columns;
+}
 
 Columns Block::getColumns() const
 {
@@ -460,7 +582,7 @@ void Block::setColumn(size_t position, ColumnWithTypeAndName column)
 {
     if (position >= data.size())
         throw Exception(ErrorCodes::POSITION_OUT_OF_BOUND, "Position {} out of bound in Block::setColumn(), max position {}",
-                        position, toString(data.size()));
+                        position, data.size());
 
     if (data[position].name != column.name)
     {
@@ -477,6 +599,17 @@ Block Block::cloneWithColumns(MutableColumns && columns) const
     Block res;
 
     size_t num_columns = data.size();
+
+    if (num_columns != columns.size())
+    {
+        auto dump_columns = std::views::transform([](const auto & col) { return col->dumpStructure(); });
+        throw Exception(
+            ErrorCodes::LOGICAL_ERROR,
+            "Cannot clone block with columns because block [{}] has {} columns, but {} columns given [{}]",
+            dumpStructure(), num_columns,
+            columns.size(), fmt::join(columns | dump_columns, ", "));
+    }
+
     res.reserve(num_columns);
 
     for (size_t i = 0; i < num_columns; ++i)
@@ -493,8 +626,14 @@ Block Block::cloneWithColumns(const Columns & columns) const
     size_t num_columns = data.size();
 
     if (num_columns != columns.size())
-        throw Exception("Cannot clone block with columns because block has " + toString(num_columns) + " columns, "
-                        "but " + toString(columns.size()) + " columns given.", ErrorCodes::LOGICAL_ERROR);
+    {
+        auto dump_columns = std::views::transform([](const auto & col) { return col->dumpStructure(); });
+        throw Exception(
+            ErrorCodes::LOGICAL_ERROR,
+            "Cannot clone block with columns because block [{}] has {} columns, but {} columns given [{}]",
+            dumpStructure(), num_columns,
+            columns.size(), fmt::join(columns | dump_columns, ", "));
+    }
 
     res.reserve(num_columns);
 
@@ -550,6 +689,32 @@ Block Block::sortColumns() const
     return sorted_block;
 }
 
+Block Block::shrinkToFit() const
+{
+    Columns new_columns(data.size(), nullptr);
+    for (size_t i = 0; i < data.size(); ++i)
+        new_columns[i] = data[i].column->cloneResized(data[i].column->size());
+    return cloneWithColumns(new_columns);
+}
+
+Block Block::compress() const
+{
+    size_t num_columns = data.size();
+    Columns new_columns(num_columns);
+    for (size_t i = 0; i < num_columns; ++i)
+        new_columns[i] = data[i].column->compress(/*force_compression=*/false);
+    return cloneWithColumns(new_columns);
+}
+
+Block Block::decompress() const
+{
+    size_t num_columns = data.size();
+    Columns new_columns(num_columns);
+    for (size_t i = 0; i < num_columns; ++i)
+        new_columns[i] = data[i].column->decompress();
+    return cloneWithColumns(new_columns);
+}
+
 
 const ColumnsWithTypeAndName & Block::getColumnsWithTypeAndName() const
 {
@@ -567,6 +732,16 @@ NamesAndTypesList Block::getNamesAndTypesList() const
     return res;
 }
 
+NamesAndTypes Block::getNamesAndTypes() const
+{
+    NamesAndTypes res;
+    res.reserve(columns());
+
+    for (const auto & elem : data)
+        res.emplace_back(elem.name, elem.type);
+
+    return res;
+}
 
 Names Block::getNames() const
 {
@@ -591,6 +766,7 @@ DataTypes Block::getDataTypes() const
     return res;
 }
 
+
 Names Block::getDataTypeNames() const
 {
     Names res;
@@ -602,7 +778,6 @@ Names Block::getDataTypeNames() const
     return res;
 }
 
-
 bool blocksHaveEqualStructure(const Block & lhs, const Block & rhs)
 {
     return checkBlockStructure<bool>(lhs, rhs, "", false);
@@ -614,6 +789,29 @@ void assertBlocksHaveEqualStructure(const Block & lhs, const Block & rhs, std::s
     checkBlockStructure<void>(lhs, rhs, context_description, false);
 }
 
+namespace
+{
+
+Block replaceReplicatedColumnsToNested(const Block & block)
+{
+    Block block_without_replicated;
+    for (const auto & column : block.getColumnsWithTypeAndName())
+    {
+        if (const auto * column_replicated = typeid_cast<const ColumnReplicated *>(column.column.get()))
+            block_without_replicated.insert(ColumnWithTypeAndName(column_replicated->getNestedColumn(), column.type, column.name));
+        else
+            block_without_replicated.insert(column);
+    }
+    return block_without_replicated;
+}
+
+}
+
+void assertBlocksHaveEqualStructureAllowReplicated(const Block & lhs, const Block & rhs, std::string_view context_description)
+{
+    checkBlockStructure<void>(replaceReplicatedColumnsToNested(lhs), replaceReplicatedColumnsToNested(rhs), context_description, false);
+}
+
 
 bool isCompatibleHeader(const Block & actual, const Block & desired)
 {
@@ -677,18 +875,22 @@ void getBlocksDifference(const Block & lhs, const Block & rhs, std::string & out
     while (r > 0)
         right_columns.push_back(rhs.safeGetByPosition(--r));
 
-    WriteBufferFromString lhs_diff_writer(out_lhs_diff);
-    WriteBufferFromString rhs_diff_writer(out_rhs_diff);
-
-    for (auto it = left_columns.rbegin(); it != left_columns.rend(); ++it)
     {
-        lhs_diff_writer << it->dumpStructure();
-        lhs_diff_writer << ", position: " << lhs.getPositionByName(it->name) << '\n';
+        WriteBufferFromString lhs_diff_writer(out_lhs_diff);
+        for (auto it = left_columns.rbegin(); it != left_columns.rend(); ++it)
+        {
+            lhs_diff_writer << it->dumpStructure();
+            lhs_diff_writer << ", position: " << lhs.getPositionByName(it->name) << '\n';
+        }
     }
-    for (auto it = right_columns.rbegin(); it != right_columns.rend(); ++it)
+
     {
-        rhs_diff_writer << it->dumpStructure();
-        rhs_diff_writer << ", position: " << rhs.getPositionByName(it->name) << '\n';
+        WriteBufferFromString rhs_diff_writer(out_rhs_diff);
+        for (auto it = right_columns.rbegin(); it != right_columns.rend(); ++it)
+        {
+            rhs_diff_writer << it->dumpStructure();
+            rhs_diff_writer << ", position: " << rhs.getPositionByName(it->name) << '\n';
+        }
     }
 }
 
@@ -715,27 +917,43 @@ void Block::updateHash(SipHash & hash) const
             col.column->updateHashWithValue(row_no, hash);
 }
 
-void convertToFullIfSparse(Block & block)
+Serializations Block::getSerializations() const
+{
+    Serializations res;
+    res.reserve(data.size());
+
+    for (const auto & column : data)
+        res.push_back(column.type->getDefaultSerialization());
+
+    return res;
+}
+
+Serializations Block::getSerializations(const SerializationInfoByName & hints) const
+{
+    Serializations res;
+    res.reserve(data.size());
+
+    for (const auto & column : data)
+    {
+        auto it = hints.find(column.name);
+        if (it == hints.end())
+            res.push_back(column.type->getSerialization(hints.getSettings()));
+        else
+            res.push_back(column.type->getSerialization(*it->second));
+    }
+
+    return res;
+}
+
+void removeSpecialColumnRepresentations(Block & block)
 {
     for (auto & column : block)
-        column.column = recursiveRemoveSparse(column.column);
+        column.column = removeSpecialRepresentations(column.column);
 }
 
-ColumnPtr getColumnFromBlock(const Block & block, const NameAndTypePair & column)
+Block materializeBlock(const Block & block, bool remove_special_column_representations)
 {
-    auto current_column = block.getByName(column.getNameInStorage()).column;
-    current_column = current_column->decompress();
-
-    if (column.isSubcolumn())
-        return column.getTypeInStorage()->getSubcolumn(column.getSubcolumnName(), current_column);
-
-    return current_column;
-}
-
-
-Block materializeBlock(const Block & block)
-{
-    if (!block)
+    if (block.empty())
         return block;
 
     Block res = block;
@@ -743,16 +961,22 @@ Block materializeBlock(const Block & block)
     for (size_t i = 0; i < columns; ++i)
     {
         auto & element = res.getByPosition(i);
-        element.column = recursiveRemoveSparse(element.column->convertToFullColumnIfConst());
+        element.column = element.column->convertToFullColumnIfConst();
+        if (remove_special_column_representations)
+            element.column = removeSpecialRepresentations(element.column);
     }
 
     return res;
 }
 
-void materializeBlockInplace(Block & block)
+void materializeBlockInplace(Block & block, bool remove_special_column_representations)
 {
     for (size_t i = 0; i < block.columns(); ++i)
-        block.getByPosition(i).column = recursiveRemoveSparse(block.getByPosition(i).column->convertToFullColumnIfConst());
+    {
+        block.getByPosition(i).column = block.getByPosition(i).column->convertToFullColumnIfConst();
+        if (remove_special_column_representations)
+            block.getByPosition(i).column = removeSpecialRepresentations(block.getByPosition(i).column);
+    }
 }
 
 Block concatenateBlocks(const std::vector<Block> & blocks)
@@ -760,6 +984,9 @@ Block concatenateBlocks(const std::vector<Block> & blocks)
     if (blocks.empty())
         return {};
 
+    if (blocks.size() == 1)
+        return blocks[0];
+
     size_t num_rows = 0;
     for (const auto & block : blocks)
         num_rows += block.rows();
@@ -781,4 +1008,30 @@ Block concatenateBlocks(const std::vector<Block> & blocks)
     return out;
 }
 
+String addDummyColumnWithRowCount(Block & block, size_t num_rows)
+{
+    bool has_columns = false;
+    for (const auto & column : block)
+    {
+        if (column.column)
+        {
+            assert(column.column->size() == num_rows);
+            has_columns = true;
+            break;
+        }
+    }
+
+    if (has_columns)
+        return {};
+
+    ColumnWithTypeAndName dummy_column;
+    dummy_column.column = DataTypeUInt8().createColumnConst(num_rows, Field(1));
+    dummy_column.type = std::make_shared<DataTypeUInt8>();
+    /// Generate a random name to avoid collisions with real columns.
+    dummy_column.name = "....dummy...." + toString(UUIDHelpers::generateV4());
+    block.insert(dummy_column);
+
+    return dummy_column.name;
+}
+
 }
