@@ -100,6 +100,18 @@ static constexpr auto DEFAULT_VECTOR_SIMILARITY_INDEX_CACHE_POLICY = "SLRU";
 static constexpr auto DEFAULT_VECTOR_SIMILARITY_INDEX_CACHE_MAX_SIZE = 5_GiB;
 static constexpr auto DEFAULT_VECTOR_SIMILARITY_INDEX_CACHE_SIZE_RATIO = 0.5;
 static constexpr auto DEFAULT_VECTOR_SIMILARITY_INDEX_CACHE_MAX_ENTRIES = 10'000'000;
+static constexpr auto DEFAULT_TEXT_INDEX_DICTIONARY_BLOCK_CACHE_POLICY = "SLRU";
+static constexpr auto DEFAULT_TEXT_INDEX_DICTIONARY_BLOCK_CACHE_MAX_SIZE = 1_GiB;
+static constexpr auto DEFAULT_TEXT_INDEX_DICTIONARY_BLOCK_CACHE_SIZE_RATIO = 0.5;
+static constexpr auto DEFAULT_TEXT_INDEX_DICTIONARY_BLOCK_CACHE_MAX_ENTRIES = 1'000'000;
+static constexpr auto DEFAULT_TEXT_INDEX_HEADER_CACHE_POLICY = "SLRU";
+static constexpr auto DEFAULT_TEXT_INDEX_HEADER_CACHE_MAX_SIZE = 1_GiB;
+static constexpr auto DEFAULT_TEXT_INDEX_HEADER_CACHE_SIZE_RATIO = 0.5;
+static constexpr auto DEFAULT_TEXT_INDEX_HEADER_CACHE_MAX_ENTRIES = 100'000;
+static constexpr auto DEFAULT_TEXT_INDEX_POSTINGS_CACHE_POLICY = "SLRU";
+static constexpr auto DEFAULT_TEXT_INDEX_POSTINGS_CACHE_MAX_SIZE = 2_GiB;
+static constexpr auto DEFAULT_TEXT_INDEX_POSTINGS_CACHE_SIZE_RATIO = 0.5;
+static constexpr auto DEFAULT_TEXT_INDEX_POSTINGS_CACHE_MAX_ENTRIES = 1'000'000;
 static constexpr auto DEFAULT_MMAP_CACHE_MAX_SIZE = 1_KiB; /// chosen by rolling dice
 static constexpr auto DEFAULT_COMPILED_EXPRESSION_CACHE_MAX_SIZE = 128_MiB;
 static constexpr auto DEFAULT_COMPILED_EXPRESSION_CACHE_MAX_ENTRIES = 10'000;
