@@ -1,11 +1,11 @@
 #pragma once
 
+#include <boost/noncopyable.hpp>
 #include <IO/ReadBuffer.h>
 #include <IO/WriteBuffer.h>
-#include "IMySQLReadPacket.h"
-#include "IMySQLWritePacket.h"
-#include "IO/MySQLPacketPayloadReadBuffer.h"
-#include <base/shared_ptr_helper.h>
+#include <Core/MySQL/IMySQLReadPacket.h>
+#include <Core/MySQL/IMySQLWritePacket.h>
+#include <IO/MySQLPacketPayloadReadBuffer.h>
 
 namespace DB
 {
@@ -16,13 +16,19 @@ namespace MySQLProtocol
 /* Writes and reads packets, keeping sequence-id.
  * Throws ProtocolError, if packet with incorrect sequence-id was received.
  */
-class PacketEndpoint : public shared_ptr_helper<PacketEndpoint>
+class PacketEndpoint : boost::noncopyable
 {
 public:
     uint8_t & sequence_id;
     ReadBuffer * in;
     WriteBuffer * out;
 
+    /// For writing.
+    PacketEndpoint(WriteBuffer & out_, uint8_t & sequence_id_);
+
+    /// For reading and writing.
+    PacketEndpoint(ReadBuffer & in_, WriteBuffer & out_, uint8_t & sequence_id_);
+
     MySQLPacketPayloadReadBuffer getPayload();
 
     void receivePacket(IMySQLReadPacket & packet);
@@ -33,7 +39,7 @@ public:
     void resetSequenceId();
 
     template<class T>
-    void sendPacket(const T & packet, bool flush = false)
+    void sendPacket(const T & packet, bool flush = true)
     {
         static_assert(std::is_base_of<IMySQLWritePacket, T>());
         packet.writePayload(*out, sequence_id);
@@ -43,15 +49,6 @@ public:
 
     /// Converts packet to text. Is used for debug output.
     static String packetToText(const String & payload);
-
-protected:
-    /// For writing.
-    PacketEndpoint(WriteBuffer & out_, uint8_t & sequence_id_);
-
-    /// For reading and writing.
-    PacketEndpoint(ReadBuffer & in_, WriteBuffer & out_, uint8_t & sequence_id_);
-
-    friend struct shared_ptr_helper<PacketEndpoint>;
 };
 
 using PacketEndpointPtr = std::shared_ptr<PacketEndpoint>;
