@@ -1,10 +1,9 @@
 #pragma once
 
-#include "config_core.h"
+#include "config.h"
 
 #if USE_LIBPQXX
 
-#include <Core/Block.h>
 #include <Core/ExternalResultDescription.h>
 #include <Core/Field.h>
 
@@ -22,11 +21,11 @@ struct PostgreSQLArrayInfo
 
 void insertPostgreSQLValue(
         IColumn & column, std::string_view value,
-        const ExternalResultDescription::ValueType type, const DataTypePtr data_type,
-        std::unordered_map<size_t, PostgreSQLArrayInfo> & array_info, size_t idx);
+        ExternalResultDescription::ValueType type, DataTypePtr data_type,
+        const std::unordered_map<size_t, PostgreSQLArrayInfo> & array_info, size_t idx);
 
 void preparePostgreSQLArrayInfo(
-        std::unordered_map<size_t, PostgreSQLArrayInfo> & array_info, size_t column_idx, const DataTypePtr data_type);
+        std::unordered_map<size_t, PostgreSQLArrayInfo> & array_info, size_t column_idx, DataTypePtr data_type);
 
 void insertDefaultPostgreSQLValue(IColumn & column, const IColumn & sample_column);
 
