@@ -1,10 +1,17 @@
-#include "PostgreSQLProtocol.h"
+#include <Core/PostgreSQLProtocol.h>
+#include <DataTypes/IDataType.h>
 
 namespace DB::PostgreSQLProtocol::Messaging
 {
 
-ColumnTypeSpec convertTypeIndexToPostgresColumnTypeSpec(TypeIndex type_index)
+ColumnTypeSpec convertDataTypeToPostgresColumnTypeSpec(const DataTypePtr & data_type)
 {
+    // Check for Bool type first
+    if (isBool(data_type))
+        return {ColumnType::BOOL, 1};
+
+    // Otherwise use TypeIndex
+    TypeIndex type_index = data_type->getTypeId();
     switch (type_index)
     {
         case TypeIndex::Int8:
@@ -37,6 +44,7 @@ ColumnTypeSpec convertTypeIndexToPostgresColumnTypeSpec(TypeIndex type_index)
         case TypeIndex::Decimal32:
         case TypeIndex::Decimal64:
         case TypeIndex::Decimal128:
+        case TypeIndex::Decimal256:
             return {ColumnType::NUMERIC, -1};
 
         case TypeIndex::UUID:
