@@ -11,9 +11,6 @@ namespace DB
   *
   * Mostly the same as Int64.
   * But also tagged with interval kind.
-  *
-  * Intended usage is for temporary elements in expressions,
-  *  not for storing values in tables.
   */
 class DataTypeInterval final : public DataTypeNumberBase<Int64>
 {
@@ -25,16 +22,18 @@ public:
 
     IntervalKind getKind() const { return kind; }
 
-    DataTypeInterval(IntervalKind kind_) : kind(kind_) {}
+    explicit DataTypeInterval(IntervalKind kind_) : kind(kind_) {}
 
+    SerializationPtr doGetDefaultSerialization() const override;
     std::string doGetName() const override { return fmt::format("Interval{}", kind.toString()); }
     const char * getFamilyName() const override { return "Interval"; }
     TypeIndex getTypeId() const override { return TypeIndex::Interval; }
+    TypeIndex getColumnType() const override { return TypeIndex::Int64; }
 
     bool equals(const IDataType & rhs) const override;
+    void updateHashImpl(SipHash & hash) const override;
 
     bool isParametric() const override { return true; }
-    bool cannotBeStoredInTables() const override { return true; }
     bool isCategorial() const override { return false; }
     bool canBeInsideNullable() const override { return true; }
 };
