@@ -117,6 +117,10 @@ Aggregate function parameter binary encoding (binary encoding of a Field, see sr
 String encodeDataType(const DataTypePtr & type);
 void encodeDataType(const DataTypePtr & type, WriteBuffer & buf);
 
+/// Special version of type encoding that is used for generic hash calculation.
+/// It can skip serializing some data types parameters.
+void encodeDataTypeForHashCalculation(const DataTypePtr & type, WriteBuffer & buf);
+
 DataTypePtr decodeDataType(const String & data);
 DataTypePtr decodeDataType(ReadBuffer & buf);
 
