@@ -22,28 +22,30 @@ class IDataTypeDummy : public IDataType
 private:
     [[noreturn]] void throwNoSerialization() const
     {
-        throw Exception("Serialization is not implemented for data type " + getName(), ErrorCodes::NOT_IMPLEMENTED);
+        throw Exception(ErrorCodes::NOT_IMPLEMENTED, "Serialization is not implemented for data type {}", getName());
     }
 
 public:
     MutableColumnPtr createColumn() const override
     {
-        throw Exception("Method createColumn() is not implemented for data type " + getName(), ErrorCodes::NOT_IMPLEMENTED);
+        throw Exception(ErrorCodes::NOT_IMPLEMENTED, "Method createColumn is not implemented for data type {}", getName());
     }
 
     Field getDefault() const override
     {
-        throw Exception("Method getDefault() is not implemented for data type " + getName(), ErrorCodes::NOT_IMPLEMENTED);
+        throw Exception(ErrorCodes::NOT_IMPLEMENTED, "Method getDefault is not implemented for data type {}", getName());
     }
 
     void insertDefaultInto(IColumn &) const override
     {
-        throw Exception("Method insertDefaultInto() is not implemented for data type " + getName(), ErrorCodes::NOT_IMPLEMENTED);
+        throw Exception(ErrorCodes::NOT_IMPLEMENTED, "Method insertDefaultInto is not implemented for data type {}", getName());
     }
 
     bool haveSubtypes() const override { return false; }
     bool cannotBeStoredInTables() const override { return true; }
 
+    void updateHashImpl(SipHash &) const override {}
+
     SerializationPtr doGetDefaultSerialization() const override { throwNoSerialization(); }
 };
 
