@@ -68,4 +68,10 @@ private:
     std::map<String, BlockPtr> nested_tables;
 };
 
+/// Returns type of scalars of Array of arbitrary dimensions and takes into account Tuples of Nested.
+DataTypePtr getBaseTypeOfArray(DataTypePtr type, const Names & tuple_elements);
+
+/// Returns Array type with requested scalar type and number of dimensions.
+DataTypePtr createArrayOfType(DataTypePtr type, size_t num_dimensions);
+
 }
