@@ -57,13 +57,13 @@ void SerializationMap::deserializeBinary(Field & field, ReadBuffer & istr, const
 {
     size_t size;
     readVarUInt(size, istr);
-    if (settings.binary.max_binary_string_size && size > settings.binary.max_binary_string_size)
+    if (settings.binary.max_binary_array_size && size > settings.binary.max_binary_array_size)
         throw Exception(
             ErrorCodes::TOO_LARGE_ARRAY_SIZE,
             "Too large map size: {}. The maximum is: {}. To increase the maximum, use setting "
             "format_binary_max_array_size",
             size,
-            settings.binary.max_binary_string_size);
+            settings.binary.max_binary_array_size);
     field = Map();
     Map & map = field.safeGet<Map>();
     map.reserve(size);
@@ -86,6 +86,11 @@ void SerializationMap::deserializeBinary(IColumn & column, ReadBuffer & istr, co
     nested->deserializeBinary(extractNestedColumn(column), istr, settings);
 }
 
+void SerializationMap::serializeForHashCalculation(const IColumn & column, size_t row_num, WriteBuffer & ostr) const
+{
+    nested->serializeForHashCalculation(extractNestedColumn(column), row_num, ostr);
+}
+
 
 template <typename KeyWriter, typename ValueWriter>
 void SerializationMap::serializeTextImpl(
