@@ -179,6 +179,16 @@ void SerializationNullable::serializeBinary(const IColumn & column, size_t row_n
         nested->serializeBinary(col.getNestedColumn(), row_num, ostr, settings);
 }
 
+void SerializationNullable::serializeForHashCalculation(const IColumn & column, size_t row_num, WriteBuffer & ostr) const
+{
+    const ColumnNullable & col = assert_cast<const ColumnNullable &>(column);
+
+    bool is_null = col.isNullAt(row_num);
+    writeBinary(is_null, ostr);
+    if (!is_null)
+        nested->serializeForHashCalculation(col.getNestedColumn(), row_num, ostr);
+}
+
 template <typename ReturnType>
 ReturnType safeAppendToNullMap(ColumnNullable & column, bool is_null)
 {
