@@ -1,3 +1,4 @@
+#include <memory>
 #include <Databases/DataLake/DatabaseDataLake.h>
 #include <Core/SettingsEnums.h>
 #include <Databases/DataLake/HiveCatalog.h>
@@ -52,6 +53,9 @@ namespace DatabaseDataLakeSetting
     extern const DatabaseDataLakeSettingsString aws_access_key_id;
     extern const DatabaseDataLakeSettingsString aws_secret_access_key;
     extern const DatabaseDataLakeSettingsString region;
+    extern const DatabaseDataLakeSettingsString onelake_tenant_id;
+    extern const DatabaseDataLakeSettingsString onelake_client_id;
+    extern const DatabaseDataLakeSettingsString onelake_client_secret;
 }
 
 namespace Setting
@@ -146,6 +150,20 @@ std::shared_ptr<DataLake::ICatalog> DatabaseDataLake::getCatalog() const
                 Context::getGlobalContextInstance());
             break;
         }
+        case DB::DatabaseDataLakeCatalogType::ICEBERG_ONELAKE:
+        {
+            catalog_impl = std::make_shared<DataLake::RestCatalog>(
+                settings[DatabaseDataLakeSetting::warehouse].value,
+                url,
+                settings[DatabaseDataLakeSetting::onelake_tenant_id].value,
+                settings[DatabaseDataLakeSetting::onelake_client_id].value,
+                settings[DatabaseDataLakeSetting::onelake_client_secret].value,
+                settings[DatabaseDataLakeSetting::auth_scope].value,
+                settings[DatabaseDataLakeSetting::oauth_server_uri].value,
+                settings[DatabaseDataLakeSetting::oauth_server_use_request_body].value,
+                Context::getGlobalContextInstance());
+            break;
+        }
         case DB::DatabaseDataLakeCatalogType::UNITY:
         {
             catalog_impl = std::make_shared<DataLake::UnityCatalog>(
@@ -195,7 +213,23 @@ std::shared_ptr<StorageObjectStorageConfiguration> DatabaseDataLake::getConfigur
     auto catalog = getCatalog();
     switch (catalog->getCatalogType())
     {
-        case DB::DatabaseDataLakeCatalogType::ICEBERG_HIVE:
+        case DatabaseDataLakeCatalogType::ICEBERG_ONELAKE:
+        {
+            switch (type)
+            {
+#if USE_AZURE_BLOB_STORAGE
+                case DB::DatabaseDataLakeStorageType::Azure:
+                {
+                    return std::make_shared<StorageAzureIcebergConfiguration>(storage_settings);
+                }
+#endif
+                default:
+                    throw Exception(ErrorCodes::BAD_ARGUMENTS,
+                                    "Server does not contain support for storage type {} for Iceberg OneLake catalog",
+                                    type);
+            }
+        }
+        case DatabaseDataLakeCatalogType::ICEBERG_HIVE:
         case DatabaseDataLakeCatalogType::ICEBERG_REST:
         {
             switch (type)
@@ -389,7 +423,7 @@ StoragePtr DatabaseDataLake::tryGetTableImpl(const String & name, ContextPtr con
                 LOG_DEBUG(log, "Has no credentials");
             }
         }
-        else if (!lightweight && table_metadata.requiresCredentials())
+        else if (!lightweight && table_metadata.requiresCredentials() && catalog->getCatalogType() != DatabaseDataLakeCatalogType::ICEBERG_ONELAKE)
         {
             throw Exception(
                ErrorCodes::BAD_ARGUMENTS,
@@ -437,6 +471,22 @@ StoragePtr DatabaseDataLake::tryGetTableImpl(const String & name, ContextPtr con
     settings_copy[Setting::use_hive_partitioning] = false;
     context_copy->setSettings(settings_copy);
 
+    if (catalog->getCatalogType() == DatabaseDataLakeCatalogType::ICEBERG_ONELAKE)
+    {
+        auto azure_configuration = std::static_pointer_cast<StorageAzureIcebergConfiguration>(configuration);
+        if (!azure_configuration)
+            throw Exception(ErrorCodes::LOGICAL_ERROR, "Configuration is not azure type for one lake catalog");
+        auto rest_catalog = std::static_pointer_cast<DataLake::RestCatalog>(catalog);
+        if (!rest_catalog)
+            throw Exception(ErrorCodes::LOGICAL_ERROR, "Catalog is not equals to one lake");
+
+        azure_configuration->setInitializationAsOneLake(
+            rest_catalog->getClientId(),
+            rest_catalog->getClientSecret(),
+            rest_catalog->getTenantId()
+        );
+    }
+
     /// with_table_structure = false: because there will be
     /// no table structure in table definition AST.
     StorageObjectStorageConfiguration::initialize(*configuration, args, context_copy, /* with_table_structure */false);
@@ -660,11 +710,12 @@ DatabaseTablesIteratorPtr DatabaseDataLake::getLightweightTablesIterator(
     return std::make_unique<DatabaseTablesSnapshotIterator>(tables, getDatabaseName());
 }
 
-ASTPtr DatabaseDataLake::getCreateDatabaseQuery() const
+ASTPtr DatabaseDataLake::getCreateDatabaseQueryImpl() const
 {
     const auto & create_query = std::make_shared<ASTCreateQuery>();
-    create_query->setDatabase(getDatabaseName());
+    create_query->setDatabase(database_name);
     create_query->set(create_query->storage, database_engine_definition);
+    create_query->uuid = db_uuid;
     return create_query;
 }
 
@@ -787,6 +838,7 @@ void registerDatabaseDataLake(DatabaseFactory & factory)
 
         switch (catalog_type)
         {
+            case DatabaseDataLakeCatalogType::ICEBERG_ONELAKE:
             case DatabaseDataLakeCatalogType::ICEBERG_REST:
             {
                 if (!args.create_query.attach
