@@ -19,6 +19,7 @@
 #include <unordered_set>
 #include <vector>
 
+
 namespace DB
 {
 
@@ -31,7 +32,7 @@ enum class MySQLDataTypesSupport : uint8_t;
  *  It doesn't make any manipulations with filesystem.
  *  All tables are created by calling code after real-time pull-out structure from remote MySQL
  */
-class DatabaseMySQL final : public IDatabase, WithContext
+class DatabaseMySQL final : public DatabaseWithAltersOnDiskBase, WithContext
 {
 public:
     ~DatabaseMySQL() override;
@@ -56,8 +57,6 @@ public:
 
     DatabaseTablesIteratorPtr getTablesIterator(ContextPtr context, const FilterByNameFunction & filter_by_table_nam, bool skip_not_loaded) const override;
 
-    ASTPtr getCreateDatabaseQuery() const override;
-
     bool isTableExist(const String & name, ContextPtr context) const override;
 
     StoragePtr tryGetTable(const String & name, ContextPtr context) const override;
@@ -82,11 +81,10 @@ public:
 
     void attachTable(ContextPtr context, const String & table_name, const StoragePtr & storage, const String & relative_table_path) override;
 
-    void alterDatabaseComment(const AlterCommand & command) override;
-
     std::vector<std::pair<ASTPtr, StoragePtr>> getTablesForBackup(const FilterByNameFunction &, const ContextPtr &) const override { return {}; }
 
 protected:
+    ASTPtr getCreateDatabaseQueryImpl() const override TSA_REQUIRES(mutex);
     ASTPtr getCreateTableQueryImpl(const String & name, ContextPtr context, bool throw_on_error) const override;
 
 private:
