@@ -21,7 +21,7 @@ struct AlterCommand;
  *  If `cache_tables` == 1 (default: 0) table structure is cached and not checked for being modififed,
  *  but it will be updated during detach->attach.
  */
-class DatabasePostgreSQL final : public IDatabase, WithContext
+class DatabasePostgreSQL final : public DatabaseWithAltersOnDiskBase, WithContext
 {
 
 public:
@@ -42,8 +42,6 @@ public:
 
     bool shouldBeEmptyOnDetach() const override { return false; }
 
-    ASTPtr getCreateDatabaseQuery() const override;
-
     bool empty() const override;
 
     void loadStoredObjects(ContextMutablePtr, LoadingStrictnessLevel /*mode*/) override;
@@ -62,11 +60,10 @@ public:
     void drop(ContextPtr /*context*/) override;
     void shutdown() override;
 
-    void alterDatabaseComment(const AlterCommand & command) override;
-
     std::vector<std::pair<ASTPtr, StoragePtr>> getTablesForBackup(const FilterByNameFunction &, const ContextPtr &) const override { return {}; }
 
 protected:
+    ASTPtr getCreateDatabaseQueryImpl() const override TSA_REQUIRES(mutex);
     ASTPtr getCreateTableQueryImpl(const String & table_name, ContextPtr context, bool throw_on_error) const override;
 
 private:
