@@ -14,7 +14,7 @@ namespace DB
 {
 struct AlterCommand;
 
-class DatabaseSQLite final : public IDatabase, WithContext
+class DatabaseSQLite final : public DatabaseWithAltersOnDiskBase, WithContext
 {
 public:
     using SQLitePtr = std::shared_ptr<sqlite3>;
@@ -34,13 +34,10 @@ public:
 
     bool empty() const override;
 
-    ASTPtr getCreateDatabaseQuery() const override;
-
     void shutdown() override {}
 
-    void alterDatabaseComment(const AlterCommand & command) override;
-
 protected:
+    ASTPtr getCreateDatabaseQueryImpl() const override TSA_REQUIRES(mutex);
     ASTPtr getCreateTableQueryImpl(const String & table_name, ContextPtr context, bool throw_on_error) const override;
 
 private:
