@@ -1,6 +1,7 @@
 #pragma once
 
 #include <Poco/Timestamp.h>
+#include <QueryPipeline/BlockIO.h>
 #include <Dictionaries/IDictionarySource.h>
 #include <Core/Block.h>
 #include <Interpreters/Context_fwd.h>
@@ -21,19 +22,19 @@ public:
 
     FileDictionarySource(const FileDictionarySource & other);
 
-    QueryPipeline loadAll() override;
+    BlockIO loadAll() override;
 
-    QueryPipeline loadUpdatedAll() override
+    BlockIO loadUpdatedAll() override
     {
         throw Exception(ErrorCodes::NOT_IMPLEMENTED, "Method loadUpdatedAll is unsupported for FileDictionarySource");
     }
 
-    QueryPipeline loadIds(const std::vector<UInt64> & /*ids*/) override
+    BlockIO loadIds(const std::vector<UInt64> & /*ids*/) override
     {
         throw Exception(ErrorCodes::NOT_IMPLEMENTED, "Method loadIds is unsupported for FileDictionarySource");
     }
 
-    QueryPipeline loadKeys(const Columns & /*key_columns*/, const std::vector<size_t> & /*requested_rows*/) override
+    BlockIO loadKeys(const Columns & /*key_columns*/, const std::vector<size_t> & /*requested_rows*/) override
     {
         throw Exception(ErrorCodes::NOT_IMPLEMENTED, "Method loadKeys is unsupported for FileDictionarySource");
     }
