@@ -94,8 +94,9 @@ namespace DB
 
     RedisDictionarySource::~RedisDictionarySource() = default;
 
-    QueryPipeline RedisDictionarySource::loadAll()
+    BlockIO RedisDictionarySource::loadAll()
     {
+        BlockIO io;
         auto connection = getRedisConnection(pool, configuration);
 
         RedisCommand command_for_keys("KEYS");
@@ -104,9 +105,12 @@ namespace DB
         /// Get only keys for specified storage type.
         auto all_keys = connection->client->execute<RedisArray>(command_for_keys);
         if (all_keys.isNull())
-            return QueryPipeline(std::make_shared<RedisSource>(
+        {
+            io.pipeline = QueryPipeline(std::make_shared<RedisSource>(
                 std::move(connection), RedisArray{},
                 configuration.storage_type, sample_block, REDIS_MAX_BLOCK_SIZE));
+            return io;
+        }
 
         RedisArray keys;
         auto key_type = storageTypeToKeyType(configuration.storage_type);
@@ -119,12 +123,13 @@ namespace DB
             keys = *getRedisHashMapKeys(connection, keys);
         }
 
-        return QueryPipeline(std::make_shared<RedisSource>(
+        io.pipeline = QueryPipeline(std::make_shared<RedisSource>(
             std::move(connection), std::move(keys),
             configuration.storage_type, sample_block, REDIS_MAX_BLOCK_SIZE));
+        return io;
     }
 
-    QueryPipeline RedisDictionarySource::loadIds(const std::vector<UInt64> & ids)
+    BlockIO RedisDictionarySource::loadIds(const std::vector<UInt64> & ids)
     {
         auto connection = getRedisConnection(pool, configuration);
 
@@ -139,12 +144,14 @@ namespace DB
         for (UInt64 id : ids)
             keys << DB::toString(id);
 
-        return QueryPipeline(std::make_shared<RedisSource>(
+        BlockIO io;
+        io.pipeline = QueryPipeline(std::make_shared<RedisSource>(
             std::move(connection), std::move(keys),
             configuration.storage_type, sample_block, REDIS_MAX_BLOCK_SIZE));
+        return io;
     }
 
-    QueryPipeline RedisDictionarySource::loadKeys(const Columns & key_columns, const std::vector<size_t> & requested_rows)
+    BlockIO RedisDictionarySource::loadKeys(const Columns & key_columns, const std::vector<size_t> & requested_rows)
     {
         auto connection = getRedisConnection(pool, configuration);
 
@@ -169,9 +176,11 @@ namespace DB
             keys.add(key);
         }
 
-        return QueryPipeline(std::make_shared<RedisSource>(
+        BlockIO io;
+        io.pipeline = QueryPipeline(std::make_shared<RedisSource>(
             std::move(connection), std::move(keys),
             configuration.storage_type, sample_block, REDIS_MAX_BLOCK_SIZE));
+        return io;
     }
 
     String RedisDictionarySource::toString() const
