@@ -316,15 +316,20 @@ void RegExpTreeDictionary::loadData()
 {
     if (!source_ptr->hasUpdateField())
     {
-        QueryPipeline pipeline(source_ptr->loadAll());
-        DictionaryPipelineExecutor executor(pipeline, configuration.use_async_executor);
-        pipeline.setConcurrencyControl(false);
+        BlockIO io = source_ptr->loadAll();
 
-        Block block;
-        while (executor.pull(block))
+        DictionaryPipelineExecutor executor(io.pipeline, configuration.use_async_executor);
+        io.pipeline.setConcurrencyControl(false);
+
+        io.executeWithCallbacks([&]()
         {
-            initRegexNodes(block);
-        }
+            Block block;
+            while (executor.pull(block))
+            {
+                initRegexNodes(block);
+            }
+        });
+
         initGraph();
         if (simple_regexps.empty() && complex_regexp_nodes.empty())
             throw Exception(ErrorCodes::INCORRECT_DICTIONARY_DEFINITION, "There are no available regular expression. Please check your config");
