@@ -8,6 +8,6 @@ namespace DB
 class QueryPipeline;
 
 /// Using in MySQLDictionarySource and XDBCDictionarySource after processing invalidate_query.
-std::string readInvalidateQuery(QueryPipeline pipeline);
+std::string readInvalidateQuery(QueryPipeline & pipeline);
 
 }
