@@ -1,6 +1,6 @@
 #pragma once
 
-#include <Core/BackgroundSchedulePool.h>
+#include <Core/BackgroundSchedulePoolTaskHolder.h>
 #include <Interpreters/Context_fwd.h>
 
 namespace Poco
@@ -29,10 +29,10 @@ private:
 
     DiskLocal * disk;
     size_t check_period_ms;
-    Poco::Logger * log;
+    LoggerPtr log;
     std::atomic<bool> need_stop{false};
 
-    BackgroundSchedulePool::TaskHolder task;
+    BackgroundSchedulePoolTaskHolder task;
     size_t retry{};
 };
 
