@@ -1,5 +1,7 @@
 #pragma once
 
+#include <Disks/ObjectStorages/InMemoryDirectoryTree.h>
+
 #include <Common/ObjectStorageKeyGenerator.h>
 
 #include <memory>
@@ -7,11 +9,10 @@
 namespace DB
 {
 
-class InMemoryDirectoryPathMap;
 class FlatDirectoryStructureKeyGenerator : public IObjectStorageKeysGenerator
 {
 public:
-    explicit FlatDirectoryStructureKeyGenerator(String storage_key_prefix_, std::weak_ptr<InMemoryDirectoryPathMap> path_map_);
+    explicit FlatDirectoryStructureKeyGenerator(String storage_key_prefix_, std::weak_ptr<InMemoryDirectoryTree> tree_);
 
     ObjectStorageKey generate(const String & path, bool is_directory, const std::optional<String> & key_prefix) const override;
 
@@ -20,7 +21,7 @@ public:
 private:
     const String storage_key_prefix;
 
-    std::weak_ptr<InMemoryDirectoryPathMap> path_map;
+    std::weak_ptr<InMemoryDirectoryTree> tree;
 };
 
 }
