@@ -1,14 +1,14 @@
 #include <Disks/ObjectStorages/MetadataStorageFromPlainObjectStorageOperations.h>
 
-#include <Disks/ObjectStorages/InMemoryDirectoryPathMap.h>
+#include <Disks/ObjectStorages/InMemoryDirectoryTree.h>
 #include <Disks/ObjectStorages/StoredObject.h>
 #include <IO/ReadSettings.h>
 #include <IO/WriteSettings.h>
 
 #include <filesystem>
-#include <mutex>
 #include <IO/ReadHelpers.h>
 #include <IO/WriteHelpers.h>
+#include <fmt/ranges.h>
 #include <Poco/Timestamp.h>
 #include <Common/Exception.h>
 #include <Common/FailPoint.h>
@@ -23,20 +23,24 @@ namespace DB
 
 namespace ErrorCodes
 {
-extern const int FILE_DOESNT_EXIST;
-extern const int FILE_ALREADY_EXISTS;
-extern const int INCORRECT_DATA;
-extern const int FAULT_INJECTED;
-extern const int LOGICAL_ERROR;
+    extern const int FILE_DOESNT_EXIST;
+    extern const int FILE_ALREADY_EXISTS;
+    extern const int DIRECTORY_DOESNT_EXIST;
+    extern const int DIRECTORY_ALREADY_EXISTS;
+    extern const int INCORRECT_DATA;
+    extern const int FAULT_INJECTED;
+    extern const int LOGICAL_ERROR;
+    extern const int CANNOT_RMDIR;
+    extern const int CANNOT_CREATE_DIRECTORY;
 };
 
 namespace FailPoints
 {
-extern const char plain_object_storage_write_fail_on_directory_create[];
-extern const char plain_object_storage_write_fail_on_directory_move[];
-extern const char plain_object_storage_copy_fail_on_file_move[];
-extern const char plain_object_storage_copy_temp_source_file_fail_on_file_move[];
-extern const char plain_object_storage_copy_temp_target_file_fail_on_file_move[];
+    extern const char plain_object_storage_write_fail_on_directory_create[];
+    extern const char plain_object_storage_write_fail_on_directory_move[];
+    extern const char plain_object_storage_copy_fail_on_file_move[];
+    extern const char plain_object_storage_copy_temp_source_file_fail_on_file_move[];
+    extern const char plain_object_storage_copy_temp_target_file_fail_on_file_move[];
 }
 
 namespace
@@ -50,50 +54,36 @@ ObjectStorageKey createMetadataObjectKey(const std::string & object_key_prefix,
     return ObjectStorageKey::createAsRelative(prefix.string(), PREFIX_PATH_FILE_NAME);
 }
 
-std::vector<std::string> listDirectoryRecursive(const InMemoryDirectoryPathMap & path_map, const std::string & root)
-{
-    std::vector<std::string> subdirs = {""};
-    std::queue<std::string> unlisted_nodes;
-    unlisted_nodes.push(root);
-
-    while (!unlisted_nodes.empty())
-    {
-        std::string next_to_list = std::move(unlisted_nodes.front());
-        unlisted_nodes.pop();
-
-        for (const auto & child : path_map.listSubdirectories(next_to_list))
-        {
-            subdirs.push_back((next_to_list + child).substr(root.size()));
-            unlisted_nodes.push(next_to_list + child);
-        }
-    }
-
-    return subdirs;
-}
-
 }
 
 MetadataStorageFromPlainObjectStorageCreateDirectoryOperation::MetadataStorageFromPlainObjectStorageCreateDirectoryOperation(
     std::filesystem::path && path_,
-    InMemoryDirectoryPathMap & path_map_,
+    InMemoryDirectoryTree & fs_tree_,
     ObjectStoragePtr object_storage_,
-    const std::string & metadata_key_prefix_)
+    const std::string & metadata_key_prefix_,
+    bool recursive_)
     : path(std::move(path_))
-    , path_map(path_map_)
+    , fs_tree(fs_tree_)
     , object_storage(object_storage_)
     , metadata_key_prefix(metadata_key_prefix_)
-    , object_key_prefix(object_storage->generateObjectKeyPrefixForDirectoryPath(path, "" /* object_key_prefix */).serialize())
+    , recursive(recursive_)
 {
     chassert(path.empty() || path.string().ends_with('/'));
 }
 
 void MetadataStorageFromPlainObjectStorageCreateDirectoryOperation::execute()
 {
-    /// parent_path() removes the trailing '/'
-    const auto base_path = path.parent_path();
-    if (path_map.existsLocalPath(base_path))
+    if (fs_tree.existsDirectory(path).first)
         return;
 
+    if (fs_tree.existsFile(path))
+        throw Exception(ErrorCodes::CANNOT_CREATE_DIRECTORY, "File '{}' already exists", path.parent_path());
+
+    if (!recursive)
+        if (!fs_tree.existsDirectory(path.parent_path().parent_path()).first)
+            throw Exception(ErrorCodes::DIRECTORY_DOESNT_EXIST, "Directory '{}' does not exist", path.parent_path().parent_path());
+
+    object_key_prefix = object_storage->generateObjectKeyPrefixForDirectoryPath(path, "" /* object_key_prefix */).serialize();
     auto metadata_object_key = createMetadataObjectKey(object_key_prefix, metadata_key_prefix);
 
     LOG_TRACE(
@@ -104,12 +94,11 @@ void MetadataStorageFromPlainObjectStorageCreateDirectoryOperation::execute()
 
     auto metadata_object = StoredObject(/*remote_path*/ metadata_object_key.serialize(), /*local_path*/ path / PREFIX_PATH_FILE_NAME);
 
-    size_t buf_size = std::bit_ceil(path.string().size()) << 1;
     auto buf = object_storage->writeObject(
         metadata_object,
         WriteMode::Rewrite,
         /*object_attributes*/ std::nullopt,
-        /*buf_size*/ std::clamp(buf_size, 32lu, size_t(DBMS_DEFAULT_BUFFER_SIZE)),
+        /*buf_size*/ 128,
         /*settings*/ getWriteSettings());
 
     writeString(path.string(), *buf);
@@ -120,15 +109,18 @@ void MetadataStorageFromPlainObjectStorageCreateDirectoryOperation::execute()
 
     auto event = object_storage->getMetadataStorageMetrics().directory_created;
     ProfileEvents::increment(event);
-    auto metadata = object_storage->getObjectMetadata(metadata_object.remote_path);
-    path_map.addOrReplacePath(base_path, InMemoryDirectoryPathMap::RemotePathInfo{object_key_prefix, metadata.etag, metadata.last_modified.epochTime(), {}});
+    auto metadata = object_storage->getObjectMetadata(metadata_object.remote_path, /*with_tags=*/ false);
+    fs_tree.recordDirectoryPath(path, DirectoryRemoteInfo{object_key_prefix, metadata.etag, metadata.last_modified.epochTime(), {}});
+    created_directory = true;
 }
 
 void MetadataStorageFromPlainObjectStorageCreateDirectoryOperation::undo()
 {
     LOG_TRACE(getLogger("MetadataStorageFromPlainObjectStorageCreateDirectoryOperation"), "Reversing directory creation for path '{}'", path);
-    const auto base_path = path.parent_path();
-    path_map.removePathIfExists(base_path);
+
+    if (created_directory)
+        fs_tree.unlinkTree(path);
+
     auto metadata_object_key = createMetadataObjectKey(object_key_prefix, metadata_key_prefix);
     object_storage->removeObjectIfExists(StoredObject(metadata_object_key.serialize(), path / PREFIX_PATH_FILE_NAME));
 }
@@ -136,12 +128,12 @@ void MetadataStorageFromPlainObjectStorageCreateDirectoryOperation::undo()
 MetadataStorageFromPlainObjectStorageMoveDirectoryOperation::MetadataStorageFromPlainObjectStorageMoveDirectoryOperation(
     std::filesystem::path && path_from_,
     std::filesystem::path && path_to_,
-    InMemoryDirectoryPathMap & path_map_,
+    InMemoryDirectoryTree & fs_tree_,
     ObjectStoragePtr object_storage_,
     const std::string & metadata_key_prefix_)
     : path_from(std::move(path_from_))
     , path_to(std::move(path_to_))
-    , path_map(path_map_)
+    , fs_tree(fs_tree_)
     , object_storage(object_storage_)
     , metadata_key_prefix(metadata_key_prefix_)
 {
@@ -150,27 +142,14 @@ MetadataStorageFromPlainObjectStorageMoveDirectoryOperation::MetadataStorageFrom
 }
 
 std::unique_ptr<WriteBufferFromFileBase> MetadataStorageFromPlainObjectStorageMoveDirectoryOperation::createWriteBuf(
-    const std::filesystem::path & expected_path, const std::filesystem::path & new_path, bool validate_content)
+    const DirectoryRemoteInfo & remote_info, std::optional<std::string> expected_content)
 {
-    /// parent_path() removes the trailing '/'.
-    auto remote_path_info = path_map.getRemotePathInfoIfExists(expected_path.parent_path());
-    if (!remote_path_info)
-        throw Exception(
-            ErrorCodes::FILE_DOESNT_EXIST, "Metadata object for the expected (source) path '{}' does not exist", expected_path);
+    auto metadata_object_key = createMetadataObjectKey(remote_info.remote_path, metadata_key_prefix);
+    StoredObject metadata_object(metadata_object_key.serialize());
 
-    if (path_map.existsLocalPath(new_path.parent_path()))
-        throw Exception(
-            ErrorCodes::FILE_ALREADY_EXISTS, "Metadata object for the new (destination) path '{}' already exists", new_path);
-
-    std::filesystem::path remote_path = remote_path_info->path;
-
-    auto metadata_object_key = createMetadataObjectKey(remote_path, metadata_key_prefix);
-
-    auto metadata_object
-        = StoredObject(/*remote_path*/ metadata_object_key.serialize(), /*local_path*/ expected_path / PREFIX_PATH_FILE_NAME);
-
-    if (validate_content)
+    if (expected_content)
     {
+        chassert(expected_content.value().ends_with('/'));
         LockMemoryExceptionInThread temporarily_lock_exceptions;
 
         std::string data;
@@ -181,52 +160,39 @@ std::unique_ptr<WriteBufferFromFileBase> MetadataStorageFromPlainObjectStorageMo
 
         auto read_buf = object_storage->readObject(metadata_object, read_settings);
         readStringUntilEOF(data, *read_buf);
-        if (data != expected_path)
+        if (data != expected_content.value())
             throw Exception(
                 ErrorCodes::INCORRECT_DATA,
                 "Incorrect data for object key {}, expected {}, got {}",
                 metadata_object_key.serialize(),
-                expected_path,
+                expected_content.value(),
                 data);
     }
 
-    size_t buf_size = std::bit_ceil(new_path.string().size()) << 1;
     auto write_buf = object_storage->writeObject(
         metadata_object,
         WriteMode::Rewrite,
         /*object_attributes*/ std::nullopt,
-        /*buf_size*/ std::clamp(buf_size, 32lu, size_t(DBMS_DEFAULT_BUFFER_SIZE)),
+        /*buf_size*/ 128,
         /*settings*/ getWriteSettings());
 
     return write_buf;
 }
 
-void MetadataStorageFromPlainObjectStorageMoveDirectoryOperation::executeMoveImpl(const std::filesystem::path & from, const std::filesystem::path & to, bool validate_content)
+void MetadataStorageFromPlainObjectStorageMoveDirectoryOperation::rewriteSingleDirectory(const std::filesystem::path & from, const std::filesystem::path & to, WriteBuffer & buffer)
 {
-    LOG_TRACE(getLogger("MetadataStorageFromPlainObjectStorageMoveDirectoryOperation"), "Moving directory '{}' to '{}'", from, to);
+    LOG_TRACE(getLogger("MetadataStorageFromPlainObjectStorageMoveDirectoryOperation"), "Rewriting '{}' to '{}'", from, to);
 
-    for (const auto & subdir : listDirectoryRecursive(path_map, from))
+    writeString(to.string(), buffer);
+
+    fiu_do_on(FailPoints::plain_object_storage_write_fail_on_directory_move,
     {
-        auto sub_path_to = to / subdir;
-        auto sub_path_from = from / subdir;
+        throw Exception(ErrorCodes::FAULT_INJECTED, "Injecting fault when moving from '{}' to '{}'", from, to);
+    });
 
-        auto write_buf = createWriteBuf(sub_path_from, sub_path_to, validate_content);
+    buffer.finalize();
 
-        /// parent_path() removes the trailing '/'.
-        /// Let's move in memory first to be able to see this move in undo in case of error.
-        path_map.moveDirectory(sub_path_from.parent_path(), sub_path_to.parent_path());
-
-        writeString(sub_path_to.string(), *write_buf);
-
-        fiu_do_on(FailPoints::plain_object_storage_write_fail_on_directory_move,
-        {
-            throw Exception(ErrorCodes::FAULT_INJECTED, "Injecting fault when moving from '{}' to '{}'", sub_path_from, sub_path_to);
-        });
-
-        write_buf->finalize();
-
-        LOG_TRACE(getLogger("MetadataStorageFromPlainObjectStorageMoveDirectoryOperation"), "Moved directory '{}' to '{}'", sub_path_from, sub_path_to);
-    }
+    LOG_TRACE(getLogger("MetadataStorageFromPlainObjectStorageMoveDirectoryOperation"), "Updated '{}' to '{}'", from, to);
 }
 
 void MetadataStorageFromPlainObjectStorageMoveDirectoryOperation::execute()
@@ -237,47 +203,88 @@ void MetadataStorageFromPlainObjectStorageMoveDirectoryOperation::execute()
     constexpr bool validate_content = false;
 #endif
 
-    executeMoveImpl(path_from, path_to, validate_content);
+    if (!fs_tree.existsDirectory(path_from).first)
+        throw Exception(ErrorCodes::DIRECTORY_DOESNT_EXIST, "Directory '{}' does not exist", path_from);
+    else if (fs_tree.existsDirectory(path_to).first)
+        throw Exception(ErrorCodes::DIRECTORY_ALREADY_EXISTS, "Directory '{}' already exists", path_to);
+    else if (path_from == "/")
+        throw Exception(ErrorCodes::LOGICAL_ERROR, "Can't move root folder");
+
+    from_tree_info = fs_tree.getSubtreeRemoteInfo(path_from);
+
+    for (const auto & [subdir, remote_info] : from_tree_info)
+    {
+        auto sub_path_to = path_to / subdir / "";
+        auto sub_path_from = path_from / subdir / "";
+
+        if (!remote_info.has_value())
+        {
+            LOG_TRACE(getLogger("MetadataStorageFromPlainObjectStorageMoveDirectoryOperation"), "Directory '{}' is virtual will not update in object storage", sub_path_from);
+            continue;
+        }
+
+        auto write_buf = createWriteBuf(remote_info.value(), /*expected_content*/validate_content ? std::make_optional(sub_path_from) : std::nullopt);
+
+        changed_paths.insert(sub_path_from);
+        rewriteSingleDirectory(sub_path_from, sub_path_to, *write_buf);
+    }
+
+    fs_tree.moveDirectory(path_from, path_to);
+    moved_in_memory = true;
 }
 
 void MetadataStorageFromPlainObjectStorageMoveDirectoryOperation::undo()
 {
     LOG_TRACE(getLogger("MetadataStorageFromPlainObjectStorageMoveDirectoryOperation"), "Reversing directory move from '{}' to '{}'", path_from, path_to);
-    executeMoveImpl(path_to, path_from, /*validate_content=*/false);
+
+    if (moved_in_memory)
+        fs_tree.moveDirectory(path_to, path_from);
+
+    for (const auto & [subdir, remote_info] : from_tree_info)
+    {
+        auto sub_path_to = path_to / subdir / "";
+        auto sub_path_from = path_from / subdir / "";
+
+        if (!changed_paths.contains(sub_path_from))
+            continue;
+
+        auto write_buf = createWriteBuf(remote_info.value(), /*expected_content*/std::nullopt);
+        rewriteSingleDirectory(sub_path_to, sub_path_from, *write_buf);
+    }
 }
 
 MetadataStorageFromPlainObjectStorageRemoveDirectoryOperation::MetadataStorageFromPlainObjectStorageRemoveDirectoryOperation(
     std::filesystem::path && path_,
-    InMemoryDirectoryPathMap & path_map_,
+    InMemoryDirectoryTree & fs_tree_,
     ObjectStoragePtr object_storage_,
     const std::string & metadata_key_prefix_)
-    : path(std::move(path_)), path_map(path_map_), object_storage(object_storage_), metadata_key_prefix(metadata_key_prefix_)
+    : path(std::move(path_)), fs_tree(fs_tree_), object_storage(object_storage_), metadata_key_prefix(metadata_key_prefix_)
 {
     chassert(path.empty() || path.string().ends_with('/'));
 }
 
 void MetadataStorageFromPlainObjectStorageRemoveDirectoryOperation::execute( /* metadata_lock */)
 {
-    /// parent_path() removes the trailing '/'
-    const auto base_path = path.parent_path();
-    auto optional_info = path_map.getRemotePathInfoIfExists(base_path);
-    if (!optional_info)
+    auto [exists, remote_info] = fs_tree.existsDirectory(path);
+    if (!exists)
+        throw Exception(ErrorCodes::DIRECTORY_DOESNT_EXIST, "Directory '{}' does not exist", path);
+    else if (auto children = fs_tree.listDirectory(path); !children.empty())
+        throw Exception(ErrorCodes::CANNOT_RMDIR, "Directory '{}' is not empty. Children: [{}]", path, fmt::join(children, ", "));
+    else if (path == "/")
         return;
-    info = *std::move(optional_info);
+
+    chassert(remote_info.has_value());
+    info = std::move(remote_info.value());
 
     LOG_TRACE(getLogger("MetadataStorageFromPlainObjectStorageRemoveDirectoryOperation"), "Removing directory '{}'", path);
 
-    auto metadata_object_key = createMetadataObjectKey(info.path, metadata_key_prefix);
+    remove_attempted = true;
+    auto metadata_object_key = createMetadataObjectKey(info.remote_path, metadata_key_prefix);
     auto metadata_object = StoredObject(/*remote_path*/ metadata_object_key.serialize(), /*local_path*/ path, path.string().length());
     object_storage->removeObjectIfExists(metadata_object);
 
-    if (path_map.removePathIfExists(base_path))
-    {
-        auto event = object_storage->getMetadataStorageMetrics().directory_removed;
-        ProfileEvents::increment(event);
-    }
-
-    remove_attempted = true;
+    fs_tree.unlinkTree(path);
+    ProfileEvents::increment(object_storage->getMetadataStorageMetrics().directory_removed);
 }
 
 void MetadataStorageFromPlainObjectStorageRemoveDirectoryOperation::undo()
@@ -286,44 +293,38 @@ void MetadataStorageFromPlainObjectStorageRemoveDirectoryOperation::undo()
         return;
 
     LOG_TRACE(getLogger("MetadataStorageFromPlainObjectStorageRemoveDirectoryOperation"), "Reversing directory removal for '{}'", path);
-    path_map.addOrReplacePath(path.parent_path(), info);
 
-    auto metadata_object_key = createMetadataObjectKey(info.path, metadata_key_prefix);
+    if (!fs_tree.existsDirectory(path).first)
+        fs_tree.recordDirectoryPath(path, info);
+
+    auto metadata_object_key = createMetadataObjectKey(info.remote_path, metadata_key_prefix);
     auto metadata_object = StoredObject(metadata_object_key.serialize(), path / PREFIX_PATH_FILE_NAME);
 
-    size_t buf_size = std::bit_ceil(path.string().size()) << 1;
     auto buf = object_storage->writeObject(
         metadata_object,
         WriteMode::Rewrite,
         /*object_attributes*/ std::nullopt,
-        /*buf_size*/ std::clamp(buf_size, 32lu, size_t(DBMS_DEFAULT_BUFFER_SIZE)),
+        /*buf_size*/ 128,
         /*settings*/ DB::getWriteSettings());
     writeString(path.string(), *buf);
     buf->finalize();
 }
 
 MetadataStorageFromPlainObjectStorageWriteFileOperation::MetadataStorageFromPlainObjectStorageWriteFileOperation(
-    const std::string & path_, InMemoryDirectoryPathMap & path_map_, ObjectStoragePtr object_storage_)
-    : path(path_), path_map(path_map_), object_storage(object_storage_)
+    const std::string & path_, const StoredObject & object_, InMemoryDirectoryTree & fs_tree_, ObjectStoragePtr object_storage_)
+    : path(path_), object(object_), fs_tree(fs_tree_), object_storage(object_storage_)
 {
 }
 
 void MetadataStorageFromPlainObjectStorageWriteFileOperation::execute()
 {
-    LOG_TEST(getLogger("MetadataStorageFromPlainObjectStorageWriteFileOperation"), "Creating metadata for a file '{}'", path);
+    LOG_TEST(getLogger("MetadataStorageFromPlainObjectStorageWriteFileOperation"), "Creating metadata for a file '{}', size: {}", path, object.bytes_size);
 
-    if (path_map.addFile(path))
+    if (!fs_tree.existsFile(path))
     {
+        fs_tree.recordFile(path, {object.bytes_size, std::time(nullptr)});
         written = true;
     }
-    else
-    {
-        /// Some paths (e.g., clickhouse_access_check) may not have parent directories.
-        LOG_TRACE(
-            getLogger("MetadataStorageFromPlainObjectStorageWriteFileOperation"),
-            "Parent directory does not exist, skipping path {}",
-            path);
-    }
 }
 
 void MetadataStorageFromPlainObjectStorageWriteFileOperation::undo()
@@ -331,14 +332,13 @@ void MetadataStorageFromPlainObjectStorageWriteFileOperation::undo()
     if (!written)
         return;
 
-    path_map.removeFile(path);
+    fs_tree.removeFile(path);
 }
 
 MetadataStorageFromPlainObjectStorageUnlinkMetadataFileOperation::MetadataStorageFromPlainObjectStorageUnlinkMetadataFileOperation(
-    std::filesystem::path && path_, InMemoryDirectoryPathMap & path_map_, ObjectStoragePtr object_storage_)
+    std::filesystem::path && path_, InMemoryDirectoryTree & fs_tree_, ObjectStoragePtr object_storage_)
     : path(path_)
-    , remote_path(std::filesystem::path(object_storage_->generateObjectKeyForPath(path_, std::nullopt).serialize()))
-    , path_map(path_map_)
+    , fs_tree(fs_tree_)
     , object_storage(object_storage_)
 {
 }
@@ -351,8 +351,14 @@ void MetadataStorageFromPlainObjectStorageUnlinkMetadataFileOperation::execute()
         path,
         remote_path);
 
-    if (path_map.removeFile(path))
-        unlinked = true;
+    if (!fs_tree.existsFile(path))
+        throw Exception(ErrorCodes::FILE_DOESNT_EXIST, "File '{}' does not exist", path);
+
+    remote_path = object_storage->generateObjectKeyForPath(path, std::nullopt).serialize();
+    file_remote_info = fs_tree.getFileRemoteInfo(path);
+
+    fs_tree.removeFile(path);
+    unlinked = true;
 }
 
 void MetadataStorageFromPlainObjectStorageUnlinkMetadataFileOperation::undo()
@@ -360,25 +366,18 @@ void MetadataStorageFromPlainObjectStorageUnlinkMetadataFileOperation::undo()
     if (!unlinked)
         return;
 
-    if (!path_map.addFile(path))
-    {
-        /// Some paths (e.g., clickhouse_access_check) may not have parent directories.
-        LOG_TRACE(
-            getLogger("MetadataStorageFromPlainObjectStorageWriteFileOperation"),
-            "Parent directory does not exist, skipping path {}",
-            path);
-    }
+    chassert(file_remote_info.has_value());
+    fs_tree.recordFile(path, std::move(file_remote_info.value()));
 }
+
 MetadataStorageFromPlainObjectStorageCopyFileOperation::MetadataStorageFromPlainObjectStorageCopyFileOperation(
     std::filesystem::path path_from_,
     std::filesystem::path path_to_,
-    InMemoryDirectoryPathMap & path_map_,
+    InMemoryDirectoryTree & fs_tree_,
     ObjectStoragePtr object_storage_)
     : path_from(path_from_)
-    , remote_path_from(object_storage_->generateObjectKeyForPath(path_from_, std::nullopt).serialize())
     , path_to(path_to_)
-    , remote_path_to(object_storage_->generateObjectKeyForPath(path_to_, std::nullopt).serialize())
-    , path_map(path_map_)
+    , fs_tree(fs_tree_)
     , object_storage(object_storage_)
 {
 }
@@ -387,26 +386,25 @@ void MetadataStorageFromPlainObjectStorageCopyFileOperation::execute()
 {
     LOG_TEST(getLogger("MetadataStorageFromPlainObjectStorageCopyFileOperation"), "Copying file from '{}' to '{}'", path_from, path_to);
 
-    if (!path_map.existsFile(path_from))
+    if (!fs_tree.existsFile(path_from))
         throw Exception(ErrorCodes::FILE_DOESNT_EXIST, "Metadata object for the source path '{}' does not exist", path_from);
-
-    const auto directory_to = path_to.parent_path();
-    if (!path_map.existsLocalPath(directory_to))
-        throw Exception(ErrorCodes::FILE_DOESNT_EXIST, "Metadata object for the target directory path '{}' does not exist", path_to);
-
-    if (path_map.existsFile(path_to))
+    else if (auto [exists, remote_info] = fs_tree.existsDirectory(path_to.parent_path()); !exists)
+        throw Exception(ErrorCodes::DIRECTORY_DOESNT_EXIST, "Directory '{}' does not exist", path_to.parent_path());
+    else if (!remote_info.has_value())
+        throw Exception(ErrorCodes::LOGICAL_ERROR, "Directory '{}' is virtual", path_to.parent_path());
+    else if (fs_tree.existsFile(path_to))
         throw Exception(ErrorCodes::FILE_ALREADY_EXISTS, "Target file '{}' already exists", path_to);
 
+    copy_attempted = true;
+    remote_path_from = object_storage->generateObjectKeyForPath(path_from, std::nullopt).serialize();
+    remote_path_to = object_storage->generateObjectKeyForPath(path_to, std::nullopt).serialize();
     object_storage->copyObject(StoredObject(remote_path_from), StoredObject(remote_path_to), getReadSettings(), getWriteSettings());
-
-    copied = true;
-    [[maybe_unused]] bool added = path_map.addFile(path_to);
-    chassert(added);
+    fs_tree.recordFile(path_to, fs_tree.getFileRemoteInfo(path_from).value());
 }
 
 void MetadataStorageFromPlainObjectStorageCopyFileOperation::undo()
 {
-    if (!copied)
+    if (!copy_attempted)
         return;
 
     LOG_WARNING(
@@ -416,32 +414,23 @@ void MetadataStorageFromPlainObjectStorageCopyFileOperation::undo()
         path_from);
 
     object_storage->removeObjectIfExists(StoredObject(remote_path_to));
-    path_map.removeFile(path_to);
-}
 
+    if (fs_tree.existsFile(path_to))
+        fs_tree.removeFile(path_to);
+}
 
 MetadataStorageFromPlainObjectStorageMoveFileOperation::MetadataStorageFromPlainObjectStorageMoveFileOperation(
     bool replaceable_,
     std::filesystem::path path_from_,
     std::filesystem::path path_to_,
-    InMemoryDirectoryPathMap & path_map_,
+    InMemoryDirectoryTree & fs_tree_,
     ObjectStoragePtr object_storage_)
     : replaceable(replaceable_)
     , path_from(path_from_)
-    , remote_path_from(object_storage_->generateObjectKeyForPath(path_from_, std::nullopt).serialize())
     , path_to(path_to_)
-    , remote_path_to(object_storage_->generateObjectKeyForPath(path_to_, std::nullopt).serialize())
-    , path_map(path_map_)
+    , fs_tree(fs_tree_)
     , object_storage(object_storage_)
 {
-    {
-        auto tmp_path_from = path_to.string() + "." + getRandomASCIIString(16) + ".tmp_move_from";
-        tmp_remote_path_from = object_storage->generateObjectKeyForPath(tmp_path_from, std::nullopt).serialize();
-    }
-    {
-        auto tmp_path_to = path_to.string() + "." + getRandomASCIIString(16) + ".tmp_move_to";
-        tmp_remote_path_to = object_storage->generateObjectKeyForPath(tmp_path_to, std::nullopt).serialize();
-    }
 }
 
 void MetadataStorageFromPlainObjectStorageMoveFileOperation::execute()
@@ -453,17 +442,22 @@ void MetadataStorageFromPlainObjectStorageMoveFileOperation::execute()
         path_from,
         path_to);
 
-    if (!path_map.existsFile(path_from))
-        throw Exception(ErrorCodes::FILE_DOESNT_EXIST, "Metadata object for the source path '{}' does not exist", path_from);
-
-    const auto directory_to = path_to.parent_path();
-    if (!path_map.existsLocalPath(directory_to))
-        throw Exception(ErrorCodes::FILE_DOESNT_EXIST, "Metadata object for the target directory path '{}' does not exist", path_to);
+    if (!fs_tree.existsFile(path_from))
+        throw Exception(ErrorCodes::FILE_DOESNT_EXIST, "File '{}' does not exist", path_from);
+    else if (auto [exists, remote_info] = fs_tree.existsDirectory(path_to.parent_path()); !exists)
+        throw Exception(ErrorCodes::DIRECTORY_DOESNT_EXIST, "Directory '{}' does not exist", path_to.parent_path());
+    else if (!remote_info.has_value())
+        throw Exception(ErrorCodes::LOGICAL_ERROR, "Directory '{}' is virtual", path_to.parent_path());
 
+    remote_path_from = object_storage->generateObjectKeyForPath(path_from, std::nullopt).serialize();
+    remote_path_to = object_storage->generateObjectKeyForPath(path_to, std::nullopt).serialize();
+    tmp_remote_path_from = object_storage->generateObjectKeyForPath(path_to.string() + "." + getRandomASCIIString(16) + ".tmp_move_from", std::nullopt).serialize();
+    tmp_remote_path_to = object_storage->generateObjectKeyForPath(path_to.string() + "." + getRandomASCIIString(16) + ".tmp_move_to", std::nullopt).serialize();
+    file_from_remote_info = fs_tree.getFileRemoteInfo(path_from).value();
     const auto read_settings = getReadSettingsForMetadata();
     const auto write_settings = getWriteSettingsForMetadata();
 
-    if (path_map.existsFile(path_to))
+    if (fs_tree.existsFile(path_to))
     {
         if (!replaceable)
             throw Exception(ErrorCodes::FILE_ALREADY_EXISTS, "Target file '{}' already exists", path_to);
@@ -478,12 +472,17 @@ void MetadataStorageFromPlainObjectStorageMoveFileOperation::execute()
             read_settings,
             write_settings);
         moved_existing_target_file = true;
+
+        file_to_remote_info = fs_tree.getFileRemoteInfo(path_to);
+        fs_tree.removeFile(path_to);
+        fs_tree.recordFile(path_to, file_from_remote_info.value());
+
         object_storage->removeObjectIfExists(StoredObject(remote_path_to));
     }
     else
     {
-        [[maybe_unused]] bool added = path_map.addFile(path_to);
-        chassert(added);
+        fs_tree.recordFile(path_to, file_from_remote_info.value());
+        created_target_file = true;
     }
 
     {
@@ -509,12 +508,13 @@ void MetadataStorageFromPlainObjectStorageMoveFileOperation::execute()
         moved_file = true;
     }
 
-    path_map.removeFile(path_from);
+    fs_tree.removeFile(path_from);
 }
 
 void MetadataStorageFromPlainObjectStorageMoveFileOperation::undo()
 {
-    path_map.addFile(path_from);
+    if (moved_file)
+        fs_tree.recordFile(path_from, file_from_remote_info.value());
 
     const auto read_settings = getReadSettings();
     const auto write_settings = getWriteSettings();
@@ -550,11 +550,15 @@ void MetadataStorageFromPlainObjectStorageMoveFileOperation::undo()
             read_settings,
             write_settings);
 
+        fs_tree.removeFile(path_to);
+        fs_tree.recordFile(path_to, file_to_remote_info.value());
+
         object_storage->removeObjectIfExists(StoredObject(tmp_remote_path_to));
     }
-    else
+
+    if (created_target_file)
     {
-        path_map.removeFile(path_to);
+        fs_tree.removeFile(path_to);
     }
 }
 
@@ -570,12 +574,12 @@ void MetadataStorageFromPlainObjectStorageMoveFileOperation::finalize()
 MetadataStorageFromPlainObjectStorageRemoveRecursiveOperation::MetadataStorageFromPlainObjectStorageRemoveRecursiveOperation(
     /// path_ must end with a trailing '/'.
     std::filesystem::path && path_,
-    InMemoryDirectoryPathMap & path_map_,
+    InMemoryDirectoryTree & fs_tree_,
     ObjectStoragePtr object_storage_,
     ObjectMetadataCachePtr object_metadata_cache_,
     const std::string & metadata_key_prefix_)
     : path(std::move(path_))
-    , path_map(path_map_)
+    , fs_tree(fs_tree_)
     , object_storage(std::move(object_storage_))
     , object_metadata_cache(std::move(object_metadata_cache_))
     , metadata_key_prefix(metadata_key_prefix_)
@@ -587,12 +591,18 @@ MetadataStorageFromPlainObjectStorageRemoveRecursiveOperation::MetadataStorageFr
 
     path = base_path;
     tmp_path = "remove_recursive." + getRandomASCIIString(16);
-    move_to_tmp_op = std::make_unique<MetadataStorageFromPlainObjectStorageMoveDirectoryOperation>(path / "", tmp_path / "", path_map, object_storage, metadata_key_prefix);
+    move_to_tmp_op = std::make_unique<MetadataStorageFromPlainObjectStorageMoveDirectoryOperation>(path / "", tmp_path / "", fs_tree, object_storage, metadata_key_prefix);
 }
 
 void MetadataStorageFromPlainObjectStorageRemoveRecursiveOperation::execute()
 {
-    if (path_map.existsLocalPath(path))
+    /// Unfortunately we are able to create merge tree unlinked from database directory.
+    /// In this case during the dropAllData method removeRecursive can be called pointing to the root folder.
+    /// I don't know what to do in this case, so right now it is a no-op.
+    if (path.empty())
+        return;
+
+    if (fs_tree.existsDirectory(path).first)
     {
         move_tried = true;
         move_to_tmp_op->execute();
@@ -613,20 +623,29 @@ void MetadataStorageFromPlainObjectStorageRemoveRecursiveOperation::finalize()
         return;
 
     StoredObjects objects_to_remove;
-    for (const auto & subdir : listDirectoryRecursive(path_map, tmp_path / ""))
+    for (const auto & [subdir, remote_info] : fs_tree.getSubtreeRemoteInfo(tmp_path))
     {
         auto subdir_path = tmp_path / subdir;
+
+        if (!remote_info.has_value())
+        {
+            LOG_TRACE(log, "Directory '{}' is virtual, will not cleanup remote", subdir_path);
+            continue;
+        }
+
         LOG_TRACE(log, "Removing directory '{}'", subdir_path);
 
         /// Info should exist since it's lifetime is bounded to execution of this operation, because tmp path is unique.
-        auto directory_info = path_map.getRemotePathInfoIfExists(subdir_path.parent_path()).value();
-        auto metadata_object_key = createMetadataObjectKey(directory_info.path, metadata_key_prefix);
+        auto metadata_object_key = createMetadataObjectKey(remote_info->remote_path, metadata_key_prefix);
         objects_to_remove.emplace_back(metadata_object_key.serialize(), path / PREFIX_PATH_FILE_NAME);
 
         /// We also need to remove all files inside each of the subdirectories.
-        for (const auto & file : path_map.listFiles(subdir_path))
+        for (const auto & child : fs_tree.listDirectory(subdir_path))
         {
-            auto file_path = subdir_path / file;
+            auto file_path = subdir_path / child;
+            if (!fs_tree.existsFile(file_path))
+                continue;
+
             LOG_TRACE(log, "Removing file '{}'", file_path);
 
             auto file_object_key = object_storage->generateObjectKeyForPath(file_path, std::nullopt).serialize();
@@ -639,12 +658,9 @@ void MetadataStorageFromPlainObjectStorageRemoveRecursiveOperation::finalize()
                 object_metadata_cache->remove(hash.get128());
             }
         }
-
-        const bool is_removed = path_map.removePathIfExists(subdir_path.parent_path());
-        if (!is_removed)
-            throw Exception(ErrorCodes::LOGICAL_ERROR, "Can't remove '{}' path from in memory map. Probably it does not exist. It is a bug.", subdir_path);
     }
 
+    fs_tree.unlinkTree(tmp_path);
     object_storage->removeObjectsIfExist(objects_to_remove);
 }
 
