@@ -53,8 +53,10 @@ FormatSchemaInfo::FormatSchemaInfo(
     const String & format,
     bool require_message,
     bool is_server,
-    const String & format_schema_path)
+    const String & format_schema_path,
+    bool generated_)
     : message_name(format_schema_message_name)
+    , generated(generated_)
     , log(getLogger("FormatSchemaInfo"))
 {
     if (format_schema_source.empty())
@@ -356,7 +358,7 @@ String FormatSchemaInfo::generateSchemaFileName(const String & hashing_content,
 
 template <typename SchemaGenerator>
 MaybeAutogeneratedFormatSchemaInfo<SchemaGenerator>::MaybeAutogeneratedFormatSchemaInfo(
-    const FormatSettings & settings, const String & format, const Block & header, bool use_autogenerated_schema)
+    const FormatSettings & settings, const String & format, const Block & header, bool use_autogenerated_schema, bool with_envelope)
 {
     if (!use_autogenerated_schema || !settings.schema.format_schema.empty())
     {
@@ -409,7 +411,7 @@ MaybeAutogeneratedFormatSchemaInfo<SchemaGenerator>::MaybeAutogeneratedFormatSch
     }
 
     WriteBufferFromFile buf(path.string());
-    SchemaGenerator::writeSchema(buf, "Message", header.getNamesAndTypesList());
+    SchemaGenerator::writeSchema(buf, "Message", header.getNamesAndTypesList(), with_envelope);
     buf.finalize();
 
     schema_info = std::make_unique<FormatSchemaInfo>(
@@ -419,7 +421,8 @@ MaybeAutogeneratedFormatSchemaInfo<SchemaGenerator>::MaybeAutogeneratedFormatSch
         /*format=*/format,
         /*require_message=*/true,
         /*is_server*/ settings.schema.is_server,
-        /*format_schema_path=*/settings.schema.format_schema_path);
+        /*format_schema_path=*/settings.schema.format_schema_path,
+        /*generated*/ true);
 }
 
 template <typename SchemaGenerator>
