@@ -11,9 +11,11 @@
 #include <Common/typeid_cast.h>
 #include <Common/UTF8Helpers.h>
 
-#include "IArraySource.h"
-#include "IValueSource.h"
-#include "Slices.h"
+#include <DataTypes/EnumValues.h>
+
+#include <Functions/GatherUtils/IArraySource.h>
+#include <Functions/GatherUtils/IValueSource.h>
+#include <Functions/GatherUtils/Slices.h>
 #include <Functions/FunctionHelpers.h>
 
 
@@ -28,8 +30,6 @@ namespace ErrorCodes
 
 namespace GatherUtils
 {
-#pragma GCC visibility push(hidden)
-
 template <typename T> struct NumericArraySink;
 struct StringSink;
 struct FixedStringSink;
@@ -58,8 +58,8 @@ struct NumericArraySource : public ArraySourceImpl<NumericArraySource<T>>
     }
 
     explicit NumericArraySource(const ColumnArray & arr)
-            : column(typeid_cast<const ColVecType &>(arr.getData()))
-            , elements(typeid_cast<const ColVecType &>(arr.getData()).getData()), offsets(arr.getOffsets())
+        : column(typeid_cast<const ColVecType &>(arr.getData()))
+        , elements(typeid_cast<const ColVecType &>(arr.getData()).getData()), offsets(arr.getOffsets())
     {
     }
 
@@ -140,18 +140,11 @@ struct NumericArraySource : public ArraySourceImpl<NumericArraySource<T>>
 
 
 /// The methods can be virtual or not depending on the template parameter. See IStringSource.
-#if !defined(__clang__)
-#   pragma GCC diagnostic push
-#   pragma GCC diagnostic ignored "-Wsuggest-override"
-#elif __clang_major__ >= 11
-#   pragma GCC diagnostic push
-#   ifdef HAS_SUGGEST_OVERRIDE
-#       pragma GCC diagnostic ignored "-Wsuggest-override"
-#   endif
-#   ifdef HAS_SUGGEST_DESTRUCTOR_OVERRIDE
-#       pragma GCC diagnostic ignored "-Wsuggest-destructor-override"
-#   endif
-#endif
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wsuggest-override"
+#pragma clang diagnostic ignored "-Wsuggest-destructor-override"
+
+/// NOLINTBEGIN(hicpp-use-override, modernize-use-override)
 
 template <typename Base>
 struct ConstSource : public Base
@@ -165,17 +158,22 @@ struct ConstSource : public Base
     size_t row_num = 0;
 
     explicit ConstSource(const ColumnConst & col_)
-            : Base(static_cast<const typename Base::Column &>(col_.getDataColumn())), total_rows(col_.size())
+        : Base(static_cast<const typename Base::Column &>(col_.getDataColumn()))
+        , total_rows(col_.size())
     {
     }
 
     template <typename ColumnType>
-    ConstSource(const ColumnType & col_, size_t total_rows_) : Base(col_), total_rows(total_rows_)
+    ConstSource(const ColumnType & col_, size_t total_rows_)
+        : Base(col_)
+        , total_rows(total_rows_)
     {
     }
 
     template <typename ColumnType>
-    ConstSource(const ColumnType & col_, const NullMap & null_map_, size_t total_rows_) : Base(col_, null_map_), total_rows(total_rows_)
+    ConstSource(const ColumnType & col_, const NullMap & null_map_, size_t total_rows_)
+        : Base(col_, null_map_)
+        , total_rows(total_rows_)
     {
     }
 
@@ -184,22 +182,24 @@ struct ConstSource : public Base
 
     virtual void accept(ArraySourceVisitor & visitor) // override
     {
-        if constexpr (std::is_base_of<IArraySource, Base>::value)
+        if constexpr (std::is_base_of_v<IArraySource, Base>)
             visitor.visit(*this);
         else
-            throw Exception(
-                    "accept(ArraySourceVisitor &) is not implemented for " + demangle(typeid(ConstSource<Base>).name())
-                    + " because " + demangle(typeid(Base).name()) + " is not derived from IArraySource", ErrorCodes::NOT_IMPLEMENTED);
+            throw Exception(ErrorCodes::NOT_IMPLEMENTED,
+                            "accept(ArraySourceVisitor &) is not implemented for {} because {} "
+                            "is not derived from IArraySource",
+                            demangle(typeid(ConstSource<Base>).name()), demangle(typeid(Base).name()));
     }
 
     virtual void accept(ValueSourceVisitor & visitor) // override
     {
-        if constexpr (std::is_base_of<IValueSource, Base>::value)
+        if constexpr (std::is_base_of_v<IValueSource, Base>)
             visitor.visit(*this);
         else
-            throw Exception(
-                    "accept(ValueSourceVisitor &) is not implemented for " + demangle(typeid(ConstSource<Base>).name())
-                    + " because " + demangle(typeid(Base).name()) + " is not derived from IValueSource", ErrorCodes::NOT_IMPLEMENTED);
+            throw Exception(ErrorCodes::NOT_IMPLEMENTED,
+                            "accept(ValueSourceVisitor &) is not implemented for {} because {} "
+                            "is not derived from IValueSource",
+                            demangle(typeid(ConstSource<Base>).name()), demangle(typeid(Base).name()));
     }
 
     void next()
@@ -233,9 +233,9 @@ struct ConstSource : public Base
     }
 };
 
-#if !defined(__clang__) || __clang_major__ >= 11
-#   pragma GCC diagnostic pop
-#endif
+/// NOLINTEND(hicpp-use-override, modernize-use-override)
+
+#pragma clang diagnostic pop
 
 struct StringSource
 {
@@ -251,7 +251,8 @@ struct StringSource
     ColumnString::Offset prev_offset = 0;
 
     explicit StringSource(const ColumnString & col)
-            : elements(col.getChars()), offsets(col.getOffsets())
+        : elements(col.getChars())
+        , offsets(col.getOffsets())
     {
     }
 
@@ -278,7 +279,7 @@ struct StringSource
 
     size_t getElementSize() const
     {
-        return offsets[row_num] - prev_offset - 1;
+        return offsets[row_num] - prev_offset;
     }
 
     size_t getColumnSize() const
@@ -288,12 +289,12 @@ struct StringSource
 
     Slice getWhole() const
     {
-        return {&elements[prev_offset], offsets[row_num] - prev_offset - 1};
+        return {&elements[prev_offset], offsets[row_num] - prev_offset};
     }
 
     Slice getSliceFromLeft(size_t offset) const
     {
-        size_t elem_size = offsets[row_num] - prev_offset - 1;
+        size_t elem_size = offsets[row_num] - prev_offset;
         if (offset >= elem_size)
             return {&elements[prev_offset], 0};
         return {&elements[prev_offset + offset], elem_size - offset};
@@ -301,7 +302,7 @@ struct StringSource
 
     Slice getSliceFromLeft(size_t offset, size_t length) const
     {
-        size_t elem_size = offsets[row_num] - prev_offset - 1;
+        size_t elem_size = offsets[row_num] - prev_offset;
         if (offset >= elem_size)
             return {&elements[prev_offset], 0};
         return {&elements[prev_offset + offset], std::min(length, elem_size - offset)};
@@ -309,7 +310,7 @@ struct StringSource
 
     Slice getSliceFromRight(size_t offset) const
     {
-        size_t elem_size = offsets[row_num] - prev_offset - 1;
+        size_t elem_size = offsets[row_num] - prev_offset;
         if (offset > elem_size)
             return {&elements[prev_offset], elem_size};
         return {&elements[prev_offset + elem_size - offset], offset};
@@ -317,11 +318,103 @@ struct StringSource
 
     Slice getSliceFromRight(size_t offset, size_t length) const
     {
-        size_t elem_size = offsets[row_num] - prev_offset - 1;
+        size_t elem_size = offsets[row_num] - prev_offset;
         if (offset > elem_size)
             return {&elements[prev_offset], length + elem_size > offset ? std::min(elem_size, length + elem_size - offset) : 0};
         return {&elements[prev_offset + elem_size - offset], std::min(length, offset)};
     }
+
+    const ColumnString::Chars & getElements() const { return elements; }
+};
+
+/// Treats Enum values as Strings, modeled after StringSource
+template <typename EnumDataType>
+struct EnumSource
+{
+    using Column = typename EnumDataType::ColumnType;
+    using Slice = NumericArraySlice<UInt8>;
+
+    using SinkType = StringSink;
+
+    const typename Column::Container & data;
+    const EnumDataType & data_type;
+
+    size_t row_num = 0;
+
+    EnumSource(const Column & col, const EnumDataType & data_type_)
+        : data(col.getData())
+        , data_type(data_type_)
+    {
+    }
+
+    void next()
+    {
+        ++row_num;
+    }
+
+    bool isEnd() const
+    {
+        return row_num == data.size();
+    }
+
+    size_t rowNum() const
+    {
+        return row_num;
+    }
+
+    size_t getSizeForReserve() const
+    {
+        return data.size();
+    }
+
+    size_t getElementSize() const
+    {
+        std::string_view name = data_type.getNameForValue(data[row_num]).toView();
+        return name.size();
+    }
+
+    size_t getColumnSize() const
+    {
+        return data.size();
+    }
+
+    Slice getWhole() const
+    {
+        std::string_view name = data_type.getNameForValue(data[row_num]).toView();
+        return {reinterpret_cast<const UInt8 *>(name.data()), name.size()};
+    }
+
+    Slice getSliceFromLeft(size_t offset) const
+    {
+        std::string_view name = data_type.getNameForValue(data[row_num]).toView();
+        if (offset >= name.size())
+            return {reinterpret_cast<const UInt8 *>(name.data()), 0};
+        return {reinterpret_cast<const UInt8 *>(name.data()) + offset, name.size() - offset};
+    }
+
+    Slice getSliceFromLeft(size_t offset, size_t length) const
+    {
+        std::string_view name = data_type.getNameForValue(data[row_num]).toView();
+        if (offset >= name.size())
+            return {reinterpret_cast<const UInt8 *>(name.data()), 0};
+        return {reinterpret_cast<const UInt8 *>(name.data()) + offset, std::min(length, name.size() - offset)};
+    }
+
+    Slice getSliceFromRight(size_t offset) const
+    {
+        std::string_view name = data_type.getNameForValue(data[row_num]).toView();
+        if (offset > name.size())
+            return {reinterpret_cast<const UInt8 *>(name.data()), name.size()};
+        return {reinterpret_cast<const UInt8 *>(name.data()) + name.size() - offset, offset};
+    }
+
+    Slice getSliceFromRight(size_t offset, size_t length) const
+    {
+        std::string_view name = data_type.getNameForValue(data[row_num]).toView();
+        if (offset > name.size())
+            return {reinterpret_cast<const UInt8 *>(name.data()), length + name.size() > offset ? std::min(name.size(), length + name.size() - offset) : 0};
+        return {reinterpret_cast<const UInt8 *>(name.data()) + name.size() - offset, std::min(length, offset)};
+    }
 };
 
 
@@ -366,7 +459,7 @@ struct UTF8StringSource : public StringSource
     Slice getSliceFromLeft(size_t offset) const
     {
         const auto * begin = &elements[prev_offset];
-        const auto * end = elements.data() + offsets[row_num] - 1;
+        const auto * end = elements.data() + offsets[row_num];
         const auto * res_begin = skipCodePointsForward(begin, offset, end);
 
         if (res_begin >= end)
@@ -378,7 +471,7 @@ struct UTF8StringSource : public StringSource
     Slice getSliceFromLeft(size_t offset, size_t length) const
     {
         const auto * begin = &elements[prev_offset];
-        const auto * end = elements.data() + offsets[row_num] - 1;
+        const auto * end = elements.data() + offsets[row_num];
         const auto * res_begin = skipCodePointsForward(begin, offset, end);
 
         if (res_begin >= end)
@@ -395,7 +488,7 @@ struct UTF8StringSource : public StringSource
     Slice getSliceFromRight(size_t offset) const
     {
         const auto * begin = &elements[prev_offset];
-        const auto * end = elements.data() + offsets[row_num] - 1;
+        const auto * end = elements.data() + offsets[row_num];
         const auto * res_begin = skipCodePointsBackward(end, offset, begin);
 
         return {res_begin, size_t(end - res_begin)};
@@ -404,7 +497,7 @@ struct UTF8StringSource : public StringSource
     Slice getSliceFromRight(size_t offset, size_t length) const
     {
         const auto * begin = &elements[prev_offset];
-        const auto * end = elements.data() + offsets[row_num] - 1;
+        const auto * end = elements.data() + offsets[row_num];
         const auto * res_begin = skipCodePointsBackward(end, offset, begin);
         const auto * res_end = skipCodePointsForward(res_begin, length, end);
 
@@ -426,11 +519,12 @@ struct FixedStringSource
     const UInt8 * pos;
     const UInt8 * end;
     size_t string_size;
+    const typename ColumnString::Chars & elements;
+
     size_t row_num = 0;
     size_t column_size = 0;
 
-    explicit FixedStringSource(const ColumnFixedString & col)
-            : string_size(col.getN())
+    explicit FixedStringSource(const ColumnFixedString & col) : string_size(col.getN()), elements(col.getChars())
     {
         const auto & chars = col.getChars();
         pos = chars.data();
@@ -501,6 +595,8 @@ struct FixedStringSource
             return {pos, length + string_size > offset ? std::min(string_size, length + string_size - offset) : 0};
         return {pos + string_size - offset, std::min(length, offset)};
     }
+
+    const ColumnString::Chars & getElements() const { return elements; }
 };
 
 
@@ -539,7 +635,7 @@ inline std::unique_ptr<IStringSource> createDynamicStringSource(const IColumn &
         return std::make_unique<DynamicStringSource<ConstSource<StringSource>>>(col);
     if (checkColumnConst<ColumnFixedString>(&col))
         return std::make_unique<DynamicStringSource<ConstSource<FixedStringSource>>>(col);
-    throw Exception("Unexpected type of string column: " + col.getName(), ErrorCodes::ILLEGAL_COLUMN);
+    throw Exception(ErrorCodes::ILLEGAL_COLUMN, "Unexpected type of string column: {}", col.getName());
 }
 
 using StringSources = std::vector<std::unique_ptr<IStringSource>>;
@@ -564,7 +660,8 @@ struct GenericArraySource : public ArraySourceImpl<GenericArraySource>
     }
 
     explicit GenericArraySource(const ColumnArray & arr)
-            : elements(arr.getData()), offsets(arr.getOffsets())
+        : elements(arr.getData())
+        , offsets(arr.getOffsets())
     {
     }
 
@@ -824,7 +921,10 @@ struct NullableValueSource : public ValueSource
     const NullMap & null_map;
 
     template <typename Column>
-    explicit NullableValueSource(const Column & col, const NullMap & null_map_) : ValueSource(col), null_map(null_map_) {}
+    NullableValueSource(const Column & col, const NullMap & null_map_)
+        : ValueSource(col)
+        , null_map(null_map_)
+    {}
 
     void accept(ValueSourceVisitor & visitor) override { visitor.visit(*this); }
 
@@ -837,6 +937,4 @@ struct NullableValueSource : public ValueSource
 };
 
 }
-
-#pragma GCC visibility pop
 }
