@@ -2,10 +2,11 @@
 
 #include <Core/ColumnNumbers.h>
 #include <Core/ColumnsWithTypeAndName.h>
+#include <Core/IResolvedFunction.h>
 #include <Core/Names.h>
-#include <DataTypes/IDataType.h>
+#include <Core/ValuesWithType.h>
 
-#include "config_core.h"
+#include "config.h"
 
 #include <memory>
 
@@ -22,13 +23,13 @@ namespace llvm
 namespace DB
 {
 
-namespace ErrorCodes
-{
-    extern const int NOT_IMPLEMENTED;
-    extern const int ILLEGAL_TYPE_OF_ARGUMENT;
-}
+class IDataType;
+struct DataTypeWithConstInfo;
+using DataTypesWithConstInfo = std::vector<DataTypeWithConstInfo>;
 
 class Field;
+struct FieldInterval;
+using FieldIntervalPtr = std::shared_ptr<FieldInterval>;
 
 /// The simplest executable object.
 /// Motivation:
@@ -38,6 +39,7 @@ class Field;
 class IExecutableFunction
 {
 public:
+    IExecutableFunction();
 
     virtual ~IExecutableFunction() = default;
 
@@ -63,6 +65,11 @@ protected:
       */
     virtual bool useDefaultImplementationForNulls() const { return true; }
 
+    /** Default implementation in presence of arguments with type Nothing is the following:
+      *  If some of arguments have type Nothing then default implementation is to return constant column with type Nothing
+      */
+    virtual bool useDefaultImplementationForNothing() const { return true; }
+
     /** If the function have non-zero number of arguments,
       *  and if all arguments are constant, that we could automatically provide default implementation:
       *  arguments are converted to ordinary columns with single value, then function is executed as usual,
@@ -83,6 +90,11 @@ protected:
       */
     virtual bool useDefaultImplementationForSparseColumns() const { return true; }
 
+    /** If function arguments have replicated columns with the same indexes and all other arguments are constants, call function on nested columns.
+      * Otherwise, convert all replicated columns to ordinary columns.
+      */
+    virtual bool useDefaultImplementationForReplicatedColumns() const { return true; }
+
     /** Some arguments could remain constant during this implementation.
       */
     virtual ColumnNumbers getArgumentsThatAreAlwaysConstant() const { return {}; }
@@ -100,37 +112,43 @@ private:
     ColumnPtr defaultImplementationForNulls(
             const ColumnsWithTypeAndName & args, const DataTypePtr & result_type, size_t input_rows_count, bool dry_run) const;
 
+    ColumnPtr defaultImplementationForNothing(
+            const ColumnsWithTypeAndName & args, const DataTypePtr & result_type, size_t input_rows_count) const;
+
     ColumnPtr executeWithoutLowCardinalityColumns(
             const ColumnsWithTypeAndName & args, const DataTypePtr & result_type, size_t input_rows_count, bool dry_run) const;
 
     ColumnPtr executeWithoutSparseColumns(
             const ColumnsWithTypeAndName & arguments, const DataTypePtr & result_type, size_t input_rows_count, bool dry_run) const;
+
+    ColumnPtr executeWithoutReplicatedColumns(
+         const ColumnsWithTypeAndName & arguments, const DataTypePtr & result_type, size_t input_rows_count, bool dry_run) const;
+
+    bool short_circuit_function_evaluation_for_nulls = false;
+    double short_circuit_function_evaluation_for_nulls_threshold = 0.0;
 };
 
 using ExecutableFunctionPtr = std::shared_ptr<IExecutableFunction>;
 
-using Values = std::vector<llvm::Value *>;
-
 /** Function with known arguments and return type (when the specific overload was chosen).
   * It is also the point where all function-specific properties are known.
   */
-class IFunctionBase
+class IFunctionBase : public IResolvedFunction
 {
 public:
 
-    virtual ~IFunctionBase() = default;
+    ~IFunctionBase() override = default;
 
-    virtual ColumnPtr execute(
-        const ColumnsWithTypeAndName & arguments, const DataTypePtr & result_type, size_t input_rows_count, bool dry_run = false) const
-    {
-        return prepare(arguments)->execute(arguments, result_type, input_rows_count, dry_run);
-    }
+    virtual ColumnPtr execute( /// NOLINT
+        const ColumnsWithTypeAndName & arguments,
+        const DataTypePtr & result_type,
+        size_t input_rows_count,
+        bool dry_run) const;
 
     /// Get the main function name.
     virtual String getName() const = 0;
 
-    virtual const DataTypes & getArgumentTypes() const = 0;
-    virtual const DataTypePtr & getResultType() const = 0;
+    const Array & getParameters() const final;
 
     /// Do preparations and return executable.
     /// sample_columns should contain data types of arguments and values of constants, if relevant.
@@ -148,10 +166,7 @@ public:
       *       templates with default arguments is impossible and including LLVM in such a generic header
       *       as this one is a major pain.
       */
-    virtual llvm::Value * compile(llvm::IRBuilderBase & /*builder*/, Values /*values*/) const
-    {
-        throw Exception(getName() + " is not JIT-compilable", ErrorCodes::NOT_IMPLEMENTED);
-    }
+    virtual llvm::Value * compile(llvm::IRBuilderBase & /*builder*/, const ValuesWithType & /*arguments*/) const;
 
 #endif
 
@@ -163,11 +178,11 @@ public:
       */
     virtual bool isSuitableForConstantFolding() const { return true; }
 
-    /** If function isSuitableForConstantFolding then, this method will be called during query analyzis
+    /** If function isSuitableForConstantFolding then, this method will be called during query analysis
       * if some arguments are constants. For example logical functions (AndFunction, OrFunction) can
-      * return they result based on some constant arguments.
-      * Arguments are passed without modifications, useDefaultImplementationForNulls, useDefaultImplementationForConstants,
-      * useDefaultImplementationForLowCardinality are not applied.
+      * return the result based on some constant arguments.
+      * Arguments are passed without modifications, useDefaultImplementationForNulls, useDefaultImplementationForNothing,
+      * useDefaultImplementationForConstants, useDefaultImplementationForLowCardinality are not applied.
       */
     virtual ColumnPtr getConstantResultForNonConstArguments(
         const ColumnsWithTypeAndName & /* arguments */, const DataTypePtr & /* result_type */) const { return nullptr; }
@@ -211,6 +226,17 @@ public:
 
     virtual bool isDeterministicInScopeOfQuery() const { return true; }
 
+    /** This is a special flags for functions which return constant value for the server,
+      * but the result could be different for different servers in distributed query.
+      *
+      * This functions can't support constant folding on the initiator, but can on the follower.
+      * We can't apply some optimizations as well (e.g. can't remove constant result from GROUP BY key).
+      * So, it is convenient to have a special flag for them.
+      *
+      * Examples are: "__getScalar" and every function from serverConstants.cpp
+      */
+    virtual bool isServerConstant() const { return false; }
+
     /** Lets you know if the function is monotonic in a range of values.
       * This is used to work with the index in a sorted chunk of data.
       * And allows to use the index not only when it is written, for example `date >= const`, but also, for example, `toMonth(date) >= 11`.
@@ -218,11 +244,19 @@ public:
       */
     virtual bool hasInformationAboutMonotonicity() const { return false; }
 
+    /** Lets you know if the function has its definition of preimage.
+      * This is used to work with predicate optimizations, where the comparison between
+      * f(x) and a constant c could be converted to the comparison between x and f's preimage [b, e).
+      */
+    virtual bool hasInformationAboutPreimage() const { return false; }
+
     struct ShortCircuitSettings
     {
-        /// Should we enable lazy execution for the first argument of short-circuit function?
-        /// Example: if(cond, then, else), we don't need to execute cond lazily.
-        bool enable_lazy_execution_for_first_argument;
+        /// Should we enable lazy execution for the nth argument of short-circuit function?
+        /// Example 1st argument: if(cond, then, else), we don't need to execute cond lazily.
+        /// Example other arguments: 1st, 2nd, 3rd argument of dictGetOrDefault should always be calculated.
+        std::unordered_set<size_t> arguments_with_disabled_lazy_execution;
+
         /// Should we enable lazy execution for functions, that are common descendants of
         /// different short-circuit function arguments?
         /// Example 1: if (cond, expr1(..., expr, ...), expr2(..., expr, ...)), we don't need
@@ -257,27 +291,29 @@ public:
     /// The property of monotonicity for a certain range.
     struct Monotonicity
     {
-        bool is_monotonic = false;    /// Is the function monotonous (non-decreasing or non-increasing).
-        bool is_positive = true;    /// true if the function is non-decreasing, false if non-increasing. If is_monotonic = false, then it does not matter.
+        bool is_monotonic = false;   /// Is the function monotonous (non-decreasing or non-increasing).
+        bool is_positive = true;     /// true if the function is non-decreasing, false if non-increasing. If is_monotonic = false, then it does not matter.
         bool is_always_monotonic = false; /// Is true if function is monotonic on the whole input range I
+        bool is_strict = false;      /// true if the function is strictly decreasing or increasing.
     };
 
     /** Get information about monotonicity on a range of values. Call only if hasInformationAboutMonotonicity.
       * NULL can be passed as one of the arguments. This means that the corresponding range is unlimited on the left or on the right.
       */
-    virtual Monotonicity getMonotonicityForRange(const IDataType & /*type*/, const Field & /*left*/, const Field & /*right*/) const
-    {
-        throw Exception("Function " + getName() + " has no information about its monotonicity.", ErrorCodes::NOT_IMPLEMENTED);
-    }
+    virtual Monotonicity getMonotonicityForRange(const IDataType & /*type*/, const Field & /*left*/, const Field & /*right*/) const;
 
+    /** Get the preimage of a function in the form of a left-closed and right-open interval. Call only if hasInformationAboutPreimage.
+      * nullptr might be returned if the point (a single value) is invalid for this function.
+      */
+    virtual FieldIntervalPtr getPreimage(const IDataType & /*type*/, const Field & /*point*/) const;
 };
 
-using FunctionBasePtr = std::shared_ptr<IFunctionBase>;
+using FunctionBasePtr = std::shared_ptr<const IFunctionBase>;
 
 
 /** Creates IFunctionBase from argument types list (chooses one function overload).
   */
-class IFunctionOverloadResolver
+class IFunctionOverloadResolver : public std::enable_shared_from_this<IFunctionOverloadResolver>
 {
 public:
     virtual ~IFunctionOverloadResolver() = default;
@@ -299,6 +335,8 @@ public:
     virtual bool isDeterministic() const { return true; }
     virtual bool isDeterministicInScopeOfQuery() const { return true; }
     virtual bool isInjective(const ColumnsWithTypeAndName &) const { return false; }
+    virtual bool isServerConstant() const { return false; }
+    virtual bool isShortCircuit(IFunctionBase::ShortCircuitSettings & /*settings*/, size_t /*number_of_arguments*/) const { return false; }
 
     /// Override and return true if function needs to depend on the state of the data.
     virtual bool isStateful() const { return false; }
@@ -310,10 +348,7 @@ public:
     /// For higher-order functions (functions, that have lambda expression as at least one argument).
     /// You pass data types with empty DataTypeFunction for lambda arguments.
     /// This function will replace it with DataTypeFunction containing actual types.
-    virtual void getLambdaArgumentTypesImpl(DataTypes & arguments [[maybe_unused]]) const
-    {
-        throw Exception("Function " + getName() + " can't have lambda-expressions as arguments", ErrorCodes::ILLEGAL_TYPE_OF_ARGUMENT);
-    }
+    virtual void getLambdaArgumentTypesImpl(DataTypes & arguments [[maybe_unused]]) const;
 
     /// Returns indexes of arguments, that must be ColumnConst
     virtual ColumnNumbers getArgumentsThatAreAlwaysConstant() const { return {}; }
@@ -322,17 +357,17 @@ public:
     /// (for functions like isNull(x))
     virtual ColumnNumbers getArgumentsThatDontImplyNullableReturnType(size_t number_of_arguments [[maybe_unused]]) const { return {}; }
 
+    /// Returns type that should be used as the result type in default implementation for Dynamic.
+    /// Function should implement this method if its result type doesn't depend on the arguments types.
+    virtual DataTypePtr getReturnTypeForDefaultImplementationForDynamic() const { return nullptr; }
+
+    DataTypePtr getReturnType(const ColumnsWithTypeAndName & arguments) const;
+
 protected:
 
-    virtual FunctionBasePtr buildImpl(const ColumnsWithTypeAndName & /* arguments */, const DataTypePtr & /* result_type */) const
-    {
-        throw Exception("buildImpl is not implemented for " + getName(), ErrorCodes::NOT_IMPLEMENTED);
-    }
+    virtual FunctionBasePtr buildImpl(const ColumnsWithTypeAndName & /* arguments */, const DataTypePtr & /* result_type */) const;
 
-    virtual DataTypePtr getReturnTypeImpl(const DataTypes & /*arguments*/) const
-    {
-        throw Exception("getReturnType is not implemented for " + getName(), ErrorCodes::NOT_IMPLEMENTED);
-    }
+    virtual DataTypePtr getReturnTypeImpl(const DataTypes & /*arguments*/) const;
 
     /// This function will be called in default implementation. You can overload it or the previous one.
     virtual DataTypePtr getReturnTypeImpl(const ColumnsWithTypeAndName & arguments) const
@@ -354,7 +389,13 @@ protected:
       */
     virtual bool useDefaultImplementationForNulls() const { return true; }
 
-    /** If useDefaultImplementationForNulls() is true, then change arguments for getReturnType() and build().
+    /** If useDefaultImplementationForNothing() is true, then change arguments for getReturnType() and build():
+      *  if some of arguments are Nothing then don't call getReturnType(), call build() with return_type = Nothing,
+      * Otherwise build returns build(arguments, getReturnType(arguments));
+      */
+    virtual bool useDefaultImplementationForNothing() const { return true; }
+
+    /** If useDefaultImplementationForLowCardinalityColumns() is true, then change arguments for getReturnType() and build().
       * If function arguments has low cardinality types, convert them to ordinary types.
       * getReturnType returns ColumnLowCardinality if at least one argument type is ColumnLowCardinality.
       */
@@ -367,12 +408,21 @@ protected:
       */
     virtual bool useDefaultImplementationForSparseColumns() const { return true; }
 
-    // /// If it isn't, will convert all ColumnLowCardinality arguments to full columns.
+    /// If it isn't, will convert all ColumnLowCardinality arguments to full columns.
     virtual bool canBeExecutedOnLowCardinalityDictionary() const { return true; }
 
-private:
+    /** If useDefaultImplementationForDynamic() is true, then special FunctionBaseDynamicAdaptor will be used
+     *  if function arguments has Dynamic column. This adaptor will build and execute this function for all
+     *  internal types inside Dynamic column separately and construct result based on results for these types.
+     *  If getReturnTypeForDefaultImplementationForDynamic() returns T, then result of such function
+     *  will be Nullable(T), otherwise the result will be Dynamic.
+     *
+     *  We cannot use default implementation for Dynamic if function doesn't use default implementation for NULLs,
+     *  because Dynamic column can contain NULLs and we should know how to process them.
+      */
+    virtual bool useDefaultImplementationForDynamic() const { return useDefaultImplementationForNulls(); }
 
-    DataTypePtr getReturnType(const ColumnsWithTypeAndName & arguments) const;
+private:
 
     DataTypePtr getReturnTypeWithoutLowCardinality(const ColumnsWithTypeAndName & arguments) const;
 };
@@ -380,7 +430,7 @@ private:
 using FunctionOverloadResolverPtr = std::shared_ptr<IFunctionOverloadResolver>;
 
 /// Old function interface. Check documentation in IFunction.h.
-/// If client do not need statefull properties it can implement this interface.
+/// If client do not need stateful properties it can implement this interface.
 class IFunction
 {
 public:
@@ -403,6 +453,11 @@ public:
       */
     virtual bool useDefaultImplementationForNulls() const { return true; }
 
+    /** Default implementation in presence of arguments with type Nothing is the following:
+      *  If some of arguments have type Nothing then default implementation is to return constant column with type Nothing
+      */
+    virtual bool useDefaultImplementationForNothing() const { return true; }
+
     /** If the function have non-zero number of arguments,
       *  and if all arguments are constant, that we could automatically provide default implementation:
       *  arguments are converted to ordinary columns with single value, then function is executed as usual,
@@ -427,9 +482,17 @@ public:
       */
     virtual bool useDefaultImplementationForSparseColumns() const { return true; }
 
+    /** If function arguments have replicated columns with the same indexes and all other arguments are constants, call function on nested columns.
+      * Otherwise, convert all replicated columns to ordinary columns.
+      */
+    virtual bool useDefaultImplementationForReplicatedColumns() const { return true; }
+
     /// If it isn't, will convert all ColumnLowCardinality arguments to full columns.
     virtual bool canBeExecutedOnLowCardinalityDictionary() const { return true; }
 
+    virtual bool useDefaultImplementationForDynamic() const { return useDefaultImplementationForNulls(); }
+    virtual DataTypePtr getReturnTypeForDefaultImplementationForDynamic() const { return nullptr; }
+
     /** True if function can be called on default arguments (include Nullable's) and won't throw.
       * Counterexample: modulo(0, 0)
       */
@@ -441,6 +504,7 @@ public:
     virtual bool isInjective(const ColumnsWithTypeAndName & /*sample_columns*/) const { return false; }
     virtual bool isDeterministic() const { return true; }
     virtual bool isDeterministicInScopeOfQuery() const { return true; }
+    virtual bool isServerConstant() const { return false; }
     virtual bool isStateful() const { return false; }
 
     using ShortCircuitSettings = IFunctionBase::ShortCircuitSettings;
@@ -448,20 +512,16 @@ public:
     virtual bool isSuitableForShortCircuitArgumentsExecution(const DataTypesWithConstInfo & /*arguments*/) const = 0;
 
     virtual bool hasInformationAboutMonotonicity() const { return false; }
+    virtual bool hasInformationAboutPreimage() const { return false; }
 
     using Monotonicity = IFunctionBase::Monotonicity;
-    virtual Monotonicity getMonotonicityForRange(const IDataType & /*type*/, const Field & /*left*/, const Field & /*right*/) const
-    {
-        throw Exception("Function " + getName() + " has no information about its monotonicity.", ErrorCodes::NOT_IMPLEMENTED);
-    }
+    virtual Monotonicity getMonotonicityForRange(const IDataType & /*type*/, const Field & /*left*/, const Field & /*right*/) const;
+    virtual FieldIntervalPtr getPreimage(const IDataType & /*type*/, const Field & /*point*/) const;
 
     /// For non-variadic functions, return number of arguments; otherwise return zero (that should be ignored).
     virtual size_t getNumberOfArguments() const = 0;
 
-    virtual DataTypePtr getReturnTypeImpl(const DataTypes & /*arguments*/) const
-    {
-        throw Exception("getReturnType is not implemented for " + getName(), ErrorCodes::NOT_IMPLEMENTED);
-    }
+    virtual DataTypePtr getReturnTypeImpl(const DataTypes & /*arguments*/) const;
 
     /// Get the result type by argument type. If the function does not apply to these arguments, throw an exception.
     virtual DataTypePtr getReturnTypeImpl(const ColumnsWithTypeAndName & arguments) const
@@ -475,19 +535,16 @@ public:
 
     virtual bool isVariadic() const { return false; }
 
-    virtual void getLambdaArgumentTypes(DataTypes & /*arguments*/) const
-    {
-        throw Exception("Function " + getName() + " can't have lambda-expressions as arguments", ErrorCodes::ILLEGAL_TYPE_OF_ARGUMENT);
-    }
+    virtual void getLambdaArgumentTypes(DataTypes & /*arguments*/) const;
 
     virtual ColumnNumbers getArgumentsThatDontImplyNullableReturnType(size_t /*number_of_arguments*/) const { return {}; }
 
 
 #if USE_EMBEDDED_COMPILER
 
-    bool isCompilable(const DataTypes & arguments) const;
+    bool isCompilable(const DataTypes & arguments, const DataTypePtr & result_type) const;
 
-    llvm::Value * compile(llvm::IRBuilderBase &, const DataTypes & arguments, Values values) const;
+    llvm::Value * compile(llvm::IRBuilderBase & builder, const ValuesWithType & arguments, const DataTypePtr & result_type) const;
 
 #endif
 
@@ -495,12 +552,9 @@ protected:
 
 #if USE_EMBEDDED_COMPILER
 
-    virtual bool isCompilableImpl(const DataTypes &) const { return false; }
+    virtual bool isCompilableImpl(const DataTypes & /*arguments*/, const DataTypePtr & /*result_type*/) const { return false; }
 
-    virtual llvm::Value * compileImpl(llvm::IRBuilderBase &, const DataTypes &, Values) const
-    {
-        throw Exception(getName() + " is not JIT-compilable", ErrorCodes::NOT_IMPLEMENTED);
-    }
+    virtual llvm::Value * compileImpl(llvm::IRBuilderBase & /*builder*/, const ValuesWithType & /*arguments*/, const DataTypePtr & /*result_type*/) const;
 
 #endif
 };
