@@ -1,6 +1,11 @@
 #pragma once
 
+#include <Columns/ColumnConst.h>
+#include <Columns/ColumnTuple.h>
 #include <Functions/IFunction.h>
+#include <Functions/FunctionHelpers.h>
+#include <Interpreters/Context_fwd.h>
+
 
 namespace DB
 {
