@@ -3,6 +3,12 @@
 
 namespace DB
 {
+
+namespace ErrorCodes
+{
+extern const int NOT_IMPLEMENTED;
+}
+
 namespace
 {
 
@@ -21,7 +27,14 @@ namespace
         template <typename T>
         static void execute(const T * src, size_t size, T * dst)
         {
-            NFastOps::Sigmoid<>(src, size, dst);
+            if constexpr (std::is_same_v<T, BFloat16>)
+            {
+                throw Exception(ErrorCodes::NOT_IMPLEMENTED, "Function `{}` is not implemented for BFloat16", name);
+            }
+            else
+            {
+                NFastOps::Sigmoid<>(src, size, dst);
+            }
         }
     };
 }
@@ -30,7 +43,7 @@ using FunctionSigmoid = FunctionMathUnary<Impl>;
 
 #else
 
-static double sigmoid(double x)
+double sigmoid(double x)
 {
     return 1.0 / (1.0 + exp(-x));
 }
@@ -41,10 +54,9 @@ using FunctionSigmoid = FunctionMathUnary<UnaryFunctionVectorized<SigmoidName, s
 
 }
 
-void registerFunctionSigmoid(FunctionFactory & factory)
+REGISTER_FUNCTION(Sigmoid)
 {
     factory.registerFunction<FunctionSigmoid>();
 }
 
 }
-
