@@ -1,9 +1,14 @@
 #include <IO/Archives/ZipArchiveReader.h>
 
 #if USE_MINIZIP
+#include <IO/Archives/ZipArchiveWriter.h>
 #include <IO/ReadBufferFromFileBase.h>
 #include <Common/quoteString.h>
+#include <base/errnoToString.h>
+
 #include <unzip.h>
+#include <zip.h>
+#include <mz.h>
 
 
 namespace DB
@@ -13,11 +18,26 @@ namespace ErrorCodes
     extern const int CANNOT_UNPACK_ARCHIVE;
     extern const int LOGICAL_ERROR;
     extern const int SEEK_POSITION_OUT_OF_BOUND;
+    extern const int CANNOT_SEEK_THROUGH_FILE;
 }
 
 using RawHandle = unzFile;
 
 
+namespace
+{
+    void checkCompressionMethodIsEnabled(int compression_method_)
+    {
+        ZipArchiveWriter::checkCompressionMethodIsEnabled(compression_method_);
+    }
+
+    void checkEncryptionIsEnabled()
+    {
+        ZipArchiveWriter::checkEncryptionIsEnabled();
+    }
+}
+
+
 /// Holds a raw handle, calls acquireRawHandle() in the constructor and releaseRawHandle() in the destructor.
 class ZipArchiveReader::HandleHolder
 {
@@ -42,12 +62,12 @@ public:
         }
     }
 
-    HandleHolder(HandleHolder && src)
+    HandleHolder(HandleHolder && src) noexcept
     {
         *this = std::move(src);
     }
 
-    HandleHolder & operator =(HandleHolder && src)
+    HandleHolder & operator=(HandleHolder && src) noexcept
     {
         reader = std::exchange(src.reader, nullptr);
         raw_handle = std::exchange(src.raw_handle, nullptr);
@@ -59,14 +79,35 @@ public:
     RawHandle getRawHandle() const { return raw_handle; }
     std::shared_ptr<ZipArchiveReader> getReader() const { return reader; }
 
-    void locateFile(const String & file_name_)
+    bool locateFile(const String & file_name_)
     {
         resetFileInfo();
         bool case_sensitive = true;
         int err = unzLocateFile(raw_handle, file_name_.c_str(), reinterpret_cast<unzFileNameComparer>(static_cast<size_t>(case_sensitive)));
         if (err == UNZ_END_OF_LIST_OF_FILE)
-            showError("File " + quoteString(file_name_) + " not found");
+            return false;
         file_name = file_name_;
+        return true;
+    }
+
+    bool locateFile(NameFilter filter)
+    {
+        int err = unzGoToFirstFile(raw_handle);
+        if (err == UNZ_END_OF_LIST_OF_FILE)
+            return false;
+
+        do
+        {
+            checkResult(err);
+            resetFileInfo();
+            retrieveFileInfo();
+            if (filter(getFileName()))
+                return true;
+
+            err = unzGoToNextFile(raw_handle);
+        } while (err != UNZ_END_OF_LIST_OF_FILE);
+
+        return false;
     }
 
     bool tryLocateFile(const String & file_name_)
@@ -108,13 +149,34 @@ public:
         return *file_name;
     }
 
-    const FileInfo & getFileInfo() const
+    const FileInfoImpl & getFileInfo() const
     {
         if (!file_info)
             retrieveFileInfo();
         return *file_info;
     }
 
+    std::vector<std::string> getAllFiles(NameFilter filter)
+    {
+        std::vector<std::string> files;
+        resetFileInfo();
+        int err = unzGoToFirstFile(raw_handle);
+        if (err == UNZ_END_OF_LIST_OF_FILE)
+            return files;
+
+        do
+        {
+            checkResult(err);
+            resetFileInfo();
+            retrieveFileInfo();
+            if (!filter || filter(getFileName()))
+                files.push_back(*file_name);
+            err = unzGoToNextFile(raw_handle);
+        } while (err != UNZ_END_OF_LIST_OF_FILE);
+
+        return files;
+    }
+
     void closeFile()
     {
         int err = unzCloseCurrentFile(raw_handle);
@@ -161,7 +223,7 @@ private:
     std::shared_ptr<ZipArchiveReader> reader;
     RawHandle raw_handle = nullptr;
     mutable std::optional<String> file_name;
-    mutable std::optional<FileInfo> file_info;
+    mutable std::optional<FileInfoImpl> file_info;
 };
 
 
@@ -174,7 +236,7 @@ public:
         , handle(std::move(handle_))
     {
         const auto & file_info = handle.getFileInfo();
-        checkCompressionMethodIsEnabled(static_cast<CompressionMethod>(file_info.compression_method));
+        checkCompressionMethodIsEnabled(file_info.compression_method);
 
         const char * password_cstr = nullptr;
         if (file_info.is_encrypted)
@@ -202,13 +264,13 @@ public:
         else if (whence == SEEK_CUR)
             new_pos = off + current_pos;
         else
-            throw Exception("Only SEEK_SET and SEEK_CUR seek modes allowed.", ErrorCodes::SEEK_POSITION_OUT_OF_BOUND);
+            throw Exception(ErrorCodes::SEEK_POSITION_OUT_OF_BOUND, "Only SEEK_SET and SEEK_CUR seek modes allowed.");
 
         if (new_pos == current_pos)
             return current_pos; /// The position is the same.
 
         if (new_pos < 0)
-            throw Exception("Seek position is out of bound", ErrorCodes::SEEK_POSITION_OUT_OF_BOUND);
+            throw Exception(ErrorCodes::SEEK_POSITION_OUT_OF_BOUND, "Seek position is out of bound");
 
         off_t working_buffer_start_pos = current_pos - offset();
         off_t working_buffer_end_pos = current_pos + available();
@@ -225,25 +287,29 @@ public:
         /// Check that the new position is now beyond the end of the file.
         const auto & file_info = handle.getFileInfo();
         if (new_pos > static_cast<off_t>(file_info.uncompressed_size))
-            throw Exception("Seek position is out of bound", ErrorCodes::SEEK_POSITION_OUT_OF_BOUND);
+            throw Exception(ErrorCodes::SEEK_POSITION_OUT_OF_BOUND, "Seek position is out of bound");
 
-        if (file_info.compression_method == static_cast<int>(CompressionMethod::kStore))
-        {
-            /// unzSeek64() works only for non-compressed files.
-            checkResult(unzSeek64(raw_handle, off, whence));
-            return unzTell64(raw_handle);
-        }
+        /// unzSeek64() works only for non-compressed files.
+        ///
+        /// We used to have a fallback here, where we would:
+        ///  * ignore() to "seek" forward,
+        ///  * unzCloseCurrentFile(raw_handle) + unzOpenCurrentFile(raw_handle) to seek to the
+        ///    beginning of the file.
+        /// But the close+open didn't work: after closing+reopening once, the second
+        /// unzCloseCurrentFile() was failing with MZ_CRC_ERROR in mz_zip_entry_read_close(). Maybe
+        /// it's a bug in minizip where some state was inadvertently left over after close+reopen.
+        /// Didn't investigate because re-reading the whole file should be avoided anyway.
+        if (file_info.compression_method != MZ_COMPRESS_METHOD_STORE)
+            throw Exception(ErrorCodes::CANNOT_SEEK_THROUGH_FILE, "Seek in compressed archive is not supported.");
 
-        /// As a last try we go slow way, we're going to simply ignore all data before the new position.
-        if (new_pos < current_pos)
-        {
-            checkResult(unzCloseCurrentFile(raw_handle));
-            checkResult(unzOpenCurrentFile(raw_handle));
-            current_pos = 0;
-        }
+        checkResult(unzSeek64(raw_handle, off, whence));
+        return unzTell64(raw_handle);
+    }
 
-        ignore(new_pos - current_pos);
-        return new_pos;
+    bool checkIfActuallySeekable() override
+    {
+        /// The library doesn't support seeking in compressed files.
+        return handle.getFileInfo().compression_method == MZ_COMPRESS_METHOD_STORE;
     }
 
     off_t getPosition() override
@@ -254,6 +320,8 @@ public:
 
     String getFileName() const override { return handle.getFileName(); }
 
+    std::optional<size_t> tryGetFileSize() override { return handle.getFileInfo().uncompressed_size; }
+
     /// Releases owned handle to pass it to an enumerator.
     HandleHolder releaseHandle() &&
     {
@@ -265,7 +333,7 @@ private:
     bool nextImpl() override
     {
         RawHandle raw_handle = handle.getRawHandle();
-        auto bytes_read = unzReadCurrentFile(raw_handle, internal_buffer.begin(), internal_buffer.size());
+        auto bytes_read = unzReadCurrentFile(raw_handle, internal_buffer.begin(), static_cast<int>(internal_buffer.size()));
 
         if (bytes_read < 0)
             checkResult(bytes_read);
@@ -443,6 +511,11 @@ ZipArchiveReader::~ZipArchiveReader()
     }
 }
 
+const std::string & ZipArchiveReader::getPath() const
+{
+    return path_to_archive;
+}
+
 bool ZipArchiveReader::fileExists(const String & filename)
 {
     return acquireHandle().tryLocateFile(filename);
@@ -451,7 +524,9 @@ bool ZipArchiveReader::fileExists(const String & filename)
 ZipArchiveReader::FileInfo ZipArchiveReader::getFileInfo(const String & filename)
 {
     auto handle = acquireHandle();
-    handle.locateFile(filename);
+    if (!handle.locateFile(filename))
+        showError(fmt::format("File {} was not found in archive", quoteString(filename)));
+
     return handle.getFileInfo();
 }
 
@@ -463,17 +538,38 @@ std::unique_ptr<ZipArchiveReader::FileEnumerator> ZipArchiveReader::firstFile()
     return std::make_unique<FileEnumeratorImpl>(std::move(handle));
 }
 
-std::unique_ptr<ReadBufferFromFileBase> ZipArchiveReader::readFile(const String & filename)
+std::unique_ptr<ReadBufferFromFileBase> ZipArchiveReader::readFile(const String & filename, bool throw_on_not_found)
 {
     auto handle = acquireHandle();
-    handle.locateFile(filename);
+    if (!handle.locateFile(filename))
+    {
+        if (throw_on_not_found)
+            showError(fmt::format("File {} was not found in archive", quoteString(filename)));
+
+        return nullptr;
+    }
+
+    return std::make_unique<ReadBufferFromZipArchive>(std::move(handle));
+}
+
+std::unique_ptr<ReadBufferFromFileBase> ZipArchiveReader::readFile(NameFilter filter, bool throw_on_not_found)
+{
+    auto handle = acquireHandle();
+    if (!handle.locateFile(filter))
+    {
+        if (throw_on_not_found)
+            showError(fmt::format("No file satisfying filter in archive"));
+
+        return nullptr;
+    }
+
     return std::make_unique<ReadBufferFromZipArchive>(std::move(handle));
 }
 
 std::unique_ptr<ReadBufferFromFileBase> ZipArchiveReader::readFile(std::unique_ptr<FileEnumerator> enumerator)
 {
     if (!dynamic_cast<FileEnumeratorImpl *>(enumerator.get()))
-        throw Exception("Wrong enumerator passed to readFile()", ErrorCodes::LOGICAL_ERROR);
+        throw Exception(ErrorCodes::LOGICAL_ERROR, "Wrong enumerator passed to readFile()");
     auto enumerator_impl = std::unique_ptr<FileEnumeratorImpl>(static_cast<FileEnumeratorImpl *>(enumerator.release()));
     auto handle = std::move(*enumerator_impl).releaseHandle();
     return std::make_unique<ReadBufferFromZipArchive>(std::move(handle));
@@ -482,7 +578,7 @@ std::unique_ptr<ReadBufferFromFileBase> ZipArchiveReader::readFile(std::unique_p
 std::unique_ptr<ZipArchiveReader::FileEnumerator> ZipArchiveReader::nextFile(std::unique_ptr<ReadBuffer> read_buffer)
 {
     if (!dynamic_cast<ReadBufferFromZipArchive *>(read_buffer.get()))
-        throw Exception("Wrong ReadBuffer passed to nextFile()", ErrorCodes::LOGICAL_ERROR);
+        throw Exception(ErrorCodes::LOGICAL_ERROR, "Wrong ReadBuffer passed to nextFile()");
     auto read_buffer_from_zip = std::unique_ptr<ReadBufferFromZipArchive>(static_cast<ReadBufferFromZipArchive *>(read_buffer.release()));
     auto handle = std::move(*read_buffer_from_zip).releaseHandle();
     if (!handle.nextFile())
@@ -490,6 +586,26 @@ std::unique_ptr<ZipArchiveReader::FileEnumerator> ZipArchiveReader::nextFile(std
     return std::make_unique<FileEnumeratorImpl>(std::move(handle));
 }
 
+std::unique_ptr<ZipArchiveReader::FileEnumerator> ZipArchiveReader::currentFile(std::unique_ptr<ReadBuffer> read_buffer)
+{
+    if (!dynamic_cast<ReadBufferFromZipArchive *>(read_buffer.get()))
+        throw Exception(ErrorCodes::LOGICAL_ERROR, "Wrong ReadBuffer passed to nextFile()");
+    auto read_buffer_from_zip = std::unique_ptr<ReadBufferFromZipArchive>(static_cast<ReadBufferFromZipArchive *>(read_buffer.release()));
+    auto handle = std::move(*read_buffer_from_zip).releaseHandle();
+    return std::make_unique<FileEnumeratorImpl>(std::move(handle));
+}
+
+std::vector<std::string> ZipArchiveReader::getAllFiles()
+{
+    return getAllFiles({});
+}
+
+std::vector<std::string> ZipArchiveReader::getAllFiles(NameFilter filter)
+{
+    auto handle = acquireHandle();
+    return handle.getAllFiles(filter);
+}
+
 void ZipArchiveReader::setPassword(const String & password_)
 {
     std::lock_guard lock{mutex};
@@ -538,11 +654,11 @@ void ZipArchiveReader::checkResult(int code) const
     if (code >= UNZ_OK)
         return;
 
-    String message = "Code= ";
+    String message = "Code = ";
     switch (code)
     {
         case UNZ_OK: return;
-        case UNZ_ERRNO: message += "ERRNO, errno= " + String{strerror(errno)}; break;
+        case UNZ_ERRNO: message += "ERRNO, errno = " + errnoToString(); break;
         case UNZ_PARAMERROR: message += "PARAMERROR"; break;
         case UNZ_BADZIPFILE: message += "BADZIPFILE"; break;
         case UNZ_INTERNALERROR: message += "INTERNALERROR"; break;
