@@ -1,16 +1,16 @@
 #pragma once
 
 #include <IO/ReadBuffer.h>
-#include <IO/BufferWithOwnMemory.h>
+#include <IO/CompressedReadBufferWrapper.h>
 
 
 namespace DB
 {
 
-class BrotliReadBuffer : public BufferWithOwnMemory<ReadBuffer>
+class BrotliReadBuffer : public CompressedReadBufferWrapper
 {
 public:
-    BrotliReadBuffer(
+    explicit BrotliReadBuffer(
             std::unique_ptr<ReadBuffer> in_,
             size_t buf_size = DBMS_DEFAULT_BUFFER_SIZE,
             char * existing_memory = nullptr,
@@ -21,8 +21,6 @@ public:
 private:
     bool nextImpl() override;
 
-    std::unique_ptr<ReadBuffer> in;
-
     class BrotliStateWrapper;
     std::unique_ptr<BrotliStateWrapper> brotli;
 
