@@ -1,16 +1,13 @@
 #pragma once
 
-#ifdef __clang__
 #pragma clang diagnostic push
 #pragma clang diagnostic ignored "-Wdouble-promotion"
-#endif
 
+#include <base/defines.h>
 #include <double-conversion/double-conversion.h>
 #include <boost/noncopyable.hpp>
 
-#ifdef __clang__
 #pragma clang diagnostic pop
-#endif
 
 
 namespace DB
@@ -29,9 +26,6 @@ template <> struct DoubleToStringConverterFlags<true>
 template <bool emit_decimal_point>
 class DoubleConverter : private boost::noncopyable
 {
-    DoubleConverter(const DoubleConverter &) = delete;
-    DoubleConverter & operator=(const DoubleConverter &) = delete;
-
     DoubleConverter() = default;
 
 public:
