@@ -1,6 +1,6 @@
 #pragma once
 
-#include <IO/BufferWithOwnMemory.h>
+#include <IO/CompressedReadBufferWrapper.h>
 #include <IO/CompressionMethod.h>
 #include <IO/ReadBuffer.h>
 
@@ -11,10 +11,10 @@
 namespace DB
 {
 
-class Lz4InflatingReadBuffer : public BufferWithOwnMemory<ReadBuffer>
+class Lz4InflatingReadBuffer : public CompressedReadBufferWrapper
 {
 public:
-    Lz4InflatingReadBuffer(
+    explicit Lz4InflatingReadBuffer(
         std::unique_ptr<ReadBuffer> in_,
         size_t buf_size = DBMS_DEFAULT_BUFFER_SIZE,
         char * existing_memory = nullptr,
@@ -25,17 +25,9 @@ public:
 private:
     bool nextImpl() override;
 
-    std::unique_ptr<ReadBuffer> in;
-
     LZ4F_dctx* dctx;
 
-    void * in_data;
-    void * out_data;
-
-    size_t in_available;
-    size_t out_available;
-
-    bool eof_flag = false;
+    bool in_eof = false;
 };
 
 }
