@@ -2,6 +2,8 @@
 
 #include <IO/BufferWithOwnMemory.h>
 #include <IO/SeekableReadBuffer.h>
+#include <IO/WithFileName.h>
+#include <Interpreters/Context_fwd.h>
 #include <base/time.h>
 
 #include <functional>
@@ -19,7 +21,8 @@
 
 namespace DB
 {
-class ReadBufferFromFileBase : public BufferWithOwnMemory<SeekableReadBuffer>
+
+class ReadBufferFromFileBase : public BufferWithOwnMemory<SeekableReadBuffer>, public WithFileName, public WithFileSize
 {
 public:
     ReadBufferFromFileBase();
@@ -29,7 +32,6 @@ public:
         size_t alignment,
         std::optional<size_t> file_size_ = std::nullopt);
     ~ReadBufferFromFileBase() override;
-    virtual std::string getFileName() const = 0;
 
     /// It is possible to get information about the time of each reading.
     struct ProfileInfo
@@ -48,6 +50,18 @@ public:
         clock_type = clock_type_;
     }
 
+    std::optional<size_t> tryGetFileSize() override;
+
+    void setProgressCallback(ContextPtr context);
+
+    /// Returns true if this file is on local filesystem, and getFileName() is its path.
+    /// I.e. it can be read using open() or mmap(). If this buffer is a "view" into a subrange of the
+    /// file, *out_view_offset is set to the start of that subrange, i.e. the difference between actual
+    /// file offset and what getPosition() returns.
+    virtual bool isRegularLocalFile(size_t * /*out_view_offsee*/) { return false; }
+
+    virtual bool isCached() const { return false; }
+
 protected:
     std::optional<size_t> file_size;
     ProfileCallback profile_callback;
