@@ -21,6 +21,14 @@ public:
 
     bool nextImpl() override;
 
+    bool isWithFileSize() const { return dynamic_cast<const WithFileSize *>(impl.get()) != nullptr; }
+
+    const ReadBuffer & getWrappedReadBuffer() const { return *impl; }
+
+    ReadBuffer & getWrappedReadBuffer() { return *impl; }
+
+    std::optional<size_t> tryGetFileSize() override;
+
 protected:
     std::unique_ptr<SeekableReadBuffer> impl;
     String file_name;
