@@ -1,6 +1,6 @@
 #include <IO/TimeoutSetter.h>
-
-#include <base/logger_useful.h>
+#include <Common/Exception.h>
+#include <Common/logger_useful.h>
 
 
 namespace DB
@@ -29,17 +29,32 @@ TimeoutSetter::TimeoutSetter(Poco::Net::StreamSocket & socket_, Poco::Timespan t
 
 TimeoutSetter::~TimeoutSetter()
 {
+    if (was_reset)
+        return;
+
     try
     {
-        socket.setSendTimeout(old_send_timeout);
-        socket.setReceiveTimeout(old_receive_timeout);
+        reset();
     }
     catch (...)
     {
-        /// Sometimes caught on Mac OS X. This message can be safely ignored.
-        /// If you are developer using Mac, please debug this error message by yourself.
+        /// It's known that setting timeouts for a socket often does not work on Apple macOS.
+        /// Let's not confuse the users of Apple macOS with extraneous error messages.
+#if !defined(OS_DARWIN)
         tryLogCurrentException("Client", "TimeoutSetter: Can't reset timeouts");
+#endif
     }
 }
 
+void TimeoutSetter::reset()
+{
+    bool connected = socket.impl()->initialized();
+    if (!connected)
+        return;
+
+    socket.setSendTimeout(old_send_timeout);
+    socket.setReceiveTimeout(old_receive_timeout);
+    was_reset = true;
+}
+
 }
