@@ -1,6 +1,6 @@
 #pragma once
 
-#include <Common/config.h>
+#include "config.h"
 #include <Common/assert_cast.h>
 
 #if USE_SSL
@@ -28,7 +28,7 @@ public:
 
     void sync() override;
 
-    std::string getFileName() const override { return assert_cast<WriteBufferFromFileBase *>(out.get())->getFileName(); }
+    std::string getFileName() const override { return assert_cast<WriteBufferFromFileBase *>(out)->getFileName(); }
 
 private:
     void nextImpl() override;
@@ -39,6 +39,8 @@ private:
     bool flush_header = false;
 
     FileEncryption::Encryptor encryptor;
+
+    LoggerPtr log = getLogger("WriteBufferFromEncryptedFile");
 };
 
 }
