@@ -1,9 +1,10 @@
 #pragma once
 
 #include <IO/ReadBuffer.h>
-#include <IO/BufferWithOwnMemory.h>
+#include <IO/CompressedReadBufferWrapper.h>
 #include <IO/CompressionMethod.h>
 
+#include <limits>
 #include <zlib.h>
 
 
@@ -16,7 +17,7 @@ namespace ErrorCodes
 
 /// Reads compressed data from ReadBuffer in_ and performs decompression using zlib library.
 /// This buffer is able to seamlessly decompress multiple concatenated zlib streams.
-class ZlibInflatingReadBuffer : public BufferWithOwnMemory<ReadBuffer>
+class ZlibInflatingReadBuffer : public CompressedReadBufferWrapper
 {
 public:
     ZlibInflatingReadBuffer(
@@ -31,9 +32,13 @@ public:
 private:
     bool nextImpl() override;
 
-    std::unique_ptr<ReadBuffer> in;
     z_stream zstr;
     bool eof_flag;
+
+    /// Limit size of buffer because zlib uses
+    /// UInt32 for sizes of internal buffers.
+    using BufferSizeType =  decltype(zstr.avail_in);
+    static constexpr auto max_buffer_size = std::numeric_limits<BufferSizeType>::max();
 };
 
 }
