@@ -1,11 +1,12 @@
-#include <string>
-
 #include <IO/ReadBufferFromFileDescriptor.h>
 #include <IO/LimitReadBuffer.h>
 #include <IO/WriteBufferFromFileDescriptor.h>
 #include <IO/copyData.h>
 #include <IO/WriteHelpers.h>
 
+#include <iostream>
+#include <string>
+
 
 int main(int argc, char ** argv)
 {
@@ -24,18 +25,20 @@ int main(int argc, char ** argv)
 
     writeCString("--- first ---\n", out);
     {
-        LimitReadBuffer limit_in(in, limit, false);
+        LimitReadBuffer limit_in(in, {.read_no_more=limit});
         copyData(limit_in, out);
     }
 
     writeCString("\n--- second ---\n", out);
     {
-        LimitReadBuffer limit_in(in, limit, false);
+        LimitReadBuffer limit_in(in,{.read_no_more=limit});
         copyData(limit_in, out);
     }
 
     writeCString("\n--- the rest ---\n", out);
     copyData(in, out);
 
+    out.finalize();
+
     return 0;
 }
