@@ -1,3 +1,4 @@
+#include <Interpreters/Context.h>
 #include <Interpreters/InterpreterFactory.h>
 #include <Interpreters/Access/InterpreterShowPrivilegesQuery.h>
 #include <Interpreters/executeQuery.h>
@@ -13,7 +14,11 @@ InterpreterShowPrivilegesQuery::InterpreterShowPrivilegesQuery(const ASTPtr & qu
 
 BlockIO InterpreterShowPrivilegesQuery::execute()
 {
-    return executeQuery("SELECT * FROM system.privileges", context, QueryFlags{ .internal = true }).second;
+    auto query_context = Context::createCopy(context);
+    query_context->makeQueryContext();
+    query_context->setCurrentQueryId("");
+
+    return executeQuery("SELECT * FROM system.privileges", query_context, QueryFlags{ .internal = true }).second;
 }
 
 void registerInterpreterShowPrivilegesQuery(InterpreterFactory & factory)
