@@ -20,14 +20,10 @@ class ActionLocksManager : WithContext
 public:
     explicit ActionLocksManager(ContextPtr context);
 
-    /// Adds new locks for each table
-    void add(StorageActionBlockType action_type, ContextPtr context);
     /// Add new lock for a table if it has not been already added
     void add(const StorageID & table_id, StorageActionBlockType action_type);
     void add(const StoragePtr & table, StorageActionBlockType action_type);
 
-    /// Remove locks for all tables
-    void remove(StorageActionBlockType action_type);
     /// Removes a lock for a table if it exists
     void remove(const StorageID & table_id, StorageActionBlockType action_type);
     void remove(const StoragePtr & table, StorageActionBlockType action_type);
