@@ -1,25 +1,20 @@
-#include "ExternalLoader.h"
+#include <Interpreters/ExternalLoader.h>
 
 #include <mutex>
-#include <pcg_random.hpp>
-#include <Common/Config/AbstractConfigurationComparison.h>
-#include <Common/Exception.h>
-#include <Common/StringUtils/StringUtils.h>
-#include <Common/ThreadPool.h>
-#include <Common/randomSeed.h>
-#include <Common/setThreadName.h>
-#include <Common/StatusInfo.h>
+#include <unordered_set>
 #include <base/chrono_io.h>
-#include <base/scope_guard_safe.h>
 #include <boost/range/adaptor/map.hpp>
 #include <boost/range/algorithm/copy.hpp>
-#include <unordered_set>
-
-
-namespace CurrentStatusInfo
-{
-    extern const Status DictionaryStatus;
-}
+#include <Common/Config/AbstractConfigurationComparison.h>
+#include <Common/CurrentThread.h>
+#include <Common/Exception.h>
+#include <Common/MemoryTrackerBlockerInThread.h>
+#include <Common/StringUtils.h>
+#include <Common/ThreadPool.h>
+#include <Common/logger_useful.h>
+#include <Common/randomSeed.h>
+#include <Common/scope_guard_safe.h>
+#include <Common/setThreadName.h>
 
 
 namespace DB
@@ -56,7 +51,7 @@ namespace
             static_assert(std::is_same_v<ReturnType, ExternalLoader::Loadables>);
             ExternalLoader::Loadables objects;
             objects.reserve(results.size());
-            for (const auto & result : results)
+            for (auto && result : results)
             {
                 if (auto object = std::move(result.object))
                     objects.push_back(std::move(object));
@@ -101,10 +96,7 @@ namespace
 class ExternalLoader::LoadablesConfigReader : private boost::noncopyable
 {
 public:
-    LoadablesConfigReader(const String & type_name_, Poco::Logger * log_)
-        : type_name(type_name_), log(log_)
-    {
-    }
+    LoadablesConfigReader(const String & type_name_, LoggerPtr log_) : type_name(type_name_), log(log_) { }
     ~LoadablesConfigReader() = default;
 
     using Repository = IExternalLoaderConfigRepository;
@@ -173,7 +165,7 @@ public:
 private:
     struct FileInfo
     {
-        Poco::Timestamp last_update_time = 0;
+        std::optional<Poco::Timestamp> last_update_time;
         bool in_use = true; // Whether the `FileInfo` should be destroyed because the correspondent file is deleted.
         Poco::AutoPtr<Poco::Util::AbstractConfiguration> file_contents; // Parsed contents of the file.
         std::unordered_map<String /* object name */, String /* key in file_contents */> objects;
@@ -272,7 +264,7 @@ private:
             // is updated, but in the same second).
             // The solution to this is probably switching to std::filesystem
             // -- the work is underway to do so.
-            if (update_time_from_repository == file_info.last_update_time)
+            if (update_time_from_repository && (update_time_from_repository == file_info.last_update_time))
             {
                 file_info.in_use = true;
                 return false;
@@ -383,7 +375,7 @@ private:
     }
 
     const String type_name;
-    Poco::Logger * log;
+    const LoggerPtr log;
 
     std::mutex mutex;
     ExternalLoaderConfigSettings settings;
@@ -400,17 +392,8 @@ private:
 class ExternalLoader::LoadingDispatcher : private boost::noncopyable
 {
 public:
-    /// Called to load or reload an object.
-    using CreateObjectFunction = std::function<LoadablePtr(
-        const String & /* name */, const ObjectConfig & /* config */, const LoadablePtr & /* previous_version */)>;
-
-    LoadingDispatcher(
-        const CreateObjectFunction & create_object_function_,
-        const String & type_name_,
-        Poco::Logger * log_)
-        : create_object(create_object_function_)
-        , type_name(type_name_)
-        , log(log_)
+    LoadingDispatcher(const String & type_name_, LoggerPtr log_, const ExternalLoader & external_loader_)
+        : type_name(type_name_), log(log_), external_loader(external_loader_)
     {
     }
 
@@ -463,16 +446,25 @@ public:
             else
             {
                 const auto & new_config = new_config_it->second;
-                bool config_is_same = isSameConfiguration(*info.config->config, info.config->key_in_config, *new_config->config, new_config->key_in_config);
+                auto previous_config = info.config;
                 info.config = new_config;
-                if (!config_is_same)
+
+                bool config_changed = !isSameConfiguration(*previous_config->config, previous_config->key_in_config, *new_config->config, new_config->key_in_config);
+                if (config_changed)
                 {
+                    if (info.object)
+                        external_loader.updateObjectFromConfigWithoutReloading(*info.object, *new_config->config, new_config->key_in_config);
+
                     if (info.triedToLoad())
                     {
-                        /// The object has been tried to load before, so it is currently in use or was in use
-                        /// and we should try to reload it with the new config.
-                        LOG_TRACE(log, "Will reload '{}' because its configuration has been changed and there were attempts to load it before", name);
-                        startLoading(info, true);
+                        bool config_change_requires_reloading = external_loader.doesConfigChangeRequiresReloadingObject(*previous_config->config, previous_config->key_in_config, *new_config->config, new_config->key_in_config);
+                        if (config_change_requires_reloading)
+                        {
+                            /// The object has been tried to load before, so it is currently in use or was in use
+                            /// and we should try to reload it with the new config.
+                            LOG_TRACE(log, "Will reload '{}' because its configuration has been changed and there were attempts to load it before", name);
+                            startLoading(info, true);
+                        }
                     }
                 }
             }
@@ -481,7 +473,7 @@ public:
         /// Insert to the map those objects which added to the new configuration.
         for (const auto & [name, config] : new_configs->configs_by_name)
         {
-            if (infos.find(name) == infos.end())
+            if (!infos.contains(name))
             {
                 Info & info = infos.emplace(name, Info{name, config}).first->second;
                 if (always_load_everything)
@@ -714,7 +706,10 @@ public:
                         /// Object was never loaded successfully and should be reloaded.
                         startLoading(info);
                     }
-                    LOG_TRACE(log, "Object '{}' is neither loaded nor failed, so it will not be reloaded as outdated.", info.name);
+                    else
+                    {
+                        LOG_TRACE(log, "Object '{}' is neither loaded nor failed, so it will not be reloaded as outdated.", info.name);
+                    }
                 }
             }
         }
@@ -736,10 +731,9 @@ private:
         {
             if (object)
                 return isLoading() ? Status::LOADED_AND_RELOADING : Status::LOADED;
-            else if (exception)
+            if (exception)
                 return isLoading() ? Status::FAILED_AND_RELOADING : Status::FAILED;
-            else
-                return isLoading() ? Status::LOADING : Status::NOT_LOADED;
+            return isLoading() ? Status::LOADING : Status::NOT_LOADED;
         }
 
         Duration loadingDuration() const
@@ -761,6 +755,7 @@ private:
                 result.exception = exception;
                 result.loading_start_time = loading_start_time;
                 result.last_successful_update_time = last_successful_update_time;
+                result.error_count = error_count;
                 result.loading_duration = loadingDuration();
                 result.config = config;
                 return result;
@@ -773,7 +768,7 @@ private:
         }
 
         String name;
-        LoadablePtr object;
+        LoadableMutablePtr object;
         std::shared_ptr<const ObjectConfig> config;
         TimePoint loading_start_time;
         TimePoint loading_end_time;
@@ -927,7 +922,16 @@ private:
         if (enable_async_loading)
         {
             /// Put a job to the thread pool for the loading.
-            auto thread = ThreadFromGlobalPool{&LoadingDispatcher::doLoading, this, info.name, loading_id, forced_to_reload, min_id_to_finish_loading_dependencies_, true, CurrentThread::getGroup()};
+            ThreadFromGlobalPool thread;
+            try
+            {
+                thread = ThreadFromGlobalPool{&LoadingDispatcher::doLoading, this, info.name, loading_id, forced_to_reload, min_id_to_finish_loading_dependencies_, true, CurrentThread::getGroup()};
+            }
+            catch (...)
+            {
+                cancelLoading(info);
+                throw;
+            }
             loading_threads.try_emplace(loading_id, std::move(thread));
         }
         else
@@ -964,15 +968,12 @@ private:
     }
 
     /// Does the loading, possibly in the separate thread.
-    void doLoading(const String & name, size_t loading_id, bool forced_to_reload, size_t min_id_to_finish_loading_dependencies_, bool async, ThreadGroupStatusPtr thread_group = {})
+    void doLoading(const String & name, size_t loading_id, bool forced_to_reload, size_t min_id_to_finish_loading_dependencies_, bool async, ThreadGroupPtr thread_group = {})
     {
-        SCOPE_EXIT_SAFE(
-            if (thread_group)
-                CurrentThread::detachQueryIfNotDetached();
-        );
+        ThreadGroupSwitcher switcher(thread_group, ThreadName::EXTERNAL_LOADER);
 
-        if (thread_group)
-            CurrentThread::attachTo(thread_group);
+        /// Do not account memory that was occupied by the dictionaries for the query/user context.
+        MemoryTrackerBlockerInThread memory_blocker;
 
         LOG_TRACE(log, "Start loading object '{}'", name);
         try
@@ -997,7 +998,7 @@ private:
             /// Loading.
             auto [new_object, new_exception] = loadSingleObject(name, *info->config, previous_version_as_base_for_loading);
             if (!new_object && !new_exception)
-                throw Exception("No object created and no exception raised for " + type_name, ErrorCodes::LOGICAL_ERROR);
+                throw Exception(ErrorCodes::LOGICAL_ERROR, "No object created and no exception raised for {}", type_name);
 
             /// Saving the result of the loading.
             {
@@ -1030,17 +1031,17 @@ private:
     }
 
     /// Load one object, returns object ptr or exception.
-    std::pair<LoadablePtr, std::exception_ptr>
+    std::pair<LoadableMutablePtr, std::exception_ptr>
     loadSingleObject(const String & name, const ObjectConfig & config, LoadablePtr previous_version)
     {
         /// Use `create_function` to perform the actual loading.
         /// It's much better to do it with `mutex` unlocked because the loading can take a lot of time
         /// and require access to other objects.
-        LoadablePtr new_object;
+        LoadableMutablePtr new_object;
         std::exception_ptr new_exception;
         try
         {
-            new_object = create_object(name, config, previous_version);
+            new_object = external_loader.createOrCloneObject(name, config, previous_version);
         }
         catch (...)
         {
@@ -1054,7 +1055,7 @@ private:
         const String & name,
         size_t loading_id,
         LoadablePtr previous_version,
-        LoadablePtr new_object,
+        LoadableMutablePtr new_object,
         std::exception_ptr new_exception,
         size_t error_count,
         const LoadingGuardForAsyncLoad &)
@@ -1117,7 +1118,10 @@ private:
         }
 
         if (new_object)
+        {
+            external_loader.updateObjectFromConfigWithoutReloading(*new_object, *info->config->config, info->config->key_in_config);
             info->object = new_object;
+        }
 
         info->exception = new_exception;
         info->error_count = error_count;
@@ -1137,7 +1141,6 @@ private:
         if (info && (info->loading_id == loading_id))
         {
             info->loading_id = info->state_id;
-            CurrentStatusInfo::set(CurrentStatusInfo::DictionaryStatus, name, static_cast<Int8>(info->status()));
         }
         min_id_to_finish_loading_dependencies.erase(std::this_thread::get_id());
 
@@ -1184,17 +1187,15 @@ private:
             LOG_TRACE(log, "Supposed update time for '{}' is {} (backoff, {} errors)", loaded_object->getLoadableName(), to_string(result), error_count);
             return result;
         }
-        else
-        {
-            auto result = std::chrono::system_clock::now() + std::chrono::seconds(calculateDurationWithBackoff(rnd_engine, error_count));
-            LOG_TRACE(log, "Supposed update time for unspecified object is {} (backoff, {} errors.", to_string(result), error_count);
-            return result;
-        }
+
+        auto result = std::chrono::system_clock::now() + std::chrono::seconds(calculateDurationWithBackoff(rnd_engine, error_count));
+        LOG_TRACE(log, "Supposed update time for unspecified object is {} (backoff, {} errors)", to_string(result), error_count);
+        return result;
     }
 
-    const CreateObjectFunction create_object;
     const String type_name;
-    Poco::Logger * log;
+    const LoggerPtr log;
+    const ExternalLoader & external_loader;
 
     mutable std::mutex mutex;
     std::condition_variable event;
@@ -1215,8 +1216,8 @@ class ExternalLoader::PeriodicUpdater : private boost::noncopyable
 public:
     static constexpr UInt64 check_period_sec = 5;
 
-    PeriodicUpdater(LoadablesConfigReader & config_files_reader_, LoadingDispatcher & loading_dispatcher_)
-        : config_files_reader(config_files_reader_), loading_dispatcher(loading_dispatcher_)
+    PeriodicUpdater(LoadablesConfigReader & config_files_reader_, LoadingDispatcher & loading_dispatcher_, LoggerPtr log_)
+        : config_files_reader(config_files_reader_), loading_dispatcher(loading_dispatcher_), log(log_)
     {
     }
 
@@ -1227,12 +1228,22 @@ public:
         std::unique_lock lock{mutex};
         enabled = enable_;
 
+        LOG_DEBUG(log, "Periodic updates {}", enabled ? "enabled" : "disabled");
+
         if (enable_)
         {
             if (!thread.joinable())
             {
-                /// Starts the thread which will do periodic updates.
-                thread = ThreadFromGlobalPool{&PeriodicUpdater::doPeriodicUpdates, this};
+                try
+                {
+                    /// Starts the thread which will do periodic updates.
+                    thread = ThreadFromGlobalPool{&PeriodicUpdater::doPeriodicUpdates, this};
+                }
+                catch (Exception & e)
+                {
+                    e.addMessage("while enabling periodic updates");
+                    throw;
+                }
             }
         }
         else
@@ -1252,21 +1263,35 @@ public:
 private:
     void doPeriodicUpdates()
     {
-        setThreadName("ExterLdrReload");
+        DB::setThreadName(ThreadName::EXTERNAL_LOADER);
+
+        LOG_DEBUG(log, "Starting periodic updates");
+        SCOPE_EXIT_SAFE({
+            LOG_DEBUG(log, "Stopped periodic updates (enabled: {})", enabled);
+        });
 
         std::unique_lock lock{mutex};
         auto pred = [this] { return !enabled; };
         while (!event.wait_for(lock, std::chrono::seconds(check_period_sec), pred))
         {
             lock.unlock();
-            loading_dispatcher.setConfiguration(config_files_reader.read());
-            loading_dispatcher.reloadOutdated();
+            try
+            {
+                loading_dispatcher.setConfiguration(config_files_reader.read());
+                loading_dispatcher.reloadOutdated();
+            }
+            catch (...)
+            {
+                LOG_ERROR(log, "Received uncaught exception: {}", getCurrentExceptionMessage(true));
+            }
             lock.lock();
         }
     }
 
     LoadablesConfigReader & config_files_reader;
     LoadingDispatcher & loading_dispatcher;
+    LoggerPtr log;
+
     mutable std::mutex mutex;
     bool enabled = false;
     ThreadFromGlobalPool thread;
@@ -1274,13 +1299,10 @@ private:
 };
 
 
-ExternalLoader::ExternalLoader(const String & type_name_, Poco::Logger * log_)
+ExternalLoader::ExternalLoader(const String & type_name_, LoggerPtr log_)
     : config_files_reader(std::make_unique<LoadablesConfigReader>(type_name_, log_))
-    , loading_dispatcher(std::make_unique<LoadingDispatcher>(
-          [this](auto && a, auto && b, auto && c) { return createObject(a, b, c); },
-          type_name_,
-          log_))
-    , periodic_updater(std::make_unique<PeriodicUpdater>(*config_files_reader, *loading_dispatcher))
+    , loading_dispatcher(std::make_unique<LoadingDispatcher>(type_name_, log_, *this))
+    , periodic_updater(std::make_unique<PeriodicUpdater>(*config_files_reader, *loading_dispatcher, log_))
     , type_name(type_name_)
     , log(log_)
 {
@@ -1293,8 +1315,18 @@ scope_guard ExternalLoader::addConfigRepository(std::unique_ptr<IExternalLoaderC
     auto * ptr = repository.get();
     String name = ptr->getName();
 
-    config_files_reader->addConfigRepository(std::move(repository));
-    reloadConfig(name);
+    /// Avoid leaving dangling repository in case of reloadConfig() fails
+    /// (it can be possible in case of CANNOT_SCHEDULE_TASK)
+    try
+    {
+        config_files_reader->addConfigRepository(std::move(repository));
+        reloadConfig(name);
+    }
+    catch (...)
+    {
+        config_files_reader->removeConfigRepository(ptr);
+        throw;
+    }
 
     return [this, ptr, name]()
     {
@@ -1436,7 +1468,7 @@ void ExternalLoader::checkLoaded(const ExternalLoader::LoadResult & result,
     if (result.object && (!check_no_errors || !result.exception))
         return;
     if (result.status == ExternalLoader::Status::LOADING)
-        throw Exception(type_name + " '" + result.name + "' is still loading", ErrorCodes::BAD_ARGUMENTS);
+        throw Exception(ErrorCodes::BAD_ARGUMENTS, "{} '{}' is still loading", type_name, result.name);
     if (result.exception)
     {
         // Exception is shared for multiple threads.
@@ -1462,9 +1494,9 @@ void ExternalLoader::checkLoaded(const ExternalLoader::LoadResult & result,
         }
     }
     if (result.status == ExternalLoader::Status::NOT_EXIST)
-        throw Exception(type_name + " '" + result.name + "' not found", ErrorCodes::BAD_ARGUMENTS);
+        throw Exception(ErrorCodes::BAD_ARGUMENTS, "{} '{}' not found", type_name, result.name);
     if (result.status == ExternalLoader::Status::NOT_LOADED)
-        throw Exception(type_name + " '" + result.name + "' not tried to load", ErrorCodes::BAD_ARGUMENTS);
+        throw Exception(ErrorCodes::BAD_ARGUMENTS, "{} '{}' not tried to load", type_name, result.name);
 }
 
 void ExternalLoader::checkLoaded(const ExternalLoader::LoadResults & results,
@@ -1506,13 +1538,13 @@ void ExternalLoader::reloadConfig(const String & repository_name, const String &
     loading_dispatcher->setConfiguration(config_files_reader->read(repository_name, path));
 }
 
-ExternalLoader::LoadablePtr ExternalLoader::createObject(
+ExternalLoader::LoadableMutablePtr ExternalLoader::createOrCloneObject(
     const String & name, const ObjectConfig & config, const LoadablePtr & previous_version) const
 {
     if (previous_version)
         return previous_version->clone();
 
-    return create(name, *config.config, config.key_in_config, config.repository_name);
+    return createObject(name, *config.config, config.key_in_config, config.repository_name);
 }
 
 template ExternalLoader::LoadablePtr ExternalLoader::getLoadResult<ExternalLoader::LoadablePtr>(const String &) const;
