@@ -1,6 +1,5 @@
 #pragma once
 
-#include <Core/Block.h>
 #include <Interpreters/IInterpreter.h>
 #include <Parsers/IAST_fwd.h>
 
@@ -18,6 +17,8 @@ public:
 
 private:
     ASTPtr query_ptr;
+
+    LoggerPtr log = getLogger("InterpreterCheckQuery");
 };
 
 }
