@@ -8,11 +8,6 @@
 #include <Common/NamedCollections/NamedCollectionsFactory.h>
 
 
-namespace CurrentMetrics
-{
-    extern const Metric NamedCollection;
-}
-
 namespace DB
 {
 
@@ -32,7 +27,6 @@ BlockIO InterpreterDropNamedCollectionQuery::execute()
     }
 
     NamedCollectionFactory::instance().removeFromSQL(query);
-    CurrentMetrics::sub(CurrentMetrics::NamedCollection);
 
     return {};
 }
