@@ -140,7 +140,8 @@ void InterpreterSelectIntersectExceptQuery::buildQueryPlan(QueryPlan & query_pla
             auto actions_dag = ActionsDAG::makeConvertingActions(
                     plans[i]->getCurrentHeader()->getColumnsWithTypeAndName(),
                     result_header->getColumnsWithTypeAndName(),
-                    ActionsDAG::MatchColumnsMode::Position);
+                    ActionsDAG::MatchColumnsMode::Position,
+                    context);
             auto converting_step = std::make_unique<ExpressionStep>(plans[i]->getCurrentHeader(), std::move(actions_dag));
             converting_step->setStepDescription("Conversion before UNION");
             plans[i]->addStep(std::move(converting_step));
