@@ -134,6 +134,8 @@ public:
     {
         UInt64 limit_length{0};
         UInt64 limit_offset{0};
+        Float64 fractional_limit{0};
+        Float64 fractional_offset{0};
         bool is_limit_length_negative{false};
         bool is_limit_offset_negative{false};
     };
