@@ -232,18 +232,20 @@ BlockIO InterpreterShowTablesQuery::execute()
     }
     auto rewritten_query = getRewrittenQuery();
     String database = getContext()->resolveDatabase(query.getFrom());
-    if (DatabaseCatalog::instance().isDatalakeCatalog(database))
+    if (query.databases || DatabaseCatalog::instance().isDatalakeCatalog(database))
     {
-        auto context_copy = Context::createCopy(getContext());
+        auto query_context = Context::createCopy(getContext());
+        query_context->makeQueryContext();
+        query_context->setCurrentQueryId("");
         /// HACK To always show them in explicit "SHOW TABLES" queries
-        context_copy->setSetting("show_data_lake_catalogs_in_system_tables", true);
-        return executeQuery(rewritten_query, context_copy, QueryFlags{ .internal = true }).second;
-    }
-    else
-    {
-        return executeQuery(rewritten_query, getContext(), QueryFlags{ .internal = true }).second;
+        query_context->setSetting("show_data_lake_catalogs_in_system_tables", true);
+        return executeQuery(rewritten_query, std::move(query_context), QueryFlags{ .internal = true }).second;
     }
 
+    auto query_context = Context::createCopy(getContext());
+    query_context->makeQueryContext();
+    query_context->setCurrentQueryId("");
+    return executeQuery(rewritten_query, std::move(query_context), QueryFlags{ .internal = true }).second;
 }
 
 /// (*) Sorting is strictly speaking not necessary but 1. it is convenient for users, 2. SQL currently does not allow to
