@@ -392,7 +392,7 @@ std::unique_ptr<llvm::Module> CHJIT::createModuleForCompilation()
 {
     std::unique_ptr<llvm::Module> module = std::make_unique<llvm::Module>("jit" + std::to_string(current_module_key), context);
     module->setDataLayout(layout);
-    module->setTargetTriple(machine->getTargetTriple().getTriple());
+    module->setTargetTriple(machine->getTargetTriple());
 
     return module;
 }
