@@ -74,6 +74,7 @@ struct QueryStatusInfo
     bool is_cancelled;
     CancelReason cancel_reason;
     bool is_all_data_sent;
+    bool is_internal;
 
     /// Optional fields, filled by query
     std::vector<UInt64> thread_ids;
@@ -190,6 +191,8 @@ protected:
     /// This field is unused in this class, but it
     /// increments/decrements metric in constructor/destructor.
     CurrentMetrics::Increment num_queries_increment;
+
+    bool is_internal;
 public:
     QueryStatus(
         ContextPtr context_,
@@ -201,7 +204,8 @@ public:
         ThreadGroupPtr && thread_group_,
         IAST::QueryKind query_kind_,
         const Settings & query_settings_,
-        UInt64 watch_start_nanoseconds);
+        UInt64 watch_start_nanoseconds,
+        bool is_internal);
 
     ~QueryStatus();
 
@@ -276,6 +280,14 @@ public:
 
     /// Get the reference for the start of the query. Used to synchronize with other Stopwatches
     UInt64 getQueryCPUStartTime() { return watch.getStart(); }
+
+    bool isInternal() const
+    {
+        return is_internal;
+    }
+
+    /// Manually release query slot (if any).
+    void releaseQuerySlot() { query_slot.reset(); }
 };
 
 using QueryStatusPtr = std::shared_ptr<QueryStatus>;
@@ -302,6 +314,7 @@ struct ProcessListForUser
     /// query_id -> ProcessListElement(s). There can be multiple queries with the same query_id as long as all queries except one are cancelled.
     using QueryToElement = std::unordered_map<String, QueryStatusPtr>;
     QueryToElement queries;
+    size_t non_internal_queries = 0;
 
     ProfileEvents::Counters user_performance_counters{VariableContext::User, &ProfileEvents::global_counters};
     /// Limit and counter for memory of all simultaneously running queries of single user.
@@ -314,9 +327,6 @@ struct ProcessListForUser
     /// Count network usage for all simultaneously running queries of single user.
     ThrottlerPtr user_throttler;
 
-    /// Number of queries waiting on load jobs
-    std::atomic<UInt64> waiting_queries_amount{0};
-
     ProcessListForUserInfo getInfo(bool get_profile_events = false) const;
 
     /// Clears MemoryTracker for the user.
@@ -391,6 +401,8 @@ protected:
 
     /// List of queries
     Container processes;
+    size_t non_internal_processes = 0;
+
     /// Notify about cancelled queries (done with ProcessListBase::mutex acquired).
     mutable std::condition_variable cancelled_cv;
 
@@ -423,24 +435,17 @@ protected:
     /// timeout in millisecond for low priority query to wait
     size_t low_priority_query_wait_time_ms = 0;
 
-    /// amount of queries by query kind.
+    /// amount of queries by query kind, excludes internal queries
     QueryKindAmounts query_kind_amounts;
 
     /// limit for waiting queries. 0 means no limit. Otherwise, when limit exceeded, an exception is thrown.
     std::atomic<UInt64> max_waiting_queries_amount{0};
 
-    /// amounts of waiting queries
-    std::atomic<UInt64> waiting_queries_amount{0};
-    std::atomic<UInt64> waiting_insert_queries_amount{0};
-    std::atomic<UInt64> waiting_select_queries_amount{0};
-
+    /// WARNING: for non-internal queries only
     void increaseQueryKindAmount(const IAST::QueryKind & query_kind);
     void decreaseQueryKindAmount(const IAST::QueryKind & query_kind);
     QueryAmount getQueryKindAmount(const IAST::QueryKind & query_kind) const;
 
-    void increaseWaitingQueryAmount(const QueryStatusPtr & status);
-    void decreaseWaitingQueryAmount(const QueryStatusPtr & status);
-
 public:
     using EntryPtr = std::shared_ptr<ProcessListEntry>;
 
@@ -449,9 +454,10 @@ public:
       * If timeout is passed - throw an exception.
       * Don't count KILL QUERY queries or async insert flush queries
       */
-    EntryPtr insert(const String & query_, UInt64 normalized_query_hash, const IAST * ast, ContextMutablePtr query_context, UInt64 watch_start_nanoseconds);
+    EntryPtr insert(const String & query_, UInt64 normalized_query_hash, const IAST * ast, ContextMutablePtr query_context, UInt64 watch_start_nanoseconds, bool is_internal);
 
     /// Number of currently executing queries.
+    /// WARNING: includes internal queries (e.g. those executed by dictionaries, RMVs, async inserts).
     size_t size() const { return processes.size(); }
 
     /// Get current state of process list.
@@ -527,10 +533,6 @@ public:
         return max_waiting_queries_amount.load();
     }
 
-    // Handlers for AsyncLoader waiters
-    void incrementWaiters();
-    void decrementWaiters();
-
     /// Try call cancel() for input and output streams of query with specified id and user
     CancellationCode sendCancelToQuery(const String & current_query_id, const String & current_user);
     CancellationCode sendCancelToQuery(QueryStatusPtr elem);
