@@ -2,15 +2,11 @@
 
 #include <Interpreters/SystemLog.h>
 #include <Interpreters/ClientInfo.h>
+#include <Common/setThreadName.h>
+#include <Common/ProfileEvents.h>
 #include <Core/NamesAndTypes.h>
 #include <Core/NamesAndAliases.h>
-
-
-namespace ProfileEvents
-{
-    class Counters;
-}
-
+#include <Storages/ColumnsDescription.h>
 
 namespace DB
 {
@@ -37,7 +33,7 @@ struct QueryThreadLogElement
     Int64 memory_usage{};
     Int64 peak_memory_usage{};
 
-    String thread_name;
+    ThreadName thread_name;
     UInt64 thread_id{};
     UInt64 master_thread_id{};
 
@@ -51,7 +47,7 @@ struct QueryThreadLogElement
 
     static std::string name() { return "QueryThreadLog"; }
 
-    static NamesAndTypesList getNamesAndTypes();
+    static ColumnsDescription getColumnsDescription();
     static NamesAndAliases getNamesAndAliases();
     void appendToBlock(MutableColumns & columns) const;
 };
@@ -64,5 +60,3 @@ class QueryThreadLog : public SystemLog<QueryThreadLogElement>
 
 
 }
-
-
