@@ -92,7 +92,7 @@ public:
     SystemLogs(ContextPtr global_context, const Poco::Util::AbstractConfiguration & config);
     SystemLogs(const SystemLogs & other) = default;
 
-    void flush(const Strings & names);
+    void flush(const std::vector<std::pair<String, String>> & names);
     void flushAndShutdown();
     void shutdown();
     void handleCrash();
@@ -109,7 +109,7 @@ public:
 private:
     std::vector<ISystemLog *> getAllLogs() const;
 
-    void flushImpl(const Strings & names, bool should_prepare_tables_anyway, bool ignore_errors);
+    void flushImpl(const std::vector<std::pair<String, String>>  & names, bool should_prepare_tables_anyway, bool ignore_errors);
 };
 
 struct SystemLogSettings
