@@ -717,7 +717,8 @@ static void mergeDags(std::optional<ActionsDAG> & result_dag, std::optional<Acti
 std::pair<std::optional<ActionsDAG>, std::optional<ActionsDAG>>
 TableJoin::createConvertingActions(
     const ColumnsWithTypeAndName & left_sample_columns,
-    const ColumnsWithTypeAndName & right_sample_columns)
+    const ColumnsWithTypeAndName & right_sample_columns,
+    const ContextPtr & context)
 {
     std::optional<ActionsDAG> left_dag;
     std::optional<ActionsDAG> right_dag;
@@ -738,8 +739,8 @@ TableJoin::createConvertingActions(
     inferJoinKeyCommonType(left_sample_columns, right_sample_columns, !isSpecialStorage(), require_strict_keys_match);
     if (!left_type_map.empty() || !right_type_map.empty())
     {
-        left_dag = applyKeyConvertToTable(left_sample_columns, left_type_map, JoinTableSide::Left, left_column_rename);
-        right_dag = applyKeyConvertToTable(right_sample_columns, right_type_map, JoinTableSide::Right, right_column_rename);
+        left_dag = applyKeyConvertToTable(left_sample_columns, left_type_map, JoinTableSide::Left, left_column_rename, context);
+        right_dag = applyKeyConvertToTable(right_sample_columns, right_type_map, JoinTableSide::Right, right_column_rename, context);
     }
 
     /**
@@ -776,7 +777,8 @@ TableJoin::createConvertingActions(
     {
         auto new_left_dag = applyJoinUseNullsConversion(
             left_dag ? left_dag->getResultColumns() : left_sample_columns,
-            left_column_rename);
+            left_column_rename,
+            context);
         mergeDags(left_dag, std::move(new_left_dag));
     }
 
@@ -784,7 +786,8 @@ TableJoin::createConvertingActions(
     {
         auto new_right_dag = applyJoinUseNullsConversion(
             right_dag ? right_dag->getResultColumns() : right_sample_columns,
-            right_column_rename);
+            right_column_rename,
+            context);
         mergeDags(right_dag, std::move(new_right_dag));
     }
 
@@ -883,7 +886,8 @@ void TableJoin::inferJoinKeyCommonType(const LeftNamesAndTypes & left, const Rig
 static std::optional<ActionsDAG> changeKeyTypes(const ColumnsWithTypeAndName & cols_src,
                                     const TableJoin::NameToTypeMap & type_mapping,
                                     bool add_new_cols,
-                                    NameToNameMap & key_column_rename)
+                                    NameToNameMap & key_column_rename,
+                                    const ContextPtr & context)
 {
     ColumnsWithTypeAndName cols_dst = cols_src;
     bool has_some_to_do = false;
@@ -903,6 +907,7 @@ static std::optional<ActionsDAG> changeKeyTypes(const ColumnsWithTypeAndName & c
         /* source= */ cols_src,
         /* result= */ cols_dst,
         /* mode= */ ActionsDAG::MatchColumnsMode::Name,
+        /* context= */ context,
         /* ignore_constant_values= */ true,
         /* add_cast_columns= */ add_new_cols,
         /* new_names= */ &key_column_rename);
@@ -910,7 +915,8 @@ static std::optional<ActionsDAG> changeKeyTypes(const ColumnsWithTypeAndName & c
 
 static std::optional<ActionsDAG> changeTypesToNullable(
     const ColumnsWithTypeAndName & cols_src,
-    const NameSet & exception_cols)
+    const NameSet & exception_cols,
+    const ContextPtr & context)
 {
     ColumnsWithTypeAndName cols_dst = cols_src;
     bool has_some_to_do = false;
@@ -930,6 +936,7 @@ static std::optional<ActionsDAG> changeTypesToNullable(
         /* source= */ cols_src,
         /* result= */ cols_dst,
         /* mode= */ ActionsDAG::MatchColumnsMode::Name,
+        /* context= */ context,
         /* ignore_constant_values= */ true,
         /* add_cast_columns= */ false,
         /* new_names= */ nullptr);
@@ -939,13 +946,14 @@ std::optional<ActionsDAG> TableJoin::applyKeyConvertToTable(
     const ColumnsWithTypeAndName & cols_src,
     const NameToTypeMap & type_mapping,
     JoinTableSide table_side,
-    NameToNameMap & key_column_rename)
+    NameToNameMap & key_column_rename,
+    const ContextPtr & context)
 {
     if (type_mapping.empty())
         return {};
 
     /// Create DAG to convert key columns
-    auto convert_dag = changeKeyTypes(cols_src, type_mapping, !hasUsing(), key_column_rename);
+    auto convert_dag = changeKeyTypes(cols_src, type_mapping, !hasUsing(), key_column_rename, context);
     applyRename(table_side, key_column_rename);
     return convert_dag;
 }
@@ -988,7 +996,8 @@ std::optional<ActionsDAG> TableJoin::applyNullsafeWrapper(
 
 std::optional<ActionsDAG> TableJoin::applyJoinUseNullsConversion(
     const ColumnsWithTypeAndName & cols_src,
-    const NameToNameMap & key_column_rename)
+    const NameToNameMap & key_column_rename,
+    const ContextPtr & context)
 {
     /// Do not need to make nullable temporary columns that would be used only as join keys, but is not visible to user
     NameSet exclude_columns;
@@ -996,7 +1005,7 @@ std::optional<ActionsDAG> TableJoin::applyJoinUseNullsConversion(
         exclude_columns.insert(it.second);
 
     /// Create DAG to make columns nullable if needed
-    return changeTypesToNullable(cols_src, exclude_columns);
+    return changeTypesToNullable(cols_src, exclude_columns, context);
 }
 
 void TableJoin::setStorageJoin(std::shared_ptr<const IKeyValueEntity> storage)
@@ -1182,7 +1191,8 @@ bool allowParallelHashJoin(
 {
     if (std::ranges::none_of(join_algorithms, [](auto algo) { return algo == JoinAlgorithm::PARALLEL_HASH; }))
         return false;
-    if (kind != JoinKind::Left && kind != JoinKind::Inner)
+    if (kind != JoinKind::Left && kind != JoinKind::Inner
+        && kind != JoinKind::Right && kind != JoinKind::Full)
         return false;
     if (strictness == JoinStrictness::Asof)
         return false;
