@@ -103,7 +103,7 @@ void TraceCollector::run()
 {
     [[maybe_unused]] MemoryTrackerDebugBlockerInThread blocker;
 
-    setThreadName("TraceCollector");
+    DB::setThreadName(ThreadName::TRACE_COLLECTOR);
 
     MemoryTrackerBlockerInThread untrack_lock(VariableContext::Global);
     ReadBufferFromFileDescriptor in(TraceSender::pipe.fds_rw[0]);
@@ -154,6 +154,9 @@ void TraceCollector::run()
             UInt64 thread_id;
             readPODBinary(thread_id, in);
 
+            UInt8 thread_name_id = 0;
+            readPODBinary(thread_name_id, in);
+
             Int64 size;
             readPODBinary(size, in);
 
@@ -188,6 +191,7 @@ void TraceCollector::run()
                     .timestamp_ns = timestamp_ns,
                     .trace_type = trace_type,
                     .thread_id = thread_id,
+                    .thread_name = static_cast<ThreadName>(thread_name_id),
                     .query_id = query_id,
                     .trace = std::move(trace),
                     .size = size,
