@@ -662,50 +662,6 @@ Field convertFieldToTypeImpl(const Field & src, const IDataType & type, const ID
 
         return src;
     }
-    else if (isObjectDeprecated(type))
-    {
-        if (src.getType() == Field::Types::Object)
-            return src; /// Already in needed type.
-
-        const auto * from_type_tuple = typeid_cast<const DataTypeTuple *>(from_type_hint);
-        if (src.getType() == Field::Types::Tuple && from_type_tuple && from_type_tuple->hasExplicitNames())
-        {
-            const auto & names = from_type_tuple->getElementNames();
-            const auto & tuple = src.safeGet<Tuple>();
-
-            if (names.size() != tuple.size())
-                throw Exception(
-                    ErrorCodes::TYPE_MISMATCH,
-                    "Bad size of tuple in IN or VALUES section (while converting to Object). Expected size: {}, actual size: {}",
-                    names.size(),
-                    tuple.size());
-
-            Object object;
-            for (size_t i = 0; i < names.size(); ++i)
-                object[names[i]] = tuple[i];
-
-            return object;
-        }
-
-        if (src.getType() == Field::Types::Map)
-        {
-            Object object;
-            const auto & map = src.safeGet<Map>();
-            for (const auto & element : map)
-            {
-                const auto & map_entry = element.safeGet<Tuple>();
-                const auto & key = map_entry[0];
-                const auto & value = map_entry[1];
-
-                if (key.getType() != Field::Types::String)
-                    throw Exception(ErrorCodes::TYPE_MISMATCH, "Cannot convert from Map with key of type {} to Object", key.getTypeName());
-
-                object[key.safeGet<String>()] = value;
-            }
-
-            return object;
-        }
-    }
     else if (const DataTypeVariant * type_variant = typeid_cast<const DataTypeVariant *>(&type))
     {
         if (src.isNull())
