@@ -19,7 +19,6 @@
 #include <DataTypes/DataTypeArray.h>
 #include <DataTypes/DataTypesNumber.h>
 #include <DataTypes/NestedUtils.h>
-#include <DataTypes/ObjectUtils.h>
 #include <Interpreters/RequiredSourceColumnsVisitor.h>
 #include <Storages/ColumnsDescription.h>
 #include <Storages/StorageInMemoryMetadata.h>
@@ -299,13 +298,14 @@ static std::unordered_map<String, ColumnPtr> collectOffsetsColumns(
         if (res_columns[i] == nullptr || isColumnConst(*res_columns[i]))
             continue;
 
-        auto serialization = IDataType::getSerialization(*available_column);
+        auto serialization_info = available_column->type->getSerializationInfo(*res_columns[i]);
+        auto serialization = available_column->type->getSerialization(serialization_info->getKindStack(), IDataType::getSerialization(*available_column));
         serialization->enumerateStreams([&](const auto & subpath)
         {
             if (subpath.empty() || subpath.back().type != ISerialization::Substream::ArraySizes)
                 return;
 
-            auto stream_name = ISerialization::getFileNameForStream(*available_column, subpath);
+            auto stream_name = ISerialization::getFileNameForStream(*available_column, subpath, {});
             const auto & current_offsets_column = subpath.back().data.column;
 
             /// If for some reason multiple offsets columns are present
@@ -424,10 +424,13 @@ void fillMissingColumns(
         const auto * array_type = typeid_cast<const DataTypeArray *>(requested_column->type.get());
         if (array_type && !offsets_columns.empty())
         {
-            num_dimensions = getNumberOfDimensions(*array_type);
+            num_dimensions = array_type->getNumberOfDimensions();
             current_offsets.resize(num_dimensions);
 
-            auto serialization = IDataType::getSerialization(*requested_column);
+            SerializationPtr serialization = IDataType::getSerialization(*requested_column);
+            if (res_columns[i])
+                serialization = requested_column->type->getSerialization(requested_column->type->getSerializationInfo(*res_columns[i])->getKindStack(), serialization);
+
             serialization->enumerateStreams([&](const auto & subpath)
             {
                 if (subpath.empty() || subpath.back().type != ISerialization::Substream::ArraySizes)
@@ -438,7 +441,7 @@ void fillMissingColumns(
                 if (level >= num_dimensions)
                     return;
 
-                auto stream_name = ISerialization::getFileNameForStream(*requested_column, subpath);
+                auto stream_name = ISerialization::getFileNameForStream(*requested_column, subpath, {});
                 auto it = offsets_columns.find(stream_name);
                 if (it != offsets_columns.end())
                     current_offsets[level] = it->second;
@@ -457,7 +460,9 @@ void fillMissingColumns(
         if (!current_offsets.empty())
         {
             Names tuple_elements;
-            auto serialization = IDataType::getSerialization(*requested_column);
+            SerializationPtr serialization = IDataType::getSerialization(*requested_column);
+            if (res_columns[i])
+                serialization = requested_column->type->getSerialization(requested_column->type->getSerializationInfo(*res_columns[i])->getKindStack(), serialization);
 
             /// For Nested columns collect names of tuple elements and skip them while getting the base type of array.
             IDataType::forEachSubcolumn([&](const auto & path, const auto &, const auto &)
