@@ -1,7 +1,6 @@
 #include <Parsers/ASTIdentifier.h>
 #include <Parsers/ASTQueryWithTableAndOutput.h>
-#include <Common/quoteString.h>
-#include <IO/Operators.h>
+#include <Parsers/IAST_erase.h>
 
 
 namespace DB
@@ -64,11 +63,5 @@ void ASTQueryWithTableAndOutput::cloneTableOptions(ASTQueryWithTableAndOutput &
         cloned.children.push_back(cloned.table);
     }
 }
-void ASTQueryWithTableAndOutput::formatHelper(const FormatSettings & settings, const char * name) const
-{
-    settings.ostr << (settings.hilite ? hilite_keyword : "") << name << " " << (settings.hilite ? hilite_none : "");
-    settings.ostr << (database ? backQuoteIfNeed(getDatabase()) + "." : "") << backQuoteIfNeed(getTable());
-}
 
 }
-
