@@ -1,6 +1,7 @@
 #pragma once
 
 #include <Parsers/ASTSelectQuery.h>
+#include <Parsers/ExpressionListParsers.h>
 
 
 namespace DB
@@ -13,21 +14,26 @@ public:
 
     ASTPtr clone() const override;
 
-    enum class Operator
+    enum class Operator : uint8_t
     {
         UNKNOWN,
-        INTERSECT,
-        EXCEPT
+        EXCEPT_ALL,
+        EXCEPT_DISTINCT,
+        INTERSECT_ALL,
+        INTERSECT_DISTINCT,
     };
 
-    void formatImpl(const FormatSettings & settings, FormatState & state, FormatStateStacked frame) const override;
-
-    virtual QueryKind getQueryKind() const override { return QueryKind::SelectIntersectExcept; }
+    QueryKind getQueryKind() const override { return QueryKind::Select; }
 
     ASTs getListOfSelects() const;
 
+    static const char * fromOperator(Operator op);
+
     /// Final operator after applying visitor.
     Operator final_operator = Operator::UNKNOWN;
+
+protected:
+    void formatImpl(WriteBuffer & ostr, const FormatSettings & settings, FormatState & state, FormatStateStacked frame) const override;
 };
 
 }
