@@ -37,20 +37,19 @@ public:
 
     ASTPtr clone() const override;
 
-    const ASTPtr ttl() const { return getExpression(ttl_expr_pos); }
-    const ASTPtr where() const { return getExpression(where_expr_pos); }
+    ASTPtr ttl() const { return getExpression(ttl_expr_pos); }
+    ASTPtr where() const { return getExpression(where_expr_pos); }
 
     void setTTL(ASTPtr && ast) { setExpression(ttl_expr_pos, std::forward<ASTPtr>(ast)); }
     void setWhere(ASTPtr && ast) { setExpression(where_expr_pos, std::forward<ASTPtr>(ast)); }
 
 protected:
-    void formatImpl(const FormatSettings & settings, FormatState & state, FormatStateStacked frame) const override;
+    void formatImpl(WriteBuffer & ostr, const FormatSettings & settings, FormatState & state, FormatStateStacked frame) const override;
 
 private:
     int ttl_expr_pos;
     int where_expr_pos;
 
-private:
     void setExpression(int & pos, ASTPtr && ast);
     ASTPtr getExpression(int pos, bool clone = false) const;
 };
