@@ -31,8 +31,10 @@ public:
 
     void replaceEmptyDatabase(const String & current_database);
 
+    QueryKind getQueryKind() const override { return QueryKind::Show; }
+
 protected:
-    void formatQueryImpl(const FormatSettings & settings, FormatState &, FormatStateStacked) const override;
+    void formatQueryImpl(WriteBuffer & ostr, const FormatSettings & settings, FormatState &, FormatStateStacked) const override;
 
 private:
     String getKeyword() const;
