@@ -25,7 +25,6 @@ namespace DB
     MR_MACROS(ALL_SETTINGS, "ALL SETTINGS") \
     MR_MACROS(ALTER_COLUMN, "ALTER COLUMN") \
     MR_MACROS(ALTER_DATABASE, "ALTER DATABASE") \
-    MR_MACROS(ALTER_LIVE_VIEW, "ALTER LIVE VIEW") \
     MR_MACROS(ALTER_POLICY, "ALTER POLICY") \
     MR_MACROS(ALTER_PROFILE, "ALTER PROFILE") \
     MR_MACROS(ALTER_PROFILES, "ALTER PROFILES") \
@@ -186,6 +185,7 @@ namespace DB
     MR_MACROS(EXCHANGE_DICTIONARIES, "EXCHANGE DICTIONARIES") \
     MR_MACROS(EXCHANGE_TABLES, "EXCHANGE TABLES") \
     MR_MACROS(EXECUTE, "EXECUTE") \
+    MR_MACROS(EXECUTE_AS, "EXECUTE AS") \
     MR_MACROS(EXISTS, "EXISTS") \
     MR_MACROS(EXPLAIN, "EXPLAIN") \
     MR_MACROS(EXPRESSION, "EXPRESSION") \
@@ -261,6 +261,7 @@ namespace DB
     MR_MACROS(INVOKER, "INVOKER") \
     MR_MACROS(IP, "IP") \
     MR_MACROS(IS_NOT_DISTINCT_FROM, "IS NOT DISTINCT FROM") \
+    MR_MACROS(IS_DISTINCT_FROM, "IS DISTINCT FROM") \
     MR_MACROS(IS_NOT_NULL, "IS NOT NULL") \
     MR_MACROS(IS_NULL, "IS NULL") \
     MR_MACROS(JOIN, "JOIN") \
