@@ -739,7 +739,9 @@ bool ParserCreateTableQuery::parseImpl(Pos & pos, ASTPtr & node, Expected & expe
     else
         return false;
 
-    if (!replace && !or_replace && s_temporary.ignore(pos, expected))
+    /// Support `REPLACE TEMPORARY TABLE t ...` and `CREATE OR REPLACE TEMPORARY TABLE t ...`,
+    /// but forbid wrong syntax `ATTACH TEMPORARY TABLE t`.
+    if (!attach && s_temporary.ignore(pos, expected))
     {
         is_temporary = true;
     }
@@ -993,133 +995,6 @@ bool ParserCreateTableQuery::parseImpl(Pos & pos, ASTPtr & node, Expected & expe
     return true;
 }
 
-bool ParserCreateLiveViewQuery::parseImpl(Pos & pos, ASTPtr & node, Expected & expected)
-{
-    ParserKeyword s_create(Keyword::CREATE);
-    ParserKeyword s_attach(Keyword::ATTACH);
-    ParserKeyword s_if_not_exists(Keyword::IF_NOT_EXISTS);
-    ParserCompoundIdentifier table_name_p(/*table_name_with_optional_uuid*/ true, /*allow_query_parameter*/ true);
-    ParserKeyword s_as(Keyword::AS);
-    ParserKeyword s_view(Keyword::VIEW);
-    ParserKeyword s_live(Keyword::LIVE);
-    ParserToken s_dot(TokenType::Dot);
-    ParserToken s_lparen(TokenType::OpeningRoundBracket);
-    ParserToken s_rparen(TokenType::ClosingRoundBracket);
-    ParserStorage storage_p{ParserStorage::TABLE_ENGINE};
-    ParserStorage storage_inner{ParserStorage::TABLE_ENGINE};
-    ParserTablePropertiesDeclarationList table_properties_p;
-    ParserSelectWithUnionQuery select_p;
-    ParserSQLSecurity sql_security_p;
-
-    ASTPtr table;
-    ASTPtr to_table;
-    ASTPtr columns_list;
-    ASTPtr as_database;
-    ASTPtr as_table;
-    ASTPtr select;
-    ASTPtr sql_security;
-
-    String cluster_str;
-    bool attach = false;
-    bool if_not_exists = false;
-
-    if (!s_create.ignore(pos, expected))
-    {
-        if (s_attach.ignore(pos, expected))
-            attach = true;
-        else
-            return false;
-    }
-
-    sql_security_p.parse(pos, sql_security, expected);
-
-    if (!s_live.ignore(pos, expected))
-        return false;
-
-    if (!s_view.ignore(pos, expected))
-       return false;
-
-    if (s_if_not_exists.ignore(pos, expected))
-       if_not_exists = true;
-
-    if (!table_name_p.parse(pos, table, expected))
-        return false;
-
-    if (ParserKeyword{Keyword::ON}.ignore(pos, expected))
-    {
-        if (!ASTQueryWithOnCluster::parse(pos, cluster_str, expected))
-            return false;
-    }
-
-    // TO [db.]table
-    if (ParserKeyword{Keyword::TO}.ignore(pos, expected))
-    {
-        if (!table_name_p.parse(pos, to_table, expected))
-            return false;
-    }
-
-    std::shared_ptr<ASTViewTargets> targets;
-    if (to_table)
-    {
-        targets = std::make_shared<ASTViewTargets>();
-        targets->setTableID(ViewTarget::To, to_table->as<ASTTableIdentifier>()->getTableId());
-    }
-
-    /// Optional - a list of columns can be specified. It must fully comply with SELECT.
-    if (s_lparen.ignore(pos, expected))
-    {
-        if (!table_properties_p.parse(pos, columns_list, expected))
-            return false;
-
-        if (!s_rparen.ignore(pos, expected))
-            return false;
-    }
-
-    if (!sql_security && !sql_security_p.parse(pos, sql_security, expected))
-        sql_security = std::make_shared<ASTSQLSecurity>();
-
-    /// AS SELECT ...
-    if (!s_as.ignore(pos, expected))
-        return false;
-
-    if (!select_p.parse(pos, select, expected))
-        return false;
-
-    auto comment = parseComment(pos, expected);
-
-    auto query = std::make_shared<ASTCreateQuery>();
-    node = query;
-
-    query->attach = attach;
-    query->if_not_exists = if_not_exists;
-    query->is_live_view = true;
-
-    auto * table_id = table->as<ASTTableIdentifier>();
-    query->database = table_id->getDatabase();
-    query->table = table_id->getTable();
-    query->uuid = table_id->uuid;
-    query->cluster = cluster_str;
-
-    if (query->database)
-        query->children.push_back(query->database);
-    if (query->table)
-        query->children.push_back(query->table);
-
-    query->set(query->columns_list, columns_list);
-
-    tryGetIdentifierNameInto(as_database, query->as_database);
-    tryGetIdentifierNameInto(as_table, query->as_table);
-    query->set(query->select, select);
-    query->set(query->targets, targets);
-
-    if (comment)
-        query->set(query->comment, comment);
-
-    if (sql_security)
-        query->sql_security = typeid_cast<std::shared_ptr<ASTSQLSecurity>>(sql_security);
-
-    return true;
-}
 
 bool ParserCreateWindowViewQuery::parseImpl(Pos & pos, ASTPtr & node, Expected & expected)
 {
@@ -1988,14 +1863,12 @@ bool ParserCreateQuery::parseImpl(Pos & pos, ASTPtr & node, Expected & expected)
     ParserCreateDatabaseQuery database_p;
     ParserCreateViewQuery view_p;
     ParserCreateDictionaryQuery dictionary_p;
-    ParserCreateLiveViewQuery live_view_p;
     ParserCreateWindowViewQuery window_view_p;
 
     return table_p.parse(pos, node, expected)
         || database_p.parse(pos, node, expected)
         || view_p.parse(pos, node, expected)
         || dictionary_p.parse(pos, node, expected)
-        || live_view_p.parse(pos, node, expected)
         || window_view_p.parse(pos, node, expected);
 }
 
