@@ -12,10 +12,16 @@ class ParserQueryWithOutput : public IParserBase
 {
 protected:
     const char * end;
+    bool allow_settings_after_format_in_insert;
+
     const char * getName() const override { return "Query with output"; }
     bool parseImpl(Pos & pos, ASTPtr & node, Expected & expected) override;
+
 public:
-    ParserQueryWithOutput(const char * end_) : end(end_) {}
+    explicit ParserQueryWithOutput(const char * end_, bool allow_settings_after_format_in_insert_ = false)
+        : end(end_)
+        , allow_settings_after_format_in_insert(allow_settings_after_format_in_insert_)
+    {}
 };
 
 }
