@@ -61,6 +61,7 @@ namespace Setting
 {
 
 extern const SettingsBool correlated_subqueries_substitute_equivalent_expressions;
+extern const SettingsBool use_variant_as_common_type;
 extern const SettingsDecorrelationJoinKind correlated_subqueries_default_join_kind;
 extern const SettingsBool join_use_nulls;
 extern const SettingsMaxThreads max_threads;
@@ -378,6 +379,7 @@ QueryPlan decorrelateQueryPlan(
         ///     FROM t2
         ///     WHERE t.x = t2.y
         /// )
+        const auto & settings = context.planner_context->getQueryContext()->getSettingsRef();
         auto process_isolated_subplan = [](
             DecorrelationContext & current_context,
             QueryPlan::Node * subplan_root
@@ -398,8 +400,11 @@ QueryPlan decorrelateQueryPlan(
         child_plans.emplace_back(std::make_unique<QueryPlan>(std::move(decorrelated_lhs_plan)));
         child_plans.emplace_back(std::make_unique<QueryPlan>(std::move(decorrelated_rhs_plan)));
 
-        Block union_common_header = buildCommonHeaderForUnion(query_plans_headers, SelectUnionMode::UNION_ALL); // Union mode doesn't matter here
-        addConvertingToCommonHeaderActionsIfNeeded(child_plans, union_common_header, query_plans_headers);
+        Block union_common_header = buildCommonHeaderForUnion(
+            query_plans_headers,
+            SelectUnionMode::UNION_ALL,
+            settings[Setting::use_variant_as_common_type]); // Union mode doesn't matter here
+        addConvertingToCommonHeaderActionsIfNeeded(child_plans, union_common_header, query_plans_headers, context.planner_context->getQueryContext());
 
         union_step->updateInputHeaders(std::move(query_plans_headers));
 
@@ -607,7 +612,8 @@ Planner buildPlannerForCorrelatedSubquery(
 
 void addStepForResultRenaming(
     const CorrelatedSubquery & correlated_subquery,
-    QueryPlan & correlated_subquery_plan
+    QueryPlan & correlated_subquery_plan,
+    const PlannerContextPtr & planner_context
 )
 {
     const auto & header = correlated_subquery_plan.getCurrentHeader();
@@ -637,7 +643,8 @@ void addStepForResultRenaming(
         result_node = &dag.addCast(
             *dag.getOutputs()[0],
             expected_result_type,
-            correlated_subquery.action_node_name);
+            correlated_subquery.action_node_name,
+            planner_context->getQueryContext());
     }
     else
     {
@@ -675,8 +682,8 @@ void buildQueryPlanForCorrelatedSubquery(
     const CorrelatedSubquery & correlated_subquery,
     const SelectQueryOptions & select_query_options)
 {
-    auto * query_node = correlated_subquery.query_tree->as<QueryNode>();
-    auto * union_node = correlated_subquery.query_tree->as<UnionNode>();
+    auto * query_node = correlated_subquery.query_tree->as<QueryNode>();  /// NOLINT(clang-analyzer-deadcode.DeadStores)
+    auto * union_node = correlated_subquery.query_tree->as<UnionNode>();  /// NOLINT(clang-analyzer-deadcode.DeadStores)
     chassert(query_node != nullptr && query_node->isCorrelated() || union_node != nullptr && union_node->isCorrelated());
 
     switch (correlated_subquery.kind)
@@ -687,7 +694,7 @@ void buildQueryPlanForCorrelatedSubquery(
             /// Logical plan for correlated subquery
             auto & correlated_query_plan = subquery_planner.getQueryPlan();
 
-            addStepForResultRenaming(correlated_subquery, correlated_query_plan);
+            addStepForResultRenaming(correlated_subquery, correlated_query_plan, planner_context);
 
             /// Mark all query plan steps if they or their subplans contain usage of correlated subqueries.
             /// It's needed to identify the moment when dependent join can be replaced by CROSS JOIN.
