@@ -1,10 +1,10 @@
 #include <Processors/ConcatProcessor.h>
-
+#include <Processors/Port.h>
 
 namespace DB
 {
 
-ConcatProcessor::ConcatProcessor(const Block & header, size_t num_inputs)
+ConcatProcessor::ConcatProcessor(SharedHeader header, size_t num_inputs)
     : IProcessor(InputPorts(num_inputs, header), OutputPorts{header}), current_input(inputs.begin())
 {
 }
