@@ -1,11 +1,11 @@
 #pragma once
-#include "config_formats.h"
+#include "config.h"
 
 #if USE_ARROW
 
 #include <Formats/FormatSettings.h>
 #include <Processors/Formats/IOutputFormat.h>
-#include "ArrowBufferedStreams.h"
+#include <Processors/Formats/Impl/ArrowBufferedStreams.h>
 
 namespace arrow { class Schema; }
 namespace arrow::ipc { class RecordBatchWriter; }
@@ -18,15 +18,14 @@ class CHColumnToArrowColumn;
 class ArrowBlockOutputFormat : public IOutputFormat
 {
 public:
-    ArrowBlockOutputFormat(WriteBuffer & out_, const Block & header_, bool stream_, const FormatSettings & format_settings_);
+    ArrowBlockOutputFormat(WriteBuffer & out_, SharedHeader header_, bool stream_, const FormatSettings & format_settings_);
 
-    String getName() const override { return "ArrowBlockOutputFormat"; }
-
-    String getContentType() const override { return "application/octet-stream"; }
+    String getName() const override { return "Arrow"; }
 
 private:
     void consume(Chunk) override;
     void finalizeImpl() override;
+    void resetFormatterImpl() override;
 
     void prepareWriter(const std::shared_ptr<arrow::Schema> & schema);
 
