@@ -1,12 +1,11 @@
 #pragma once
 
 #include <Processors/Formats/IRowOutputFormat.h>
-#include <Core/Block.h>
-
 
 namespace DB
 {
 
+class Block;
 class IColumn;
 class IDataType;
 class WriteBuffer;
@@ -17,11 +16,11 @@ class WriteBuffer;
 class BinaryRowOutputFormat final: public IRowOutputFormat
 {
 public:
-    BinaryRowOutputFormat(WriteBuffer & out_, const Block & header, bool with_names_, bool with_types_, const RowOutputFormatParams & params_);
+    BinaryRowOutputFormat(WriteBuffer & out_, SharedHeader header, bool with_names_, bool with_types_, const FormatSettings & format_settings_);
 
     String getName() const override { return "BinaryRowOutputFormat"; }
 
-    String getContentType() const override { return "application/octet-stream"; }
+    bool supportsSpecialSerializationKinds() const override { return format_settings.allow_special_serialization_kinds; }
 
 private:
     void writeField(const IColumn & column, const ISerialization & serialization, size_t row_num) override;
@@ -29,6 +28,7 @@ private:
 
     bool with_names;
     bool with_types;
+    const FormatSettings format_settings;
 };
 
 }
