@@ -1,6 +1,5 @@
 #pragma once
 
-#include <Core/Block.h>
 #include <Processors/Formats/IRowOutputFormat.h>
 #include <Formats/FormatSettings.h>
 
@@ -8,6 +7,7 @@
 namespace DB
 {
 
+class Block;
 class WriteBuffer;
 
 
@@ -20,20 +20,20 @@ public:
     /** with_names - output in the first line a header with column names
       * with_types - output in the next line header with the names of the types
       */
-    CSVRowOutputFormat(WriteBuffer & out_, const Block & header_, bool with_names_, bool with_types, const RowOutputFormatParams & params_, const FormatSettings & format_settings_);
+    CSVRowOutputFormat(WriteBuffer & out_, SharedHeader header_, bool with_names_, bool with_types, const FormatSettings & format_settings_);
 
-    String getName() const override { return "CSVRowOutputFormat"; }
+    String getName() const override { return "CSV"; }
 
-    /// https://www.iana.org/assignments/media-types/text/csv
-    String getContentType() const override
-    {
-        return String("text/csv; charset=UTF-8; header=") + (with_names ? "present" : "absent");
-    }
+    bool supportsSpecialSerializationKinds() const override { return format_settings.allow_special_serialization_kinds; }
 
 private:
     void writeField(const IColumn & column, const ISerialization & serialization, size_t row_num) override;
     void writeFieldDelimiter() override;
     void writeRowEndDelimiter() override;
+
+    bool supportTotals() const override { return true; }
+    bool supportExtremes() const override { return true; }
+
     void writeBeforeTotals() override;
     void writeBeforeExtremes() override;
 
