@@ -2,6 +2,9 @@
 #include <Columns/ColumnTuple.h>
 #include <Columns/ColumnMap.h>
 #include <Columns/ColumnsNumber.h>
+#include <Common/SipHash.h>
+#include <Formats/FormatSettings.h>
+#include <Core/Settings.h>
 #include <DataTypes/DataTypesNumber.h>
 #include <DataTypes/DataTypeNullable.h>
 #include <DataTypes/DataTypeString.h>
@@ -12,12 +15,13 @@
 #include <Processors/Formats/IRowInputFormat.h>
 #include <Functions/FunctionFactory.h>
 #include <Interpreters/ExpressionAnalyzer.h>
+#include <Interpreters/ExpressionActions.h>
 #include <Interpreters/ReplaceQueryParameterVisitor.h>
 #include <Interpreters/TreeRewriter.h>
 #include <Interpreters/Context.h>
 #include <Interpreters/convertFieldToType.h>
-#include <Interpreters/ExpressionActions.h>
 #include <Interpreters/castColumn.h>
+#include <IO/Operators.h>
 #include <IO/ReadHelpers.h>
 #include <Parsers/ASTExpressionList.h>
 #include <Parsers/ASTFunction.h>
@@ -28,11 +32,15 @@
 #include <Processors/Formats/Impl/ConstantExpressionTemplate.h>
 #include <Parsers/ExpressionElementParsers.h>
 #include <boost/functional/hash.hpp>
-#include <base/sort.h>
 
 
 namespace DB
 {
+namespace Setting
+{
+    extern const SettingsUInt64 max_parser_backtracks;
+    extern const SettingsUInt64 max_parser_depth;
+}
 
 namespace ErrorCodes
 {
@@ -130,8 +138,8 @@ static void fillLiteralInfo(DataTypes & nested_types, LiteralInfo & info)
             field_type = Field::Types::Map;
         }
         else
-            throw Exception("Unexpected literal type inside Array: " + nested_type->getName() + ". It's a bug",
-                            ErrorCodes::LOGICAL_ERROR);
+            throw Exception(ErrorCodes::LOGICAL_ERROR, "Unexpected literal type inside Array: {}. It's a bug",
+                            nested_type->getName());
 
         if (is_nullable)
             nested_type = std::make_shared<DataTypeNullable>(nested_type);
@@ -159,9 +167,9 @@ public:
         else if (ast->as<ASTQueryParameter>())
             return;
         else if (ast->as<ASTIdentifier>())
-            throw DB::Exception("Identifier in constant expression", ErrorCodes::SYNTAX_ERROR);
+            throw DB::Exception(ErrorCodes::SYNTAX_ERROR, "Identifier in constant expression");
         else
-            throw DB::Exception("Syntax error in constant expression", ErrorCodes::SYNTAX_ERROR);
+            throw DB::Exception(ErrorCodes::SYNTAX_ERROR, "Syntax error in constant expression");
     }
 
 private:
@@ -177,6 +185,14 @@ private:
         if (function.name == "lambda")
             return;
 
+        /// Parsing of INTERVALs is quite hacky. Expressions are rewritten during parsing like this:
+        /// "now() + interval 1 day" -> "now() + toIntervalDay(1)"
+        /// "select now() + INTERVAL '1 day 1 hour 1 minute'" -> "now() + (toIntervalDay(1), toIntervalHour(1), toIntervalMinute(1))"
+        /// so the AST is completely different from the original expression .
+        /// Avoid extracting these literals and simply compare tokens. It makes the template less flexible but much simpler.
+        if (function.name.starts_with("toInterval"))
+            return;
+
         FunctionOverloadResolverPtr builder = FunctionFactory::instance().get(function.name, context);
         /// Do not replace literals which must be constant
         ColumnNumbers dont_visit_children = builder->getArgumentsThatAreAlwaysConstant();
@@ -201,27 +217,32 @@ private:
             /// Do not replace empty array and array of NULLs
             if (literal->value.getType() == Field::Types::Array)
             {
-                const Array & array = literal->value.get<Array>();
+                const Array & array = literal->value.safeGet<Array>();
                 auto not_null = std::find_if_not(array.begin(), array.end(), [](const auto & elem) { return elem.isNull(); });
                 if (not_null == array.end())
                     return true;
             }
             else if (literal->value.getType() == Field::Types::Map)
             {
-                const Map & map = literal->value.get<Map>();
+                const Map & map = literal->value.safeGet<Map>();
                 if (map.size() % 2)
                     return false;
             }
             else if (literal->value.getType() == Field::Types::Tuple)
             {
-                const Tuple & tuple = literal->value.get<Tuple>();
+                const Tuple & tuple = literal->value.safeGet<Tuple>();
 
                 for (const auto & value : tuple)
                     if (value.isNull())
                         return true;
             }
 
-            String column_name = "_dummy_" + std::to_string(replaced_literals.size());
+            /// When generating placeholder names, ensure that we use names
+            /// requiring quotes to be valid identifiers. This prevents the
+            /// tuple() function from generating named tuples. Otherwise,
+            /// inserting named tuples with different names into another named
+            /// tuple will result in only default values being inserted.
+            String column_name = "-dummy-" + std::to_string(replaced_literals.size());
             replaced_literals.emplace_back(literal, column_name, force_nullable);
             setDataType(replaced_literals.back());
             ast = std::make_shared<ASTIdentifier>(column_name);
@@ -314,9 +335,6 @@ ConstantExpressionTemplate::TemplateStructure::TemplateStructure(LiteralsInfo &
     for (size_t i = 0; i < replaced_literals.size(); ++i)
     {
         const LiteralInfo & info = replaced_literals[i];
-        if (info.literal->begin.value() < prev_end)
-            throw Exception("Cannot replace literals", ErrorCodes::LOGICAL_ERROR);
-
         while (prev_end < info.literal->begin.value())
         {
             tokens.emplace_back(prev_end->begin, prev_end->size());
@@ -350,6 +368,31 @@ ConstantExpressionTemplate::TemplateStructure::TemplateStructure(LiteralsInfo &
 
 }
 
+String ConstantExpressionTemplate::TemplateStructure::dumpTemplate() const
+{
+    WriteBufferFromOwnString res;
+
+    size_t cur_column = 0;
+    size_t cur_token = 0;
+    size_t num_columns = literals.columns();
+    while (cur_column < num_columns)
+    {
+        size_t skip_tokens_until = token_after_literal_idx[cur_column];
+        while (cur_token < skip_tokens_until)
+            res << quote << tokens[cur_token++] << ", ";
+
+        const DataTypePtr & type = literals.getByPosition(cur_column).type;
+        res << type->getName() << ", ";
+        ++cur_column;
+    }
+
+    while (cur_token < tokens.size())
+        res << quote << tokens[cur_token++] << ", ";
+
+    res << "eof";
+    return res.str();
+}
+
 size_t ConstantExpressionTemplate::TemplateStructure::getTemplateHash(const ASTPtr & expression,
                                                                       const LiteralsInfo & replaced_literals,
                                                                       const DataTypePtr & result_column_type,
@@ -360,7 +403,7 @@ size_t ConstantExpressionTemplate::TemplateStructure::getTemplateHash(const ASTP
     SipHash hash_state;
     hash_state.update(result_column_type->getName());
 
-    expression->updateTreeHash(hash_state);
+    expression->updateTreeHash(hash_state, /*ignore_aliases=*/ true);
 
     for (const auto & info : replaced_literals)
         hash_state.update(info.type->getName());
@@ -369,11 +412,10 @@ size_t ConstantExpressionTemplate::TemplateStructure::getTemplateHash(const ASTP
     /// Allows distinguish expression in the last column in Values format
     hash_state.update(salt);
 
-    IAST::Hash res128;
-    hash_state.get128(res128);
+    const auto res128 = getSipHash128AsPair(hash_state);
     size_t res = 0;
-    boost::hash_combine(res, res128.first);
-    boost::hash_combine(res, res128.second);
+    boost::hash_combine(res, res128.low64);
+    boost::hash_combine(res, res128.high64);
     return res;
 }
 
@@ -418,12 +460,13 @@ ConstantExpressionTemplate::Cache::getFromCacheOrConstruct(const DataTypePtr & r
     return res;
 }
 
-bool ConstantExpressionTemplate::parseExpression(ReadBuffer & istr, const FormatSettings & format_settings, const Settings & settings)
+bool ConstantExpressionTemplate::parseExpression(
+    ReadBuffer & istr, const TokenIterator & token_iterator, const FormatSettings & format_settings, const Settings & settings)
 {
     size_t cur_column = 0;
     try
     {
-        if (tryParseExpression(istr, format_settings, cur_column, settings))
+        if (tryParseExpression(istr, token_iterator, format_settings, cur_column, settings))
         {
             ++rows_count;
             return true;
@@ -445,7 +488,12 @@ bool ConstantExpressionTemplate::parseExpression(ReadBuffer & istr, const Format
     return false;
 }
 
-bool ConstantExpressionTemplate::tryParseExpression(ReadBuffer & istr, const FormatSettings & format_settings, size_t & cur_column, const Settings & settings)
+bool ConstantExpressionTemplate::tryParseExpression(
+    ReadBuffer & istr,
+    const TokenIterator & token_iterator,
+    const FormatSettings & format_settings,
+    size_t & cur_column,
+    const Settings & settings)
 {
     size_t cur_token = 0;
     size_t num_columns = structure->literals.columns();
@@ -464,7 +512,7 @@ bool ConstantExpressionTemplate::tryParseExpression(ReadBuffer & istr, const For
         const DataTypePtr & type = structure->literals.getByPosition(cur_column).type;
         if (format_settings.values.accurate_types_of_literals && !structure->special_parser[cur_column].useDefaultParser())
         {
-            if (!parseLiteralAndAssertType(istr, type.get(), cur_column, settings))
+            if (!parseLiteralAndAssertType(istr, token_iterator, type.get(), cur_column, settings))
                 return false;
         }
         else
@@ -482,7 +530,8 @@ bool ConstantExpressionTemplate::tryParseExpression(ReadBuffer & istr, const For
     return true;
 }
 
-bool ConstantExpressionTemplate::parseLiteralAndAssertType(ReadBuffer & istr, const IDataType * complex_type, size_t column_idx, const Settings & settings)
+bool ConstantExpressionTemplate::parseLiteralAndAssertType(
+    ReadBuffer & istr, const TokenIterator & token_iterator, const IDataType * complex_type, size_t column_idx, const Settings & settings)
 {
     using Type = Field::Types::Which;
 
@@ -497,12 +546,13 @@ bool ConstantExpressionTemplate::parseLiteralAndAssertType(ReadBuffer & istr, co
 
     if (type_info.is_array || type_info.is_tuple || type_info.is_map)
     {
-        /// TODO faster way to check types without using Parsers
         ParserArrayOfLiterals parser_array;
         ParserTupleOfLiterals parser_tuple;
 
-        Tokens tokens_number(istr.position(), istr.buffer().end());
-        IParser::Pos iterator(tokens_number, settings.max_parser_depth);
+        IParser::Pos iterator(
+            token_iterator, static_cast<unsigned>(settings[Setting::max_parser_depth]), static_cast<unsigned>(settings[Setting::max_parser_backtracks]));
+        while (iterator->begin < istr.position())
+            ++iterator;
         Expected expected;
         ASTPtr ast;
         if (!parser_array.parse(iterator, ast, expected) && !parser_tuple.parse(iterator, ast, expected))
@@ -515,11 +565,26 @@ bool ConstantExpressionTemplate::parseLiteralAndAssertType(ReadBuffer & istr, co
 
         DataTypes nested_types;
         if (type_info.is_array)
-            nested_types = { assert_cast<const DataTypeArray &>(*collection_type).getNestedType() };
+        {
+            const auto * array_type = typeid_cast<const DataTypeArray *>(collection_type.get());
+            if (!array_type)
+                return false;
+            nested_types = {array_type->getNestedType()};
+        }
         else if (type_info.is_tuple)
-            nested_types = assert_cast<const DataTypeTuple &>(*collection_type).getElements();
+        {
+            const auto * tuple_type = typeid_cast<const DataTypeTuple *>(collection_type.get());
+            if (!tuple_type)
+                return false;
+            nested_types = tuple_type->getElements();
+        }
         else
-            nested_types = assert_cast<const DataTypeMap &>(*collection_type).getKeyValueTypes();
+        {
+            const auto * map_type = typeid_cast<const DataTypeMap *>(collection_type.get());
+            if (!map_type)
+                return false;
+            nested_types = map_type->getKeyValueTypes();
+        }
 
         for (size_t i = 0; i < nested_types.size(); ++i)
         {
@@ -542,60 +607,62 @@ bool ConstantExpressionTemplate::parseLiteralAndAssertType(ReadBuffer & istr, co
         columns[column_idx]->insert(array_same_types);
         return true;
     }
+
+    Field number;
+    if (type_info.is_nullable && 4 <= istr.available() && 0 == strncasecmp(istr.position(), "NULL", 4))
+    {
+        istr.position() += 4;
+    }
     else
     {
-        Field number;
-        if (type_info.is_nullable && 4 <= istr.available() && 0 == strncasecmp(istr.position(), "NULL", 4))
-            istr.position() += 4;
-        else
+        /// ParserNumber::parse(...) is about 20x slower than strtod(...)
+        /// because of using ASTPtr, Expected and Tokens, which are not needed here.
+        /// Parse numeric literal in the same way, as ParserNumber does, but use strtod and strtoull directly.
+        bool negative = *istr.position() == '-';
+        if (negative || *istr.position() == '+')
+            ++istr.position();
+
+        static constexpr size_t MAX_LENGTH_OF_NUMBER = 319;
+        char buf[MAX_LENGTH_OF_NUMBER + 1];
+        size_t bytes_to_copy = std::min(istr.available(), MAX_LENGTH_OF_NUMBER);
+        memcpy(buf, istr.position(), bytes_to_copy);
+        buf[bytes_to_copy] = 0;
+
+        const bool hex_like = bytes_to_copy >= 2 && buf[0] == '0' && (buf[1] == 'x' || buf[1] == 'X');
+
+        char * pos_double = buf;
+        errno = 0;
+        Float64 float_value = std::strtod(buf, &pos_double);
+        if (pos_double == buf || errno == ERANGE || float_value < 0)
+            return false;
+
+        if (negative)
+            float_value = -float_value;
+
+        char * pos_integer = buf;
+        errno = 0;
+        UInt64 uint_value = std::strtoull(buf, &pos_integer, hex_like ? 16 : 10);
+        if (pos_integer == pos_double && errno != ERANGE && (!negative || uint_value <= (1ULL << 63)))
         {
-            /// ParserNumber::parse(...) is about 20x slower than strtod(...)
-            /// because of using ASTPtr, Expected and Tokens, which are not needed here.
-            /// Parse numeric literal in the same way, as ParserNumber does, but use strtod and strtoull directly.
-            bool negative = *istr.position() == '-';
-            if (negative || *istr.position() == '+')
-                ++istr.position();
-
-            static constexpr size_t MAX_LENGTH_OF_NUMBER = 319;
-            char buf[MAX_LENGTH_OF_NUMBER + 1];
-            size_t bytes_to_copy = std::min(istr.available(), MAX_LENGTH_OF_NUMBER);
-            memcpy(buf, istr.position(), bytes_to_copy);
-            buf[bytes_to_copy] = 0;
-
-            char * pos_double = buf;
-            errno = 0;
-            Float64 float_value = std::strtod(buf, &pos_double);
-            if (pos_double == buf || errno == ERANGE || float_value < 0)
-                return false;
-
-            if (negative)
-                float_value = -float_value;
-
-            char * pos_integer = buf;
-            errno = 0;
-            UInt64 uint_value = std::strtoull(buf, &pos_integer, 0);
-            if (pos_integer == pos_double && errno != ERANGE && (!negative || uint_value <= (1ULL << 63)))
-            {
-                istr.position() += pos_integer - buf;
-                if (negative && type_info.main_type == Type::Int64)
-                    number = static_cast<Int64>(-uint_value);
-                else if (!negative && type_info.main_type == Type::UInt64)
-                    number = uint_value;
-                else
-                    return false;
-            }
-            else if (type_info.main_type == Type::Float64)
-            {
-                istr.position() += pos_double - buf;
-                number = float_value;
-            }
+            istr.position() += pos_integer - buf;
+            if (negative && type_info.main_type == Type::Int64)
+                number = static_cast<Int64>(-uint_value);
+            else if (type_info.main_type == Type::UInt64 && (!negative || uint_value == 0))
+                number = uint_value;
             else
                 return false;
         }
-
-        columns[column_idx]->insert(number);
-        return true;
+        else if (type_info.main_type == Type::Float64)
+        {
+            istr.position() += pos_double - buf;
+            number = float_value;
+        }
+        else
+            return false;
     }
+
+    columns[column_idx]->insert(number);
+    return true;
 }
 
 ColumnPtr ConstantExpressionTemplate::evaluateAll(BlockMissingValues & nulls, size_t column_idx, const DataTypePtr & expected_type, size_t offset)
@@ -606,14 +673,13 @@ ColumnPtr ConstantExpressionTemplate::evaluateAll(BlockMissingValues & nulls, si
         evaluated.insert({ColumnConst::create(ColumnUInt8::create(1, 0), rows_count), std::make_shared<DataTypeUInt8>(), "_dummy"});
     structure->actions_on_literals->execute(evaluated);
 
-    if (!evaluated || evaluated.rows() != rows_count)
-        throw Exception("Number of rows mismatch after evaluation of batch of constant expressions: got " +
-                        std::to_string(evaluated.rows()) + " rows for " + std::to_string(rows_count) + " expressions",
-                        ErrorCodes::LOGICAL_ERROR);
+    if (evaluated.empty() || evaluated.rows() != rows_count)
+        throw Exception(ErrorCodes::LOGICAL_ERROR, "Number of rows mismatch after evaluation of batch of constant expressions: "
+                        "got {} rows for {} expressions", evaluated.rows(), rows_count);
 
     if (!evaluated.has(structure->result_column_name))
-        throw Exception("Cannot evaluate template " + structure->result_column_name + ", block structure:\n" + evaluated.dumpStructure(),
-                        ErrorCodes::LOGICAL_ERROR);
+        throw Exception(ErrorCodes::LOGICAL_ERROR, "Cannot evaluate template {}, block structure:\n{}",
+                        structure->result_column_name, evaluated.dumpStructure());
 
     rows_count = 0;
     auto res = evaluated.getByName(structure->result_column_name);
@@ -624,7 +690,7 @@ ColumnPtr ConstantExpressionTemplate::evaluateAll(BlockMissingValues & nulls, si
     /// Extract column with evaluated expression and mask for NULLs
     const auto & tuple = assert_cast<const ColumnTuple &>(*res.column);
     if (tuple.tupleSize() != 2)
-        throw Exception("Invalid tuple size, it'a a bug", ErrorCodes::LOGICAL_ERROR);
+        throw Exception(ErrorCodes::LOGICAL_ERROR, "Invalid tuple size, it'a a bug");
     const auto & is_null = assert_cast<const ColumnUInt8 &>(tuple.getColumn(1));
 
     for (size_t i = 0; i < is_null.size(); ++i)
