@@ -169,7 +169,7 @@ JSONAsObjectRowInputFormat::JSONAsObjectRowInputFormat(
     : JSONAsRowInputFormat(header_, in_, params_, format_settings_)
 {
     const auto & type = header_->getByPosition(0).type;
-    if (!isObject(type) && !isObjectDeprecated(type))
+    if (!isObject(type))
         throw Exception(ErrorCodes::BAD_ARGUMENTS,
             "Input format JSONAsObject is only suitable for tables with a single column of type JSON but the column type is {}",
             type->getName());
