@@ -1,15 +1,13 @@
 #pragma once
 
-#include <Core/Block.h>
-#include <IO/WriteBuffer.h>
-#include <IO/WriteBufferValidUTF8.h>
 #include <Processors/Formats/Impl/JSONRowOutputFormat.h>
 
-
 namespace DB
 {
 
+class Block;
 struct FormatSettings;
+class WriteBuffer;
 
 /** The stream for outputting data in the JSONCompact- formats.
   */
@@ -18,8 +16,7 @@ class JSONCompactRowOutputFormat final : public JSONRowOutputFormat
 public:
     JSONCompactRowOutputFormat(
         WriteBuffer & out_,
-        const Block & header,
-        const RowOutputFormatParams & params_,
+        SharedHeader header,
         const FormatSettings & settings_,
         bool yield_strings_);
 
@@ -31,17 +28,15 @@ private:
     void writeRowStartDelimiter() override;
     void writeRowEndDelimiter() override;
 
+    bool supportTotals() const override { return true; }
+    bool supportExtremes() const override { return true; }
+
     void writeBeforeTotals() override;
     void writeAfterTotals() override;
 
     void writeExtremesElement(const char * title, const Columns & columns, size_t row_num) override;
 
-    void writeTotalsField(const IColumn & column, const ISerialization & serialization, size_t row_num) override
-    {
-        return writeField(column, serialization, row_num);
-    }
-
-    void writeTotalsFieldDelimiter() override;
+    void writeTotals(const Columns & columns, size_t row_num) override;
 };
 
 }
