@@ -1,7 +1,6 @@
 #pragma once
 
 #include <Processors/Formats/IRowOutputFormat.h>
-#include <Formats/FormatFactory.h>
 #include <Formats/FormatSettings.h>
 
 namespace DB
@@ -12,10 +11,12 @@ class ReadBuffer;
 class MarkdownRowOutputFormat final : public IRowOutputFormat
 {
 public:
-    MarkdownRowOutputFormat(WriteBuffer & out_, const Block & header_, const RowOutputFormatParams & params_, const FormatSettings & format_settings_);
+    MarkdownRowOutputFormat(WriteBuffer & out_, SharedHeader header_, const FormatSettings & format_settings_);
 
     String getName() const override { return "MarkdownRowOutputFormat"; }
 
+    bool supportsSpecialSerializationKinds() const override { return format_settings.allow_special_serialization_kinds; }
+
 private:
     /// Write higher part of markdown table like this:
     /// |columnName1|columnName2|...|columnNameN|
@@ -28,8 +29,8 @@ private:
     /// Write '|' between values
     void writeFieldDelimiter() override;
 
-    /// Write '|\n' after each row
-    void writeRowEndDelimiter() override ;
+    /// Write '|\n' at the end of each row
+    void writeRowEndDelimiter() override;
 
     void writeField(const IColumn & column, const ISerialization & serialization, size_t row_num) override;
 
