@@ -1,8 +1,8 @@
 #pragma once
 
-#include <Core/Block.h>
 #include <Formats/FormatSettings.h>
 #include <Processors/Formats/IRowOutputFormat.h>
+#include <IO/WriteBufferFromString.h>
 
 
 namespace DB
@@ -20,24 +20,26 @@ public:
       */
     TabSeparatedRowOutputFormat(
         WriteBuffer & out_,
-        const Block & header_,
+        SharedHeader header_,
         bool with_names_,
         bool with_types_,
         bool is_raw_,
-        const RowOutputFormatParams & params_,
         const FormatSettings & format_settings_);
 
     String getName() const override { return "TabSeparatedRowOutputFormat"; }
 
-    /// https://www.iana.org/assignments/media-types/text/tab-separated-values
-    String getContentType() const override { return "text/tab-separated-values; charset=UTF-8"; }
+    bool supportsSpecialSerializationKinds() const override { return format_settings.allow_special_serialization_kinds; }
 
 protected:
     void writeField(const IColumn & column, const ISerialization & serialization, size_t row_num) override;
-    void writeFieldDelimiter() override final;
+    void writeFieldDelimiter() final;
     void writeRowEndDelimiter() override;
-    void writeBeforeTotals() override final;
-    void writeBeforeExtremes() override final;
+
+    bool supportTotals() const override { return true; }
+    bool supportExtremes() const override { return true; }
+
+    void writeBeforeTotals() final;
+    void writeBeforeExtremes() final;
 
     void writePrefix() override;
     void writeLine(const std::vector<String> & values);
