@@ -1,5 +1,6 @@
 #pragma once
 
+#include <Core/Block.h>
 #include <Processors/Merges/IMergingTransform.h>
 #include <Processors/Merges/Algorithms/FinishAggregatingInOrderAlgorithm.h>
 
@@ -13,20 +14,20 @@ class FinishAggregatingInOrderTransform final : public IMergingTransform<FinishA
 {
 public:
     FinishAggregatingInOrderTransform(
-        const Block & header,
+        SharedHeader header,
         size_t num_inputs,
         AggregatingTransformParamsPtr params,
         SortDescription description,
-        size_t max_block_size,
-        size_t max_block_bytes)
+        size_t max_block_size_rows,
+        size_t max_block_size_bytes)
         : IMergingTransform(
-            num_inputs, header, {}, /*have_all_inputs_=*/ true, /*limit_hint_=*/ 0,
+            num_inputs, header, std::make_shared<Block>(Block{}), /*have_all_inputs_=*/ true, /*limit_hint_=*/ 0, /*always_read_till_end_=*/ false,
             header,
             num_inputs,
             params,
             std::move(description),
-            max_block_size,
-            max_block_bytes)
+            max_block_size_rows,
+            max_block_size_bytes)
     {
     }
 
