@@ -11,15 +11,22 @@ class GraphiteRollupSortedTransform final : public IMergingTransform<GraphiteRol
 {
 public:
     GraphiteRollupSortedTransform(
-        const Block & header, size_t num_inputs,
-        SortDescription description_, size_t max_block_size,
-        Graphite::Params params_, time_t time_of_merge_)
+        SharedHeader header,
+        size_t num_inputs,
+        SortDescription description_,
+        size_t max_block_size_rows,
+        size_t max_block_size_bytes,
+        std::optional<size_t> max_dynamic_subcolumns_,
+        Graphite::Params params_,
+        time_t time_of_merge_)
         : IMergingTransform(
-            num_inputs, header, header, /*have_all_inputs_=*/ true, /*limit_hint_=*/ 0,
+            num_inputs, header, header, /*have_all_inputs_=*/ true, /*limit_hint_=*/ 0, /*always_read_till_end_=*/ false,
             header,
             num_inputs,
             std::move(description_),
-            max_block_size,
+            max_block_size_rows,
+            max_block_size_bytes,
+            max_dynamic_subcolumns_,
             std::move(params_),
             time_of_merge_)
     {
