@@ -18,6 +18,7 @@ MergingSortedTransform::MergingSortedTransform(
     const SortDescription & description_,
     size_t max_block_size_rows,
     size_t max_block_size_bytes,
+    std::optional<size_t> max_dynamic_subcolumns_,
     SortingQueueStrategy sorting_queue_strategy,
     UInt64 limit_,
     bool always_read_till_end_,
@@ -38,6 +39,7 @@ MergingSortedTransform::MergingSortedTransform(
         description_,
         max_block_size_rows,
         max_block_size_bytes,
+        max_dynamic_subcolumns_,
         sorting_queue_strategy,
         limit_,
         out_row_sources_buf_,
