@@ -5,14 +5,14 @@
 namespace DB
 {
 
-void addConvertingActions(QueryPlan & plan, const Block & header, bool has_missing_objects)
+void addConvertingActions(QueryPlan & plan, const Block & header, const ContextPtr & context)
 {
     if (blocksHaveEqualStructure(*plan.getCurrentHeader(), header))
         return;
 
-    auto mode = has_missing_objects ? ActionsDAG::MatchColumnsMode::Position : ActionsDAG::MatchColumnsMode::Name;
+    auto mode = ActionsDAG::MatchColumnsMode::Name;
 
-    auto get_converting_dag = [mode](const Block & block_, const Block & header_)
+    auto get_converting_dag = [mode, context](const Block & block_, const Block & header_)
     {
         /// Convert header structure to expected.
         /// Also we ignore constants from result and replace it with constants from header.
@@ -21,6 +21,7 @@ void addConvertingActions(QueryPlan & plan, const Block & header, bool has_missi
             block_.getColumnsWithTypeAndName(),
             header_.getColumnsWithTypeAndName(),
             mode,
+            context,
             true);
     };
 
