@@ -44,7 +44,7 @@ void LimitByStep::transformPipeline(QueryPipelineBuilder & pipeline, const Build
         if (stream_type != QueryPipelineBuilder::StreamType::Main)
             return nullptr;
 
-        return std::make_shared<LimitByTransform>(header, group_length, group_offset, columns);
+        return std::make_shared<LimitByTransform>(header, group_length, group_offset, in_order, columns);
     });
 }
 
@@ -113,6 +113,11 @@ std::unique_ptr<IQueryPlanStep> LimitByStep::deserialize(Deserialization & ctx)
     return std::make_unique<LimitByStep>(ctx.input_headers.front(), group_length, group_offset, std::move(columns));
 }
 
+void LimitByStep::applyOrder(SortDescription sort_description)
+{
+    in_order = sort_description.hasPrefix(columns);
+}
+
 void registerLimitByStep(QueryPlanStepRegistry & registry)
 {
     registry.registerStep("LimitBy", LimitByStep::deserialize);
