@@ -9,8 +9,8 @@ class LimitByStep : public ITransformingStep
 {
 public:
     explicit LimitByStep(
-            const DataStream & input_stream_,
-            size_t group_length_, size_t group_offset_, const Names & columns_);
+            const SharedHeader & input_header_,
+            size_t group_length_, size_t group_offset_, Names columns_);
 
     String getName() const override { return "LimitBy"; }
 
@@ -19,10 +19,24 @@ public:
     void describeActions(JSONBuilder::JSONMap & map) const override;
     void describeActions(FormatSettings & settings) const override;
 
+    void serialize(Serialization & ctx) const override;
+    bool isSerializable() const override { return true; }
+
+    static std::unique_ptr<IQueryPlanStep> deserialize(Deserialization & ctx);
+
+    void applyOrder(SortDescription sort_desc);
 private:
+    void updateOutputHeader() override
+    {
+        output_header = input_headers.front();
+    }
+
     size_t group_length;
     size_t group_offset;
+
     Names columns;
+
+    bool in_order = false;
 };
 
 }
