@@ -1,4 +1,7 @@
+#include <Processors/Port.h>
 #include <Processors/QueryPlan/OffsetStep.h>
+#include <Processors/QueryPlan/QueryPlanStepRegistry.h>
+#include <Processors/QueryPlan/Serialization.h>
 #include <Processors/OffsetTransform.h>
 #include <QueryPipeline/QueryPipelineBuilder.h>
 #include <IO/Operators.h>
@@ -12,7 +15,6 @@ static ITransformingStep::Traits getTraits()
     return ITransformingStep::Traits
     {
         {
-            .preserves_distinct_columns = true,
             .returns_single_stream = false,
             .preserves_number_of_streams = true,
             .preserves_sorting = true,
@@ -23,8 +25,8 @@ static ITransformingStep::Traits getTraits()
     };
 }
 
-OffsetStep::OffsetStep(const DataStream & input_stream_, size_t offset_)
-    : ITransformingStep(input_stream_, input_stream_.header, getTraits())
+OffsetStep::OffsetStep(const SharedHeader & input_header_, size_t offset_)
+    : ITransformingStep(input_header_, input_header_, getTraits())
     , offset(offset_)
 {
 }
@@ -47,4 +49,22 @@ void OffsetStep::describeActions(JSONBuilder::JSONMap & map) const
     map.add("Offset", offset);
 }
 
+void OffsetStep::serialize(Serialization & ctx) const
+{
+    writeVarUInt(offset, ctx.out);
+}
+
+std::unique_ptr<IQueryPlanStep> OffsetStep::deserialize(Deserialization & ctx)
+{
+    UInt64 offset;
+    readVarUInt(offset, ctx.in);
+
+    return std::make_unique<OffsetStep>(ctx.input_headers.front(), offset);
+}
+
+void registerOffsetStep(QueryPlanStepRegistry & registry)
+{
+    registry.registerStep("Offset", OffsetStep::deserialize);
+}
+
 }
