@@ -224,11 +224,18 @@ std::optional<String> optimizeUseNormalProjections(
     {
         if (!has_all_required_columns(projection))
         {
-            /// Check if projection can be used to filter parts
+            /// Check if projection can be used to filter parts or building projection index filters
             if (query.filter_node && optimize_use_projection_filtering)
             {
-                filterPartsUsingProjection(
-                    *projection, reader, empty_mutations_snapshot, *parent_reading_select_result, projection_query_info, context);
+                filterPartsAndCollectProjectionCandidates(
+                    *reading,
+                    *projection,
+                    reader,
+                    empty_mutations_snapshot,
+                    *parent_reading_select_result,
+                    projection_query_info,
+                    query.filter_node,
+                    context);
             }
 
             continue;
