@@ -11,6 +11,7 @@
 #include <QueryPipeline/QueryPipelineBuilder.h>
 #include <Storages/MergeTree/KeyCondition.h>
 #include <Storages/System/StorageSystemNumbers.h>
+#include <base/types.h>
 #include <fmt/format.h>
 #include <Common/iota.h>
 #include <Common/typeid_cast.h>
@@ -119,9 +120,9 @@ private:
 
 struct RangeWithStep
 {
-    UInt64 left;
-    UInt64 step;
-    UInt128 size;
+    UInt64 left;    /// first value in the range
+    UInt64 step;    /// step between values
+    UInt128 size;   /// how many values in this range
 };
 
 using RangesWithStep = std::vector<RangeWithStep>;
@@ -165,11 +166,11 @@ auto sizeOfRanges(const RangesWithStep & rs)
 class NumbersRangedSource : public ISource
 {
 public:
-    /// Represent a position in Ranges list.
+    /// Represent a position in `Ranges` list.
     struct RangesPos
     {
-        size_t offset_in_ranges;
-        UInt128 offset_in_range;
+        size_t offset_in_ranges; /// which range in the vector
+        UInt128 offset_in_range; /// how many values already consumed inside that range
     };
 
     struct RangesState
@@ -196,56 +197,61 @@ public:
 
     String getName() const override { return "NumbersRange"; }
 
-protected:
-    /// Find the data range in ranges and return how many item found.
-    /// If no data left in ranges return 0.
-    UInt64 findRanges(RangesPos & start, RangesPos & end, UInt64 base_block_size_)
+private:
+    /// Find the data range in ranges and return (start position, how many items found).
+    /// If no data left in ranges return 0 items found
+    std::pair<RangesPos, UInt64> findRanges()
     {
         std::lock_guard lock(ranges_state->mutex);
 
 
-        UInt64 need = base_block_size_;
-        UInt64 size = 0; /// how many item found.
+        UInt64 maximum_can_take = base_block_size;
 
-        /// find start
-        start = ranges_state->pos;
-        end = start;
+        /// `start` is the current global position.
+        RangesPos start = ranges_state->pos;
+        RangesPos end = start;
 
-        /// find end
-        while (need != 0)
+        /// Find end
+        while (maximum_can_take != 0)
         {
-            UInt128 can_provide = end.offset_in_ranges == ranges.size() ? static_cast<UInt128>(0)
-                                                                        : ranges[end.offset_in_ranges].size - end.offset_in_range;
-            if (can_provide == 0)
-                break;
+            if (end.offset_in_ranges == ranges.size())
+                break; /// No ranges left at all
 
-            if (can_provide > need)
+
+            auto & current_range = ranges[end.offset_in_ranges];
+
+            UInt128 remaining_in_current_range = current_range.size - end.offset_in_range;
+
+            chassert(remaining_in_current_range > 0);
+
+            UInt128 take128 = std::min<UInt128>(remaining_in_current_range, maximum_can_take);
+
+            UInt64 take = static_cast<UInt64>(take128);
+
+            /// Advance 'end' by 'take' elements inside this range.
+            chassert(end.offset_in_range + take <= current_range.size);
+
+            end.offset_in_range += take;
+
+            /// If we've exactly exhausted this range, move to the next one.
+            if (end.offset_in_range == current_range.size)
             {
-                end.offset_in_range += need;
-                size += need;
-                need = 0;
-            }
-            else if (can_provide == need)
-            {
-                end.offset_in_ranges++;
+                ++end.offset_in_ranges;
                 end.offset_in_range = 0;
-                size += need;
-                need = 0;
-            }
-            else
-            {
-                end.offset_in_ranges++;
-                end.offset_in_range = 0;
-                size += static_cast<UInt64>(can_provide);
-                need -= static_cast<UInt64>(can_provide);
             }
+
+            maximum_can_take -= take;
         }
 
+        /// Publish new global position.
         ranges_state->pos = end;
 
-        return size;
+        UInt64 block_size = base_block_size - maximum_can_take;
+
+        return {start, block_size};
     }
 
+protected:
     Chunk generate() override
     {
         if (ranges.empty())
@@ -253,86 +259,58 @@ protected:
 
         /// Find the data range.
         /// If data left is small, shrink block size.
-        RangesPos start;
-        RangesPos end;
-        auto block_size = findRanges(start, end, base_block_size);
+        auto [start, block_size] = findRanges();
 
+        chassert(block_size <= base_block_size);
+
+        /// No data left.
         if (!block_size)
             return {};
 
+        chassert(start.offset_in_ranges < ranges.size());
+
         auto column = ColumnUInt64::create(block_size);
         ColumnUInt64::Container & vec = column->getData();
 
-        /// This will accelerates the code.
         UInt64 * pos = vec.data();
 
         UInt64 provided = 0;
         RangesPos cursor = start;
 
-        while (block_size - provided != 0)
+        while (provided < block_size)
         {
-            UInt64 need = block_size - provided;
+            chassert(cursor.offset_in_ranges < ranges.size());
+
             auto & range = ranges[cursor.offset_in_ranges];
 
-            UInt128 can_provide = cursor.offset_in_ranges == end.offset_in_ranges
-                ? end.offset_in_range - cursor.offset_in_range
-                : range.size - cursor.offset_in_range;
+            UInt64 need = block_size - provided;
+            UInt128 remaining_in_current_range = range.size - cursor.offset_in_range;
 
-            /// set value to block
-            auto set_value = [&pos, this](UInt128 & start_value, UInt128 & end_value)
-            {
-                if (end_value > std::numeric_limits<UInt64>::max())
-                {
-                    while (start_value < end_value)
-                    {
-                        *(pos++) = start_value;
-                        start_value += this->step;
-                    }
-                }
-                else
-                {
-                    auto start_value_64 = static_cast<UInt64>(start_value);
-                    auto end_value_64 = static_cast<UInt64>(end_value);
-                    auto size = (end_value_64 - start_value_64) / this->step;
-                    iotaWithStepOptimized(pos, static_cast<size_t>(size), start_value_64, step);
-                    pos += size;
-                }
-            };
+            /// How many items from the current range should belong to this block starting at cursor.
+            UInt128 can_provide = std::min<UInt128>(remaining_in_current_range, need);
+            chassert(can_provide > 0);
 
-            if (can_provide > need)
-            {
-                UInt64 start_value = range.left + cursor.offset_in_range * step;
-                /// end_value will never overflow
-                iotaWithStepOptimized(pos, static_cast<size_t>(need), start_value, step);
-                pos += need;
-                provided += need;
-                cursor.offset_in_range += need;
-            }
-            else if (can_provide == need)
-            {
-                /// to avoid UInt64 overflow
-                UInt128 start_value = static_cast<UInt128>(range.left) + cursor.offset_in_range * step;
-                UInt128 end_value = start_value + need * step;
-                set_value(start_value, end_value);
+            UInt64 take = static_cast<UInt64>(can_provide);
 
-                provided += need;
-                cursor.offset_in_ranges++;
-                cursor.offset_in_range = 0;
-            }
-            else
-            {
-                /// to avoid UInt64 overflow
-                UInt128 start_value = static_cast<UInt128>(range.left) + cursor.offset_in_range * step;
-                UInt128 end_value = start_value + can_provide * step;
-                set_value(start_value, end_value);
+            /// Compute logical start value (in 128-bit to avoid overflow), then narrow to UInt64 for the column.
+            UInt128 start_value_128 = static_cast<UInt128>(range.left) + cursor.offset_in_range * step;
 
-                provided += static_cast<UInt64>(can_provide);
-                cursor.offset_in_ranges++;
+            chassert(start_value_128 <= std::numeric_limits<UInt64>::max());
+            UInt64 start_value = static_cast<UInt64>(start_value_128);
+
+            iotaWithStepOptimized<UInt64>(pos, take, start_value, step);
+            pos += take;
+            provided += take;
+
+            cursor.offset_in_range += take;
+            if (cursor.offset_in_range == range.size)
+            {
+                ++cursor.offset_in_ranges;
                 cursor.offset_in_range = 0;
             }
         }
 
-        chassert(block_size == UInt64(pos - vec.begin()));
+        chassert(provided == block_size);
         progress(column->size(), column->byteSize());
 
         return {Columns{std::move(column)}, block_size};
@@ -356,25 +334,57 @@ private:
 namespace
 {
 /// Whether we should push limit down to scan.
-bool shouldPushdownLimit(const SelectQueryInfo & query_info, UInt64 limit_length)
+bool shouldPushdownLimit(const SelectQueryInfo & query_info, const InterpreterSelectQuery::LimitInfo & lim_info)
 {
-    if (!query_info.query)
+    /// Reject negative, fractional, and zero limits for pushdown
+    if (lim_info.is_limit_length_negative
+        || lim_info.fractional_limit > 0
+        || lim_info.fractional_offset > 0
+        || lim_info.limit_length == 0)
         return false;
 
+    chassert(query_info.query);
+
     const auto & query = query_info.query->as<ASTSelectQuery &>();
+
     /// Just ignore some minor cases, such as:
     ///     select * from system.numbers order by number asc limit 10
-    return !query.distinct && !query.limitBy() && !query_info.has_order_by
+    return !query.distinct
+        && !query.limitBy()
+        && !query_info.has_order_by
         && !query_info.need_aggregate
-        /// For new analyzer, window will be delete from AST, so we should not use query.window()
-        && !query_info.has_window && !query_info.additional_filter_ast && (limit_length > 0 && !query.limit_with_ties);
+        /// For new analyzer, window will be deleted from AST, so we should not use query.window()
+        && !query_info.has_window
+        && !query_info.additional_filter_ast
+        && !query.limit_with_ties;
 }
 
 /// Shrink ranges to size.
 ///     For example: ranges: [1, 5], [8, 100]; size: 7, we will get [1, 5], [8, 9]
 void shrinkRanges(RangesWithStep & ranges, size_t size)
 {
+    if (size == 0)
+    {
+        ranges.clear();
+        return;
+    }
+
+    chassert(!ranges.empty());
+
+#ifndef NDEBUG
+    {
+        UInt128 total_size{};
+        for (const auto & r : ranges)
+        {
+            chassert(r.size > 0);
+            total_size += r.size;
+        }
+        chassert(static_cast<UInt128>(size) <= total_size);
+    }
+#endif
+
     size_t last_range_idx = 0;
+    [[maybe_unused]] bool found = false; /// for invariant checking only
     for (size_t i = 0; i < ranges.size(); i++)
     {
         auto range_size = ranges[i].size;
@@ -386,15 +396,21 @@ void shrinkRanges(RangesWithStep & ranges, size_t size)
         if (range_size == size)
         {
             last_range_idx = i;
+            found = true;
             break;
         }
 
         auto & range = ranges[i];
         range.size = static_cast<UInt128>(size);
         last_range_idx = i;
+        found = true;
         break;
     }
 
+    /// With the preconditions above, we must have found a truncation point.
+    chassert(found);
+    chassert(last_range_idx < ranges.size());
+
     /// delete the additional ranges
     ranges.erase(ranges.begin() + (last_range_idx + 1), ranges.end());
 }
@@ -407,10 +423,7 @@ std::optional<size_t> getLimitFromQueryInfo(const SelectQueryInfo & query_info,
 
     const auto lim_info = InterpreterSelectQuery::getLimitLengthAndOffset(query_info.query->as<ASTSelectQuery &>(), context);
 
-    if (lim_info.is_limit_length_negative)
-        return {};
-
-    if (!shouldPushdownLimit(query_info, lim_info.limit_length))
+    if (!shouldPushdownLimit(query_info, lim_info))
         return {};
 
     return lim_info.limit_length + lim_info.limit_offset;
@@ -539,17 +552,23 @@ Pipe ReadFromSystemNumbersStep::makePipe()
         /// intersection with overflowed_table_range goes back.
         if (overflowed_table_range.has_value())
         {
+            auto step = numbers_storage.step;
+
+            UInt64 offset_mod = numbers_storage.offset % step;
+
+            /// 2^64 % step, computed safely in 128-bit
+            UInt64 wrap_mod = static_cast<UInt64>((static_cast<UInt128>(std::numeric_limits<UInt64>::max()) + 1) % step);
+
+            /// remainder for the wrapped segment: (offset - 2^64) % step
+            UInt128 tmp = static_cast<UInt128>(offset_mod) + step - wrap_mod;
+            UInt64 remainder_overflow = static_cast<UInt64>(tmp % step);
+
             for (auto & r : ranges)
             {
                 auto intersected_range = overflowed_table_range->intersectWith(r);
                 if (intersected_range)
                 {
-                    auto range_with_step = steppedRangeFromRange(
-                        intersected_range.value(),
-                        numbers_storage.step,
-                        static_cast<UInt64>(
-                            (static_cast<UInt128>(numbers_storage.offset) + std::numeric_limits<UInt64>::max() + 1)
-                            % numbers_storage.step));
+                    auto range_with_step = steppedRangeFromRange(intersected_range.value(), step, remainder_overflow);
                     if (range_with_step)
                         intersected_ranges.push_back(*range_with_step);
                 }
