@@ -1,4 +1,5 @@
 #pragma once
+#include <Interpreters/Aggregator.h>
 #include <Processors/QueryPlan/ITransformingStep.h>
 #include <QueryPipeline/SizeLimits.h>
 
@@ -12,14 +13,21 @@ using AggregatingTransformParamsPtr = std::shared_ptr<AggregatingTransformParams
 class RollupStep : public ITransformingStep
 {
 public:
-    RollupStep(const DataStream & input_stream_, AggregatingTransformParamsPtr params_);
+    RollupStep(const SharedHeader & input_header_, Aggregator::Params params_, bool final_, bool use_nulls_);
 
     String getName() const override { return "Rollup"; }
 
     void transformPipeline(QueryPipelineBuilder & pipeline, const BuildQueryPipelineSettings &) override;
 
+    const Aggregator::Params & getParams() const { return params; }
+
 private:
-    AggregatingTransformParamsPtr params;
+    void updateOutputHeader() override;
+
+    Aggregator::Params params;
+    size_t keys_size;
+    bool final;
+    bool use_nulls;
 };
 
 }
