@@ -1,6 +1,8 @@
 #pragma once
 #include <Processors/IInflatingTransform.h>
 #include <Processors/Transforms/AggregatingTransform.h>
+#include <Processors/Transforms/RollupTransform.h>
+#include <Processors/Transforms/finalizeChunk.h>
 
 
 namespace DB
@@ -8,28 +10,23 @@ namespace DB
 
 /// Takes blocks after grouping, with non-finalized aggregate functions.
 /// Calculates all subsets of columns and aggregates over them.
-class CubeTransform : public IAccumulatingTransform
+class CubeTransform : public GroupByModifierTransform
 {
 public:
-    CubeTransform(Block header, AggregatingTransformParamsPtr params);
+    CubeTransform(SharedHeader header, AggregatingTransformParamsPtr params, bool use_nulls_);
     String getName() const override { return "CubeTransform"; }
 
 protected:
-    void consume(Chunk chunk) override;
     Chunk generate() override;
 
 private:
-    AggregatingTransformParamsPtr params;
-    ColumnNumbers keys;
+    const ColumnsMask aggregates_mask;
 
-    Chunks consumed_chunks;
-    Chunk cube_chunk;
     Columns current_columns;
     Columns current_zero_columns;
 
     UInt64 mask = 0;
-
-    Chunk merge(Chunks && chunks, bool final);
+    UInt64 grouping_set = 0;
 };
 
 }
