@@ -1,6 +1,7 @@
 #include <exception>
 #include <Processors/Transforms/ExceptionKeepingTransform.h>
 #include <Common/ThreadStatus.h>
+#include <Common/setThreadName.h>
 #include <Common/Stopwatch.h>
 #include <base/scope_guard.h>
 
@@ -94,7 +95,7 @@ IProcessor::Status ExceptionKeepingTransform::prepare()
 
 static std::exception_ptr runStep(std::function<void()> step, ThreadGroupPtr & thread_group)
 {
-    ThreadGroupSwitcher switcher(thread_group, "RuntimeData", /*allow_existing_group*/ true);
+    ThreadGroupSwitcher switcher(thread_group, ThreadName::RUNTIME_DATA, /*allow_existing_group*/ true);
 
     std::exception_ptr res;
 
