@@ -1,6 +1,7 @@
 #pragma once
-#include <Processors/Transforms/ExceptionKeepingTransform.h>
 #include <Processors/ISimpleTransform.h>
+#include <Processors/Transforms/ExceptionKeepingTransform.h>
+#include <Core/Block_fwd.h>
 
 namespace DB
 {
@@ -12,19 +13,17 @@ class ActionsDAG;
 
 /** Executes a certain expression over the block.
   * The expression consists of column identifiers from the block, constants, common functions.
-  * For example: hits * 2 + 3, url LIKE '%yandex%'
+  * For example: hits * 2 + 3, url LIKE '%clickhouse%'
   * The expression processes each row independently of the others.
   */
 class ExpressionTransform final : public ISimpleTransform
 {
 public:
-    ExpressionTransform(
-            const Block & header_,
-            ExpressionActionsPtr expression_);
+    ExpressionTransform(SharedHeader header_, ExpressionActionsPtr expression_);
 
     String getName() const override { return "ExpressionTransform"; }
 
-    static Block transformHeader(Block header, const ActionsDAG & expression);
+    static Block transformHeader(const Block & header, const ActionsDAG & expression);
 
 protected:
     void transform(Chunk & chunk) override;
@@ -37,8 +36,8 @@ class ConvertingTransform final : public ExceptionKeepingTransform
 {
 public:
     ConvertingTransform(
-            const Block & header_,
-            ExpressionActionsPtr expression_);
+        SharedHeader header_,
+        ExpressionActionsPtr expression_);
 
     String getName() const override { return "ConvertingTransform"; }
 
