@@ -8,12 +8,13 @@ namespace DB
 class MaterializingTransform : public ISimpleTransform
 {
 public:
-    explicit MaterializingTransform(const Block & header);
+    explicit MaterializingTransform(SharedHeader header, bool remove_special_representations_ = true);
 
     String getName() const override { return "MaterializingTransform"; }
 
 protected:
     void transform(Chunk & chunk) override;
+    bool remove_special_representations;
 };
 
 }
