@@ -1,22 +1,22 @@
 #pragma once
+
+#include <Interpreters/PreparedSets.h>
 #include <Processors/IAccumulatingTransform.h>
 #include <Storages/MergeTree/MergeTreeData.h>
-#include <Storages/MergeTree/IMergeTreeDataPart.h>
-#include <Core/Block.h>
-#include <Storages/MergeTree/MergeTreeDataPartTTLInfo.h>
 #include <Processors/TTL/ITTLAlgorithm.h>
 #include <Processors/TTL/TTLDeleteAlgorithm.h>
 
-#include <Common/DateLUT.h>
-
 namespace DB
 {
 
+class Block;
+
 class TTLTransform : public IAccumulatingTransform
 {
 public:
     TTLTransform(
-        const Block & header_,
+        const ContextPtr & context,
+        SharedHeader header_,
         const MergeTreeData & storage_,
         const StorageMetadataPtr & metadata_snapshot_,
         const MergeTreeData::MutableDataPartPtr & data_part_,
@@ -28,6 +28,8 @@ public:
 
     Status prepare() override;
 
+    PreparedSets::Subqueries getSubqueries() { return std::move(subqueries_for_sets); }
+
 protected:
     void consume(Chunk chunk) override;
     Chunk generate() override;
@@ -40,9 +42,11 @@ private:
     const TTLDeleteAlgorithm * delete_algorithm = nullptr;
     bool all_data_dropped = false;
 
+    PreparedSets::Subqueries subqueries_for_sets;
+
     /// ttl_infos and empty_columns are updating while reading
     const MergeTreeData::MutableDataPartPtr & data_part;
-    Poco::Logger * log;
+    LoggerPtr log;
 };
 
 }
