@@ -1,6 +1,6 @@
 #pragma once
 
-#include <Parsers/IAST.h>
+#include <Parsers/IAST_fwd.h>
 #include <Interpreters/Context_fwd.h>
 #include <Processors/Formats/IInputFormat.h>
 #include <cstddef>
@@ -14,19 +14,27 @@ class Pipe;
 
 /// Prepares a input format, which produce data containing in INSERT query.
 InputFormatPtr getInputFormatFromASTInsertQuery(
-        const ASTPtr & ast,
-        bool with_buffers,
-        const Block & header,
-        ContextPtr context,
-        const ASTPtr & input_function);
+    const ASTPtr & ast,
+    bool with_buffers,
+    const Block & header,
+    ContextPtr context,
+    const ASTPtr & input_function);
+
+/// Prepares a pipe from input format got from ASTInsertQuery,
+/// which produce data containing in INSERT query.
+Pipe getSourceFromInputFormat(
+    const ASTPtr & ast,
+    InputFormatPtr format,
+    ContextPtr context,
+    const ASTPtr & input_function);
 
 /// Prepares a pipe which produce data containing in INSERT query.
 Pipe getSourceFromASTInsertQuery(
-        const ASTPtr & ast,
-        bool with_buffers,
-        const Block & header,
-        ContextPtr context,
-        const ASTPtr & input_function);
+    const ASTPtr & ast,
+    bool with_buffers,
+    const Block & header,
+    ContextPtr context,
+    const ASTPtr & input_function);
 
 class ReadBuffer;
 
