@@ -26,7 +26,7 @@ struct QueryPlanResourceHolder
 
     /// Custom move assignment does not destroy data from lhs. It appends data from rhs to lhs.
     QueryPlanResourceHolder & operator=(QueryPlanResourceHolder &&) noexcept;
-    QueryPlanResourceHolder & append(QueryPlanResourceHolder &&) noexcept;
+    QueryPlanResourceHolder & append(const QueryPlanResourceHolder & rhs) noexcept;
 
     /// Some processors may implicitly use Context or temporary Storage created by Interpreter.
     /// But lifetime of Streams is not nested in lifetime of Interpreters, so we have to store it here,
