@@ -1,14 +1,12 @@
 #pragma once
 
 #include <Core/Block.h>
-#include <Common/Throttler.h>
 #include <IO/ConnectionTimeouts.h>
-#include <Interpreters/ClientInfo.h>
-
 
 namespace DB
 {
 
+class ClientInfo;
 class Connection;
 class ReadBuffer;
 struct Settings;
@@ -26,6 +24,14 @@ public:
         const Settings & settings_,
         const ClientInfo & client_info_);
 
+    void initialize();
+
+    Block initializeAndGetHeader()
+    {
+        initialize();
+        return header;
+    }
+
     void write(Block block);
     void onFinish();
 
@@ -38,8 +44,13 @@ public:
     UInt64 getServerRevision() const { return server_revision; }
 
 private:
+    const Settings & insert_settings;
+    const ClientInfo & client_info;
+    const ConnectionTimeouts & timeouts;
+
     Connection & connection;
     String query;
+
     Block header;
     bool finished = false;
     UInt64 server_revision;
