@@ -8,11 +8,17 @@
 #include <Poco/Net/Context.h>
 #include <Poco/Net/SSLManager.h>
 #include <Poco/Net/Utility.h>
+#include <Server/ACME/Client.h>
 
 
 namespace DB
 {
 
+namespace ErrorCodes
+{
+    extern const int INVALID_CONFIG_PARAMETER;
+}
+
 namespace
 {
 
@@ -32,6 +38,7 @@ int callSetCertificate(SSL * ssl, void * arg)
 int CertificateReloader::setCertificate(SSL * ssl, const CertificateReloader::MultiData * pdata)
 {
     auto current = pdata->data.get();
+
     if (!current)
         return -1;
     return setCertificateCallback(ssl, current.get(), log);
@@ -86,9 +93,9 @@ void CertificateReloader::init(MultiData * pdata)
     LOG_DEBUG(log, "Initializing certificate reloader.");
 
     /// Set a callback for OpenSSL to allow get the updated cert and key.
-
     SSL_CTX_set_cert_cb(pdata->ctx, callSetCertificate, reinterpret_cast<void *>(pdata));
-    pdata->init_was_not_made = false;
+
+    pdata->initialized = true;
 }
 
 
@@ -133,46 +140,87 @@ std::list<CertificateReloader::MultiData>::iterator CertificateReloader::findOrI
     return it;
 }
 
+void CertificateReloader::tryLoadACMECertificate(SSL_CTX * ctx, const std::string & prefix)
+{
+    try
+    {
+        auto it = findOrInsert(ctx, prefix);
+        if (!it->initialized)
+            init(&*it);
+
+        auto key_certificate_pair = ACME::Client::instance().requestCertificate();
+        if (!key_certificate_pair)
+        {
+            LOG_WARNING(log, "ACME certificate is not ready yet.");
+            return;
+        }
+
+        auto current_version = it->data.get();
+        if (current_version && current_version->hash == key_certificate_pair->version)
+            return;
+
+        LOG_DEBUG(log, "Reloading ACME certificate and key.");
+        it->data.set(std::make_unique<const Data>(
+            std::move(key_certificate_pair->private_key),
+            std::move(key_certificate_pair->certificate),
+            key_certificate_pair->version
+        ));
+        LOG_INFO(log, "Reloaded ACME certificate and key.");
+    }
+    catch (...)
+    {
+        LOG_ERROR(log, getCurrentExceptionMessageAndPattern(/* with_stacktrace */ false));
+    }
+}
 
 void CertificateReloader::tryLoadImpl(const Poco::Util::AbstractConfiguration & config, SSL_CTX * ctx, const std::string & prefix)
 {
     /// If at least one of the files is modified - recreate
-
     std::string new_cert_path = config.getString(prefix + "certificateFile", "");
     std::string new_key_path = config.getString(prefix + "privateKeyFile", "");
 
+    if (config.has("acme") && prefix == Poco::Net::SSLManager::CFG_SERVER_PREFIX)
+    {
+        if (!new_cert_path.empty() || !new_key_path.empty())
+            throw Exception(ErrorCodes::INVALID_CONFIG_PARAMETER, "Static TLS keys and ACME provider are enabled at the same time.");
+
+        tryLoadACMECertificate(ctx, prefix);
+
+        return;
+    }
+
     /// For empty paths (that means, that user doesn't want to use certificates)
     /// no processing required
-
     if (new_cert_path.empty() || new_key_path.empty())
     {
         LOG_INFO(log, "One of paths is empty. Cannot apply new configuration for certificates. Fill all paths and try again.");
+        return;
     }
-    else
+
+    try
     {
-        try
+        auto it = findOrInsert(ctx, prefix);
+
+        bool cert_file_changed = it->cert_file.changeIfModified(std::move(new_cert_path), log);
+        bool key_file_changed = it->key_file.changeIfModified(std::move(new_key_path), log);
+
+        if (cert_file_changed || key_file_changed)
         {
-            auto it = findOrInsert(ctx, prefix);
+            LOG_DEBUG(log, "Reloading certificate ({}) and key ({}).", it->cert_file.path, it->key_file.path);
 
-            bool cert_file_changed = it->cert_file.changeIfModified(std::move(new_cert_path), log);
-            bool key_file_changed = it->key_file.changeIfModified(std::move(new_key_path), log);
+            std::string pass_phrase = config.getString(prefix + "privateKeyPassphraseHandler.options.password", "");
+            it->data.set(std::make_unique<const Data>(it->cert_file.path, it->key_file.path, pass_phrase));
 
-            if (cert_file_changed || key_file_changed)
-            {
-                LOG_DEBUG(log, "Reloading certificate ({}) and key ({}).", it->cert_file.path, it->key_file.path);
-                std::string pass_phrase = config.getString(prefix + "privateKeyPassphraseHandler.options.password", "");
-                it->data.set(std::make_unique<const Data>(it->cert_file.path, it->key_file.path, pass_phrase));
-                LOG_INFO(log, "Reloaded certificate ({}) and key ({}).", it->cert_file.path, it->key_file.path);
-            }
-
-            /// If callback is not set yet
-            if (it->init_was_not_made)
-                init(&*it);
-        }
-        catch (...)
-        {
-            LOG_ERROR(log, getCurrentExceptionMessageAndPattern(/* with_stacktrace */ false));
+            LOG_INFO(log, "Reloaded certificate ({}) and key ({}).", it->cert_file.path, it->key_file.path);
         }
+
+        /// If callback is not set yet
+        if (!it->initialized)
+            init(&*it);
+    }
+    catch (...)
+    {
+        LOG_ERROR(log, getCurrentExceptionMessageAndPattern(/* with_stacktrace */ false));
     }
 }
 
@@ -190,6 +238,11 @@ CertificateReloader::Data::Data(std::string cert_path, std::string key_path, std
 {
 }
 
+CertificateReloader::Data::Data(KeyPair _pkey, X509Certificate::List _certs_chain, std::string _hash)
+    : certs_chain(std::move(_certs_chain)), key(std::move(_pkey)), hash(std::move(_hash))
+{
+}
+
 
 bool CertificateReloader::File::changeIfModified(std::string new_path, LoggerPtr logger)
 {
@@ -197,8 +250,12 @@ bool CertificateReloader::File::changeIfModified(std::string new_path, LoggerPtr
     std::filesystem::file_time_type new_modification_time = std::filesystem::last_write_time(new_path, ec);
     if (ec)
     {
-        LOG_ERROR(logger, "Cannot obtain modification time for {} file {}, skipping update. {}",
-            description, new_path, errnoToString(ec.value()));
+        LOG_ERROR(
+            logger,
+            "Cannot obtain modification time for {} file {}, skipping update. {}",
+            description,
+            new_path,
+            errnoToString(ec.value()));
         return false;
     }
 
