@@ -1,19 +1,22 @@
-#include <Server/HTTPHandlerFactory.h>
-
 #include <Server/HTTP/HTTPRequestHandler.h>
+#include <Server/HTTPHandler.h>
+#include <Server/HTTPHandlerFactory.h>
+#include <Server/IServer.h>
+#include <Server/IndexRequestHandler.h>
+#include <Server/InterserverIOHTTPHandler.h>
 #include <Server/PrometheusMetricsWriter.h>
 #include <Server/PrometheusRequestHandlerFactory.h>
-#include <Server/IServer.h>
+#include <Server/ReplicasStatusHandler.h>
+#include <Server/StaticRequestHandler.h>
+#include <Server/WebUIRequestHandler.h>
+
+#if USE_SSL
+#include <Server/ACME/RequestHandler.h>
+#include <Server/ACME/Client.h>
+#endif
 
 #include <Poco/Util/AbstractConfiguration.h>
 
-#include <Server/HTTPHandler.h>
-#include <Server/StaticRequestHandler.h>
-#include <Server/ReplicasStatusHandler.h>
-#include <Server/InterserverIOHTTPHandler.h>
-#include <Server/IndexRequestHandler.h>
-#include <Server/WebUIRequestHandler.h>
-
 
 namespace DB
 {
@@ -233,6 +236,14 @@ static inline auto createHandlersFactoryFromConfig(
                 handler->addFiltersFromConfig(config, prefix + "." + key);
                 main_handler_factory->addHandler(std::move(handler));
             }
+#if USE_SSL
+            else if (handler_type == "acme")
+            {
+                auto handler = std::make_shared<HandlingRuleHTTPHandlerFactory<ACMERequestHandler>>(server);
+                handler->addFiltersFromConfig(config, prefix + "." + key);
+                main_handler_factory->addHandler(std::move(handler));
+            }
+#endif
             else
                 throw Exception(ErrorCodes::INVALID_CONFIG_PARAMETER, "Unknown handler type '{}' in config here: {}.{}.handler.type",
                     handler_type, prefix, key);
@@ -270,7 +281,6 @@ static inline HTTPRequestHandlerFactoryPtr createInterserverHTTPHandlerFactory(I
     return factory;
 }
 
-
 HTTPRequestHandlerFactoryPtr createHandlerFactory(IServer & server, const Poco::Util::AbstractConfiguration & config, AsynchronousMetrics & async_metrics, const std::string & name)
 {
     if (name == "HTTPHandler-factory" || name == "HTTPSHandler-factory")
@@ -348,6 +358,16 @@ void addCommonDefaultHandlersFactory(HTTPRequestHandlerFactoryMain & factory, IS
     js_handler->attachNonStrictPath("/js/");
     js_handler->allowGetAndHeadRequest();
     factory.addHandler(js_handler);
+
+#if USE_SSL
+    if (server.config().has("acme"))
+    {
+        auto acme_handler = std::make_shared<HandlingRuleHTTPHandlerFactory<ACMERequestHandler>>(server);
+        acme_handler->attachNonStrictPath(ACME::CHALLENGE_HTTP_PATH);
+        acme_handler->allowGetAndHeadRequest();
+        factory.addHandler(acme_handler);
+    }
+#endif
 }
 
 void addDefaultHandlersFactory(
