@@ -1,14 +1,15 @@
 #pragma once
 
-#include <atomic>
-#include <memory>
 #include <Server/IServer.h>
 #include <Server/TCPServerConnectionFactory.h>
+#include <Common/ProfileEvents.h>
 
-#include <Common/config.h>
+#include "config.h"
+
+#include <atomic>
 
 #if USE_SSL
-#    include <openssl/rsa.h>
+#    include <Common/Crypto/KeyPair.h>
 #endif
 
 namespace DB
@@ -19,26 +20,24 @@ class MySQLHandlerFactory : public TCPServerConnectionFactory
 {
 private:
     IServer & server;
-    Poco::Logger * log;
+    LoggerPtr log;
 
 #if USE_SSL
-    struct RSADeleter
-    {
-        void operator()(RSA * ptr) { RSA_free(ptr); }
-    };
-    using RSAPtr = std::unique_ptr<RSA, RSADeleter>;
-
-    RSAPtr public_key;
-    RSAPtr private_key;
+    KeyPair keypair;
 
     bool ssl_enabled = true;
 #else
     bool ssl_enabled = false;
 #endif
 
-    std::atomic<size_t> last_connection_id = 0;
+    bool secure_required = false;
+
+    std::atomic<unsigned> last_connection_id = 0;
+
+    ProfileEvents::Event read_event;
+    ProfileEvents::Event write_event;
 public:
-    explicit MySQLHandlerFactory(IServer & server_);
+    explicit MySQLHandlerFactory(IServer & server_, bool secure_required_, const ProfileEvents::Event & read_event_ = ProfileEvents::end(), const ProfileEvents::Event & write_event_ = ProfileEvents::end());
 
     void readRSAKeys();
 
