@@ -26,8 +26,12 @@
 #include <Server/HTTP/authenticateUserByHTTP.h>
 #include <Server/HTTP/checkHTTPHeader.h>
 #include <Server/HTTP/setReadOnlyIfHTTPMethodIdempotent.h>
+#include <IO/WriteBufferFromString.h>
+#include <IO/WriteHelpers.h>
+#include <Core/Settings.h>
 #include <Storages/TimeSeries/PrometheusRemoteReadProtocol.h>
 #include <Storages/TimeSeries/PrometheusRemoteWriteProtocol.h>
+#include <Storages/TimeSeries/PrometheusHTTPProtocolAPI.h>
 
 
 namespace DB
@@ -37,6 +41,7 @@ namespace ErrorCodes
 {
     extern const int BAD_ARGUMENTS;
     extern const int SUPPORT_IS_DISABLED;
+    extern const int NOT_IMPLEMENTED;
 }
 
 /// Base implementation of a prometheus protocol.
@@ -46,6 +51,7 @@ public:
     explicit Impl(PrometheusRequestHandler & parent) : parent_ref(parent) {}
     virtual ~Impl() = default;
     virtual void beforeHandlingRequest(HTTPServerRequest & /* request */) {}
+    virtual bool isSettingLikeParameter(const String & /* name */) { return false; }
     virtual void handleRequest(HTTPServerRequest & request, HTTPServerResponse & response) = 0;
     virtual void onException() {}
 
@@ -149,6 +155,18 @@ protected:
         return authenticateUserByHTTP(request, *params, response, *session, request_credentials, config().connection_config, server().context(), log());
     }
 
+    bool isSettingLikeParameter(const String & name) override
+    {
+        /// Empty parameter appears when URL like ?&a=b or a=b&&c=d. Just skip them for user's convenience.
+        if (name.empty())
+            return false;
+
+        /// Some parameters (database, default_format, everything used in the code above) do not
+        /// belong to the Settings class.
+        static const NameSet reserved_param_names{"user", "password", "quota_key", "stacktrace", "role", "query_id"};
+        return !reserved_param_names.contains(name);
+    }
+
     void makeContext(HTTPServerRequest & request)
     {
         context = session->makeQueryContext();
@@ -160,23 +178,10 @@ protected:
         if (!roles.empty())
             context->setCurrentRoles(roles);
 
-        /// Settings can be overridden in the URL query.
-        auto is_setting_like_parameter = [&] (const String & name)
-        {
-            /// Empty parameter appears when URL like ?&a=b or a=b&&c=d. Just skip them for user's convenience.
-            if (name.empty())
-                return false;
-
-            /// Some parameters (database, default_format, everything used in the code above) do not
-            /// belong to the Settings class.
-            static const NameSet reserved_param_names{"user", "password", "quota_key", "stacktrace", "role", "query_id"};
-            return !reserved_param_names.contains(name);
-        };
-
         SettingsChanges settings_changes;
         for (const auto & [key, value] : *params)
         {
-            if (is_setting_like_parameter(key))
+            if (isSettingLikeParameter(key))
             {
                 /// This query parameter should be considered as a ClickHouse setting.
                 settings_changes.push_back({key, value});
@@ -314,6 +319,147 @@ public:
     }
 };
 
+/// Handles Prometheus Query API endpoints (/api/v1/query, /api/v1/query_range, etc.)
+class PrometheusRequestHandler::QueryAPIImpl : public ImplWithContext
+{
+public:
+    using ImplWithContext::ImplWithContext;
+
+    void beforeHandlingRequest(HTTPServerRequest & request) override
+    {
+        LOG_INFO(log(), "Handling Prometheus Query API request from {}", request.get("User-Agent", ""));
+        chassert(config().type == PrometheusRequestHandlerConfig::Type::QueryAPI);
+    }
+
+    bool isSettingLikeParameter(const String & name) override
+    {
+        /// Empty parameter appears when URL like ?&a=b or a=b&&c=d. Just skip them for user's convenience.
+        if (name.empty())
+            return false;
+
+        /// Some parameters (database, default_format, everything used in the code above) do not
+        /// belong to the Settings class.
+        static const NameSet reserved_param_names{"user", "password", "query", "time", "start", "end", "step"};
+        return !reserved_param_names.contains(name);
+    }
+
+    void handlingRequestWithContext(HTTPServerRequest & request, HTTPServerResponse & response) override
+    {
+        auto table = DatabaseCatalog::instance().getTable(StorageID{config().time_series_table_name}, context);
+        PrometheusHTTPProtocolAPI protocol{table, context};
+
+
+        const String & uri = request.getURI();
+        LOG_DEBUG(log(), "Processing Query API request: method={}, uri={}", request.getMethod(), uri);
+
+        response.setContentType("application/json");
+        try
+        {
+            if (uri.starts_with("/api/v1/query_range"))
+            {
+                String query = params->get("query", "");
+                String start = params->get("start", "");
+                String end = params->get("end", "");
+                String step = params->get("step", "");
+
+                /// TODO: Support the following **optional** query parameters:
+                /// - timeout=<duration>: Evaluation timeout
+                /// - limit=<number>: Maximum number of returned series
+                /// - lookback_delta=<number>: Override for the lookback period for this query.
+
+                PrometheusHTTPProtocolAPI::Params params
+                {
+                    .type = PrometheusHTTPProtocolAPI::Type::Range,
+                    .promql_query = query,
+                    .time_param = "",
+                    .start_param = start,
+                    .end_param = end,
+                    .step_param = step,
+                };
+
+                protocol.executePromQLQuery(getOutputStream(response), params);
+            }
+            else if (uri.starts_with("/api/v1/query"))
+            {
+                String query = params->get("query", "");
+                String time = params->get("time", "");
+
+                /// TODO: Support optional parameters same as for the range query.
+
+                PrometheusHTTPProtocolAPI::Params params
+                {
+                    .type = PrometheusHTTPProtocolAPI::Type::Instant,
+                    .promql_query = query,
+                    .time_param = time,
+                    .start_param = "",
+                    .end_param = "",
+                    .step_param = "",
+                };
+
+                protocol.executePromQLQuery(getOutputStream(response), params);
+            }
+            else if (uri.starts_with("/api/v1/format_query"))
+            {
+                throw Exception(ErrorCodes::NOT_IMPLEMENTED, "The format_query endpoint is not implemented");
+            }
+            else if (uri.starts_with("/api/v1/parse_query"))
+            {
+                throw Exception(ErrorCodes::NOT_IMPLEMENTED, "The parse_query endpoint is not implemented");
+            }
+            else if (uri.starts_with("/api/v1/series"))
+            {
+                String match = params->get("match[]", "");
+                String start = params->get("start", "");
+                String end = params->get("end", "");
+
+                /// TODO: Support limit=<number> optional parameter
+
+                protocol.getSeries(getOutputStream(response), match, start, end);
+            }
+            else if (uri.starts_with("/api/v1/labels"))
+            {
+                String match = params->get("match[]", "");
+                String start = params->get("start", "");
+                String end = params->get("end", "");
+
+                protocol.getLabels(getOutputStream(response), match, start, end);
+            }
+            else if (uri.find("/api/v1/label/") != String::npos && uri.ends_with("/values"))
+            {
+                // Extract label name from URI: /api/v1/label/<name>/values
+                size_t start_pos = uri.find("/api/v1/label/") + 14; // length of "/api/v1/label/"
+                size_t end_pos = uri.find("/values");
+                String label_name = uri.substr(start_pos, end_pos - start_pos);
+
+                String match = params->get("match[]", "");
+                String start = params->get("start", "");
+                String end = params->get("end", "");
+
+                protocol.getLabelValues(getOutputStream(response), label_name, match, start, end);
+            }
+            else
+            {
+                LOG_ERROR(log(), "No matching endpoint found for URI: {}, method: {}", uri, request.getMethod());
+                response.setStatusAndReason(Poco::Net::HTTPResponse::HTTP_NOT_FOUND);
+                writeString(R"({"status":"error","errorType":"not_found","error":"API endpoint not found"})", getOutputStream(response));
+            }
+        }
+        catch (const Exception & e)
+        {
+            response.setStatusAndReason(Poco::Net::HTTPResponse::HTTP_BAD_REQUEST);
+            String error_str;
+            WriteBufferFromString error_buf(error_str);
+            writeString(R"({"status":"error","errorType":"bad_data","error":")", error_buf);
+            writeString(e.message(), error_buf);
+            writeString(R"("})", error_buf);
+            error_buf.finalize();
+            writeString(error_str, getOutputStream(response));
+
+            LOG_ERROR(log(), "Error executing query: {}", e.displayText());
+        }
+    }
+};
+
 
 PrometheusRequestHandler::PrometheusRequestHandler(
     IServer & server_,
@@ -352,13 +498,18 @@ void PrometheusRequestHandler::createImpl()
             impl = std::make_unique<RemoteReadImpl>(*this);
             return;
         }
+        case PrometheusRequestHandlerConfig::Type::QueryAPI:
+        {
+            impl = std::make_unique<QueryAPIImpl>(*this);
+            return;
+        }
     }
     UNREACHABLE();
 }
 
 void PrometheusRequestHandler::handleRequest(HTTPServerRequest & request, HTTPServerResponse & response, const ProfileEvents::Event & write_event_)
 {
-    setThreadName("PrometheusHndlr");
+    DB::setThreadName(ThreadName::PROMETHEUS_HANDLER);
     applyHTTPResponseHeaders(response, response_headers);
 
     try
