@@ -66,17 +66,10 @@ struct GetColumnsOptions
         return *this;
     }
 
-    GetColumnsOptions & withExtendedObjects(bool value = true)
-    {
-        with_extended_objects = value;
-        return *this;
-    }
-
     Kind kind;
     VirtualsKind virtuals_kind = VirtualsKind::None;
 
     bool with_subcolumns = false;
-    bool with_extended_objects = false;
 };
 
 /// Description of a single table column (in CREATE TABLE for example).
@@ -117,7 +110,7 @@ public:
 
     static ColumnsDescription fromNamesAndTypes(NamesAndTypes ordinary);
 
-    explicit ColumnsDescription(NamesAndTypesList ordinary);
+    explicit ColumnsDescription(NamesAndTypesList ordinary, bool with_subcolumns = true);
 
     ColumnsDescription(std::initializer_list<ColumnDescription> ordinary);
 
