@@ -1,9 +1,10 @@
 #pragma once
 
-#include <Processors/Sources/SourceWithProgress.h>
+#include <Processors/ISource.h>
 
 #include <Storages/Kafka/StorageKafka.h>
-#include <Storages/Kafka/ReadBufferFromKafkaConsumer.h>
+#include <Storages/Kafka/KafkaConsumer.h>
+#include <Common/Stopwatch.h>
 
 
 namespace Poco
@@ -13,15 +14,15 @@ namespace Poco
 namespace DB
 {
 
-class KafkaSource : public SourceWithProgress
+class KafkaSource : public ISource
 {
 public:
     KafkaSource(
         StorageKafka & storage_,
-        const StorageMetadataPtr & metadata_snapshot_,
+        const StorageSnapshotPtr & storage_snapshot_,
         const ContextPtr & context_,
         const Names & columns,
-        Poco::Logger * log_,
+        LoggerPtr log_,
         size_t max_block_size_,
         bool commit_in_suffix = false);
     ~KafkaSource() override;
@@ -31,24 +32,31 @@ public:
     Chunk generate() override;
 
     void commit();
-    bool isStalled() const { return !buffer || buffer->isStalled(); }
+    bool isStalled() const { return !consumer || consumer->isStalled(); }
+
+    void setTimeLimit(Poco::Timespan max_execution_time_) { max_execution_time = max_execution_time_; }
 
 private:
     StorageKafka & storage;
-    StorageMetadataPtr metadata_snapshot;
+    StorageSnapshotPtr storage_snapshot;
     ContextPtr context;
     Names column_names;
-    Poco::Logger * log;
+    LoggerPtr log;
     UInt64 max_block_size;
 
-    ConsumerBufferPtr buffer;
+    KafkaConsumerPtr consumer;
     bool broken = true;
     bool is_finished = false;
     bool commit_in_suffix;
 
     const Block non_virtual_header;
     const Block virtual_header;
-    const HandleKafkaErrorMode handle_error_mode;
+    const StreamingHandleErrorMode handle_error_mode;
+
+    Poco::Timespan max_execution_time = 0;
+    Stopwatch total_stopwatch {CLOCK_MONOTONIC_COARSE};
+
+    bool checkTimeLimit() const;
 
     Chunk generateImpl();
 };
