@@ -53,6 +53,7 @@ private:
 };
 
 using RangesByIndex = std::unordered_map<size_t, RangesInDataPart>;
+using ProjectionRangesByIndex = std::unordered_map<size_t, RangesInDataParts>;
 class MergeTreeIndexReadResultPool;
 using MergeTreeIndexReadResultPoolPtr = std::shared_ptr<MergeTreeIndexReadResultPool>;
 
@@ -64,12 +65,15 @@ struct MutableAtomicSizeT
 
 using PartRemainingMarks = std::unordered_map<size_t, MutableAtomicSizeT>;
 
-/// Provides shared context needed to build filtering indexes (e.g., skip indexes) during data reads.
+/// Provides shared context needed to build filtering indexes (e.g., skip indexes or projection indexes) during data reads.
 struct MergeTreeIndexBuildContext
 {
     /// For each part, stores all ranges need to be read.
     const RangesByIndex read_ranges;
 
+    /// For each part, stores a set of read ranges grouped by projection.
+    const ProjectionRangesByIndex projection_read_ranges;
+
     /// Thread-safe shared pool for reading and building index filters. Must not be null (enforced in constructor).
     const MergeTreeIndexReadResultPoolPtr index_reader_pool;
 
@@ -79,7 +83,8 @@ struct MergeTreeIndexBuildContext
 
     MergeTreeIndexBuildContext(
         RangesByIndex read_ranges_,
-        MergeTreeIndexReadResultPoolPtr index_reader_,
+        ProjectionRangesByIndex projection_read_ranges_,
+        MergeTreeIndexReadResultPoolPtr index_reader_pool_,
         PartRemainingMarks part_remaining_marks_);
 
     MergeTreeIndexReadResultPtr getPreparedIndexReadResult(const MergeTreeReadTask & task) const;
@@ -112,6 +117,12 @@ public:
 
     Block getHeader() const { return result_header; }
 
+    /// Reads a single MergeTreeReadTask in a stateless manner.
+    /// Can be called concurrently and is used, for example, by SingleProjectionIndexReader.
+    ChunkAndProgress readCurrentTask(MergeTreeReadTask & current_task, IMergeTreeSelectAlgorithm & task_algorithm) const;
+
+    /// Reads using the standard task-based algorithm, managing task state internally.
+    /// Not thread-safe: must not be called concurrently on the same MergeTreeSelectProcessor instance.
     ChunkAndProgress read();
 
     void cancel() noexcept;
@@ -131,10 +142,9 @@ public:
     void onFinish() const;
 
 private:
-    static void injectLazilyReadColumns(size_t rows, Block & block, size_t part_index, const LazilyReadInfoPtr & lazily_read_info);
+    friend class SingleProjectionIndexReader;
 
-    /// Sets up range readers corresponding to data readers
-    void initializeReadersChain();
+    static void injectLazilyReadColumns(size_t rows, Block & block, size_t part_index, const LazilyReadInfoPtr & lazily_read_info);
 
     const MergeTreeReadPoolPtr pool;
     const MergeTreeSelectAlgorithmPtr algorithm;
@@ -166,6 +176,4 @@ private:
     std::atomic<bool> is_cancelled{false};
 };
 
-using MergeTreeSelectProcessorPtr = std::unique_ptr<MergeTreeSelectProcessor>;
-
 }
