@@ -11,20 +11,20 @@ namespace DB
 /// ReplicatedMergeTreeLogEntry.
 ///
 /// Order is important, don't try to change it.
-enum class MergeType
+enum class MergeType : uint8_t
 {
     /// Just regular merge
-    REGULAR = 1,
-    /// Merge assigned to delete some data from parts (with TTLMergeSelector)
-    TTL_DELETE = 2,
+    Regular = 1,
+    /// Merge assigned to delete rows from parts
+    TTLDelete = 2,
     /// Merge with recompression
-    TTL_RECOMPRESS = 3,
+    TTLRecompress = 3,
+    /// Merge assigned to drop parts completely
+    TTLDrop = 4,
 };
 
 /// Check parsed merge_type from raw int and get enum value.
-MergeType checkAndGetMergeType(UInt64 merge_type);
-
-String toString(MergeType merge_type);
+MergeType checkAndGetMergeType(UInt32 merge_type);
 
 /// Check this merge assigned with TTL
 bool isTTLMergeType(MergeType merge_type);
