@@ -6,6 +6,7 @@
 #include <Interpreters/Context.h>
 #include <Common/ErrorCodes.h>
 #include <Common/ProfileEventsScope.h>
+#include <Common/setThreadName.h>
 #include <Core/Settings.h>
 
 namespace DB
@@ -75,6 +76,11 @@ void MutatePlainMergeTreeTask::prepare()
             time(nullptr), task_context, merge_mutate_entry->txn, merge_mutate_entry->tagger->reserved_space, table_lock_holder);
 }
 
+void MutatePlainMergeTreeTask::finish()
+{
+    if (merge_mutate_entry)
+        merge_mutate_entry->finalize();
+}
 
 bool MutatePlainMergeTreeTask::executeStep()
 {
@@ -84,7 +90,7 @@ bool MutatePlainMergeTreeTask::executeStep()
     /// Make out memory tracker a parent of current thread memory tracker
     std::optional<ThreadGroupSwitcher> switcher;
     if (merge_list_entry)
-        switcher.emplace((*merge_list_entry)->thread_group, "", /*allow_existing_group*/ true);
+        switcher.emplace((*merge_list_entry)->thread_group, ThreadName::MERGE_MUTATE, /*allow_existing_group*/ true);
 
     switch (state)
     {
@@ -137,6 +143,7 @@ bool MutatePlainMergeTreeTask::executeStep()
         case State::NEED_FINISH:
         {
             // Nothing to do
+            finish();
             state = State::SUCCESS;
             return false;
         }
@@ -156,6 +163,9 @@ void MutatePlainMergeTreeTask::cancel() noexcept
 
     if (new_part)
         new_part->removeIfNeeded();
+
+    if (merge_mutate_entry)
+        merge_mutate_entry->finalize();
 }
 
 
