@@ -1,4 +1,4 @@
-#include "PinnedPartUUIDs.h"
+#include <Storages/MergeTree/PinnedPartUUIDs.h>
 #include <IO/ReadHelpers.h>
 #include <IO/WriteHelpers.h>
 #include <Poco/JSON/JSON.h>
