@@ -10,17 +10,11 @@ namespace DB
 IMergeTreeDataPart::Checksums checkDataPart(
     MergeTreeData::DataPartPtr data_part,
     bool require_checksums,
-    std::function<bool()> is_cancelled = []{ return false; });
-
-IMergeTreeDataPart::Checksums checkDataPart(
-    const DiskPtr & disk,
-    const String & full_relative_path,
-    const NamesAndTypesList & columns_list,
-    const MergeTreeDataPartType & part_type,
-    const NameSet & files_without_checksums,
-    bool require_checksums,
-    std::function<bool()> is_cancelled = []{ return false; });
+    bool & is_broken_projection,
+    std::function<bool()> is_cancelled = []{ return false; },
+    bool throw_on_broken_projection = false);
 
 bool isNotEnoughMemoryErrorCode(int code);
+bool isRetryableException(std::exception_ptr exception_ptr);
 
 }
