@@ -16,6 +16,7 @@
 #include <Interpreters/Context.h>
 #include <azure/identity/managed_identity_credential.hpp>
 #include <azure/identity/workload_identity_credential.hpp>
+#include <azure/identity/client_secret_credential.hpp>
 #include <Core/Settings.h>
 #include <Common/RemoteHostFilter.h>
 #include <Parsers/ASTIdentifier.h>
@@ -151,6 +152,55 @@ static AzureBlobStorage::ConnectionParams getConnectionParams(
     return connection_params;
 }
 
+void StorageAzureConfiguration::fillBlobsFromURLCommon(String & connection_url, const String & suffix, const String & full_suffix)
+{
+    String container_name;
+
+    auto pos_container = connection_url.find(suffix);
+
+    if (pos_container != std::string::npos)
+    {
+        String container_blob_path = connection_url.substr(pos_container+5);
+        connection_url = connection_url.substr(0,pos_container+4);
+        container_name = connection_url.substr(pos_container+4);
+        auto pos_blob_path = container_blob_path.find('/');
+
+        if (pos_blob_path != std::string::npos)
+        {
+            container_name = container_blob_path.substr(0, pos_blob_path);
+            blob_path = container_blob_path.substr(pos_blob_path);
+        }
+    }
+
+    /// Added for Unity Catalog on top of AzureBlobStorage
+    // Sample abfss url : abfss://mycontainer@mydatalakestorage.dfs.core.windows.net/subdirectory/file.txt
+    if (connection_url.starts_with("abfss"))
+    {
+        auto pos_slash = connection_url.find("://");
+        auto pos_at = connection_url.find('@');
+        auto pos_dot = connection_url.find('.');
+        auto pos_net = connection_url.find(suffix);
+
+        if (pos_slash == std::string::npos || pos_at == std::string::npos|| pos_dot == std::string::npos || pos_net == std::string::npos
+            || pos_at-pos_slash-3 <= 0 || pos_dot-pos_at-1 <= 0)
+        {
+            throw Exception(ErrorCodes::LOGICAL_ERROR, "Incorrect url format for a abfss url {}", connection_url);
+        }
+        auto container_name_abfss = connection_url.substr(pos_slash+3, pos_at-pos_slash-3);
+        auto name = connection_url.substr(pos_at+1, pos_dot-pos_at-1);
+
+        connection_params.endpoint.storage_account_url = "https://" + name + full_suffix;
+
+        if (!container_name.empty())
+        {
+            blob_path.path = container_name + blob_path.path;
+        }
+        connection_params.endpoint.container_name = container_name_abfss;
+    }
+    blobs_paths = {blob_path};
+}
+
+
 void StorageAzureConfiguration::fromNamedCollection(const NamedCollection & collection, ContextPtr context)
 {
     validateNamedCollection(collection, required_configuration_keys, optional_configuration_keys);
@@ -291,8 +341,34 @@ void StorageAzureConfiguration::fromDisk(const String & disk_name, ASTs & args,
         structure = *parsing_result.structure;
 }
 
+void StorageAzureConfiguration::initializeForOneLake(ASTs & args, ContextPtr context)
+{
+    if (args.size() != 1)
+        throw Exception(ErrorCodes::BAD_ARGUMENTS, "Only one argument should be provided in OneLake catalog");
+
+    String connection_url = checkAndGetLiteralArgument<String>(args[0], "connection_string/storage_account_url");
+
+    fillBlobsFromURLCommon(connection_url, ".com", ".dfs.fabric.microsoft.com");
+    connection_params.endpoint.additional_params = "resource=REDACTED&directory=REDACTED&recursive=REDACTED";
+
+    connection_params.auth_method = std::make_shared<Azure::Identity::ClientSecretCredential>(
+        onelake_tenant_id,
+        onelake_client_id,
+        onelake_client_secret
+    );
+
+    auto request_settings = AzureBlobStorage::getRequestSettings(context->getSettingsRef());
+    connection_params.client_options = AzureBlobStorage::getClientOptions(context, context->getSettingsRef(), *request_settings, /*for_disk=*/ false);
+}
+
 void StorageAzureConfiguration::fromAST(ASTs & engine_args, ContextPtr context, bool with_structure)
 {
+    if (!onelake_client_id.empty())
+    {
+        initializeForOneLake(engine_args, context);
+        return;
+    }
+
     auto extra_credentials = extractExtraCredentials(engine_args);
 
     if (engine_args.empty() || engine_args.size() > getMaxNumberOfArguments(with_structure))
@@ -320,51 +396,7 @@ void StorageAzureConfiguration::fromAST(ASTs & engine_args, ContextPtr context,
     {
         String connection_url = checkAndGetLiteralArgument<String>(engine_args[0], "connection_string/storage_account_url");
         String sas_token = checkAndGetLiteralArgument<String>(engine_args[1], "sas_token");
-        String container_name;
-
-        auto pos_container = connection_url.find(".net");
-
-        if (pos_container != std::string::npos)
-        {
-            String container_blob_path = connection_url.substr(pos_container+5);
-            connection_url = connection_url.substr(0,pos_container+4);
-            container_name = connection_url.substr(pos_container+4);
-            auto pos_blob_path = container_blob_path.find('/');
-
-            if (pos_blob_path != std::string::npos)
-            {
-                container_name = container_blob_path.substr(0, pos_blob_path);
-                blob_path = container_blob_path.substr(pos_blob_path);
-            }
-        }
-
-        /// Added for Unity Catalog on top of AzureBlobStorage
-        // Sample abfss url : abfss://mycontainer@mydatalakestorage.dfs.core.windows.net/subdirectory/file.txt
-        if (connection_url.starts_with("abfss"))
-        {
-            auto pos_slash = connection_url.find("://");
-            auto pos_at = connection_url.find('@');
-            auto pos_dot = connection_url.find('.');
-            auto pos_net = connection_url.find(".net");
-
-            if (pos_slash == std::string::npos || pos_at == std::string::npos|| pos_dot == std::string::npos || pos_net == std::string::npos
-                || pos_at-pos_slash-3 <= 0 || pos_dot-pos_at-1 <= 0)
-            {
-                throw Exception(ErrorCodes::LOGICAL_ERROR, "Incorrect url format for a abfss url {}", connection_url);
-            }
-            auto container_name_abfss = connection_url.substr(pos_slash+3, pos_at-pos_slash-3);
-            auto name = connection_url.substr(pos_at+1, pos_dot-pos_at-1);
-
-            connection_params.endpoint.storage_account_url = "https://" + name + ".blob.core.windows.net";
-
-            if (!container_name.empty())
-            {
-                blob_path.path = container_name + blob_path.path;
-            }
-            connection_params.endpoint.container_name = container_name_abfss;
-        }
-
-        blobs_paths = {blob_path};
+        fillBlobsFromURLCommon(connection_url, ".net", ".blob.core.windows.net");
         connection_params.endpoint.sas_auth = sas_token;
 
         return;
