@@ -476,7 +476,8 @@ StorageInfo PostgreSQLReplicationHandler::loadFromSnapshot(postgres::Connection
         /// We should not use columns list from getTableAllowedColumns because it may have broken columns order
         Strings allowed_columns;
         for (const auto & column : table_structure->physical_columns->columns)
-            allowed_columns.push_back(column.name);
+            allowed_columns.push_back(doubleQuoteString(column.name));
+
         query_str = fmt::format("SELECT {} FROM ONLY {}", boost::algorithm::join(allowed_columns, ","), quoted_name);
     }
 
