@@ -1,6 +1,6 @@
 #pragma once
 
-#include <Storages/RabbitMQ/UVLoop.h>
+#include <Storages/UVLoop.h>
 #include <Storages/RabbitMQ/RabbitMQHandler.h>
 
 
@@ -22,7 +22,7 @@ struct RabbitMQConfiguration
 class RabbitMQConnection
 {
 public:
-    RabbitMQConnection(const RabbitMQConfiguration & configuration_, Poco::Logger * log_);
+    RabbitMQConnection(const RabbitMQConfiguration & configuration_, LoggerPtr log_);
 
     bool isConnected();
 
@@ -51,12 +51,14 @@ private:
     void disconnectImpl(bool immediately = false);
 
     RabbitMQConfiguration configuration;
-    Poco::Logger * log;
+    LoggerPtr log;
 
     UVLoop loop;
+    /// Preserve order of destruction here:
+    /// destruct connection and handler before the loop above.
     RabbitMQHandler event_handler;
-
     std::unique_ptr<AMQP::TcpConnection> connection;
+
     std::mutex mutex;
 };
 
