@@ -1,6 +1,6 @@
 #pragma once
 
-#include "config_core.h"
+#include "config.h"
 
 #if USE_MYSQL || USE_LIBPQXX
 
@@ -13,15 +13,15 @@ namespace DB
 
 bool needRewriteQueryWithFinalForStorage(const Names & column_names, const StoragePtr & storage);
 
-Pipe readFinalFromNestedStorage(
+void readFinalFromNestedStorage(
+    QueryPlan & query_plan,
     StoragePtr nested_storage,
     const Names & column_names,
-    const StorageMetadataPtr & /*metadata_snapshot*/,
     SelectQueryInfo & query_info,
     ContextPtr context,
     QueryProcessingStage::Enum processed_stage,
     size_t max_block_size,
-    unsigned int num_streams);
+    size_t num_streams);
 
 }
 
