@@ -17,13 +17,14 @@ public:
         StorageEmbeddedRocksDB & storage_,
         const StorageMetadataPtr & metadata_snapshot_);
 
-    void consume(Chunk chunk) override;
+    void consume(Chunk & chunk) override;
     String getName() const override { return "EmbeddedRocksDBSink"; }
 
 private:
     StorageEmbeddedRocksDB & storage;
     StorageMetadataPtr metadata_snapshot;
     size_t primary_key_pos = 0;
+    Serializations serializations;
 };
 
 }
