@@ -88,13 +88,6 @@ StoragePtr StorageFactory::get(
 
         name = "View";
     }
-    else if (query.is_live_view)
-    {
-        if (query.storage)
-            throw Exception(ErrorCodes::INCORRECT_QUERY, "Specifying ENGINE is not allowed for a LiveView");
-
-        name = "LiveView";
-    }
     else if (query.is_dictionary)
     {
         if (query.storage)
@@ -149,13 +142,6 @@ StoragePtr StorageFactory::get(
                     "Direct creation of tables with ENGINE MaterializedView "
                     "is not supported, use CREATE MATERIALIZED VIEW statement");
             }
-            if (name == "LiveView")
-            {
-                throw Exception(
-                    ErrorCodes::INCORRECT_QUERY,
-                    "Direct creation of tables with ENGINE LiveView "
-                    "is not supported, use CREATE LIVE VIEW statement");
-            }
             if (name == "WindowView")
             {
                 throw Exception(
