@@ -2,29 +2,38 @@
 #include <Storages/ColumnsDescription.h>
 #include <Storages/StorageGenerateRandom.h>
 #include <Storages/StorageFactory.h>
+#include <Storages/checkAndGetLiteralArgument.h>
+#include <Storages/SelectQueryInfo.h>
 #include <Processors/Sources/SourceFromSingleChunk.h>
 #include <QueryPipeline/Pipe.h>
 #include <Parsers/ASTLiteral.h>
 
-#include <DataTypes/DataTypeTuple.h>
-#include <DataTypes/DataTypeEnum.h>
-#include <DataTypes/DataTypeNullable.h>
-#include <DataTypes/DataTypeDateTime64.h>
-#include <DataTypes/DataTypeDecimalBase.h>
-#include <DataTypes/DataTypeArray.h>
-#include <DataTypes/DataTypeString.h>
-#include <DataTypes/DataTypeFixedString.h>
-#include <DataTypes/NestedUtils.h>
 #include <Columns/ColumnArray.h>
 #include <Columns/ColumnFixedString.h>
-#include <Columns/ColumnString.h>
-#include <Columns/ColumnVector.h>
+#include <Columns/ColumnLowCardinality.h>
+#include <Columns/ColumnMap.h>
 #include <Columns/ColumnNullable.h>
+#include <Columns/ColumnString.h>
 #include <Columns/ColumnTuple.h>
+#include <Columns/ColumnVector.h>
+#include <DataTypes/DataTypeArray.h>
+#include <DataTypes/DataTypeDateTime64.h>
+#include <DataTypes/DataTypeDecimalBase.h>
+#include <DataTypes/DataTypeEnum.h>
+#include <DataTypes/DataTypeFixedString.h>
+#include <DataTypes/DataTypeLowCardinality.h>
+#include <DataTypes/DataTypeMap.h>
+#include <DataTypes/DataTypeNullable.h>
+#include <DataTypes/DataTypeTuple.h>
+#include <DataTypes/DataTypesDecimal.h>
+#include <DataTypes/NestedUtils.h>
 
+#include <Core/Settings.h>
+#include <Interpreters/Context.h>
+#include <Common/DateLUTImpl.h>
 #include <Common/SipHash.h>
+#include <Common/intExp10.h>
 #include <Common/randomSeed.h>
-#include <base/unaligned.h>
 
 #include <Functions/FunctionFactory.h>
 
@@ -33,6 +42,10 @@
 
 namespace DB
 {
+namespace Setting
+{
+    extern const SettingsUInt64 preferred_block_size_bytes;
+}
 
 namespace ErrorCodes
 {
@@ -46,21 +59,105 @@ namespace ErrorCodes
 namespace
 {
 
-void fillBufferWithRandomData(char * __restrict data, size_t size, pcg64 & rng)
+struct GenerateRandomState
 {
+    std::atomic<UInt64> add_total_rows = 0;
+};
+using GenerateRandomStatePtr = std::shared_ptr<GenerateRandomState>;
+
+void fillBufferWithRandomData(char * __restrict data, size_t limit, size_t size_of_type, pcg64 & rng, [[maybe_unused]] bool flip_bytes = false)
+{
+    size_t size = limit * size_of_type;
     char * __restrict end = data + size;
     while (data < end)
     {
         /// The loop can be further optimized.
         UInt64 number = rng();
-        unalignedStore<UInt64>(data, number);
+        if constexpr (std::endian::native == std::endian::big)
+            unalignedStoreLittleEndian<UInt64>(data, number);
+        else
+            unalignedStore<UInt64>(data, number);
         data += sizeof(UInt64); /// We assume that data has at least 7-byte padding (see PaddedPODArray)
     }
+    if constexpr (std::endian::native == std::endian::big)
+    {
+        if (flip_bytes)
+        {
+            data = end - size;
+            while (data < end)
+            {
+                char * rev_end = data + size_of_type;
+                std::reverse(data, rev_end);
+                data += size_of_type;
+            }
+        }
+    }
 }
 
 
-ColumnPtr fillColumnWithRandomData(
+size_t estimateValueSize(
     const DataTypePtr type,
+    UInt64 max_array_length,
+    UInt64 max_string_length)
+{
+    if (type->haveMaximumSizeOfValue())
+        return type->getMaximumSizeOfValueInMemory();
+
+    TypeIndex idx = type->getTypeId();
+
+    switch (idx)
+    {
+        case TypeIndex::String:
+        {
+            return max_string_length + sizeof(UInt64);
+        }
+
+        /// The logic in this function should reflect the logic of fillColumnWithRandomData.
+        case TypeIndex::Array:
+        {
+            auto nested_type = typeid_cast<const DataTypeArray &>(*type).getNestedType();
+            return sizeof(size_t) + estimateValueSize(nested_type, max_array_length / 2, max_string_length);
+        }
+
+        case TypeIndex::Map:
+        {
+            const DataTypePtr & nested_type = typeid_cast<const DataTypeMap &>(*type).getNestedType();
+            return sizeof(size_t) + estimateValueSize(nested_type, max_array_length / 2, max_string_length);
+        }
+
+        case TypeIndex::Tuple:
+        {
+            auto elements = typeid_cast<const DataTypeTuple *>(type.get())->getElements();
+            const size_t tuple_size = elements.size();
+            size_t res = 0;
+
+            for (size_t i = 0; i < tuple_size; ++i)
+                res += estimateValueSize(elements[i], max_array_length, max_string_length);
+
+            return res;
+        }
+
+        case TypeIndex::Nullable:
+        {
+            auto nested_type = typeid_cast<const DataTypeNullable &>(*type).getNestedType();
+            return 1 + estimateValueSize(nested_type, max_array_length, max_string_length);
+        }
+
+        case TypeIndex::LowCardinality:
+        {
+            auto nested_type = typeid_cast<const DataTypeLowCardinality &>(*type).getDictionaryType();
+            return sizeof(size_t) + estimateValueSize(nested_type, max_array_length, max_string_length);
+        }
+
+        default:
+            throw Exception(ErrorCodes::NOT_IMPLEMENTED, "The 'GenerateRandom' is not implemented for type {}", type->getName());
+    }
+}
+
+}
+
+ColumnPtr fillColumnWithRandomData(
+    DataTypePtr type,
     UInt64 limit,
     UInt64 max_array_length,
     UInt64 max_string_length,
@@ -85,7 +182,7 @@ ColumnPtr fillColumnWithRandomData(
             {
                 size_t length = rng() % (max_string_length + 1);    /// Slow
 
-                IColumn::Offset next_offset = offset + length + 1;
+                IColumn::Offset next_offset = offset + length;
                 data_to.resize(next_offset);
                 offsets_to[row_num] = next_offset;
 
@@ -108,11 +205,8 @@ ColumnPtr fillColumnWithRandomData(
                     data_to_ptr[pos + 3] = 32 + ((rand4 * 95) >> 16);
 
                     /// NOTE gcc failed to vectorize this code (aliasing of char?)
-                    /// TODO Implement SIMD optimizations from Danila Kutenin.
                 }
 
-                data_to[offset + length] = 0;
-
                 offset = next_offset;
             }
 
@@ -157,7 +251,7 @@ ColumnPtr fillColumnWithRandomData(
 
         case TypeIndex::Array:
         {
-            auto nested_type = typeid_cast<const DataTypeArray *>(type.get())->getNestedType();
+            auto nested_type = typeid_cast<const DataTypeArray &>(*type).getNestedType();
 
             auto offsets_column = ColumnVector<ColumnArray::Offset>::create();
             auto & offsets = offsets_column->getData();
@@ -170,14 +264,25 @@ ColumnPtr fillColumnWithRandomData(
                 offsets[i] = offset;
             }
 
-            auto data_column = fillColumnWithRandomData(nested_type, offset, max_array_length, max_string_length, rng, context);
+            /// This division by two makes the size growth subexponential on depth.
+            auto data_column = fillColumnWithRandomData(nested_type, offset, max_array_length / 2, max_string_length, rng, context);
 
-            return ColumnArray::create(std::move(data_column), std::move(offsets_column));
+            return ColumnArray::create(data_column, std::move(offsets_column));
+        }
+
+        case TypeIndex::Map:
+        {
+            const DataTypePtr & nested_type = typeid_cast<const DataTypeMap &>(*type).getNestedType();
+            auto nested_column = fillColumnWithRandomData(nested_type, limit, max_array_length / 2, max_string_length, rng, context);
+            return ColumnMap::create(nested_column);
         }
 
         case TypeIndex::Tuple:
         {
             auto elements = typeid_cast<const DataTypeTuple *>(type.get())->getElements();
+            if (elements.empty())
+                return ColumnTuple::create(limit);
+
             const size_t tuple_size = elements.size();
             Columns tuple_columns(tuple_size);
 
@@ -189,7 +294,7 @@ ColumnPtr fillColumnWithRandomData(
 
         case TypeIndex::Nullable:
         {
-            auto nested_type = typeid_cast<const DataTypeNullable *>(type.get())->getNestedType();
+            auto nested_type = typeid_cast<const DataTypeNullable &>(*type).getNestedType();
             auto nested_column = fillColumnWithRandomData(nested_type, limit, max_array_length, max_string_length, rng, context);
 
             auto null_map_column = ColumnUInt8::create();
@@ -198,14 +303,23 @@ ColumnPtr fillColumnWithRandomData(
             for (UInt64 i = 0; i < limit; ++i)
                 null_map[i] = rng() % 16 == 0; /// No real motivation for this.
 
-            return ColumnNullable::create(std::move(nested_column), std::move(null_map_column));
+            return ColumnNullable::create(nested_column, std::move(null_map_column));
         }
 
         case TypeIndex::UInt8:
         {
             auto column = ColumnUInt8::create();
-            column->getData().resize(limit);
-            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit * sizeof(UInt8), rng);
+            auto & data = column->getData();
+            data.resize(limit);
+            if (isBool(type))
+            {
+                for (size_t i = 0; i < limit; ++i)
+                    data[i] = rng() % 2;
+            }
+            else
+            {
+                fillBufferWithRandomData(reinterpret_cast<char *>(data.data()), limit, sizeof(UInt8), rng);
+            }
             return column;
         }
         case TypeIndex::UInt16: [[fallthrough]];
@@ -213,14 +327,17 @@ ColumnPtr fillColumnWithRandomData(
         {
             auto column = ColumnUInt16::create();
             column->getData().resize(limit);
-            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit * sizeof(UInt16), rng);
+            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit, sizeof(UInt16), rng, true);
             return column;
         }
         case TypeIndex::Date32:
         {
             auto column = ColumnInt32::create();
             column->getData().resize(limit);
-            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit * sizeof(Int32), rng);
+
+            for (size_t i = 0; i < limit; ++i)
+                column->getData()[i] = (rng() % static_cast<UInt64>(DATE_LUT_SIZE)) - DAYNUM_OFFSET_EPOCH;
+
             return column;
         }
         case TypeIndex::UInt32: [[fallthrough]];
@@ -228,28 +345,28 @@ ColumnPtr fillColumnWithRandomData(
         {
             auto column = ColumnUInt32::create();
             column->getData().resize(limit);
-            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit * sizeof(UInt32), rng);
+            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit, sizeof(UInt32), rng, true);
             return column;
         }
         case TypeIndex::UInt64:
         {
             auto column = ColumnUInt64::create();
             column->getData().resize(limit);
-            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit * sizeof(UInt64), rng);
+            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit, sizeof(UInt64), rng, true);
             return column;
         }
         case TypeIndex::UInt128:
         {
             auto column = ColumnUInt128::create();
             column->getData().resize(limit);
-            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit * sizeof(UInt128), rng);
+            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit, sizeof(UInt128), rng, true);
             return column;
         }
         case TypeIndex::UInt256:
         {
             auto column = ColumnUInt256::create();
             column->getData().resize(limit);
-            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit * sizeof(UInt256), rng);
+            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit, sizeof(UInt256), rng, true);
             return column;
         }
         case TypeIndex::UUID:
@@ -257,95 +374,116 @@ ColumnPtr fillColumnWithRandomData(
             auto column = ColumnUUID::create();
             column->getData().resize(limit);
             /// NOTE This is slightly incorrect as random UUIDs should have fixed version 4.
-            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit * sizeof(UUID), rng);
+            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit, sizeof(UUID), rng, true);
             return column;
         }
         case TypeIndex::Int8:
         {
             auto column = ColumnInt8::create();
             column->getData().resize(limit);
-            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit * sizeof(Int8), rng);
+            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit, sizeof(Int8), rng);
             return column;
         }
         case TypeIndex::Int16:
         {
             auto column = ColumnInt16::create();
             column->getData().resize(limit);
-            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit * sizeof(Int16), rng);
+            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit, sizeof(Int16), rng, true);
             return column;
         }
         case TypeIndex::Int32:
         {
             auto column = ColumnInt32::create();
             column->getData().resize(limit);
-            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit * sizeof(Int32), rng);
+            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit, sizeof(Int32), rng, true);
             return column;
         }
         case TypeIndex::Int64:
         {
             auto column = ColumnInt64::create();
             column->getData().resize(limit);
-            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit * sizeof(Int64), rng);
+            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit, sizeof(Int64), rng, true);
             return column;
         }
         case TypeIndex::Int128:
         {
             auto column = ColumnInt128::create();
             column->getData().resize(limit);
-            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit * sizeof(Int128), rng);
+            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit, sizeof(Int128), rng, true);
             return column;
         }
         case TypeIndex::Int256:
         {
             auto column = ColumnInt256::create();
             column->getData().resize(limit);
-            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit * sizeof(Int256), rng);
+            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit, sizeof(Int256), rng, true);
             return column;
         }
         case TypeIndex::Float32:
         {
             auto column = ColumnFloat32::create();
             column->getData().resize(limit);
-            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit * sizeof(Float32), rng);
+            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit, sizeof(Float32), rng, true);
             return column;
         }
         case TypeIndex::Float64:
         {
             auto column = ColumnFloat64::create();
             column->getData().resize(limit);
-            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit * sizeof(Float64), rng);
+            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit, sizeof(Float64), rng, true);
             return column;
         }
         case TypeIndex::Decimal32:
         {
-            auto column = type->createColumn();
+            const auto & decimal_type = assert_cast<const DataTypeDecimal<Decimal32> &>(*type);
+            auto column = decimal_type.createColumn();
             auto & column_concrete = typeid_cast<ColumnDecimal<Decimal32> &>(*column);
-            column_concrete.getData().resize(limit);
-            fillBufferWithRandomData(reinterpret_cast<char *>(column_concrete.getData().data()), limit * sizeof(Decimal32), rng);
+            auto & data = column_concrete.getData();
+            data.resize(limit);
+            /// Generate numbers from range [-10^P + 1, 10^P - 1]
+            Int32 range = common::exp10_i32(decimal_type.getPrecision());
+            for (size_t i = 0; i != limit; ++i)
+                data[i] = static_cast<Int32>(rng()) % range;
             return column;
         }
-        case TypeIndex::Decimal64:  /// TODO Decimal may be generated out of range.
+        case TypeIndex::Decimal64:
         {
+            const auto & decimal_type = assert_cast<const DataTypeDecimal<Decimal64> &>(*type);
             auto column = type->createColumn();
             auto & column_concrete = typeid_cast<ColumnDecimal<Decimal64> &>(*column);
-            column_concrete.getData().resize(limit);
-            fillBufferWithRandomData(reinterpret_cast<char *>(column_concrete.getData().data()), limit * sizeof(Decimal64), rng);
+            auto & data = column_concrete.getData();
+            data.resize(limit);
+            /// Generate numbers from range [-10^P + 1, 10^P - 1]
+            Int64 range = common::exp10_i64(decimal_type.getPrecision());
+            for (size_t i = 0; i != limit; ++i)
+                data[i] = static_cast<Int64>(rng()) % range;
+
             return column;
         }
         case TypeIndex::Decimal128:
         {
+            const auto & decimal_type = assert_cast<const DataTypeDecimal<Decimal128> &>(*type);
             auto column = type->createColumn();
             auto & column_concrete = typeid_cast<ColumnDecimal<Decimal128> &>(*column);
-            column_concrete.getData().resize(limit);
-            fillBufferWithRandomData(reinterpret_cast<char *>(column_concrete.getData().data()), limit * sizeof(Decimal128), rng);
+            auto & data = column_concrete.getData();
+            data.resize(limit);
+            /// Generate numbers from range [-10^P + 1, 10^P - 1]
+            Int128 range = common::exp10_i128(decimal_type.getPrecision());
+            for (size_t i = 0; i != limit; ++i)
+                data[i] = Int128({rng(), rng()}) % range;
             return column;
         }
         case TypeIndex::Decimal256:
         {
+            const auto & decimal_type = assert_cast<const DataTypeDecimal<Decimal256> &>(*type);
             auto column = type->createColumn();
             auto & column_concrete = typeid_cast<ColumnDecimal<Decimal256> &>(*column);
-            column_concrete.getData().resize(limit);
-            fillBufferWithRandomData(reinterpret_cast<char *>(column_concrete.getData().data()), limit * sizeof(Decimal256), rng);
+            auto & data = column_concrete.getData();
+            data.resize(limit);
+            /// Generate numbers from range [-10^P + 1, 10^P - 1]
+            Int256 range = common::exp10_i256(decimal_type.getPrecision());
+            for (size_t i = 0; i != limit; ++i)
+                data[i] = Int256({rng(), rng(), rng(), rng()}) % range;
             return column;
         }
         case TypeIndex::FixedString:
@@ -353,7 +491,7 @@ ColumnPtr fillColumnWithRandomData(
             size_t n = typeid_cast<const DataTypeFixedString &>(*type).getN();
             auto column = ColumnFixedString::create(n);
             column->getChars().resize(limit * n);
-            fillBufferWithRandomData(reinterpret_cast<char *>(column->getChars().data()), limit * n, rng);
+            fillBufferWithRandomData(reinterpret_cast<char *>(column->getChars().data()), limit, n, rng);
             return column;
         }
         case TypeIndex::DateTime64:
@@ -369,20 +507,64 @@ ColumnPtr fillColumnWithRandomData(
 
             return column;
         }
+        case TypeIndex::LowCardinality:
+        {
+            /// We are generating the values using the same random distribution as for full columns
+            /// so it's not in fact "low cardinality",
+            /// but it's ok for testing purposes, because the LowCardinality data type supports high cardinality data as well.
+
+            auto nested_type = typeid_cast<const DataTypeLowCardinality &>(*type).getDictionaryType();
+            auto nested_column = fillColumnWithRandomData(nested_type, limit, max_array_length, max_string_length, rng, context);
+
+            auto column = type->createColumn();
+            typeid_cast<ColumnLowCardinality &>(*column).insertRangeFromFullColumn(*nested_column, 0, limit);
+
+            return column;
+        }
+        case TypeIndex::IPv4:
+        {
+            auto column = ColumnIPv4::create();
+            column->getData().resize(limit);
+            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit, sizeof(IPv4), rng, true);
+            return column;
+        }
+        case TypeIndex::IPv6:
+        {
+            auto column = ColumnIPv6::create();
+            column->getData().resize(limit);
+            fillBufferWithRandomData(reinterpret_cast<char *>(column->getData().data()), limit, sizeof(IPv6), rng);
+            return column;
+        }
 
         default:
-            throw Exception("The 'GenerateRandom' is not implemented for type " + type->getName(), ErrorCodes::NOT_IMPLEMENTED);
+            throw Exception(ErrorCodes::NOT_IMPLEMENTED, "The 'GenerateRandom' is not implemented for type {}", type->getName());
     }
 }
 
+namespace
+{
 
-class GenerateSource : public SourceWithProgress
+class GenerateSource : public ISource
 {
 public:
-    GenerateSource(UInt64 block_size_, UInt64 max_array_length_, UInt64 max_string_length_, UInt64 random_seed_, Block block_header_, ContextPtr context_)
-        : SourceWithProgress(Nested::flatten(prepareBlockToFill(block_header_)))
-        , block_size(block_size_), max_array_length(max_array_length_), max_string_length(max_string_length_)
-        , block_to_fill(std::move(block_header_)), rng(random_seed_), context(context_) {}
+    GenerateSource(
+        UInt64 block_size_,
+        UInt64 max_array_length_,
+        UInt64 max_string_length_,
+        UInt64 random_seed_,
+        Block block_header_,
+        ContextPtr context_,
+        GenerateRandomStatePtr state_)
+        : ISource(std::make_shared<const Block>(Nested::flattenNested(prepareBlockToFill(block_header_))))
+        , block_size(block_size_)
+        , max_array_length(max_array_length_)
+        , max_string_length(max_string_length_)
+        , block_to_fill(std::move(block_header_))
+        , rng(random_seed_)
+        , context(context_)
+        , shared_state(state_)
+    {
+    }
 
     String getName() const override { return "GenerateRandom"; }
 
@@ -395,8 +577,16 @@ protected:
         for (const auto & elem : block_to_fill)
             columns.emplace_back(fillColumnWithRandomData(elem.type, block_size, max_array_length, max_string_length, rng, context));
 
-        columns = Nested::flatten(block_to_fill.cloneWithColumns(std::move(columns))).getColumns();
-        return {std::move(columns), block_size};
+        columns = Nested::flattenNested(block_to_fill.cloneWithColumns(columns)).getColumns();
+
+        UInt64 total_rows = shared_state->add_total_rows.fetch_and(0);
+        if (total_rows)
+            addTotalRowsApprox(total_rows);
+
+        auto chunk = Chunk{std::move(columns), block_size};
+        progress(chunk.getNumRows(), chunk.bytes());
+
+        return chunk;
     }
 
 private:
@@ -408,6 +598,7 @@ private:
     pcg64 rng;
 
     ContextPtr context;
+    GenerateRandomStatePtr shared_state;
 
     static Block & prepareBlockToFill(Block & block)
     {
@@ -431,7 +622,7 @@ StorageGenerateRandom::StorageGenerateRandom(
     const String & comment,
     UInt64 max_array_length_,
     UInt64 max_string_length_,
-    std::optional<UInt64> random_seed_)
+    const std::optional<UInt64> & random_seed_)
     : IStorage(table_id_), max_array_length(max_array_length_), max_string_length(max_string_length_)
 {
     static constexpr size_t MAX_ARRAY_SIZE = 1 << 30;
@@ -459,9 +650,9 @@ void registerStorageGenerateRandom(StorageFactory & factory)
         ASTs & engine_args = args.engine_args;
 
         if (engine_args.size() > 3)
-            throw Exception("Storage GenerateRandom requires at most three arguments: "
-                "random_seed, max_string_length, max_array_length.",
-                ErrorCodes::NUMBER_OF_ARGUMENTS_DOESNT_MATCH);
+            throw Exception(ErrorCodes::NUMBER_OF_ARGUMENTS_DOESNT_MATCH,
+                            "Storage GenerateRandom requires at most three arguments: "
+                            "random_seed, max_string_length, max_array_length.");
 
         std::optional<UInt64> random_seed;
         UInt64 max_string_length = 10;
@@ -469,36 +660,33 @@ void registerStorageGenerateRandom(StorageFactory & factory)
 
         if (!engine_args.empty())
         {
-            const Field & value = engine_args[0]->as<const ASTLiteral &>().value;
-            if (!value.isNull())
-                random_seed = value.safeGet<UInt64>();
+            const auto & ast_literal = engine_args[0]->as<const ASTLiteral &>();
+            if (!ast_literal.value.isNull())
+                random_seed = checkAndGetLiteralArgument<UInt64>(ast_literal, "random_seed");
         }
 
         if (engine_args.size() >= 2)
-            max_string_length = engine_args[1]->as<const ASTLiteral &>().value.safeGet<UInt64>();
+            max_string_length = checkAndGetLiteralArgument<UInt64>(engine_args[1], "max_string_length");
 
         if (engine_args.size() == 3)
-            max_array_length = engine_args[2]->as<const ASTLiteral &>().value.safeGet<UInt64>();
+            max_array_length = checkAndGetLiteralArgument<UInt64>(engine_args[2], "max_array_length");
 
-        return StorageGenerateRandom::create(args.table_id, args.columns, args.comment, max_array_length, max_string_length, random_seed);
+        return std::make_shared<StorageGenerateRandom>(args.table_id, args.columns, args.comment, max_array_length, max_string_length, random_seed);
     });
 }
 
 Pipe StorageGenerateRandom::read(
     const Names & column_names,
-    const StorageMetadataPtr & metadata_snapshot,
-    SelectQueryInfo & /*query_info*/,
+    const StorageSnapshotPtr & storage_snapshot,
+    SelectQueryInfo & query_info,
     ContextPtr context,
     QueryProcessingStage::Enum /*processed_stage*/,
     size_t max_block_size,
-    unsigned num_streams)
+    size_t num_streams)
 {
-    metadata_snapshot->check(column_names, getVirtuals(), getStorageID());
+    storage_snapshot->check(column_names);
 
-    Pipes pipes;
-    pipes.reserve(num_streams);
-
-    const ColumnsDescription & our_columns = metadata_snapshot->getColumns();
+    const ColumnsDescription & our_columns = storage_snapshot->metadata->getColumns();
     Block block_header;
     for (const auto & name : column_names)
     {
@@ -507,11 +695,44 @@ Pipe StorageGenerateRandom::read(
         block_header.insert({std::move(column), name_type.type, name_type.name});
     }
 
+    /// Correction of block size for wide tables.
+    size_t preferred_block_size_bytes = context->getSettingsRef()[Setting::preferred_block_size_bytes];
+    if (preferred_block_size_bytes)
+    {
+        size_t estimated_row_size_bytes = estimateValueSize(std::make_shared<DataTypeTuple>(block_header.getDataTypes()), max_array_length, max_string_length);
+
+        size_t estimated_block_size_bytes = 0;
+        if (common::mulOverflow(max_block_size, estimated_row_size_bytes, estimated_block_size_bytes))
+            throw Exception(ErrorCodes::TOO_LARGE_ARRAY_SIZE, "Too large estimated block size in GenerateRandom table: its estimation leads to 64bit overflow");
+
+        if (estimated_block_size_bytes > preferred_block_size_bytes)
+        {
+            max_block_size = static_cast<size_t>(max_block_size * (static_cast<double>(preferred_block_size_bytes) / estimated_block_size_bytes));
+            if (max_block_size == 0)
+                max_block_size = 1;
+        }
+    }
+
+    UInt64 query_limit = query_info.trivial_limit;
+    if (query_limit && num_streams * max_block_size > query_limit)
+    {
+        /// We want to avoid spawning more streams than necessary
+        num_streams = std::min(num_streams, static_cast<size_t>(((query_limit + max_block_size - 1) / max_block_size)));
+    }
+    Pipes pipes;
+    pipes.reserve(num_streams);
+
     /// Will create more seed values for each source from initial seed.
     pcg64 generate(random_seed);
 
+    auto shared_state = std::make_shared<GenerateRandomState>(query_info.trivial_limit);
+
     for (UInt64 i = 0; i < num_streams; ++i)
-        pipes.emplace_back(std::make_shared<GenerateSource>(max_block_size, max_array_length, max_string_length, generate(), block_header, context));
+    {
+        auto source = std::make_shared<GenerateSource>(
+            max_block_size, max_array_length, max_string_length, generate(), block_header, context, shared_state);
+        pipes.emplace_back(std::move(source));
+    }
 
     return Pipe::unitePipes(std::move(pipes));
 }
