@@ -1,22 +1,25 @@
-#include "StorageSystemContributors.h"
+#include <Storages/System/StorageSystemContributors.h>
+
+#include <Columns/IColumn.h>
+#include <Common/thread_local_rng.h>
+#include <DataTypes/DataTypeString.h>
 
 #include <algorithm>
-#include <DataTypes/DataTypeString.h>
-#include <Common/thread_local_rng.h>
 
 
 extern const char * auto_contributors[];
 
 namespace DB
 {
-NamesAndTypesList StorageSystemContributors::getNamesAndTypes()
+ColumnsDescription StorageSystemContributors::getColumnsDescription()
 {
-    return {
-        {"name", std::make_shared<DataTypeString>()},
+    return ColumnsDescription
+    {
+        {"name", std::make_shared<DataTypeString>(), "Contributor (author) name from git log."},
     };
 }
 
-void StorageSystemContributors::fillData(MutableColumns & res_columns, ContextPtr, const SelectQueryInfo &) const
+void StorageSystemContributors::fillData(MutableColumns & res_columns, ContextPtr, const ActionsDAG::Node *, std::vector<UInt8>) const
 {
     std::vector<const char *> contributors;
     for (auto * it = auto_contributors; *it; ++it)
