@@ -1,6 +1,5 @@
 #pragma once
 
-#include <base/shared_ptr_helper.h>
 #include <Storages/IStorage.h>
 
 
@@ -8,25 +7,24 @@ namespace DB
 {
 
 /// For system.data_skipping_indices table - describes the data skipping indices in tables, similar to system.columns.
-class StorageSystemDataSkippingIndices : public shared_ptr_helper<StorageSystemDataSkippingIndices>, public IStorage
+class StorageSystemDataSkippingIndices : public IStorage
 {
-    friend struct shared_ptr_helper<StorageSystemDataSkippingIndices>;
 public:
+    explicit StorageSystemDataSkippingIndices(const StorageID & table_id_);
+
     std::string getName() const override { return "SystemDataSkippingIndices"; }
 
-    Pipe read(
+    void read(
+        QueryPlan & query_plan,
         const Names & column_names,
-        const StorageMetadataPtr & /*metadata_snapshot*/,
+        const StorageSnapshotPtr & storage_snapshot,
         SelectQueryInfo & query_info,
         ContextPtr context,
         QueryProcessingStage::Enum processed_stage,
         size_t max_block_size,
-        unsigned num_streams) override;
+        size_t num_streams) override;
 
     bool isSystemStorage() const override { return true; }
-
-protected:
-    StorageSystemDataSkippingIndices(const StorageID & table_id_);
 };
 
 }
