@@ -1,6 +1,5 @@
 #pragma once
 
-#include <base/shared_ptr_helper.h>
 #include <Storages/System/IStorageSystemOneBlock.h>
 
 
@@ -9,17 +8,16 @@ namespace DB
 class Context;
 
 /// Implements `privileges` system table, which allows you to get information about access types.
-class StorageSystemPrivileges final : public shared_ptr_helper<StorageSystemPrivileges>, public IStorageSystemOneBlock<StorageSystemPrivileges>
+class StorageSystemPrivileges final : public IStorageSystemOneBlock
 {
 public:
     std::string getName() const override { return "SystemPrivileges"; }
-    static NamesAndTypesList getNamesAndTypes();
+    static ColumnsDescription getColumnsDescription();
     static const std::vector<std::pair<String, Int16>> & getAccessTypeEnumValues();
 
 protected:
-    friend struct shared_ptr_helper<StorageSystemPrivileges>;
     using IStorageSystemOneBlock::IStorageSystemOneBlock;
-    void fillData(MutableColumns & res_columns, ContextPtr context, const SelectQueryInfo &) const override;
+    void fillData(MutableColumns & res_columns, ContextPtr, const ActionsDAG::Node *, std::vector<UInt8>) const override;
 };
 
 }
