@@ -72,6 +72,8 @@ ColumnsDescription StorageSystemProcesses::getColumnsDescription()
         {"Settings", std::make_shared<DataTypeMap>(std::make_shared<DataTypeString>(), std::make_shared<DataTypeString>()), "The list of modified user-level settings."},
 
         {"current_database", std::make_shared<DataTypeString>(), "The name of the current database."},
+
+        {"is_internal", std::make_shared<DataTypeUInt8>(), "Indicates whether it is an auxiliary query executed internally."},
     };
 
     description.setAliases({
@@ -167,6 +169,7 @@ void StorageSystemProcesses::fillData(MutableColumns & res_columns, ContextPtr c
         }
 
         res_columns[i++]->insert(process.current_database);
+        res_columns[i++]->insert(process.is_internal);
     }
 }
 
