@@ -1,6 +1,5 @@
 #pragma once
 
-#include <base/shared_ptr_helper.h>
 #include <Storages/System/IStorageSystemOneBlock.h>
 
 
@@ -11,16 +10,15 @@ class Context;
 /** Implements the `quotas_usage` system table, which allows you to get information about
   * how all users use the quotas.
   */
-class StorageSystemQuotasUsage final : public shared_ptr_helper<StorageSystemQuotasUsage>, public IStorageSystemOneBlock<StorageSystemQuotasUsage>
+class StorageSystemQuotasUsage final : public IStorageSystemOneBlock
 {
 public:
     std::string getName() const override { return "SystemQuotasUsage"; }
-    static NamesAndTypesList getNamesAndTypes();
+    static ColumnsDescription getColumnsDescription();
 
 protected:
-    friend struct shared_ptr_helper<StorageSystemQuotasUsage>;
     using IStorageSystemOneBlock::IStorageSystemOneBlock;
-    void fillData(MutableColumns & res_columns, ContextPtr context, const SelectQueryInfo &) const override;
+    void fillData(MutableColumns & res_columns, ContextPtr context, const ActionsDAG::Node *, std::vector<UInt8>) const override;
 };
 
 }
