@@ -3,6 +3,7 @@
 
 #include <Columns/ColumnString.h>
 #include <DataTypes/DataTypeString.h>
+#include <DataTypes/DataTypeUUID.h>
 #include <DataTypes/DataTypesNumber.h>
 #include <DataTypes/DataTypeNullable.h>
 #include <DataTypes/DataTypeDateTime.h>
@@ -43,6 +44,7 @@ StorageSystemReplicas::StorageSystemReplicas(const StorageID & table_id_)
     ColumnsDescription description = {
         { "database",                             std::make_shared<DataTypeString>(),   "Database name."},
         { "table",                                std::make_shared<DataTypeString>(),   "Table name."},
+        { "uuid",                                 std::make_shared<DataTypeUUID>(),     "Table UUID."},
         { "engine",                               std::make_shared<DataTypeString>(),   "Table engine name."},
         { "is_leader",                            std::make_shared<DataTypeUInt8>(),    "Whether the replica is the leader. Multiple replicas can be leaders at the same time. "
                                                                                           "A replica can be prevented from becoming a leader using the merge_tree setting replicated_can_become_leader. "
@@ -145,10 +147,11 @@ void ReadFromSystemReplicas::applyFilters(ActionDAGNodes added_filter_nodes)
         {
             { ColumnString::create(), std::make_shared<DataTypeString>(), "database" },
             { ColumnString::create(), std::make_shared<DataTypeString>(), "table" },
+            { ColumnString::create(), std::make_shared<DataTypeUUID>(), "uuid" },
             { ColumnString::create(), std::make_shared<DataTypeString>(), "engine" },
         };
 
-        auto dag = VirtualColumnUtils::splitFilterDagForAllowedInputs(filter_actions_dag->getOutputs().at(0), &block_to_filter);
+        auto dag = VirtualColumnUtils::splitFilterDagForAllowedInputs(filter_actions_dag->getOutputs().at(0), &block_to_filter, context);
         if (dag)
             virtual_columns_filter = VirtualColumnUtils::buildFilterExpression(std::move(*dag), context);
     }
@@ -225,6 +228,7 @@ public:
         size_t max_block_size_,
         ColumnPtr col_database_,
         ColumnPtr col_table_,
+        ColumnPtr col_uuid_,
         ColumnPtr col_engine_,
         std::vector<TFuture> futures_,
         ContextPtr context_)
@@ -232,6 +236,7 @@ public:
         , max_block_size(max_block_size_)
         , col_database(std::move(col_database_))
         , col_table(std::move(col_table_))
+        , col_uuid(std::move(col_uuid_))
         , col_engine(std::move(col_engine_))
         , futures(std::move(futures_))
         , context(std::move(context_))
@@ -248,6 +253,7 @@ private:
     /// Columns with table metadata.
     ColumnPtr col_database;
     ColumnPtr col_table;
+    ColumnPtr col_uuid;
     ColumnPtr col_engine;
     /// Futures for the status of each table.
     std::vector<TFuture> futures;
@@ -263,6 +269,7 @@ void ReadFromSystemReplicas::initializePipeline(QueryPipelineBuilder & pipeline,
 
     MutableColumnPtr col_database_mut = ColumnString::create();
     MutableColumnPtr col_table_mut = ColumnString::create();
+    MutableColumnPtr col_uuid_mut = ColumnUUID::create();
     MutableColumnPtr col_engine_mut = ColumnString::create();
 
     for (auto & db : replicated_tables)
@@ -271,12 +278,14 @@ void ReadFromSystemReplicas::initializePipeline(QueryPipelineBuilder & pipeline,
         {
             col_database_mut->insert(db.first);
             col_table_mut->insert(table.first);
+            col_uuid_mut->insert(table.second->getStorageID().uuid);
             col_engine_mut->insert(table.second->getName());
         }
     }
 
     ColumnPtr col_database = std::move(col_database_mut);
     ColumnPtr col_table = std::move(col_table_mut);
+    ColumnPtr col_uuid = std::move(col_uuid_mut);
     ColumnPtr col_engine = std::move(col_engine_mut);
 
     /// Determine what tables are needed by the conditions in the query.
@@ -285,6 +294,7 @@ void ReadFromSystemReplicas::initializePipeline(QueryPipelineBuilder & pipeline,
         {
             { col_database, std::make_shared<DataTypeString>(), "database" },
             { col_table, std::make_shared<DataTypeString>(), "table" },
+            { col_uuid, std::make_shared<DataTypeUUID>(), "uuid" },
             { col_engine, std::make_shared<DataTypeString>(), "engine" },
         };
 
@@ -300,6 +310,7 @@ void ReadFromSystemReplicas::initializePipeline(QueryPipelineBuilder & pipeline,
 
         col_database = filtered_block.getByName("database").column;
         col_table = filtered_block.getByName("table").column;
+        col_uuid = filtered_block.getByName("uuid").column;
         col_engine = filtered_block.getByName("engine").column;
     }
 
@@ -329,7 +340,7 @@ void ReadFromSystemReplicas::initializePipeline(QueryPipelineBuilder & pipeline,
     /// If there are more requests, they will be scheduled by the query that needs them.
     get_status_requests.scheduleRequests(max_request_id, query_status);
 
-    pipeline.init(Pipe(std::make_shared<SystemReplicasSource>(header, max_block_size, col_database, col_table, col_engine, std::move(futures), context)));
+    pipeline.init(Pipe(std::make_shared<SystemReplicasSource>(header, max_block_size, col_database, col_table, col_uuid, col_engine, std::move(futures), context)));
 }
 
 Chunk SystemReplicasSource::generate()
@@ -382,11 +393,12 @@ Chunk SystemReplicasSource::generate()
             throw;
         }
 
-        res_columns[0]->insert((*col_database)[i]);
-        res_columns[1]->insert((*col_table)[i]);
-        res_columns[2]->insert((*col_engine)[i]);
+        size_t col_num = 0;
+        res_columns[col_num++]->insert((*col_database)[i]);
+        res_columns[col_num++]->insert((*col_table)[i]);
+        res_columns[col_num++]->insert((*col_uuid)[i]);
+        res_columns[col_num++]->insert((*col_engine)[i]);
 
-        size_t col_num = 3;
         res_columns[col_num++]->insert(status->is_leader);
         res_columns[col_num++]->insert(status->can_become_leader);
         res_columns[col_num++]->insert(status->is_readonly);
