@@ -104,6 +104,9 @@
 #include <Storages/System/StorageSystemViewRefreshes.h>
 #include <Storages/System/StorageSystemDNSCache.h>
 #include <Storages/System/StorageSystemIcebergHistory.h>
+#if USE_ICU
+#   include <Storages/System/StorageSystemUnicode.h>
+#endif
 #include <Interpreters/Context.h>
 
 #include <Poco/Util/LayeredConfiguration.h>
@@ -242,6 +245,9 @@ void attachSystemTablesServer(ContextPtr context, IDatabase & system_database, b
     attach<StorageSystemWorkloads>(context, system_database, "workloads", "Contains a list of all currently existing workloads.");
     attach<StorageSystemResources>(context, system_database, "resources", "Contains a list of all currently existing resources.");
     attach<StorageSystemIcebergHistory>(context, system_database, "iceberg_history", "Displays the history of an iceberg table similar to the Spark history table");
+#if USE_ICU
+    attach<StorageSystemUnicode>(context, system_database, "unicode", "Contains all unicode codepoints.");
+#endif
 
     if (has_zookeeper)
     {
