@@ -31,6 +31,7 @@
 #include <DataTypes/DataTypeNullable.h>
 #include <DataTypes/DataTypesNumber.h>
 #include <DataTypes/DataTypeString.h>
+#include <DataTypes/DataTypeMap.h>
 #include <DataTypes/DataTypeLowCardinality.h>
 #include <DataTypes/DataTypeDateTime.h>
 
@@ -61,6 +62,14 @@
 
 namespace DB
 {
+namespace Setting
+{
+    extern const SettingsBool use_hive_partitioning;
+}
+namespace ErrorCodes
+{
+    extern const int LOGICAL_ERROR;
+}
 
 namespace VirtualColumnUtils
 {
@@ -142,13 +151,15 @@ void filterBlockWithExpression(const ExpressionActionsPtr & actions, Block & blo
 
 static NamesAndTypesList getCommonVirtualsForFileLikeStorage()
 {
-    return {{"_path", std::make_shared<DataTypeLowCardinality>(std::make_shared<DataTypeString>())},
-            {"_file", std::make_shared<DataTypeLowCardinality>(std::make_shared<DataTypeString>())},
-            {"_size", makeNullable(std::make_shared<DataTypeUInt64>())},
-            {"_time", makeNullable(std::make_shared<DataTypeDateTime>())},
-            {"_etag", std::make_shared<DataTypeLowCardinality>(std::make_shared<DataTypeString>())},
-            {"_data_lake_snapshot_version", makeNullable(std::make_shared<DataTypeUInt64>())},
-            {"_row_number", makeNullable(std::make_shared<DataTypeInt64>())}
+    return {
+        {"_path", std::make_shared<DataTypeLowCardinality>(std::make_shared<DataTypeString>())},
+        {"_file", std::make_shared<DataTypeLowCardinality>(std::make_shared<DataTypeString>())},
+        {"_size", makeNullable(std::make_shared<DataTypeUInt64>())},
+        {"_time", makeNullable(std::make_shared<DataTypeDateTime>())},
+        {"_etag", std::make_shared<DataTypeLowCardinality>(std::make_shared<DataTypeString>())},
+        {"_tags", std::make_shared<DataTypeMap>(std::make_shared<DataTypeString>(), std::make_shared<DataTypeString>())},
+        {"_data_lake_snapshot_version", makeNullable(std::make_shared<DataTypeUInt64>())},
+        {"_row_number", makeNullable(std::make_shared<DataTypeInt64>())},
     };
 }
 
@@ -157,29 +168,52 @@ NameSet getVirtualNamesForFileLikeStorage()
     return getCommonVirtualsForFileLikeStorage().getNameSet();
 }
 
-VirtualColumnsDescription getVirtualsForFileLikeStorage(ColumnsDescription & storage_columns)
+VirtualColumnsDescription getVirtualsForFileLikeStorage(
+    ColumnsDescription & storage_columns,
+    ContextPtr context,
+    const std::optional<FormatSettings> & format_settings,
+    std::optional<PartitionStrategyFactory::StrategyType> partition_strategy,
+    const std::string & path)
 {
     VirtualColumnsDescription desc;
 
     auto add_virtual = [&](const NameAndTypePair & pair)
     {
         const auto & name = pair.getNameInStorage();
-        const auto & type = pair.getTypeInStorage();
         if (storage_columns.has(name))
-        {
             return;
-        }
 
+        const auto & type = pair.getTypeInStorage();
         desc.addEphemeral(name, type, "");
     };
 
     for (const auto & item : getCommonVirtualsForFileLikeStorage())
         add_virtual(item);
 
+    if (!path.empty())
+    {
+        if (!partition_strategy.has_value())
+            throw Exception(ErrorCodes::LOGICAL_ERROR, "Expected partition strategy to be specified");
+
+        /// If partition_stategy == none, we add hive columns, if present, to virtual columns.
+        if (context->getSettingsRef()[Setting::use_hive_partitioning]
+            && partition_strategy == PartitionStrategyFactory::StrategyType::NONE)
+        {
+            auto hive_columns = HivePartitioningUtils::extractHivePartitionColumnsFromPath(storage_columns, path, format_settings, context);
+            for (const auto & column : hive_columns)
+                add_virtual(column);
+        }
+    }
+
     return desc;
 }
 
-static void addPathAndFileToVirtualColumns(Block & block, const String & path, size_t idx, const FormatSettings & format_settings, bool parse_hive_columns)
+static void addPathAndFileToVirtualColumns(
+    Block & block,
+    const String & path,
+    size_t idx,
+    const FormatSettings & format_settings,
+    bool parse_hive_columns)
 {
     if (block.has("_path"))
         block.getByName("_path").column->assumeMutableRef().insert(path);
@@ -212,7 +246,11 @@ static void addPathAndFileToVirtualColumns(Block & block, const String & path, s
     block.getByName("_idx").column->assumeMutableRef().insert(idx);
 }
 
-std::optional<ActionsDAG> createPathAndFileFilterDAG(const ActionsDAG::Node * predicate, const NamesAndTypesList & virtual_columns, const NamesAndTypesList & hive_columns)
+std::optional<ActionsDAG> createPathAndFileFilterDAG(
+    const ActionsDAG::Node * predicate,
+    const NamesAndTypesList & virtual_columns,
+    const ContextPtr & context,
+    const NamesAndTypesList & hive_columns)
 {
     if (!predicate || virtual_columns.empty())
         return {};
@@ -231,10 +269,15 @@ std::optional<ActionsDAG> createPathAndFileFilterDAG(const ActionsDAG::Node * pr
     }
 
     block.insert({ColumnUInt64::create(), std::make_shared<DataTypeUInt64>(), "_idx"});
-    return splitFilterDagForAllowedInputs(predicate, &block);
+    return splitFilterDagForAllowedInputs(predicate, &block, context);
 }
 
-ColumnPtr getFilterByPathAndFileIndexes(const std::vector<String> & paths, const ExpressionActionsPtr & actions, const NamesAndTypesList & virtual_columns, const NamesAndTypesList & hive_columns, const ContextPtr & context)
+ColumnPtr getFilterByPathAndFileIndexes(
+    const std::vector<String> & paths,
+    const ExpressionActionsPtr & actions,
+    const NamesAndTypesList & virtual_columns,
+    const NamesAndTypesList & hive_columns,
+    const ContextPtr & context)
 {
     Block block;
     NameSet common_virtuals = getVirtualNamesForFileLikeStorage();
@@ -252,7 +295,14 @@ ColumnPtr getFilterByPathAndFileIndexes(const std::vector<String> & paths, const
     block.insert({ColumnUInt64::create(), std::make_shared<DataTypeUInt64>(), "_idx"});
 
     for (size_t i = 0; i != paths.size(); ++i)
-        addPathAndFileToVirtualColumns(block, paths[i], i, getFormatSettings(context), /* parse_hive_columns */ !hive_columns.empty());
+    {
+        addPathAndFileToVirtualColumns(
+            block,
+            paths[i],
+            /* idx */i,
+            getFormatSettings(context),
+            /* parse_hive_columns */context->getSettingsRef()[Setting::use_hive_partitioning] || !hive_columns.empty());
+    }
 
     filterBlockWithExpression(actions, block);
 
@@ -260,9 +310,15 @@ ColumnPtr getFilterByPathAndFileIndexes(const std::vector<String> & paths, const
 }
 
 void addRequestedFileLikeStorageVirtualsToChunk(
-    Chunk & chunk, const NamesAndTypesList & requested_virtual_columns,
-    VirtualsForFileLikeStorage virtual_values, ContextPtr)
+    Chunk & chunk,
+    const NamesAndTypesList & requested_virtual_columns,
+    VirtualsForFileLikeStorage virtual_values,
+    ContextPtr context)
 {
+    HivePartitioningUtils::HivePartitioningKeysAndValues hive_map;
+    if (context->getSettingsRef()[Setting::use_hive_partitioning])
+        hive_map = HivePartitioningUtils::parseHivePartitioningKeysAndValues(virtual_values.path);
+
     for (const auto & virtual_column : requested_virtual_columns)
     {
         if (virtual_column.name == "_path")
@@ -303,6 +359,21 @@ void addRequestedFileLikeStorageVirtualsToChunk(
             else
                 chunk.addColumn(virtual_column.type->createColumnConstWithDefaultValue(chunk.getNumRows())->convertToFullColumnIfConst());
         }
+        else if (virtual_column.name == "_tags")
+        {
+            if (virtual_values.tags)
+            {
+                Map map_value;
+                for (const auto & [key, value] : *virtual_values.tags)
+                    map_value.push_back(Field(Tuple{Field(key), Field(value)}));
+
+                chunk.addColumn(virtual_column.type->createColumnConst(chunk.getNumRows(), map_value)->convertToFullColumnIfConst());
+            }
+            else
+            {
+                chunk.addColumn(virtual_column.type->createColumnConstWithDefaultValue(chunk.getNumRows())->convertToFullColumnIfConst());
+            }
+        }
         else if (virtual_column.name == "_data_lake_snapshot_version")
         {
             if (virtual_values.data_lake_snapshot_version)
@@ -331,6 +402,13 @@ void addRequestedFileLikeStorageVirtualsToChunk(
             /// Row numbers not known, _row_number = NULL.
             chunk.addColumn(virtual_column.type->createColumnConstWithDefaultValue(chunk.getNumRows())->convertToFullColumnIfConst());
         }
+        else if (auto it = hive_map.find(virtual_column.getNameInStorage()); it != hive_map.end())
+        {
+            chunk.addColumn(
+                virtual_column.type->createColumnConst(
+                    chunk.getNumRows(),
+                    convertFieldToType(Field(it->second), *virtual_column.type))->convertToFullColumnIfConst());
+        }
     }
 }
 
@@ -403,7 +481,7 @@ bool isDeterministicInScopeOfQuery(const ActionsDAG::Node * node)
 }
 
 static const ActionsDAG::Node * splitFilterNodeForAllowedInputs(
-    const ActionsDAG::Node * node, const Block * allowed_inputs, ActionsDAG::Nodes & additional_nodes, bool allow_partial_result)
+    const ActionsDAG::Node * node, const Block * allowed_inputs, ActionsDAG::Nodes & additional_nodes, const ContextPtr & context, bool allow_partial_result)
 {
     if (node->type == ActionsDAG::ActionType::FUNCTION)
     {
@@ -413,7 +491,7 @@ static const ActionsDAG::Node * splitFilterNodeForAllowedInputs(
             node_copy.children.clear();
             for (const auto * child : node->children)
                 if (const auto * child_copy
-                    = splitFilterNodeForAllowedInputs(child, allowed_inputs, additional_nodes, allow_partial_result))
+                    = splitFilterNodeForAllowedInputs(child, allowed_inputs, additional_nodes, context, allow_partial_result))
                     node_copy.children.push_back(child_copy);
                 /// Expression like (now_allowed AND allowed) is not allowed if allow_partial_result = true. This is important for
                 /// trivial count optimization, otherwise we can get incorrect results. For example, if the query is
@@ -433,7 +511,7 @@ static const ActionsDAG::Node * splitFilterNodeForAllowedInputs(
                 if (!res->result_type->equals(*node->result_type))
                 {
                     ActionsDAG tmp_dag;
-                    res = &tmp_dag.addCast(*res, node->result_type, {});
+                    res = &tmp_dag.addCast(*res, node->result_type, {}, context);
                     additional_nodes.splice(additional_nodes.end(), ActionsDAG::detachNodes(std::move(tmp_dag)));
                 }
 
@@ -446,7 +524,7 @@ static const ActionsDAG::Node * splitFilterNodeForAllowedInputs(
         {
             auto & node_copy = additional_nodes.emplace_back(*node);
             for (auto & child : node_copy.children)
-                if (child = splitFilterNodeForAllowedInputs(child, allowed_inputs, additional_nodes, allow_partial_result); !child)
+                if (child = splitFilterNodeForAllowedInputs(child, allowed_inputs, additional_nodes, context, allow_partial_result); !child)
                     return nullptr;
 
             return &node_copy;
@@ -461,7 +539,7 @@ static const ActionsDAG::Node * splitFilterNodeForAllowedInputs(
                     ActionsDAG::NodeRawConstPtrs atoms;
                     for (const auto & output : index_hint_dag.getOutputs())
                         if (const auto * child_copy
-                            = splitFilterNodeForAllowedInputs(output, allowed_inputs, additional_nodes, allow_partial_result))
+                            = splitFilterNodeForAllowedInputs(output, allowed_inputs, additional_nodes, context, allow_partial_result))
                             atoms.push_back(child_copy);
 
                     if (!atoms.empty())
@@ -476,7 +554,7 @@ static const ActionsDAG::Node * splitFilterNodeForAllowedInputs(
                         }
 
                         if (!res->result_type->equals(*node->result_type))
-                            res = &index_hint_dag.addCast(*res, node->result_type, {});
+                            res = &index_hint_dag.addCast(*res, node->result_type, {}, context);
 
                         additional_nodes.splice(additional_nodes.end(), ActionsDAG::detachNodes(std::move(index_hint_dag)));
                         return res;
@@ -497,13 +575,13 @@ static const ActionsDAG::Node * splitFilterNodeForAllowedInputs(
 }
 
 std::optional<ActionsDAG>
-splitFilterDagForAllowedInputs(const ActionsDAG::Node * predicate, const Block * allowed_inputs, bool allow_partial_result)
+splitFilterDagForAllowedInputs(const ActionsDAG::Node * predicate, const Block * allowed_inputs, const ContextPtr & context, bool allow_partial_result)
 {
     if (!predicate)
         return {};
 
     ActionsDAG::Nodes additional_nodes;
-    const auto * res = splitFilterNodeForAllowedInputs(predicate, allowed_inputs, additional_nodes, allow_partial_result);
+    const auto * res = splitFilterNodeForAllowedInputs(predicate, allowed_inputs, additional_nodes, context, allow_partial_result);
     if (!res)
         return {};
 
@@ -513,7 +591,7 @@ splitFilterDagForAllowedInputs(const ActionsDAG::Node * predicate, const Block *
 void filterBlockWithPredicate(
     const ActionsDAG::Node * predicate, Block & block, ContextPtr context, bool allow_filtering_with_partial_predicate)
 {
-    auto dag = splitFilterDagForAllowedInputs(predicate, &block, /*allow_partial_result=*/allow_filtering_with_partial_predicate);
+    auto dag = splitFilterDagForAllowedInputs(predicate, &block, context, /*allow_partial_result=*/allow_filtering_with_partial_predicate);
     if (dag)
         filterBlockWithExpression(buildFilterExpression(std::move(*dag), context), block);
 }
