@@ -125,7 +125,19 @@ if [[ "$USE_S3_STORAGE_FOR_MERGE_TREE" == "1" ]]; then
 elif [[ "$USE_AZURE_STORAGE_FOR_MERGE_TREE" == "1" ]]; then
     TEMP_POLICY="azure_cache"
 else
-    TEMP_POLICY="default"
+    random=$((RANDOM % 3))
+    if [[ $random -eq 0 ]]; then
+        TEMP_POLICY="default"
+        echo "Using local storage policy"
+    elif [[ $random -eq 1 ]]; then
+        TEMP_POLICY="s3_cache"
+        export USE_S3_STORAGE_FOR_MERGE_TREE=1
+        echo "Using s3 storage policy"
+    elif [[ $random -eq 2 ]]; then
+        TEMP_POLICY="azure_cache"
+        export USE_AZURE_STORAGE_FOR_MERGE_TREE=1
+        echo "Using azure storage policy"
+    fi
 fi
 
 
