@@ -2,6 +2,10 @@
 mysql_user = "root"
 mysql_pass = "ClickHouse_MySQL_P@ssw0rd"
 
+# Dremio CREDENTIALS
+dremio_user = "user"
+dremio_pass = "ClickHouse_Dremio_P@ssw0rd"
+
 # POSTGRES CREDENTIALS
 pg_user = "postgres"
 pg_pass = "ClickHouse_PostgreSQL_P@ssw0rd"
