@@ -155,10 +155,14 @@ def write_iceberg_from_df(
         if partition_by is None:
             df.writeTo(table_name).tableProperty(
                 "format-version", format_version
+            ).tableProperty(
+                "write.parquet.row-group-size-bytes", "104850"  # 1MB
             ).using("iceberg").create()
         else:
             df.writeTo(table_name).tableProperty(
                 "format-version", format_version
+            ).tableProperty(
+                "write.parquet.row-group-size-bytes", "104850"  # 1MB
             ).partitionedBy(partition_by).using("iceberg").create()
     else:
         df.writeTo(table_name).append()
