@@ -184,7 +184,8 @@ def test_backup_restore(cluster):
         f"CREATE TABLE test_simple_write_connection_string (key UInt64, data String) Engine = AzureBlobStorage('{cluster.env_variables['AZURITE_CONNECTION_STRING']}', 'cont', 'test_simple_write_c.csv', 'CSV')",
     )
     azure_query(
-        node, f"INSERT INTO test_simple_write_connection_string SETTINGS azure_truncate_on_insert = 1 VALUES (1, 'a')"
+        node,
+        f"INSERT INTO test_simple_write_connection_string SETTINGS azure_truncate_on_insert = 1 VALUES (1, 'a')",
     )
     print(get_azure_file_content("test_simple_write_c.csv", port))
     assert get_azure_file_content("test_simple_write_c.csv", port) == '1,"a"\n'
@@ -196,7 +197,9 @@ def test_backup_restore(cluster):
         f"BACKUP TABLE test_simple_write_connection_string TO {backup_destination}",
     )
     print(get_azure_file_content(f"{backup_name}/.backup", port))
-    azure_query(node, "DROP TABLE IF EXISTS test_simple_write_connection_string_restored")
+    azure_query(
+        node, "DROP TABLE IF EXISTS test_simple_write_connection_string_restored"
+    )
     azure_query(
         node,
         f"RESTORE TABLE test_simple_write_connection_string AS test_simple_write_connection_string_restored FROM {backup_destination};",
@@ -216,7 +219,8 @@ def test_backup_restore_diff_container(cluster):
         f"CREATE TABLE test_simple_write_connection_string_cont1 (key UInt64, data String) Engine = AzureBlobStorage('{cluster.env_variables['AZURITE_CONNECTION_STRING']}', 'cont', 'test_simple_write_c_cont1.csv', 'CSV')",
     )
     azure_query(
-        node, f"INSERT INTO test_simple_write_connection_string_cont1 SETTINGS azure_truncate_on_insert = 1 VALUES (1, 'a')"
+        node,
+        f"INSERT INTO test_simple_write_connection_string_cont1 SETTINGS azure_truncate_on_insert = 1 VALUES (1, 'a')",
     )
     backup_name = new_backup_name()
     backup_destination = f"AzureBlobStorage('{cluster.env_variables['AZURITE_CONNECTION_STRING']}', 'cont1', '{backup_name}')"
@@ -224,7 +228,9 @@ def test_backup_restore_diff_container(cluster):
         node,
         f"BACKUP TABLE test_simple_write_connection_string_cont1 TO {backup_destination}",
     )
-    azure_query(node, "DROP TABLE IF EXISTS test_simple_write_connection_string_restored_cont1")
+    azure_query(
+        node, "DROP TABLE IF EXISTS test_simple_write_connection_string_restored_cont1"
+    )
     azure_query(
         node,
         f"RESTORE TABLE test_simple_write_connection_string_cont1 AS test_simple_write_connection_string_restored_cont1 FROM {backup_destination};",
@@ -245,7 +251,10 @@ def test_backup_restore_with_named_collection_azure_conf1(cluster):
         node,
         f"CREATE TABLE test_write_connection_string (key UInt64, data String) Engine = AzureBlobStorage('{cluster.env_variables['AZURITE_CONNECTION_STRING']}', 'cont', 'test_simple_write.csv', 'CSV')",
     )
-    azure_query(node, f"INSERT INTO test_write_connection_string SETTINGS azure_truncate_on_insert = 1 VALUES (1, 'a')")
+    azure_query(
+        node,
+        f"INSERT INTO test_write_connection_string SETTINGS azure_truncate_on_insert = 1 VALUES (1, 'a')",
+    )
     print(get_azure_file_content("test_simple_write.csv", port))
     assert get_azure_file_content("test_simple_write.csv", port) == '1,"a"\n'
 
@@ -275,14 +284,15 @@ def test_backup_restore_with_named_collection_azure_conf2(cluster):
         node,
         f"CREATE TABLE test_write_connection_string_2 (key UInt64, data String) Engine = AzureBlobStorage('{cluster.env_variables['AZURITE_CONNECTION_STRING']}', 'cont', 'test_simple_write_2.csv', 'CSV')",
     )
-    azure_query(node, f"INSERT INTO test_write_connection_string_2 SETTINGS azure_truncate_on_insert = 1 VALUES (1, 'a')")
+    azure_query(
+        node,
+        f"INSERT INTO test_write_connection_string_2 SETTINGS azure_truncate_on_insert = 1 VALUES (1, 'a')",
+    )
     print(get_azure_file_content("test_simple_write_2.csv", port))
     assert get_azure_file_content("test_simple_write_2.csv", port) == '1,"a"\n'
 
     backup_name = new_backup_name()
-    backup_destination = (
-        f"AzureBlobStorage(azure_conf2, '{backup_name}')"
-    )
+    backup_destination = f"AzureBlobStorage(azure_conf2, '{backup_name}')"
     azure_query(
         node,
         f"BACKUP TABLE test_write_connection_string_2 TO {backup_destination}",
@@ -400,7 +410,9 @@ def test_backup_restore_correct_block_ids(cluster):
             else:
                 assert block.get("size") < expected_block_size
 
-        azure_query(node, f"DROP TABLE IF EXISTS test_simple_merge_tree_restored_{max_blocks}")
+        azure_query(
+            node, f"DROP TABLE IF EXISTS test_simple_merge_tree_restored_{max_blocks}"
+        )
         azure_query(
             node,
             f"RESTORE TABLE test_simple_merge_tree AS test_simple_merge_tree_restored_{max_blocks} FROM {backup_destination};",
@@ -409,3 +421,57 @@ def test_backup_restore_correct_block_ids(cluster):
             node,
             f"SELECT * from test_simple_merge_tree_restored_{max_blocks} ORDER BY key",
         ) == node.query(data_query)
+
+
+def test_backup_restore_with_checksum_data_file_name(cluster):
+    node = cluster.instances["node"]
+    port = cluster.env_variables["AZURITE_PORT"]
+    azure_query(node, "DROP TABLE IF EXISTS test")
+    azure_query(
+        node,
+        f"CREATE TABLE test (key UInt64, data String) Engine = AzureBlobStorage('{cluster.env_variables['AZURITE_CONNECTION_STRING']}', 'cont', 'test_simple_write_c.csv', 'CSV')",
+    )
+    azure_query(
+        node, f"INSERT INTO test SETTINGS azure_truncate_on_insert = 1 VALUES (1, 'a')"
+    )
+
+    backup_name = new_backup_name()
+    backup_destination = f"AzureBlobStorage('{cluster.env_variables['AZURITE_CONNECTION_STRING']}', 'cont', '{backup_name}')"
+    azure_query(
+        node,
+        f"BACKUP TABLE test TO {backup_destination} SETTINGS data_file_name_generator='checksum'",
+    )
+    print(get_azure_file_content(f"{backup_name}/.backup", port))
+    azure_query(node, "DROP TABLE IF EXISTS test_restored")
+    azure_query(
+        node,
+        f"RESTORE TABLE test AS test_restored FROM {backup_destination};",
+    )
+    assert azure_query(node, f"SELECT * from test_restored") == "1\ta\n"
+
+
+def test_backup_restore_on_merge_tree_with_checksum_data_file_name(cluster):
+    node = cluster.instances["node"]
+    azure_query(
+        node,
+        f"""
+        DROP TABLE IF EXISTS test;
+        CREATE TABLE test(key UInt64, data String) Engine = MergeTree() ORDER BY tuple() SETTINGS storage_policy='blob_storage_policy'
+        """,
+    )
+    azure_query(node, f"INSERT INTO test VALUES (1, 'a')")
+
+    backup_name = new_backup_name()
+    backup_destination = f"AzureBlobStorage('{cluster.env_variables['AZURITE_CONNECTION_STRING']}', 'cont', '{backup_name}')"
+    azure_query(
+        node,
+        f"BACKUP TABLE test TO {backup_destination} SETTINGS data_file_name_generator='checksum'",
+    )
+    azure_query(node, f"DROP TABLE IF EXISTS test_restored")
+    azure_query(
+        node,
+        f"RESTORE TABLE test AS test_restored FROM {backup_destination};",
+    )
+    assert azure_query(node, f"SELECT * from test_restored") == "1\ta\n"
+    azure_query(node, f"DROP TABLE test")
+    azure_query(node, f"DROP TABLE test_restored")
