@@ -1,12 +1,13 @@
 import os
 
 import pytest
+
 from helpers.cluster import ClickHouseCluster
 
 SCRIPT_DIR = os.path.dirname(os.path.realpath(__file__))
 
 cluster = ClickHouseCluster(__file__)
-node = cluster.add_instance('node', main_configs=["configs/config.d/bad.xml"])
+node = cluster.add_instance("node", main_configs=["configs/config.d/bad.xml"])
 caught_exception = ""
 
 
@@ -21,4 +22,9 @@ def start_cluster():
 
 def test_work(start_cluster):
     print(caught_exception)
-    assert caught_exception.find("Root element doesn't have the corresponding root element as the config file.") != -1
+    assert (
+        caught_exception.find(
+            "Root element doesn't have the corresponding root element as the config file."
+        )
+        != -1
+    )
