@@ -2,11 +2,11 @@
 
 import sys
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     for chunk_header in sys.stdin:
         chunk_length = int(chunk_header)
-        print(1, end='\n')
-        print(str(chunk_length), end='\n')
+        print(1, end="\n")
+        print(str(chunk_length), end="\n")
 
         while chunk_length != 0:
             line = sys.stdin.readline()
