@@ -1,9 +1,13 @@
-import pytest
-from helpers.cluster import ClickHouseCluster
 import logging
 
+import pytest
+
+from helpers.cluster import ClickHouseCluster
+
 cluster = ClickHouseCluster(__file__)
-node = cluster.add_instance("node", main_configs=["configs/config.xml"], with_zookeeper=True)
+node = cluster.add_instance(
+    "node", main_configs=["configs/config.xml"], with_zookeeper=True
+)
 
 
 @pytest.fixture(scope="module")
