@@ -3,10 +3,10 @@ import pytest
 from helpers.cluster import ClickHouseCluster
 
 cluster = ClickHouseCluster(__file__)
-node = cluster.add_instance('node', with_zookeeper=True)
+node = cluster.add_instance("node", with_zookeeper=True)
 
 
-@pytest.fixture(scope='module', autouse=True)
+@pytest.fixture(scope="module", autouse=True)
 def started_cluster():
     try:
         cluster.start()
@@ -17,7 +17,7 @@ def started_cluster():
 
 def test_replicated_engine_parse_metadata_on_attach():
     node.query(
-        '''
+        """
         CREATE TABLE data (
             key Int,
             INDEX key_idx0 key+0 TYPE minmax GRANULARITY 1,
@@ -25,15 +25,18 @@ def test_replicated_engine_parse_metadata_on_attach():
         )
         ENGINE = ReplicatedMergeTree('/ch/tables/default/data', 'node')
         ORDER BY key;
-        ''')
-    node.query('DETACH TABLE data')
+        """
+    )
+    node.query("DETACH TABLE data")
 
-    zk = cluster.get_kazoo_client('zoo1')
+    zk = cluster.get_kazoo_client("zoo1")
     # Add **extra space between indices**, to check that it will be re-parsed
     # and successfully accepted by the server.
     #
     # This metadata was obtain from the server without #11325
-    zk.set('/ch/tables/default/data/replicas/node/metadata', b"""
+    zk.set(
+        "/ch/tables/default/data/replicas/node/metadata",
+        b"""
 metadata format version: 1
 date column: 
 sampling expression: 
@@ -46,5 +49,6 @@ partition key:
 indices:  key_idx0 key + 0 TYPE minmax GRANULARITY 1,  key_idx1 key + 1 TYPE minmax GRANULARITY 1
 granularity bytes: 10485760
 
-""".lstrip())
-    node.query('ATTACH TABLE data')
+""".lstrip(),
+    )
+    node.query("ATTACH TABLE data")
