@@ -302,11 +302,11 @@ def test_ytsaurus_composite_types(
     print(create_command)
     yt.exec(create_command)
     if len(yt_data) > 0:
-        yt.write_table(table_path, yt_data_json)
+        yt.write_table(table_path, yt_data_json, False)
     instance.query(
         f"CREATE TABLE yt_test(a {ch_column_type}) ENGINE=YTsaurus('{yt_uri_helper.uri}', '{table_path}', '{yt_uri_helper.token}')"
     )
-    yt.write_table(table_path, yt_data_json)
+    yt.write_table(table_path, yt_data_json, False)
     try:
         assert instance.query("SELECT a FROM yt_test") == f"{ch_data_expected}\n"
     except:
@@ -452,7 +452,6 @@ def test_yt_multiple_endpoints(started_cluster):
     yt.remove_table("//tmp/table")
 
 
-
 def test_ytsaurus_cyrillic_strings(started_cluster):
     table_path = "//tmp/table"
     yt = YTsaurusCLI(started_cluster, instance, yt_uri_helper.host, yt_uri_helper.port)
@@ -492,3 +491,27 @@ def test_ytsaurus_select_subset_of_columns(started_cluster):
     )
     instance.query("DROP TABLE yt_test SYNC")
     yt.remove_table(f"{table_path}")
+
+
+def test_ytsaurus_replicated_table(started_cluster):
+    table_path = "//tmp/replicated_table"
+    yt = YTsaurusCLI(
+        started_cluster,
+        instance,
+        yt_uri_helper.host,
+        yt_uri_helper.port,
+        yt_uri_helper.ytcluster_name,
+    )
+
+    yt.create_replciated_table(
+        table_path,
+        yt_uri_helper.ytcluster_name,
+        '{"a":10,"b":20, "c": 1}{"a":20,"b":40, "c": 2}',
+        schema={"a": "int32", "b": "int32", "c": "int32"},
+    )
+    instance.query(
+        f"CREATE TABLE yt_test(a Int32, b Int32) ENGINE=YTsaurus('{yt_uri_helper.uri}', '{table_path}', '{yt_uri_helper.token}') SETTINGS check_table_schema = 0"
+    )
+    assert instance.query("SELECT * FROM yt_test") == "10\t20\n20\t40\n"
+    instance.query("DROP TABLE yt_test SYNC")
+    yt.remove_replicated_table(f"{table_path}")
