@@ -8,4 +8,5 @@ ANY INNER JOIN
 (
     SELECT number * 2 AS k, number AS joined FROM system.numbers LIMIT 10
 ) AS b
-USING k;
+USING k
+ORDER BY ALL;
