@@ -11,7 +11,7 @@ CREATE TABLE Issue_2231_Invalid_Nested_Columns_Size (
     PARTITION BY tuple()
     ORDER BY Date;
 
-INSERT INTO Issue_2231_Invalid_Nested_Columns_Size VALUES (today(), [2,2], [1]), (today(), [2,2], [1, 1]); -- { serverError 190 }
+INSERT INTO Issue_2231_Invalid_Nested_Columns_Size VALUES (today(), [2,2], [1]), (today(), [2,2], [1, 1]); -- { serverError SIZES_OF_ARRAYS_DONT_MATCH }
 
 SELECT * FROM Issue_2231_Invalid_Nested_Columns_Size;
 DROP TABLE Issue_2231_Invalid_Nested_Columns_Size;
