@@ -1,5 +1,5 @@
 drop table if exists tab_00717;
-create table tab_00717 (a String, b StringWithDictionary) engine = MergeTree order by a;
+create table tab_00717 (a String, b LowCardinality(String)) engine = MergeTree order by a;
 insert into tab_00717 values ('a_1', 'b_1'), ('a_2', 'b_2');
 select count() from tab_00717;
 select a from tab_00717 group by a order by a;
