@@ -15,6 +15,12 @@ SELECT quantilesExact(0.2)(d) FROM datetime;
 SELECT quantileExactWeighted(0.2)(d, 1) FROM datetime;
 SELECT quantilesExactWeighted(0.2)(d, 1) FROM datetime;
 
+SELECT quantileInterpolatedWeighted(0.2)(d, 1) FROM datetime;
+SELECT quantilesInterpolatedWeighted(0.2)(d, 1) FROM datetime;
+
+SELECT quantileExactWeightedInterpolated(0.2)(d, 1) FROM datetime;
+SELECT quantilesExactWeightedInterpolated(0.2)(d, 1) FROM datetime;
+
 SELECT quantileTiming(0.2)(d) FROM datetime;
 SELECT quantilesTiming(0.2)(d) FROM datetime;
 
