@@ -152,45 +152,45 @@ DROP TABLE arrays_test;
 
 
 select '---------BAD';
-SELECT arrayEnumerateUniqRanked(); -- { serverError 42 }
+SELECT arrayEnumerateUniqRanked(); -- { serverError TOO_FEW_ARGUMENTS_FOR_FUNCTION }
 SELECT arrayEnumerateUniqRanked([]);
-SELECT arrayEnumerateUniqRanked(1); -- { serverError 36 }
-SELECT arrayEnumerateUniqRanked(2,[]); -- { serverError 36 }
-SELECT arrayEnumerateUniqRanked(2,[],2); -- { serverError 36 }
-SELECT arrayEnumerateUniqRanked(2,[],[]); -- { serverError 36 }
-SELECT arrayEnumerateUniqRanked(2,[],[],3); -- { serverError 36 }
-SELECT arrayEnumerateUniqRanked([],2); -- { serverError 36 }
-SELECT arrayEnumerateUniqRanked([],2,[]); -- { serverError 36 }
-SELECT arrayEnumerateUniqRanked(0,[],0); -- { serverError 36 }
-SELECT arrayEnumerateUniqRanked(0,0,0); -- { serverError 36 }
-SELECT arrayEnumerateUniqRanked(1,1,1); -- { serverError 36 }
-SELECT arrayEnumerateDenseRanked(1, [10,20,10,30], 0); -- { serverError 36 }
-SELECT arrayEnumerateUniqRanked(1, [[7,8,9,10],[10,11,12]], 2, [[14,15,16],[17,18,19],[20],[21]], 2); -- { serverError 190 }
+SELECT arrayEnumerateUniqRanked(1); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateUniqRanked(2,[]); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateUniqRanked(2,[],2); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateUniqRanked(2,[],[]); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateUniqRanked(2,[],[],3); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateUniqRanked([],2); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateUniqRanked([],2,[]); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateUniqRanked(0,[],0); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateUniqRanked(0,0,0); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateUniqRanked(1,1,1); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateDenseRanked(1, [10,20,10,30], 0); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateUniqRanked(1, [[7,8,9,10],[10,11,12]], 2, [[14,15,16],[17,18,19],[20],[21]], 2); -- { serverError SIZES_OF_ARRAYS_DONT_MATCH }
 
-SELECT arrayEnumerateUniqRanked(1, [1,2], 1, ['a', 'b', 'c', 'd'],1); -- { serverError 190 }
-SELECT arrayEnumerateUniqRanked(1, [1,2], 1, [14,15,16,17,18,19], 1); -- { serverError 190 }
-SELECT arrayEnumerateUniqRanked(1, [14,15,16,17,18,19], 1, [1,2], 1); -- { serverError 190 }
-SELECT arrayEnumerateUniqRanked(1, [1,1,1,1,1,1], 1, [1,1], 1); -- { serverError 190 }
-SELECT arrayEnumerateUniqRanked(1, [1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1], 1, [1,1], 1); -- { serverError 190 }
+SELECT arrayEnumerateUniqRanked(1, [1,2], 1, ['a', 'b', 'c', 'd'],1); -- { serverError SIZES_OF_ARRAYS_DONT_MATCH }
+SELECT arrayEnumerateUniqRanked(1, [1,2], 1, [14,15,16,17,18,19], 1); -- { serverError SIZES_OF_ARRAYS_DONT_MATCH }
+SELECT arrayEnumerateUniqRanked(1, [14,15,16,17,18,19], 1, [1,2], 1); -- { serverError SIZES_OF_ARRAYS_DONT_MATCH }
+SELECT arrayEnumerateUniqRanked(1, [1,1,1,1,1,1], 1, [1,1], 1); -- { serverError SIZES_OF_ARRAYS_DONT_MATCH }
+SELECT arrayEnumerateUniqRanked(1, [1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1], 1, [1,1], 1); -- { serverError SIZES_OF_ARRAYS_DONT_MATCH }
 SELECT arrayEnumerateDenseRanked([], [], []);
 SELECT arrayEnumerateDenseRanked([], [], [], [], [], [], [], [], [], [], [], [], [], [], [], [], [], [], [], [], [], []);
 SELECT arrayEnumerateDenseRanked([1,2], [1,2], [1,2]);
 SELECT arrayEnumerateUniqRanked([1,2], [1,2], [1,2]);
-SELECT arrayEnumerateUniqRanked([1,2], 3, 4, 5); -- { serverError 36 }
-SELECT arrayEnumerateUniqRanked([1,2], 1, 2); -- { serverError 36 }
-SELECT arrayEnumerateUniqRanked([1,2], 1, 3, 4, 5); -- { serverError 36 }
-SELECT arrayEnumerateUniqRanked([1,2], 1, 3, [4], 5); -- { serverError 36 }
+SELECT arrayEnumerateUniqRanked([1,2], 3, 4, 5); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateUniqRanked([1,2], 1, 2); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateUniqRanked([1,2], 1, 3, 4, 5); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateUniqRanked([1,2], 1, 3, [4], 5); -- { serverError BAD_ARGUMENTS }
 SELECT arrayEnumerateDenseRanked([[[[[[[[[[42]]]]]]]]]]);
-SELECT arrayEnumerateUniqRanked('wat', [1,2]); -- { serverError 170 }
-SELECT arrayEnumerateUniqRanked(1, [1,2], 'boom'); -- { serverError 170 }
-SELECT arrayEnumerateDenseRanked(['\0'], -8363126); -- { serverError 170 }
-SELECT arrayEnumerateDenseRanked(-10, ['\0'], -8363126); -- { serverError 170 }
-SELECT arrayEnumerateDenseRanked(1, ['\0'], -8363126); -- { serverError 170 }
-SELECT arrayEnumerateDenseRanked(-101, ['\0']); -- { serverError 170 }
-SELECT arrayEnumerateDenseRanked(1.1, [10,20,10,30]); -- { serverError 170 }
-SELECT arrayEnumerateDenseRanked([10,20,10,30], 0.4); -- { serverError 170 }
-SELECT arrayEnumerateDenseRanked([10,20,10,30], 1.8); -- { serverError 170 }
-SELECT arrayEnumerateUniqRanked(1, [], 1000000000); -- { serverError 36 }
+SELECT arrayEnumerateUniqRanked('wat', [1,2]); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateUniqRanked(1, [1,2], 'boom'); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateDenseRanked(['\0'], -8363126); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateDenseRanked(-10, ['\0'], -8363126); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateDenseRanked(1, ['\0'], -8363126); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateDenseRanked(-101, ['\0']); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateDenseRanked(1.1, [10,20,10,30]); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateDenseRanked([10,20,10,30], 0.4); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateDenseRanked([10,20,10,30], 1.8); -- { serverError BAD_ARGUMENTS }
+SELECT arrayEnumerateUniqRanked(1, [], 1000000000); -- { serverError BAD_ARGUMENTS }
 
 
 -- skipping empty arrays
