@@ -2,7 +2,7 @@ SET joined_subquery_requires_alias = 0;
 
 DROP TABLE IF EXISTS ANIMAL;
 
-CREATE TABLE ANIMAL ( ANIMAL Nullable(String) ) engine = TinyLog;
+CREATE TABLE ANIMAL ( ANIMAL Nullable(String) ) engine = MergeTree ORDER BY tuple();
 INSERT INTO ANIMAL (ANIMAL) VALUES ('CAT'), ('FISH'), ('DOG'), ('HORSE'), ('BIRD');
 
 select * from (
