@@ -6,14 +6,14 @@ CREATE TABLE IF NOT EXISTS test_move_partition_src (
     val UInt32
 ) Engine = MergeTree()
   PARTITION BY pk
-  ORDER BY (pk, val);
+  ORDER BY (pk, val) SETTINGS index_granularity = 8192, index_granularity_bytes = '10Mi';
 
 CREATE TABLE IF NOT EXISTS test_move_partition_dest (
     pk UInt8,
     val UInt32
 ) Engine = MergeTree()
   PARTITION BY pk
-  ORDER BY (pk, val);
+  ORDER BY (pk, val) SETTINGS index_granularity = 8192, index_granularity_bytes = '10Mi';
 
 INSERT INTO test_move_partition_src SELECT number % 2, number FROM system.numbers LIMIT 10000000;
 
@@ -21,6 +21,7 @@ SELECT count() FROM test_move_partition_src;
 SELECT count() FROM test_move_partition_dest;
 
 ALTER TABLE test_move_partition_src MOVE PARTITION 1 TO TABLE test_move_partition_dest;
+ALTER TABLE test_move_partition_src MOVE PART '0_1_1_0' TO TABLE test_move_partition_dest;  -- { clientError SYNTAX_ERROR }
 
 SELECT count() FROM test_move_partition_src;
 SELECT count() FROM test_move_partition_dest;
