@@ -12,8 +12,8 @@ SELECT * FROM VALUES('n UInt64, s String, ss String', (1 + 22, '23', toString(23
 
 SELECT * FROM VALUES('a Decimal(4, 4), b String, c String', (divide(toDecimal32(5, 3), 3), 'a', 'b'));
 
-SELECT * FROM VALUES('x Float64', toUInt64(-1)); -- { serverError 69; }
-SELECT * FROM VALUES('x Float64', NULL); -- { serverError 53; }
+SELECT * FROM VALUES('x Float64', toUInt64(-1)); -- { serverError ARGUMENT_OUT_OF_BOUND }
+SELECT * FROM VALUES('x Float64', NULL); -- { serverError TYPE_MISMATCH }
 SELECT * FROM VALUES('x Nullable(Float64)', NULL);
 
 DROP TABLE values_list;
