@@ -10,4 +10,4 @@ ENGINE = SummingMergeTree([price, spend])
 PARTITION BY toYYYYMM(date)
 ORDER BY id
 SAMPLE BY id
-SETTINGS index_granularity = 8192; -- { serverError 223 }
+SETTINGS index_granularity = 8192; -- { serverError UNEXPECTED_AST_STRUCTURE }
