@@ -1,8 +1,8 @@
 DROP TABLE IF EXISTS l;
 DROP TABLE IF EXISTS r;
 
-CREATE TABLE l (a String, b Tuple(String, String)) ENGINE = Memory();
-CREATE TABLE r (a String, c Tuple(String, String)) ENGINE = Memory();
+CREATE TABLE l (a String, b Tuple(String, String)) ENGINE = MergeTree ORDER BY tuple();
+CREATE TABLE r (a String, c Tuple(String, String)) ENGINE = MergeTree ORDER BY tuple();
 
 INSERT INTO l (a, b) VALUES ('a', ('b', 'c')), ('d', ('e', 'f'));
 INSERT INTO r (a, c) VALUES ('a', ('b', 'c')), ('x', ('y', 'z'));
@@ -21,8 +21,8 @@ SELECT * from l RIGHT JOIN r USING a ORDER BY a;
 DROP TABLE l;
 DROP TABLE r;
 
-CREATE TABLE l (a String, b String) ENGINE = Memory();
-CREATE TABLE r (a String, c Array(String)) ENGINE = Memory();
+CREATE TABLE l (a String, b String) ENGINE = MergeTree ORDER BY tuple();
+CREATE TABLE r (a String, c Array(String)) ENGINE = MergeTree ORDER BY tuple();
 
 INSERT INTO l (a, b) VALUES ('a', 'b'), ('d', 'e');
 INSERT INTO r (a, c) VALUES ('a', ['b', 'c']), ('x', ['y', 'z']);
