@@ -1,4 +1,3 @@
--- Tags: no-parallel
 
 DROP TABLE IF EXISTS table_01;
 
@@ -21,4 +20,8 @@ ALTER TABLE table_01 ATTACH PART '20191001_1_1_0';
 
 SELECT COUNT() FROM table_01;
 
+ALTER TABLE table_01 DETACH PARTITION ALL;
+
+SELECT COUNT() FROM table_01;
+
 DROP TABLE IF EXISTS table_01;
