@@ -2,9 +2,12 @@
 
 SELECT arraySort(h3kRing(581276613233082367, toUInt16(1)));
 SELECT h3kRing(581276613233082367, toUInt16(0));
-SELECT h3kRing(581276613233082367, -1); -- { serverError 43 }
-SELECT h3kRing(581276613233082367, toUInt16(-1)); -- { serverError 12 }
+SELECT h3kRing(581276613233082367, -1); -- { serverError ILLEGAL_TYPE_OF_ARGUMENT }
+SELECT h3kRing(581276613233082367, toUInt16(-1)); -- { serverError PARAMETER_OUT_OF_BOUND }
 
+SELECT arraySort(h3kRing(581276613233082367, 1));
+SELECT h3kRing(581276613233082367, 0);
+SELECT h3kRing(581276613233082367, -1); -- { serverError ILLEGAL_TYPE_OF_ARGUMENT }
 
 DROP TABLE IF EXISTS h3_indexes;
 
