@@ -1,4 +1,3 @@
--- Tags: no-parallel
 
 DROP TABLE IF EXISTS t1;
 DROP TABLE IF EXISTS t2;
@@ -16,22 +15,16 @@ DROP TABLE t2;
 CREATE TABLE t3 AS numbers(10);
 DROP TABLE t3;
 
--- live view
-SET allow_experimental_live_view=1;
-CREATE LIVE VIEW lv AS SELECT * FROM t1;
-CREATE TABLE t3 AS lv; -- { serverError 80; }
-DROP TABLE lv;
-
 -- view
 CREATE VIEW v AS SELECT * FROM t1;
-CREATE TABLE t3 AS v; -- { serverError 80; }
+CREATE TABLE t3 AS v; -- { serverError INCORRECT_QUERY }
 DROP TABLE v;
 
 -- dictionary
 DROP DICTIONARY IF EXISTS dict;
-DROP DATABASE if exists test_01056_dict_data;
-CREATE DATABASE test_01056_dict_data;
-CREATE TABLE test_01056_dict_data.dict_data (key Int, value UInt16) Engine=Memory();
+DROP DATABASE if exists {CLICKHOUSE_DATABASE_1:Identifier};
+CREATE DATABASE {CLICKHOUSE_DATABASE_1:Identifier};
+CREATE TABLE {CLICKHOUSE_DATABASE_1:Identifier}.dict_data (key Int, value UInt16) Engine=Memory();
 CREATE DICTIONARY dict
 (
     `key` UInt64,
@@ -40,17 +33,17 @@ CREATE DICTIONARY dict
 PRIMARY KEY key
 SOURCE(CLICKHOUSE(
     HOST '127.0.0.1' PORT tcpPort()
-    TABLE 'dict_data' DB 'test_01056_dict_data' USER 'default' PASSWORD ''))
+    TABLE 'dict_data' DB concat(currentDatabase(), '_1') USER 'default' PASSWORD ''))
 LIFETIME(MIN 0 MAX 0)
 LAYOUT(SPARSE_HASHED());
-CREATE TABLE t3 AS dict; -- { serverError 80; }
+CREATE TABLE t3 AS dict; -- { serverError INCORRECT_QUERY }
 
 DROP TABLE IF EXISTS t1;
 DROP TABLE IF EXISTS t3;
 DROP DICTIONARY dict;
-DROP TABLE test_01056_dict_data.dict_data;
+DROP TABLE {CLICKHOUSE_DATABASE_1:Identifier}.dict_data;
 
-DROP DATABASE test_01056_dict_data;
+DROP DATABASE {CLICKHOUSE_DATABASE_1:Identifier};
 
 CREATE TABLE t1 (x String) ENGINE = Memory AS SELECT 1;
 SELECT x, toTypeName(x) FROM t1;
