@@ -1,5 +1,3 @@
--- Tags: no-parallel
-
 DROP DATABASE IF EXISTS memory_01069;
 CREATE DATABASE memory_01069 ENGINE = Memory;
 SHOW CREATE DATABASE memory_01069;
@@ -14,10 +12,10 @@ SELECT * FROM memory_01069.mt ORDER BY n;
 SELECT * FROM memory_01069.file ORDER BY n;
 
 DROP TABLE memory_01069.mt;
-SELECT * FROM memory_01069.mt ORDER BY n; -- { serverError 60 }
+SELECT * FROM memory_01069.mt ORDER BY n; -- { serverError UNKNOWN_TABLE }
 SELECT * FROM memory_01069.file ORDER BY n;
 
-SHOW CREATE TABLE memory_01069.mt; -- { serverError 60 }
+SHOW CREATE TABLE memory_01069.mt; -- { serverError UNKNOWN_TABLE }
 SHOW CREATE TABLE memory_01069.file;
 
 DROP DATABASE memory_01069;
