@@ -1,11 +1,6 @@
--- Tags: no-parallel
 
 SHOW TABLES NOT LIKE '%';
 
-DROP DATABASE IF EXISTS test_01073;
-CREATE DATABASE test_01073;
-USE test_01073;
-
 SHOW TABLES;
 SELECT '---';
 CREATE TABLE test1 (x UInt8) ENGINE = Memory;
@@ -22,14 +17,14 @@ SELECT '--';
 SHOW TABLES NOT LIKE 'tes%2';
 
 SELECT '---';
-SHOW TABLES FROM test_01073;
+SHOW TABLES FROM {CLICKHOUSE_DATABASE:Identifier};
 SELECT '--';
-SHOW TABLES FROM test_01073 LIKE 'tes%';
+SHOW TABLES FROM {CLICKHOUSE_DATABASE:Identifier} LIKE 'tes%';
 SELECT '--';
-SHOW TABLES FROM test_01073 NOT LIKE 'tes%';
+SHOW TABLES FROM {CLICKHOUSE_DATABASE:Identifier} NOT LIKE 'tes%';
 SELECT '--';
-SHOW TABLES FROM test_01073 LIKE 'tes%1';
+SHOW TABLES FROM {CLICKHOUSE_DATABASE:Identifier} LIKE 'tes%1';
 SELECT '--';
-SHOW TABLES FROM test_01073 NOT LIKE 'tes%2';
+SHOW TABLES FROM {CLICKHOUSE_DATABASE:Identifier} NOT LIKE 'tes%2';
 
-DROP DATABASE test_01073;
+DROP DATABASE {CLICKHOUSE_DATABASE:Identifier};
