@@ -6,7 +6,7 @@ CREATE TABLE test
     `a3` UInt64 DEFAULT a2 + 1,
     `a4` UInt64 ALIAS a3 + 1
 )
-ENGINE = Log; -- { serverError 174 }
+ENGINE = Log; -- { serverError CYCLIC_ALIASES }
 
 CREATE TABLE pythagoras
 (
@@ -14,6 +14,6 @@ CREATE TABLE pythagoras
     `b` Float64 DEFAULT sqrt((c * c) - (a * a)),
     `c` Float64 DEFAULT sqrt((a * a) + (b * b))
 )
-ENGINE = Log; -- { serverError 174 }
+ENGINE = Log; -- { serverError CYCLIC_ALIASES }
 
 -- TODO: It works but should not: CREATE TABLE test (a DEFAULT b, b DEFAULT a) ENGINE = Memory
