@@ -1,5 +1,5 @@
 DROP TABLE IF EXISTS pk_func;
-CREATE TABLE pk_func(d DateTime, ui UInt32) ENGINE = MergeTree ORDER BY toDate(d);
+CREATE TABLE pk_func(d DateTime, ui UInt32) ENGINE = MergeTree ORDER BY toDate(d) SETTINGS index_granularity = 8192, index_granularity_bytes = '10Mi';
 
 INSERT INTO pk_func SELECT '2020-05-05 01:00:00', number FROM numbers(1000000);
 INSERT INTO pk_func SELECT '2020-05-06 01:00:00', number FROM numbers(1000000);
@@ -10,7 +10,7 @@ SELECT * FROM pk_func ORDER BY toDate(d), ui LIMIT 5;
 DROP TABLE pk_func;
 
 DROP TABLE IF EXISTS nORX;
-CREATE TABLE nORX (`A` Int64, `B` Int64, `V` Int64) ENGINE = MergeTree ORDER BY (A, negate(B));
+CREATE TABLE nORX (`A` Int64, `B` Int64, `V` Int64) ENGINE = MergeTree ORDER BY (A, negate(B)) SETTINGS index_granularity = 8192, index_granularity_bytes = '10Mi';
 INSERT INTO nORX SELECT 111, number, number FROM numbers(10000000);
 
 SELECT *
