@@ -1,3 +1,5 @@
+-- Tags: long, no-msan, no-distributed-cache
+
 DROP TABLE IF EXISTS t;
 
 CREATE TABLE t
@@ -6,9 +8,9 @@ CREATE TABLE t
 )
 ENGINE = MergeTree
 ORDER BY number
-SETTINGS index_granularity = 128;
+SETTINGS index_granularity = 128, ratio_of_defaults_for_sparse_serialization = 1.0, index_granularity_bytes = '10Mi';
 
-SET min_insert_block_size_rows = 0, min_insert_block_size_bytes = 0;
+SET min_insert_block_size_rows = 0, min_insert_block_size_bytes = 0, max_insert_threads = 1;
 INSERT INTO t SELECT number FROM numbers(10000000);
 
 SET max_threads = 1, max_block_size = 12345;
