@@ -1,7 +1,11 @@
 -- Tags: no-parallel
 
+SET send_logs_level = 'fatal';
+
 DROP DATABASE IF EXISTS dict_db_01225;
 DROP DATABASE IF EXISTS dict_db_01225_dictionary;
+set allow_deprecated_database_ordinary=1;
+-- Creation of a database with Ordinary engine emits a warning.
 CREATE DATABASE dict_db_01225 ENGINE=Ordinary;    -- Different internal dictionary name with Atomic
 CREATE DATABASE dict_db_01225_dictionary Engine=Dictionary;
 
@@ -17,7 +21,7 @@ LIFETIME(MIN 0 MAX 0)
 LAYOUT(FLAT());
 
 SHOW CREATE TABLE dict_db_01225_dictionary.`dict_db_01225.dict` FORMAT TSVRaw;
-SHOW CREATE TABLE dict_db_01225_dictionary.`dict_db_01225.no_such_dict`; -- { serverError 487; }
+SHOW CREATE TABLE dict_db_01225_dictionary.`dict_db_01225.no_such_dict`; -- { serverError CANNOT_GET_CREATE_DICTIONARY_QUERY }
 
 DROP DATABASE dict_db_01225;
 DROP DATABASE dict_db_01225_dictionary;
