@@ -3,4 +3,7 @@ INSERT INTO enum VALUES ('hello');
 
 SELECT count() FROM enum WHERE x = 'hello';
 SELECT count() FROM enum WHERE x = 'world';
-SELECT count() FROM enum WHERE x = 'xyz'; -- { serverError 36 }
+SELECT count() FROM enum WHERE x = 'xyz';
+
+SET validate_enum_literals_in_operators = 1;
+SELECT count() FROM enum WHERE x = 'xyz'; -- { serverError UNKNOWN_ELEMENT_OF_ENUM }
