@@ -9,7 +9,7 @@ create table dist_01320 as data_01320 Engine=Distributed(test_cluster_two_shards
 
 set optimize_skip_unused_shards=1;
 set force_optimize_skip_unused_shards=1;
-select * from dist_01320 where key = 0; -- { serverError 507 }
+select * from dist_01320 where key = 0; -- { serverError UNABLE_TO_SKIP_UNUSED_SHARDS }
 
 drop table data_01320;
 drop table dist_01320;
