@@ -10,12 +10,11 @@ CURDIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
 
 echo "Starting clickhouse-server"
 
-$PORT
-
-$CLICKHOUSE_BINARY server -- --tcp_port "$CLICKHOUSE_PORT_TCP" --path /tmp/ > server.log 2>&1 &
+$CLICKHOUSE_BINARY server -- --tcp_port "$CLICKHOUSE_PORT_TCP" --path "${CLICKHOUSE_TMP}/" > "${CLICKHOUSE_TMP}/server.log" 2>&1 &
 PID=$!
 
-function finish {
+function finish()
+{
     kill $PID
     wait
 }
@@ -27,14 +26,14 @@ for i in {1..30}; do
     sleep 1
     $CLICKHOUSE_CLIENT --query "SELECT 1" 2>/dev/null && break
     if [[ $i == 30 ]]; then
-        cat server.log
+        cat "${CLICKHOUSE_TMP}/server.log"
         exit 1
     fi
 done
 
 # Check access rights
 
-$CLICKHOUSE_CLIENT -n --query "
+$CLICKHOUSE_CLIENT --query "
     DROP DATABASE IF EXISTS test;
     CREATE DATABASE test;
     USE test;
@@ -47,4 +46,15 @@ $CLICKHOUSE_CLIENT -n --query "
     CREATE TEMPORARY TABLE t (s String);
     INSERT INTO t VALUES ('World');
     SELECT * FROM t;
-";
+"
+
+kill $PID
+# Dump server.log in case wait hangs
+function trace()
+{
+    # clickhouse-test prints only stderr on timeouts
+    cat "${CLICKHOUSE_TMP}/server.log" >&2
+}
+trap trace EXIT
+wait
+trap '' EXIT
