@@ -11,8 +11,8 @@ CURDIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
 NUM_REPLICAS=6
 
 for i in $(seq 1 $NUM_REPLICAS); do
-    $CLICKHOUSE_CLIENT -n -q "
-        DROP TABLE IF EXISTS r$i;
+    $CLICKHOUSE_CLIENT -q "
+        DROP TABLE IF EXISTS r$i SYNC;
         CREATE TABLE r$i (x UInt64) ENGINE = ReplicatedMergeTree('/clickhouse/tables/$CLICKHOUSE_TEST_ZOOKEEPER_PREFIX/parallel_quorum_many', 'r$i') ORDER BY x;
     "
 done
@@ -32,19 +32,19 @@ for i in $(seq 1 $NUM_REPLICAS); do
 
         # Note: making 30 connections simultaneously is a mini-DoS when server is build with sanitizers and CI environment is overloaded.
         # That's why we repeat "socket timeout" errors.
-        thread $i $a 2>&1 | grep -v -P 'SOCKET_TIMEOUT|NETWORK_ERROR|^$' &
+        thread $i $a 2>&1 | tr '\n' ' ' | grep -v -P 'SOCKET_TIMEOUT|NETWORK_ERROR|Timeout while waiting for quorum|^$' &
     done
 done
 
 wait
 
 for i in $(seq 1 $NUM_REPLICAS); do
-    $CLICKHOUSE_CLIENT -n -q "
+    $CLICKHOUSE_CLIENT -q "
         SYSTEM SYNC REPLICA r$i;
         SELECT count(), min(x), max(x), sum(x) FROM r$i;
     "
 done
 
 for i in $(seq 1 $NUM_REPLICAS); do
-    $CLICKHOUSE_CLIENT -n -q "DROP TABLE IF EXISTS r$i SYNC;"
+    $CLICKHOUSE_CLIENT -q "DROP TABLE IF EXISTS r$i SYNC;"
 done
