@@ -8,6 +8,14 @@ CREATE TABLE visits
 )
 ENGINE = Null;
 
+CREATE TABLE goal
+(
+     `CounterID` UInt32,
+     `StartDate` Date,
+     `GoalID` UInt32,
+     `Visits` AggregateFunction(sumIf, Int8, UInt8),
+     `GoalReaches` AggregateFunction(sum, Int8)
+) ENGINE = AggregatingMergeTree PARTITION BY toStartOfMonth(StartDate) ORDER BY (CounterID, StartDate, GoalID) SETTINGS index_granularity = 256, index_granularity_bytes = '10Mi';
 
 CREATE MATERIALIZED VIEW goal_view TO goal
 (
@@ -36,15 +44,6 @@ ORDER BY
     StartDate ASC,
     GoalID ASC;
 
-CREATE TABLE goal
-(
-     `CounterID` UInt32,
-     `StartDate` Date,
-     `GoalID` UInt32,
-     `Visits` AggregateFunction(sumIf, Int8, UInt8),
-     `GoalReaches` AggregateFunction(sum, Int8)
-) ENGINE = AggregatingMergeTree PARTITION BY toStartOfMonth(StartDate) ORDER BY (CounterID, StartDate, GoalID) SETTINGS index_granularity = 256;
-
 INSERT INTO visits (`CounterID`,`StartDate`,`StartTime`,`Sign`,`GoalsID`) VALUES (1, toDate('2000-01-01'), toDateTime(toDate('2000-01-01')), 1, [1]);
 
 DROP TABLE goal;
