@@ -1,8 +1,12 @@
+-- Tags: no-fasttest
+
 DROP TABLE IF EXISTS select_final;
 
+SET allow_asynchronous_read_from_io_pool_for_merge_tree = 0;
 SET do_not_merge_across_partitions_select_final = 1;
+SET max_threads = 16;
 
-CREATE TABLE select_final (t DateTime, x Int32, string String) ENGINE = ReplacingMergeTree() PARTITION BY toYYYYMM(t) ORDER BY (x, t); 
+CREATE TABLE select_final (t DateTime, x Int32, string String) ENGINE = ReplacingMergeTree() PARTITION BY toYYYYMM(t) ORDER BY (x, t) SETTINGS index_granularity = 8192, index_granularity_bytes = '10Mi';
 
 INSERT INTO select_final SELECT toDate('2000-01-01'), number, '' FROM numbers(2);
 INSERT INTO select_final SELECT toDate('2000-01-01'), number + 1, '' FROM numbers(2);
@@ -10,8 +14,7 @@ INSERT INTO select_final SELECT toDate('2000-01-01'), number + 1, '' FROM number
 INSERT INTO select_final SELECT toDate('2020-01-01'), number, '' FROM numbers(2);
 INSERT INTO select_final SELECT toDate('2020-01-01'), number + 1, '' FROM numbers(2);
 
-
-SELECT * FROM select_final FINAL ORDER BY x;
+SELECT * FROM select_final FINAL ORDER BY x, t;
 
 TRUNCATE TABLE select_final;
 
@@ -28,13 +31,7 @@ SELECT max(x) FROM select_final FINAL where string = 'updated';
 TRUNCATE TABLE select_final;
 
 INSERT INTO select_final SELECT toDate('2000-01-01'), number, '' FROM numbers(500000);
-
 OPTIMIZE TABLE select_final FINAL;
-
 SELECT max(x) FROM select_final FINAL;
 
-SYSTEM FLUSH LOGS;
-
-SELECT length(thread_ids) FROM system.query_log WHERE query='SELECT max(x) FROM select_final FINAL;' AND type='QueryFinish' AND current_database = currentDatabase() ORDER BY event_time DESC LIMIT 1;
-
 DROP TABLE select_final;
