@@ -1,5 +1,6 @@
-SET allow_experimental_map_type = 1;
+-- Tags: log-engine
 SET output_format_write_statistics = 0;
+SET output_format_json_pretty_print = 0;
 
 DROP TABLE IF EXISTS map_formats;
 CREATE TABLE map_formats (m Map(String, UInt32), m1 Map(String, Date), m2 Map(String, Array(UInt32))) ENGINE = Log;
