@@ -1,3 +1,13 @@
+-- Tags: long
+
+SET enable_analyzer = 1;
+
+-- Too slow
+SET max_bytes_before_external_sort = 0;
+SET max_bytes_ratio_before_external_sort = 0;
+SET max_bytes_before_external_group_by = 0;
+SET max_bytes_ratio_before_external_group_by = 0;
+
 -- { echo }
 
 -- just something basic
@@ -7,7 +17,7 @@ select number, count() over (partition by intDiv(number, 3) order by number rows
 select number, max(number) over (partition by intDiv(number, 3) order by number desc rows unbounded preceding) from numbers(10) settings max_block_size = 2;
 
 -- not a window function
-select number, abs(number) over (partition by toString(intDiv(number, 3)) rows unbounded preceding) from numbers(10); -- { serverError 63 }
+select number, abs(number) over (partition by toString(intDiv(number, 3)) rows unbounded preceding) from numbers(10); -- { serverError UNKNOWN_AGGREGATE_FUNCTION }
 
 -- no partition by
 select number, avg(number) over (order by number rows unbounded preceding) from numbers(10);
@@ -18,9 +28,8 @@ select number, quantileExact(number) over (partition by intDiv(number, 3) AS val
 -- can add an alias after window spec
 select number, quantileExact(number) over (partition by intDiv(number, 3) AS value order by number rows unbounded preceding) q from numbers(10);
 
--- can't reference it yet -- the window functions are calculated at the
--- last stage of select, after all other functions.
-select q * 10, quantileExact(number) over (partition by intDiv(number, 3) rows unbounded preceding) q from numbers(10); -- { serverError 47 }
+-- now we should be able to compute expressions with window functions
+select number, q * 10, quantileExact(number) over (partition by intDiv(number, 3) order by number rows unbounded preceding) q from numbers(10) order by number;
 
 -- must work in WHERE if you wrap it in a subquery
 select * from (select count(*) over (rows unbounded preceding) c from numbers(3)) where c > 0;
@@ -376,7 +385,7 @@ settings max_block_size = 3;
 
 -- careful with auto-application of Null combinator
 select lagInFrame(toNullable(1)) over ();
-select lagInFrameOrNull(1) over (); -- { serverError 36 }
+select lagInFrameOrNull(1) over (); -- { serverError BAD_ARGUMENTS }
 -- this is the same as `select max(Null::Nullable(Nothing))`
 select intDiv(1, NULL) x, toTypeName(x), max(x) over ();
 -- to make lagInFrame return null for out-of-frame rows, cast the argument to
@@ -401,6 +410,47 @@ window w as (order by number range between 1 preceding and 1 following)
 order by number
 ;
 
+-- nth_value without specific frame range given
+select
+    number,
+    nth_value(number, 1) over w as firstValue,
+    nth_value(number, 2) over w as secondValue,
+    nth_value(number, 3) over w as thirdValue,
+    nth_value(number, 4) over w as fourthValue
+from numbers(10)
+window w as (order by number)
+order by number
+;
+
+-- nth_value with frame range specified
+select
+    number,
+    nth_value(number, 1) over w as firstValue,
+    nth_value(number, 2) over w as secondValue,
+    nth_value(number, 3) over w as thirdValue,
+    nth_value(number, 4) over w as fourthValue
+from numbers(10)
+window w as (order by number range between 1 preceding and 1 following)
+order by number
+;
+
+-- to make nth_value return null for out-of-frame rows, cast the argument to
+-- Nullable; otherwise, it returns default values.
+SELECT
+    number,
+    nth_value(toNullable(number), 1) OVER w as firstValue,
+    nth_value(toNullable(number), 3) OVER w as thridValue
+FROM numbers(5)
+WINDOW w AS (ORDER BY number ASC)
+;
+
+-- nth_value UBsan
+SELECT nth_value(1, -1) OVER (); -- { serverError BAD_ARGUMENTS }
+SELECT nth_value(1, 0) OVER (); -- { serverError BAD_ARGUMENTS }
+SELECT nth_value(1, /* INT64_MAX+1 */ 0x7fffffffffffffff+1) OVER (); -- { serverError BAD_ARGUMENTS }
+SELECT nth_value(1, /* INT64_MAX */ 0x7fffffffffffffff) OVER ();
+SELECT nth_value(1, 1) OVER ();
+
 -- lagInFrame UBsan
 SELECT lagInFrame(1, -1) OVER (); -- { serverError BAD_ARGUMENTS }
 SELECT lagInFrame(1, 0) OVER ();
@@ -415,24 +465,33 @@ SELECT leadInFrame(1, /* INT64_MAX+1 */ 0x7fffffffffffffff+1) OVER (); -- { serv
 SELECT leadInFrame(1, /* INT64_MAX */ 0x7fffffffffffffff) OVER ();
 SELECT leadInFrame(1, 1) OVER ();
 
+-- nth_value Msan
+SELECT nth_value(1, '') OVER (); -- { serverError BAD_ARGUMENTS }
+
+-- lagInFrame Msan
+SELECT lagInFrame(1, '') OVER (); -- { serverError BAD_ARGUMENTS }
+
+-- leadInFrame Msan
+SELECT leadInFrame(1, '') OVER (); -- { serverError BAD_ARGUMENTS }
+
 -- In this case, we had a problem with PartialSortingTransform returning zero-row
 -- chunks for input chunks w/o columns.
 select count() over () from numbers(4) where number < 2;
 
 -- floating point RANGE frame
 select
-    count(*) over (order by toFloat32(number) range 5. preceding),
-    count(*) over (order by toFloat64(number) range 5. preceding),
-    count(*) over (order by toFloat32(number) range between current row and 5. following),
-    count(*) over (order by toFloat64(number) range between current row and 5. following)
+    count(*) over (order by toFloat32(number) range 5 preceding),
+    count(*) over (order by toFloat64(number) range 5 preceding),
+    count(*) over (order by toFloat32(number) range between current row and 5 following),
+    count(*) over (order by toFloat64(number) range between current row and 5 following)
 from numbers(7)
 ;
 
 -- negative offsets should not be allowed
-select count() over (order by toInt64(number) range between -1 preceding and unbounded following) from numbers(1); -- { serverError 36 }
-select count() over (order by toInt64(number) range between -1 following and unbounded following) from numbers(1); -- { serverError 36 }
-select count() over (order by toInt64(number) range between unbounded preceding and -1 preceding) from numbers(1); -- { serverError 36 }
-select count() over (order by toInt64(number) range between unbounded preceding and -1 following) from numbers(1); -- { serverError 36 }
+select count() over (order by toInt64(number) range between -1 preceding and unbounded following) from numbers(1); -- { serverError BAD_ARGUMENTS }
+select count() over (order by toInt64(number) range between -1 following and unbounded following) from numbers(1); -- { serverError BAD_ARGUMENTS }
+select count() over (order by toInt64(number) range between unbounded preceding and -1 preceding) from numbers(1); -- { serverError BAD_ARGUMENTS }
+select count() over (order by toInt64(number) range between unbounded preceding and -1 following) from numbers(1); -- { serverError BAD_ARGUMENTS }
 
 -- a test with aggregate function that allocates memory in arena
 select sum(a[length(a)])
@@ -442,9 +501,29 @@ from (
     from numbers_mt(10000)
 ) settings max_block_size = 7;
 
+-- a test with aggregate function which is -state type
+select bitmapCardinality(bs)
+from
+    (
+        select groupBitmapMergeState(bm) over (order by k asc rows between unbounded preceding and current row) as bs
+        from
+            (
+                select
+                    groupBitmapState(number) as bm, k
+                from
+                    (
+                        select
+                            number,
+                            number % 3 as k
+                        from numbers(3)
+                    )
+                group by k
+            )
+    );
+
 -- -INT_MIN row offset that can lead to problems with negation, found when fuzzing
 -- under UBSan. Should be limited to at most INT_MAX.
-select count() over (rows between 2147483648 preceding and 2147493648 following) from numbers(2); -- { serverError 36 }
+select count() over (rows between 2147483648 preceding and 2147493648 following) from numbers(2); -- { serverError BAD_ARGUMENTS }
 
 -- Somehow in this case WindowTransform gets empty input chunks not marked as
 -- input end, and then two (!) empty input chunks marked as input end. Whatever.
@@ -461,16 +540,16 @@ order by p, o, number
 ;
 
 -- can't redefine PARTITION BY
-select count() over (w partition by number) from numbers(1) window w as (partition by intDiv(number, 5)); -- { serverError 36 }
+select count() over (w partition by number) from numbers(1) window w as (partition by intDiv(number, 5)); -- { serverError BAD_ARGUMENTS }
 
 -- can't redefine existing ORDER BY
-select count() over (w order by number) from numbers(1) window w as (partition by intDiv(number, 5) order by mod(number, 3)); -- { serverError 36 }
+select count() over (w order by number) from numbers(1) window w as (partition by intDiv(number, 5) order by mod(number, 3)); -- { serverError BAD_ARGUMENTS }
 
 -- parent window can't have frame
-select count() over (w range unbounded preceding) from numbers(1) window w as (partition by intDiv(number, 5) order by mod(number, 3) rows unbounded preceding); -- { serverError 36 }
+select count() over (w range unbounded preceding) from numbers(1) window w as (partition by intDiv(number, 5) order by mod(number, 3) rows unbounded preceding); -- { serverError BAD_ARGUMENTS }
 
 -- looks weird but probably should work -- this is a window that inherits and changes nothing
 select count() over (w) from numbers(1) window w as ();
 
 -- nonexistent parent window
-select count() over (w2 rows unbounded preceding); -- { serverError 36 }
+select count() over (w2 rows unbounded preceding); -- { serverError BAD_ARGUMENTS }
