@@ -1,7 +1,7 @@
 DROP TABLE IF EXISTS skip_idx_comp_parts;
 CREATE TABLE skip_idx_comp_parts (a Int, b Int, index b_idx b TYPE minmax GRANULARITY 4)
     ENGINE = MergeTree ORDER BY a
-    SETTINGS index_granularity=256, merge_max_block_size=100;
+    SETTINGS index_granularity=256, index_granularity_bytes = '10Mi', merge_max_block_size=100;
 
 SYSTEM STOP MERGES skip_idx_comp_parts;
 
