@@ -6,7 +6,7 @@ DROP TABLE IF EXISTS distributed;
 CREATE TABLE local (x UInt8) ENGINE = Memory;
 CREATE TABLE distributed AS local ENGINE = Distributed(test_cluster_two_shards, currentDatabase(), local, x);
 
-SET insert_distributed_sync = 1;
+SET distributed_foreground_insert = 1;
 
 INSERT INTO distributed SELECT number FROM numbers(256) WHERE number % 2 = 0;
 SELECT count() FROM local;
