@@ -10,3 +10,6 @@ SELECT h3GetBaseCell(0xFFFFFFFFFFFFFF) FORMAT Null;
 SELECT h3GetResolution(0xFFFFFFFFFFFFFF) FORMAT Null;
 SELECT h3kRing(0xFFFFFFFFFFFFFF, toUInt16(10)) FORMAT Null;
 SELECT h3ToGeo(0xFFFFFFFFFFFFFF) FORMAT Null;
+SELECT h3HexRing(0xFFFFFFFFFFFFFF, toUInt16(10)) FORMAT Null; -- { serverError INCORRECT_DATA }
+SELECT h3HexRing(0xFFFFFFFFFFFFFF, toUInt16(10000)) FORMAT Null; -- { serverError INCORRECT_DATA }
+SELECT length(h3HexRing(581276613233082367, toUInt16(1))) FORMAT Null;
