@@ -1,3 +1,6 @@
+SET allow_deprecated_error_prone_window_functions = 1;
+SET output_format_pretty_row_numbers = 0;
+
 SELECT
     neighbor(n, -2) AS int,
     neighbor(s, -2) AS str,
@@ -40,6 +43,7 @@ FROM
     ORDER BY val_string, rowNr
 )
 ORDER BY rowNr, val_string, str_m1, str_p1, val_low, low_m1, low_p1
+SETTINGS output_format_pretty_color=1
 format PrettyCompact;
 
 drop table if exists neighbor_test;
