@@ -16,7 +16,7 @@ INSERT INTO 01759_db.dictionary_source_table VALUES (0, 2, 3), (1, 5, 6), (2, 8,
 CREATE DICTIONARY 01759_db.test_dictionary(key UInt64, value1 UInt64, value1 UInt64)
 PRIMARY KEY key
 SOURCE(CLICKHOUSE(HOST 'localhost' PORT tcpPort() USER 'default' TABLE 'dictionary_source_table' DB '01759_db'))
-LAYOUT(COMPLEX_KEY_DIRECT()); -- {serverError 36}
+LAYOUT(COMPLEX_KEY_DIRECT()); -- {serverError BAD_ARGUMENTS}
 
 CREATE DICTIONARY 01759_db.test_dictionary(key UInt64, value1 UInt64, value2 UInt64)
 PRIMARY KEY key
