@@ -3,8 +3,6 @@ set -ue
 
 # this test doesn't need 'current_database = currentDatabase()',
 
-unset CLICKHOUSE_LOG_COMMENT
-
 CURDIR=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
 # shellcheck source=../shell_config.sh
 . "$CURDIR"/../shell_config.sh
@@ -15,8 +13,8 @@ ${CLICKHOUSE_CLIENT} -q "create table m (dummy UInt8) ENGINE = Distributed('test
 query_id=$(${CLICKHOUSE_CLIENT} -q "select lower(hex(reverse(reinterpretAsString(generateUUIDv4()))))")
 ${CLICKHOUSE_CLIENT} -q "select * from m format Null" "--query_id=$query_id"
 
-${CLICKHOUSE_CLIENT} -n -q "
-system flush logs;
+${CLICKHOUSE_CLIENT} -q "
+system flush logs query_log;
 select
     anyIf(initial_query_start_time, is_initial_query) = anyIf(initial_query_start_time, not is_initial_query),
     anyIf(initial_query_start_time_microseconds, is_initial_query) = anyIf(initial_query_start_time_microseconds, not is_initial_query)
