@@ -4,11 +4,11 @@ CREATE TABLE a
     `x` MATERIALIZED x
 )
 ENGINE = MergeTree
-ORDER BY number; --{ serverError 174}
+ORDER BY number; --{ serverError CYCLIC_ALIASES}
 
 CREATE TABLE foo
 (
     i Int32,
     j ALIAS j + 1
 )
-ENGINE = MergeTree() ORDER BY i; --{ serverError 174}
+ENGINE = MergeTree() ORDER BY i; --{ serverError CYCLIC_ALIASES}
