@@ -12,7 +12,7 @@ PARTITION BY p
 ORDER BY t
 SETTINGS number_of_free_entries_in_pool_to_execute_mutation=0;
 INSERT INTO rep_data VALUES (1, now());
-ALTER TABLE rep_data MATERIALIZE INDEX idx IN PARTITION ID 'NO_SUCH_PART'; -- { serverError 248 }
+ALTER TABLE rep_data MATERIALIZE INDEX idx IN PARTITION ID 'NO_SUCH_PART'; -- { serverError INVALID_PARTITION_VALUE }
 ALTER TABLE rep_data MATERIALIZE INDEX idx IN PARTITION ID '1';
 ALTER TABLE rep_data MATERIALIZE INDEX idx IN PARTITION ID '2';
 
@@ -28,6 +28,6 @@ PARTITION BY p
 ORDER BY t
 SETTINGS number_of_free_entries_in_pool_to_execute_mutation=0;
 INSERT INTO data VALUES (1, now());
-ALTER TABLE data MATERIALIZE INDEX idx IN PARTITION ID 'NO_SUCH_PART'; -- { serverError 248 }
+ALTER TABLE data MATERIALIZE INDEX idx IN PARTITION ID 'NO_SUCH_PART'; -- { serverError INVALID_PARTITION_VALUE }
 ALTER TABLE data MATERIALIZE INDEX idx IN PARTITION ID '1';
 ALTER TABLE data MATERIALIZE INDEX idx IN PARTITION ID '2';
