@@ -25,3 +25,8 @@ SET optimize_group_by_function_keys = 1;
 SELECT round(sum(log(2) * number), 6) AS k FROM numbers(10000)
 GROUP BY (number % 2) * (number % 3), number % 3, number % 2
 HAVING sum(log(2) * number) > 346.57353 ORDER BY k;
+
+SELECT round(sum(log(2) * number), 6) AS k FROM numbers(10000)
+GROUP BY (number % 2) * (number % 3), number % 3, number % 2
+HAVING sum(log(2) * number) > 346.57353 ORDER BY k
+SETTINGS enable_analyzer=1;
