@@ -1,5 +1,5 @@
 SELECT toStartOfWeek(toDateTime('1970-01-01 00:00:00', 'UTC'));
-SELECT toStartOfWeek(toDateTime('1970-01-01 00:00:00', 'Europe/Moscow'));
+SELECT toStartOfWeek(toDateTime('1970-01-01 00:00:00', 'Asia/Istanbul'));
 SELECT toStartOfWeek(toDateTime('1970-01-01 00:00:00', 'Canada/Atlantic'));
 SELECT toStartOfWeek(toDateTime('1970-01-04 00:00:00'));
  
