@@ -1,4 +1,4 @@
 drop table if exists t;
 create table t (x Bool) engine=Memory();
-insert into t values settings bool_true_representation='да' ('да');
+insert into t settings bool_true_representation='да' values ('да');
 drop table t;
