@@ -1,4 +1,5 @@
 -- Tags: memory-engine
+SET create_table_empty_primary_key_by_default = 0;
 SET default_table_engine = 'None';
 
 CREATE TABLE table_02184 (x UInt8); --{serverError ENGINE_REQUIRED}
@@ -8,7 +9,7 @@ SHOW CREATE TABLE table_02184;
 DROP TABLE table_02184;
 
 SET default_table_engine = 'MergeTree';
-CREATE TABLE table_02184 (x UInt8); --{serverError NUMBER_OF_ARGUMENTS_DOESNT_MATCH}
+CREATE TABLE table_02184 (x UInt8); --{serverError BAD_ARGUMENTS}
 CREATE TABLE table_02184 (x UInt8, PRIMARY KEY (x));
 SHOW CREATE TABLE table_02184;
 DROP TABLE table_02184;
