@@ -16,7 +16,7 @@ $CLICKHOUSE_CLIENT --query "CREATE TABLE url ENGINE=URL('https://clickhouse.com'
 
 $CLICKHOUSE_CLIENT  --user=user_test_02184 --password=user_test_02184  --query "CREATE TABLE t AS url" 2>&1| grep -Fo "ACCESS_DENIED" | uniq
 
-$CLICKHOUSE_CLIENT --query "GRANT URL ON *.* TO user_test_02184;"
+$CLICKHOUSE_CLIENT --query "GRANT READ, WRITE ON URL TO user_test_02184;"
 $CLICKHOUSE_CLIENT --user=user_test_02184 --password=user_test_02184  --query "CREATE TABLE t AS url"
 $CLICKHOUSE_CLIENT --query "SHOW CREATE TABLE t"
 $CLICKHOUSE_CLIENT --query "DROP TABLE t"
