@@ -2,15 +2,13 @@
 
 DROP TABLE IF EXISTS t_subcolumns_sizes;
 
-SET allow_experimental_object_type = 1;
-
-CREATE TABLE t_subcolumns_sizes (id UInt64, arr Array(UInt64), n Nullable(String), d Object('json'))
+CREATE TABLE t_subcolumns_sizes (id UInt64, arr Array(UInt64), n Nullable(String))
 ENGINE = MergeTree ORDER BY id
 SETTINGS min_bytes_for_wide_part = 0, serialization_info_version = 'basic';
 
-INSERT INTO t_subcolumns_sizes FORMAT JSONEachRow {"id": 1, "arr": [1, 2, 3], "n": null, "d": {"k1": "v1", "k2": [{"k3": 1, "k4": "v2"}, {"k3": 3}]}}
+INSERT INTO t_subcolumns_sizes FORMAT JSONEachRow {"id": 1, "arr": [1, 2, 3], "n": null}
 
-INSERT INTO t_subcolumns_sizes FORMAT JSONEachRow {"id": 2, "arr": [0], "n": "foo", "d": {"k1": "v3", "k2": [{"k4": "v4"}, {"k3": "v5", "k5": 5}]}}
+INSERT INTO t_subcolumns_sizes FORMAT JSONEachRow {"id": 2, "arr": [0], "n": "foo"}
 
 OPTIMIZE TABLE t_subcolumns_sizes FINAL;
 
