@@ -4,5 +4,5 @@ FROM system.columns
 WHERE (database = 'system') AND
       (comment = '') AND
       (table NOT ILIKE '%log%') AND
-      (table NOT IN ('numbers', 'numbers_mt', 'one', 'generate_series', 'generateSeries', 'coverage_log', 'filesystem_read_prefetches_log', 'custom_metrics', 'custom_metrics_refresher', 'prometheus_metrics')) AND
+      (table NOT IN ('numbers', 'numbers_mt', 'one', 'generate_series', 'generateSeries', 'coverage_log', 'filesystem_read_prefetches_log', 'custom_metrics', 'custom_metrics_refresher', 'prometheus_metrics', 'unicode')) AND
       (default_kind != 'ALIAS');
