@@ -1,6 +1,4 @@
-#ifdef HAS_RESERVED_IDENTIFIER
 #pragma clang diagnostic ignored "-Wreserved-identifier"
-#endif
 
 #include <memory>
 #include <cstddef>
@@ -72,7 +70,7 @@ uint64_t test(uint8_t * dst, uint8_t * src, size_t size, size_t iterations, size
                     iteration % 2 ? &dst[begin] : &src[begin],
                     end - begin,
                     [rng = RNG(), &generator]() mutable { return generator(rng); },
-                    std::forward<MemcpyImpl>(impl));
+                    std::forward<MemcpyImpl>(impl));  /// NOLINT(bugprone-use-after-move,hicpp-invalid-access-moved)
             }
         });
     }
@@ -190,7 +188,8 @@ static void * memcpySSE2Unrolled2(void * __restrict destination, const void * __
     }
 
     // medium size copy
-    __m128i c0, c1;
+    __m128i c0;
+    __m128i c1;
 
     for (; size >= 32; size -= 32)
     {
@@ -229,7 +228,10 @@ static void * memcpySSE2Unrolled4(void * __restrict destination, const void * __
     }
 
     // medium size copy
-    __m128i c0, c1, c2, c3;
+    __m128i c0;
+    __m128i c1;
+    __m128i c2;
+    __m128i c3;
 
     for (; size >= 64; size -= 64)
     {
@@ -273,7 +275,14 @@ static void * memcpySSE2Unrolled8(void * __restrict destination, const void * __
     }
 
     // medium size copy
-    __m128i c0, c1, c2, c3, c4, c5, c6, c7;
+    __m128i c0;
+    __m128i c1;
+    __m128i c2;
+    __m128i c3;
+    __m128i c4;
+    __m128i c5;
+    __m128i c6;
+    __m128i c7;
 
     for (; size >= 128; size -= 128)
     {
@@ -319,7 +328,14 @@ memcpy_my_medium_sse(uint8_t * __restrict & dst, const uint8_t * __restrict & sr
     }
 
     /// Aligned unrolled copy.
-    __m128i c0, c1, c2, c3, c4, c5, c6, c7;
+    __m128i c0;
+    __m128i c1;
+    __m128i c2;
+    __m128i c3;
+    __m128i c4;
+    __m128i c5;
+    __m128i c6;
+    __m128i c7;
 
     while (size >= 128)
     {
@@ -360,7 +376,14 @@ void memcpy_my_medium_avx(uint8_t * __restrict & __restrict dst, const uint8_t *
         size -= padding;
     }
 
-    __m256i c0, c1, c2, c3, c4, c5, c6, c7;
+    __m256i c0;
+    __m256i c1;
+    __m256i c2;
+    __m256i c3;
+    __m256i c4;
+    __m256i c5;
+    __m256i c6;
+    __m256i c7;
 
     while (size >= 256)
     {
@@ -532,7 +555,14 @@ tail:
             }
 
             /// Aligned unrolled copy.
-            __m128i c0, c1, c2, c3, c4, c5, c6, c7;
+            __m128i c0;
+            __m128i c1;
+            __m128i c2;
+            __m128i c3;
+            __m128i c4;
+            __m128i c5;
+            __m128i c6;
+            __m128i c7;
 
             while (size >= 128)
             {
@@ -619,7 +649,14 @@ static uint8_t * memcpy_my2(uint8_t * __restrict dst, const uint8_t * __restrict
         }
 
         /// Aligned unrolled copy.
-        __m128i c0, c1, c2, c3, c4, c5, c6, c7;
+        __m128i c0;
+        __m128i c1;
+        __m128i c2;
+        __m128i c3;
+        __m128i c4;
+        __m128i c5;
+        __m128i c6;
+        __m128i c7;
 
         while (size >= 128)
         {
@@ -673,7 +710,7 @@ static uint8_t * memcpy_my2(uint8_t * __restrict dst, const uint8_t * __restrict
             size -= padding;
         }
 
-        while (size >= 512)
+        while (size >= 512) /// NOLINT
         {
             __asm__(
                 "vmovups    (%[s]), %%ymm0\n"
@@ -794,19 +831,19 @@ static uint8_t * memcpy_my2(uint8_t * __restrict dst, const uint8_t * __restrict
     return ret;
 }
 
-extern "C" void * __memcpy_erms(void * __restrict destination, const void * __restrict source, size_t size);
-extern "C" void * __memcpy_sse2_unaligned(void * __restrict destination, const void * __restrict source, size_t size);
-extern "C" void * __memcpy_ssse3(void * __restrict destination, const void * __restrict source, size_t size);
-extern "C" void * __memcpy_ssse3_back(void * __restrict destination, const void * __restrict source, size_t size);
-extern "C" void * __memcpy_avx_unaligned(void * __restrict destination, const void * __restrict source, size_t size);
-extern "C" void * __memcpy_avx_unaligned_erms(void * __restrict destination, const void * __restrict source, size_t size);
-extern "C" void * __memcpy_avx512_unaligned(void * __restrict destination, const void * __restrict source, size_t size);
-extern "C" void * __memcpy_avx512_unaligned_erms(void * __restrict destination, const void * __restrict source, size_t size);
-extern "C" void * __memcpy_avx512_no_vzeroupper(void * __restrict destination, const void * __restrict source, size_t size);
+extern "C" void * __memcpy_erms(void * __restrict destination, const void * __restrict source, size_t size); /// NOLINT
+extern "C" void * __memcpy_sse2_unaligned(void * __restrict destination, const void * __restrict source, size_t size); /// NOLINT
+extern "C" void * __memcpy_ssse3(void * __restrict destination, const void * __restrict source, size_t size); /// NOLINT
+extern "C" void * __memcpy_ssse3_back(void * __restrict destination, const void * __restrict source, size_t size); /// NOLINT
+extern "C" void * __memcpy_avx_unaligned(void * __restrict destination, const void * __restrict source, size_t size); /// NOLINT
+extern "C" void * __memcpy_avx_unaligned_erms(void * __restrict destination, const void * __restrict source, size_t size); /// NOLINT
+extern "C" void * __memcpy_avx512_unaligned(void * __restrict destination, const void * __restrict source, size_t size); /// NOLINT
+extern "C" void * __memcpy_avx512_unaligned_erms(void * __restrict destination, const void * __restrict source, size_t size); /// NOLINT
+extern "C" void * __memcpy_avx512_no_vzeroupper(void * __restrict destination, const void * __restrict source, size_t size); /// NOLINT
 
 
 #define VARIANT(N, NAME) \
-    if (memcpy_variant == N) \
+    if (memcpy_variant == (N)) \
         return test(dst, src, size, iterations, num_threads, std::forward<F>(generator), NAME, #NAME);
 
 template <typename F>
